#include "patch.h"

void FreeBrkStructures()
{
	FreePalette();
	FreeTextures(&pwt);
	FreeVTex();
	FreeDir(&pdir);
	FreeResourceDefinition(&ResDef);
	FreeResourceDefinition(&pResDef);
	CloseStreams();
}

// Open the control files, make the output wad

int brkmain(int argc,char **argv)
{
	int i,j,n,p;
	char tname[256];
	int cleared=0;

	dprintf(logfile,"\n");

	resources = NULL;
	nresources = 0;

	CopyEnvironmentCLArgs("BRK",argc,argv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;
	}

	if (nclfiles<1)
	{
		dprintf(logfile,"\nBRK.EXE %s by Jim Flynn, Copyright 1996-7  -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"\nBreak a wad into its component resources in subdirectories\n");
		dprintf(logfile,"A control file (.RES) to reconstruct the wad is also generated\n");
		dprintf(logfile,"\nUsage: BRK [options] wadfile[.wad] [resname]...\n");
		dprintf(logfile,"\n Resource selection option:\n");
		dprintf(logfile," +XYZ extracts only XYZ resources, -XYZ extracts all BUT XYZ resources.\n");
		dprintf(logfile," L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
		dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
		dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
		dprintf(logfile,"\n General options, use SET BRK=options to set defaults:\n");
		dprintf(logfile," /O:filename  Specify output resource text file name\n");
		dprintf(logfile," /A Additional unmarked patches in PNAMES extracted\n");
		dprintf(logfile," /I:name[.WAD] Specify wad containing palette\n");
		dprintf(logfile," /J[:filename] Just output .RES file, do not extract resources.\n");
		dprintf(logfile,"\n Game definition options, use SET BRK=options to set defaults:\n");
		dprintf(logfile," /D:name[.DEF] Specify resource definitions file\n");
		dprintf(logfile," /DOOM2 /DOOM /HERETIC /HEXEN specify game\n");
		dprintf(logfile,"\n See BRK.TXT for details.\n");
						 
		FreeCLStrs();
		return(1);
	}

	cleared=0;
	strcpy(outname,"");
	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfLWTEMGSPF");
		if (n==strlen(cloptions[i]))
		{
			if (!cleared)	// if first selection option
			{
				if (cloptsense[i])	// clear all if it adds types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=0;
					DoGraphics=DoSprites=DoPatches=DoFlats=DoNonStandard=0;
				}
				else if (!cloptsense[i]) // set all if it subtracts types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=1;
					DoGraphics=DoSprites=DoPatches=DoFlats=DoNonStandard=1;
				}
				cleared=1;	// but don't do it for following selection opts
			}
			for (j=0;j<strlen(cloptions[i]);j++)
			{
				switch(tolower(cloptions[i][j]))
				{
					case 'l':
						DoLumps = cloptsense[i];
						break;
					case 'w':
						DoWads = cloptsense[i];
						break;
					case 't':
						DoTextures = cloptsense[i];
						break;
					case 'e':
						DoEffects = cloptsense[i];
						break;
					case 'm':
						DoMusics = cloptsense[i];
						break;
					case 'g':
						DoGraphics = cloptsense[i];
						break;
					case 's':
						DoSprites = cloptsense[i];
						break;
					case 'p':
						DoPatches = cloptsense[i];
						break;
					case 'f':
						DoFlats = cloptsense[i];
						break;
					case 'u':
						DoNonStandard = cloptsense[i];
						break;
				}
			}
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
			strcpy(defname,cloptions[i]);
		else if (tolower(cloptions[i][0])=='v')
			Verbose = 1;
		else if (tolower(cloptions[i][0])=='o' && cloptions[i][1]==':')
			strcpy(outname,cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='j')
		{
			if (cloptions[i][1]==':')
				strcpy(outname,cloptions[i]+2);
			JustRes = cloptsense[i];
		}
		else if (tolower(cloptions[i][0])=='a')
			AdditionalPatches = 1;
		else if (tolower(cloptions[i][0])=='d' && cloptions[i][1]==':')
			strcpy(defname,cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='i' && cloptions[i][1]==':')
			strcpy(iwadname,cloptions[i]+2);
		else dprintf(logfile,"Warn: command line option: %s not understood\n",cloptions[i]);
	}
	strcpy(pwadname,clfiles[0]);

	// get the specific resources to extract if any
	for (i=1;i<nclfiles;i++)
		AddDynStr(&resources,&nresources,clfiles[i]);

	FreeCLStrs();

	AddExt(pwadname,".WAD");
	AddExt(iwadname,".WAD");
	strcpy(tname,*outname=='\0'? pwadname : outname);
	AddExt(tname,".RES");
	AddExt(defname,".DEF");

	InitCategories(defname,&ResDef);

	pst = fopen(pwadname,"rb");
	if (pst!=NULL)
	{
		fread(&phdr,sizeof(WadHeader),1,pst);
		ReadDir(&phdr,&pdir,pst);
		LocateCategories(&pdir,&ResDef,&pResDef);

		// Locate a palette to be inserted in BMPs output

		p = IsIn("PLAYPAL",&pdir);				// try the wad first
		if (p>=0)
		{
			dprintf(logfile,"Using PLAYPAL from %s for palette\n",pwadname);
			palette = GetPalette(&pdir,pst,p);
		}
		else if (*iwadname)						// then the IWAD, if spec'd
		{
			FILE *ist;
			WadHeader ihdr;
			WadDir idir;

			InitDir(&idir);
			ist = fopen(iwadname,"rb");
			if (ist!=NULL)
			{
				dprintf(logfile,"Using PLAYPAL from WAD %s for palette\n",iwadname);
				fread(&ihdr,sizeof(WadHeader),1,ist);
				ReadDir(&ihdr,&idir,ist);
				p = IsIn("PLAYPAL",&idir);
				if (p>=0)
					palette = GetPalette(&idir,ist,p);
				fclose(ist);
				FreeDir(&idir);
			}
		}
		else palette = GetPalette(NULL,NULL,-1);

		ReadPNames(&pdir,pst,&pwt);
		ReadTextures(&pdir,pst,&pwt);

		errorlev = WriteResourceControl(tname);
	}

	FreeDynStr(&resources,&nresources);
	return errorlev;
}

