#include "patch.h"

/************************************************************
 BRKSUBS.C
			 /-------------------------\
			||     BRK subroutines     ||
			 \-------------------------/
									
************************************************************/

// Extract a generalized lump wad to a .BMP

void OutputLump(WadDir *dir,int idx,char *dirn,char *extn)
{
	char *buffer = NULL,resn[16];
	char path[256];
	int n,o;
	FILE *st;

	if (idx<0) return;
	n = dir->dirp[idx].reslen;
	o = dir->dirp[idx].resptr;
	strcpy(resn,Str8(dir->dirp[idx].resname));

	strcpy(path,dirn);
	strcat(path,"\\");
	strcat(path,resn);
	strcat(path,extn);
	MakeDir(dirn,(mode_t)0777);
	
	st = fopen(path,"wb");
	if (st!=NULL)
	{
		if (n>0)
		{
			buffer = malloc(n);
			fseek(pst,o,SEEK_SET);
			fread(buffer,n,1,pst);

			SafeWrite(buffer,n,1,st);
			fclose(st);
			free(buffer);
		}
	}
	else dprintf(logfile,"ERROR: Can't write lump %s\n",path);
}

// Extract a level from a wad to a level only .WAD

void OutputWad(WadDir *dir,int idx,char *dirn,char *extn)
{
	char *buffer = NULL,resn[16];
	char path[256];
	int n,o,offs=0;
	FILE *st;
	WadHeader wh;
	WadDir wd;

	if (idx<0) return;

	n = dir->dirp[idx].reslen;
	o = dir->dirp[idx].resptr;
	strcpy(resn,Str8(dir->dirp[idx].resname));

	strcpy(path,dirn);
	strcat(path,"\\");
	strcat(path,resn);
	strcat(path,extn);
	MakeDir(dirn,(mode_t)0777);
	
	st = fopen(path,"wb");
	if (st!=NULL)
	{
		InitDir(&wd);

		strncpy(wh.wadtag,"PWAD",4);
		wh.waddiraddr=0;
		wh.waddirlen=0;
		SafeWrite(&wh,sizeof(WadHeader),1,st);
		offs = sizeof(WadHeader);
		AddDirEntry(&wd,offs,0,resn);
		idx++;

		while (idx<dir->nentries && IsMapResource(dir->dirp+idx))
		{
			n = dir->dirp[idx].reslen;
			o = dir->dirp[idx].resptr;
			strcpy(resn,Str8(dir->dirp[idx].resname));

			if (n>0)
			{
				buffer = malloc(n);
				fseek(pst,o,SEEK_SET);
				fread(buffer,n,1,pst);

				SafeWrite(buffer,n,1,st);
				AddDirEntry(&wd,offs,n,resn);
				offs+=n;

				free(buffer);
			}
			idx++;
		}
		SafeWrite(wd.dirp,sizeof(WadDirEntry),wd.nentries,st);
		fseek(st,0,SEEK_SET);
		wh.waddirlen = wd.nentries;
		wh.waddiraddr = offs;
		SafeWrite(&wh,sizeof(WadHeader),1,st);
		fclose(st);
		FreeDir(&wd);
	}
	else dprintf(logfile,"Can't write level %s\n",path);
}

int IsWaveResource(WadDir *dir,FILE *st,int idx)
{
	RawHeader rh;

	fseek(st,dir->dirp[idx].resptr,SEEK_SET);
	fread(&rh,sizeof(RawHeader),1,st);
	return rh.three==3 && rh.zero==0;
}

// Extract a sound effect from a wad to a .WAV

void OutputWave(WadDir *dir,int idx,char *dirn,char *extn)
{
	char *buffer = NULL,resn[16];
	char path[256];
	int n,o;
	FILE *st;
	RawHeader rh;
	WaveHeader wh;

	if (idx<0) return;
	n = dir->dirp[idx].reslen;
	o = dir->dirp[idx].resptr;
	strcpy(resn,Str8(dir->dirp[idx].resname));

	strcpy(path,dirn);
	strcat(path,"\\");
	strcat(path,resn);
	strcat(path,extn);
	MakeDir(dirn,(mode_t)0777);
	
	st = fopen(path,"wb");
	if (st!=NULL)
	{
		if (n>0)
		{
			assert(pst);
			fseek(pst,o,SEEK_SET);
			fread(&rh,sizeof(RawHeader),1,pst);
			n -= sizeof(RawHeader);

			buffer = malloc(n);

			fread(buffer,n,1,pst);

			strncpy(wh.riff,"RIFF",4);
			wh.flen = n+36;
			strncpy(wh.wave,"WAVEfmt ",8);
			wh.a0 = 16;
			wh.a1 = 1;
			wh.a2 = 1;
			wh.s1 = rh.srate;
			wh.s2 = rh.srate;
			wh.a3 = 1;
			wh.a4 = 8;
			strncpy(wh.data,"data",4);
			wh.nsamps = n;

			SafeWrite(&wh,sizeof(WaveHeader),1,st);
			SafeWrite(buffer,n,1,st);
			free(buffer);
			fclose(st);
		}
	}
	else dprintf(logfile,"Can't write sound %s\n",path);
}

// Extract a Graphic, Patch, or Sprite from a wad to a .BMP

void OutputPicture(WadDir *dir,int idx,char *dirn,char *extn,short *x,short *y)
{
	char resn[16];
	char path[256];
	int n,o;
	Bitmap out;
	int xo,yo;

	if (idx<0) return;
	n = dir->dirp[idx].reslen;
	o = dir->dirp[idx].resptr;
	strcpy(resn,Str8(dir->dirp[idx].resname));

	strcpy(path,dirn);
	strcat(path,"\\");
	strcat(path,resn);
	strcat(path,extn);
	MakeDir(dirn,(mode_t)0777);

	if (n>0)
	{
		Wad2BMP(&out,&xo,&yo,n,o,pst);
		*x = xo; *y = yo;
		WriteBMP(&out,path,palette);
		free(out.bmp);
	}
}

// Extract a flat from a wad to a .BMP

void OutputFlat(WadDir *dir,int idx,char *dirn,char *extn)
{
	char resn[16],path[256];
	int n,o;
	Bitmap out;

	if (idx<0) return;
	n = dir->dirp[idx].reslen;
	o = dir->dirp[idx].resptr;
	strcpy(resn,Str8(dir->dirp[idx].resname));

	strcpy(path,dirn);
	strcat(path,"\\");
	strcat(path,resn);
	strcat(path,extn);
	MakeDir(dirn,(mode_t)0777);

	if (n>0)
	{
		out.bmp = malloc(max(n,64));
		fseek(pst,o,SEEK_SET);
		fread(out.bmp,n,1,pst);
		out.realw=out.logw=64;
		out.logh=max(1,n/64);
		WriteBMP(&out,path,palette);

		free(out.bmp);
	}
}

void OutputSectionHeader(Category *ctgy,FILE *cst,int nooutputdone)
{
	if (ctgy->idm!=ID_INFO && ctgy->idm!=ID_NONE)
	{
		if (nooutputdone) fprintf(cst,"\n");
		fprintf(cst,"; Resource section for %s, default location: %s\\*%s\n",ctgy->secname,ctgy->defaultdir,ctgy->Extns[0].str);
		fprintf(cst,"[%s]\n",ctgy->secname);
		nooutputdone=0;
	}
}


// This routine implements BRK
// Extract resources to files, write a control file listing them

int WriteResourceControl(char *name)
{
	FILE *cst=NULL;
	char patn[16];
	int i,j,k,n,np=0,c,lno=1;
	TextureDesc *td;
	PicData pic;
	Category *ctgy=NULL;
	TextureLump *tl;
	int first=1;
	int sectionheaderdone=0;
	int nooutputdone=0;

	if (stricmp(name,"CON:.WAD")==0 || stricmp(name,"CON.WAD")==0)
		cst = stdout;
	else
		cst = fopen(name,"w");
	if (cst!=NULL)
	{
		dprintf(logfile,"\nExtracting resources from %s writing info to %s\n",pwadname,name);
		if (nresources)
		{
			for (n=0;n<nresources;n++)
			{
				for (c=0;c<pResDef.ncats;c++)
				{
					ctgy = pResDef.cat+c;
					if (IsSectionEnabled(ctgy->secname))
					{
						if (ctgy->idm==ID_INFO || ctgy->idm==ID_NONE)
							;
						else if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
						{
							if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
							{
								j = FindTextureN(&pwt,resources[n],&lno);
								tl = GetTextureLump(c,&pwt,&lno);
								if (tl && j>=0)
								{
									td = tl->textures+j;
									AddVTex2Num
									(
										&mtxres[c],&mtxcnt[c],
										ST_RESDEFN<<SyntaxTypeShift,
										Str8(td->textname),
										NULL,
										NULL,
										td->twid,td->thgt
									);
									for (k=0;k<td->npatches;k++)
									{
										strcpy(patn,GetPName(&pwt.pnames,td->patches[k].pidx));
										AddVTex2Num
										(
											&mtxres[c],&mtxcnt[c],
											ST_RESINPUT<<SyntaxTypeShift,
											patn,
											NULL,
											NULL,
											td->patches[k].xoff,td->patches[k].yoff
										);
									}
									dprintf(logfile,"Info: Extracted %-8s from TEXTURE%d\n",resources[n],lno);
								}
							}
						}
						else if (stricmp(ctgy->secname,"SOUNDS")==0)
						{
							for (i=0;i<ctgy->nnames;i++)
							{
								if (ctgy->Names[i].didx>=0 && stricmp(resources[n],ctgy->Names[i].str)==0)
								{
									AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,0,0);
									if (!JustRes)
									{
										if (IsWaveResource(&pdir,pst,ctgy->Names[i].didx))
											OutputWave(&pdir,ctgy->Names[i].didx,ctgy->defaultdir,ctgy->Extns[0].str);
										else
											OutputLump(&pdir,ctgy->Names[i].didx,ctgy->defaultdir,ctgy->Extns[1].str);
									}
									dprintf(logfile,"Info: Extracted %-8s from SOUNDS\n",resources[n]);
								}
							}
						}
						else
						{
							for (i=0;i<ctgy->nnames;i++)
							{
								j = ctgy->Names[i].didx;
								if (j>=0 && stricmp(resources[n],ctgy->Names[i].str)==0)
								{
									switch(ctgy->Extns[0].didx)
									{
										case DT_GENERIC:
											AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,0,0);
											if (!JustRes) OutputLump(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str);
											break;
										case DT_LEVELS:
											AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,0,0);
											if (!JustRes) OutputWad(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str);
							    			break;
										case DT_PICS:
											if (!JustRes)
												OutputPicture(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str,&pic.pxoff,&pic.pyoff);
											else if (pdir.dirp[j].reslen>=4)
											{
												fseek(pst,pdir.dirp[j].resptr,SEEK_SET);
												fread(&pic,sizeof(short),4,pst);
											}
											else pic.pxoff=pic.pyoff=0;
											AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,pic.pxoff,pic.pyoff);
											break;
										case DT_FLATS:
											AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,0,0);
											if (!JustRes) OutputFlat(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str);
											break;
									}
			  					dprintf(logfile,"Info: Extracted %-8s from %-8s\n",resources[n],ctgy->secname);
								}
							}
							if (stricmp(ctgy->secname,"PATCHES")==0 && AdditionalPatches)
							{
								for (i=0;i<pwt.pnames.npnames;i++)
								{
									if (stricmp(resources[n],GetPName(&pwt.pnames,i))==0)
										if (IsInVTex(GetPName(&pwt.pnames,i),mtxres[c],mtxcnt[c])<0)
											if ((j=IsIn(GetPName(&pwt.pnames,i),&pdir))>=0)
											{
												if (!JustRes)
													OutputPicture(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str,&pic.pxoff,&pic.pyoff);
												else if (pdir.dirp[j].reslen)
												{
													fseek(pst,pdir.dirp[j].resptr,SEEK_SET);
													fread(&pic,sizeof(short),4,pst);
												}
												else pic.pxoff=pic.pyoff=0;
												AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,pic.pxoff,pic.pyoff);
						  					dprintf(logfile,"Info: Extracted %-8s from PATCHES\n",resources[n]);
											}
								}
							}
						}
					}
				}
			}
		}
		else
		{
			for (c=0;c<pResDef.ncats;c++)
			{
				ctgy = pResDef.cat+c;
				if (IsSectionEnabled(ctgy->secname))
				{
					if (ctgy->idm==ID_INFO || ctgy->idm==ID_NONE)
						;
					else if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
					{
						tl = GetTextureLump(c,&pwt,&lno);
						if (tl)
						{
							for (j=0;j<tl->N;j++)
							{
								td = tl->textures+j;
								AddVTex2Num
								(
									&mtxres[c],&mtxcnt[c],
									ST_RESDEFN<<SyntaxTypeShift,
									Str8(td->textname),
									NULL,
									NULL,
									td->twid,td->thgt
								);
								for (k=0;k<td->npatches;k++)
								{
									strcpy(patn,GetPName(&pwt.pnames,td->patches[k].pidx));
									AddVTex2Num
									(
										&mtxres[c],&mtxcnt[c],
										ST_RESINPUT<<SyntaxTypeShift,
										patn,
										NULL,
										NULL,
										td->patches[k].xoff,td->patches[k].yoff
									);
								}
							}
							if (first)
								dprintf(logfile,"Info: Extracted %5ld PNAMES\n",pwt.pnames.npnames);
							first=0;
							dprintf(logfile,"Info: Extracted %5ld TEXTURE%d\n",tl->N,lno);
						}
					}
					else if (stricmp(ctgy->secname,"SOUNDS")==0)
					{
						for (i=0;i<ctgy->nnames;i++)
						{
							if (ctgy->Names[i].didx>=0)
							{
								AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,0,0);
								if (!JustRes)
								{
									if (IsWaveResource(&pdir,pst,ctgy->Names[i].didx))
										OutputWave(&pdir,ctgy->Names[i].didx,ctgy->defaultdir,ctgy->Extns[0].str);
									else
										OutputLump(&pdir,ctgy->Names[i].didx,ctgy->defaultdir,ctgy->Extns[1].str);
								}
							}
						}
						dprintf(logfile,"Info: Extracted %5d %-8s\n",mtxcnt[c],ctgy->secname);
					}
					else
					{
						for (i=0;i<ctgy->nnames;i++)
						{
							j = ctgy->Names[i].didx;
							if (j<0) continue;

							switch(ctgy->Extns[0].didx)
							{
								case DT_GENERIC:
									AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,0,0);
									if (!JustRes) OutputLump(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str);
									break;
								case DT_LEVELS:
									AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,0,0);
									if (!JustRes) OutputWad(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str);
							    	break;
								case DT_PICS:
									if (!JustRes)
										OutputPicture(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str,&pic.pxoff,&pic.pyoff);
									else if (pdir.dirp[j].reslen>=4)
									{
										fseek(pst,pdir.dirp[j].resptr,SEEK_SET);
										fread(&pic,sizeof(short),4,pst);
									}
									else pic.pxoff=pic.pyoff=0;
									AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,pic.pxoff,pic.pyoff);
									break;
								case DT_FLATS:
									AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,0,0);
									if (!JustRes) OutputFlat(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str);
									break;
							}
						}
						if (stricmp(ctgy->secname,"PATCHES")==0 && AdditionalPatches)
						{
							for (i=0;i<pwt.pnames.npnames;i++)
							{
								if (IsInVTex(GetPName(&pwt.pnames,i),mtxres[c],mtxcnt[c])<0)
									if ((j=IsIn(GetPName(&pwt.pnames,i),&pdir))>=0)
									{
										if (!JustRes)
											OutputPicture(&pdir,j,ctgy->defaultdir,ctgy->Extns[0].str,&pic.pxoff,&pic.pyoff);
										else if (pdir.dirp[j].reslen>=4)
										{
											fseek(pst,pdir.dirp[j].resptr,SEEK_SET);
											fread(&pic,sizeof(short),4,pst);
										}
										else pic.pxoff=pic.pyoff=0;
										AddVTex2Num(&mtxres[c],&mtxcnt[c],ST_RESDEFN<<SyntaxTypeShift,ctgy->Names[i].str,NULL,NULL,pic.pxoff,pic.pyoff);
									}
							}
						}
		  				dprintf(logfile,"Info: Extracted %5d %-8s\n",mtxcnt[c],ctgy->secname);
					}
				}
			}
		}

		// write the control file and report totals
		fprintf(cst,";------------------------------------------------------------------------\n");
		fprintf(cst,";%s breakdown, written by BRK.EXE on %s\n",pwadname,GetDateTime());
		fprintf(cst,";------------------------------------------------------------------------\n");
		fprintf(cst,";\n\n");
		for (c=0;c<pResDef.ncats;c++)
		{
			ctgy = pResDef.cat+c;
			if (IsSectionEnabled(ctgy->secname))
			{
				if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
				{
					for (i=0,np=0;i<mtxcnt[c];i++)
					{
						if (SyntaxType(mtxres[c][i].id)==ST_RESINPUT)
						{
							if (!sectionheaderdone)
								OutputSectionHeader(ctgy,cst,nooutputdone);
							sectionheaderdone=1;

							fprintf(cst,"*  %s  %d  %d\n",mtxres[c][i].name,atoi(mtxres[c][i].args[0]),atoi(mtxres[c][i].args[1]));
							np++;
						}
						else
						{
							if (!sectionheaderdone)
								OutputSectionHeader(ctgy,cst,nooutputdone);
							sectionheaderdone=1;

							fprintf(cst,"%s  %d  %d\n",mtxres[c][i].name,atoi(mtxres[c][i].args[0]),atoi(mtxres[c][i].args[1]));
						}
					}
					if (!sectionheaderdone)
					{
						if (ctgy->idm!=ID_INFO && ctgy->idm!=ID_NONE)
							fprintf(cst,"; No %s in %s\n",ctgy->secname,pwadname);
						nooutputdone=1;
					}
					else
							fprintf(cst,"\n");
					sectionheaderdone = 0;
				}
				else if
				(
					stricmp(ctgy->secname,"GRAPHICS")==0 ||
					stricmp(ctgy->secname,"SPRITES")==0	||
					stricmp(ctgy->secname,"PATCHES")==0
				)
				{
					for (i=0;i<mtxcnt[c];i++)
					{
						if (!sectionheaderdone)
							OutputSectionHeader(ctgy,cst,nooutputdone);
						sectionheaderdone=1;

						fprintf(cst,"%s %d %d\n",mtxres[c][i].name,atoi(mtxres[c][i].args[0]),atoi(mtxres[c][i].args[1]));
					}
					if (!sectionheaderdone)
					{
						if (ctgy->idm!=ID_INFO && ctgy->idm!=ID_NONE)
							fprintf(cst,"; No %s in %s\n",ctgy->secname,pwadname);
						nooutputdone=1;
					}
					else
							fprintf(cst,"\n");
					sectionheaderdone = 0;
				}
				else if (ctgy->idm!=ID_INFO && ctgy->idm!=ID_NONE)
				{
					for (i=0;i<mtxcnt[c];i++)
					{
						if (!sectionheaderdone)
							OutputSectionHeader(ctgy,cst,nooutputdone);
						sectionheaderdone=1;

						fprintf(cst,"%s\n",mtxres[c][i].name);
					}
					if (!sectionheaderdone)
					{
						if (ctgy->idm!=ID_INFO && ctgy->idm!=ID_NONE)
							fprintf(cst,"; No %s in %s\n",ctgy->secname,pwadname);
						nooutputdone=1;
					}
					else
							fprintf(cst,"\n");
					sectionheaderdone = 0;
				}
			}
		}
		fclose(cst);
		dprintf(logfile,"\nResources extracted listed in %s\n",name);
	}
	return 0;
}

