#include "patch.h"

Button *CreateButton(char *caption,ButtonStyle *onsty,ButtonStyle *offsty)
{
	Button *b;

	b = (Button *)malloc(sizeof(Button));

	strcpy(b->cap,caption);
	b->stt = 0;
	b->sty[0] = offsty;
	b->sty[1] = onsty;

	return b;
}

ButtonArray *CreateButtonArray(char **caps,int nbtns,int nrows,int ncols,int xinc,int yinc,int colord,ButtonStyle *offsty,ButtonStyle *onsty)
{
	ButtonArray *b;
	int i;

	if (nbtns==0) return NULL;

	b = (ButtonArray *)malloc(sizeof(ButtonArray));

	b->btn = (Button *)malloc(sizeof(Button)*nbtns);
	b->nbtns = nbtns;
	b->nrows = nrows;
	b->ncols = ncols;
	b->xinc = xinc;
	b->yinc = yinc;
	b->colord = colord;

	for (i=0;i<b->nbtns;i++)
	{
		strcpy(b->btn[i].cap,caps[i]);
		b->btn[i].stt = 0;
		b->btn[i].sty[0] = offsty;
		b->btn[i].sty[1] = onsty;
	}

	return b;
}

void FreeButtonArray(ButtonArray **b)
{
	free((*b)->btn);
	free((*b));
	*b = NULL;
}

void DrawButton(int x,int y,Button *b)
{
	int lfill;
	int i,w,ow,h;
	char buffer[256];

	if (b->cap[0]=='\0') return;

	//pad the button caption to make len
	if (b->sty[b->stt]->len>=strlen(b->cap))
	{
		i=0;
		switch(b->sty[b->stt]->aln)
		{
			case BTN_CENTER:
				lfill = (b->sty[b->stt]->len-strlen(b->cap))/2;
				for (;i<lfill;i++)
					buffer[i]=' ';
				for (;i<lfill+strlen(b->cap);i++)
					buffer[i] = b->cap[i-lfill];
				for (;i<b->sty[b->stt]->len;i++)
					buffer[i] = ' ';
				break;
			case BTN_RIGHT:
				lfill = b->sty[b->stt]->len-strlen(b->cap);
				for (;i<lfill;i++)
					buffer[i]=' ';
				for (;i<b->sty[b->stt]->len;i++)
					buffer[i] = b->cap[i-lfill];
				break;
			case BTN_LEFT:
				for (;i<strlen(b->cap);i++)
					buffer[i] = b->cap[i];
				for (;i<b->sty[b->stt]->len;i++)
					buffer[i] = ' ';
				break;
		}
		buffer[i]='\0';
	}
	else if (b->sty[b->stt]->len<strlen(b->cap))
		strcpy(buffer,b->cap);

	w = GrStringWidth (b->cap,max(strlen(b->cap),b->sty[b->stt]->len),b->sty[b->stt]->top);
	ow = GrStringWidth (b->cap,max(strlen(b->cap),b->sty[1-b->stt]->len),b->sty[1-b->stt]->top);
	h = GrStringHeight(b->cap,max(strlen(b->cap),b->sty[b->stt]->len),b->sty[b->stt]->top);

	GrFilledBox(x-b->sty[b->stt]->thk,y-b->sty[b->stt]->thk,x+max(w,ow)-1+b->sty[b->stt]->thk,y+h-1+b->sty[b->stt]->thk,7);
	GrFramedBox(x,y,x+w-1,y+h-1,b->sty[b->stt]->thk,b->sty[b->stt]->frm);
	GrDrawString(buffer,strlen(buffer),x,y,b->sty[b->stt]->top);
}

int IsInButton(int mx,int my,int x,int y,Button *b)
{
	int x1,y1,x2,y2,w,h;

	x1 = x-b->sty[b->stt]->thk;
	y1 = y-b->sty[b->stt]->thk;
	w = GrStringWidth (b->cap,max(strlen(b->cap),b->sty[b->stt]->len),b->sty[b->stt]->top);
	h = GrStringHeight(b->cap,max(strlen(b->cap),b->sty[b->stt]->len),b->sty[b->stt]->top);
	x2 = x+w-1+b->sty[b->stt]->thk;
	y2 = y+h-1+b->sty[b->stt]->thk;
	return IsInBox(mx,my,x1,y1,x2,y2);
}

void DrawButtonArray(int x,int y,ButtonArray *b)
{
	int i,j;

	if (!b->colord)
	{
		for (i=0;i<b->nrows;i++)
			for (j=0;j<b->ncols;j++)
			{
				if (b->ncols*i+j>=b->nbtns)
					goto bdone;
				else
					DrawButton(x+b->xinc*j,y+b->yinc*i,b->btn+(b->ncols)*i+j);
			}
	}
	else
	{
		for (i=0;i<b->ncols;i++)
			for (j=0;j<b->nrows;j++)
			{
				if (b->nrows*i+j>=b->nbtns)
					goto bdone;
				else
					DrawButton(x+b->xinc*i,y+b->yinc*j,b->btn+(b->nrows)*i+j);
			}
	}
bdone:;
}

int IsInButtonArray(int mx,int my,int x,int y,ButtonArray *b)
{
	int i,j,r;

	r = 0;
	if (!b->colord)
	{
		for (i=0;i<b->nrows;i++)
			for (j=0;j<b->ncols;j++)
			{
				if (IsInButton(mx,my,x+b->xinc*j,y+b->yinc*i,b->btn+(b->ncols)*i+j))
					r = b->ncols*i+j+1;
				if (r) goto ibdone;
			}
	}
	else
	{
		for (i=0;i<b->ncols;i++)
			for (j=0;j<b->nrows;j++)
			{
				if (IsInButton(mx,my,x+b->xinc*i,y+b->yinc*j,b->btn+(b->nrows)*i+j))
					r = b->nrows*i+j+1;
				if (r) goto ibdone;
			}
	}
ibdone:
	return r;
}

// Definitions of buttons and styles

#define MODEBTNWID 39

//GrFBoxColors outfrm={7,8,5,3,5};
//GrFBoxColors infrm= {0,3,5,8,5};
//GrFBoxColors midfrm={0,3,8,5,8};

GrFBoxColors boutfr={7,8,5,3,5};
GrFBoxColors binfr= {0,3,5,8,5};

GrTextOption bouttop={&GrDefaultFont,{ 0 }, { 7 },0,0,0,0};
GrTextOption bintop= {&GrDefaultFont,{ 12 },{ 0 },0,0,0,0};
GrTextOption fouttop={&GrDefaultFont,{ 0 }, { 7 },0,0,0,0};
GrTextOption fintop= {&GrDefaultFont,{ 11 },{ 0 },0,0,0,0};

ButtonStyle modeout={4, BTN_CENTER,2,&boutfr,&bouttop};
ButtonStyle modein= {4, BTN_CENTER,2,&binfr, &bintop};
ButtonStyle filtout={7, BTN_LEFT,  2,&boutfr,&fouttop};
ButtonStyle filtin= {16,BTN_LEFT,  2,&binfr, &fintop};
ButtonStyle flinout={2, BTN_LEFT,  2,&boutfr,&fouttop};
ButtonStyle flinin= {7, BTN_LEFT,  2,&binfr, &fintop};
ButtonStyle flinin2={2, BTN_LEFT,  2,&binfr, &fintop};

// mode button bar labels

char *modelabels[]={"Thng","Line","Sect","Vert","Doom","Node",};
int nmodelabels = sizeof(modelabels)/sizeof(modelabels[0]);

char *vertlabels[]={"X   ",	"Y   ",};
int nvertlabels = sizeof(vertlabels)/sizeof(vertlabels[0]);

char *thglabels[]={"X   ","Y   ","Type","Angl","Flag",};
int nthglabels = sizeof(thglabels)/sizeof(thglabels[0]);

char *linlabels[]={"From","To  ","Flag","Type","Tag ","Lsid","Rsid",};
int nlinlabels = sizeof(linlabels)/sizeof(linlabels[0]);

char *sidlabels[]={"Uprt","Nrmt","Lwrt","Xoff","Yoff","Sect",};
int nsidlabels = sizeof(sidlabels)/sizeof(sidlabels[0]);

char *seclabels[]={"Ceil","Flor","Chgt","Fhgt","Lite","Type","Tag ",};
int nseclabels = sizeof(seclabels)/sizeof(seclabels[0]);

char *VertClassName[]=
{
	" None  ",
	"Normal ",
	"Unused ",
	"X Aln  ",
	"Y Aln  ",
	" All   ",
};
int NVertClassNames = sizeof(VertClassName)/sizeof(VertClassName[0]);

char *ThingClassName[]=
{
	" None  ",
	"Monster",
	"Start  ",
	"Armor  ",
	"Lights ",
	"Ammo   ",
	"AmmoBox",
	"Keys   ",
	"Weapon ",
	"Health ",
	"Item   ",
	"Other  ",
	" All   ",
};
int NThingClassNames = sizeof(ThingClassName)/sizeof(ThingClassName[0]);

char *SectorClassName[]=
{
	" None  ",
	"Light  ",
	"Damage ",
	"Secret ",
	"Open   ",
	"Tagged ",
	" All   ",
};
int NSectorClassNames = sizeof(SectorClassName)/sizeof(SectorClassName[0]);

char *LineClassName[]=
{
	" None  ","",
	"2s",
	"Ma",
	"So",
	"Lw",
	"Up",
	"Se",
	"Mo",
	"Im",
	"In",
	"Tr",
	" All   ",""
};
int NLineClassNames = sizeof(LineClassName)/sizeof(LineClassName[0]);

char *DiffClassName[]=
{
	"Ez",
	"Md",
	"Hd",
	"",
	"Df",
	"Mp",
};
int NDiffClassNames = sizeof(DiffClassName)/sizeof(DiffClassName[0]);

char *PrvNxtName[]=
{
	"<-",
	"->",
};
int NPrvNxtNames = sizeof(PrvNxtName)/sizeof(PrvNxtName[0]);


void CreateButtons()
{
	int i;

	// mode select button array
	modeButtons = CreateButtonArray(modelabels,nmodelabels,1,nmodelabels,MODEBTNWID,0,0,&modeout,&modein);
	modeButtons->btn[0].stt = 1; // Thing mode
	if (ViewOnly) modeButtons->nbtns = 5;      // remove advanced mode buttons

	// mode item's fields buttons
	vertButtons = CreateButtonArray(vertlabels,nvertlabels,1,nvertlabels,MODEBTNWID,0,0,&modeout,&modein);
	thngButtons = CreateButtonArray(thglabels,nthglabels,1,nthglabels,MODEBTNWID,0,0,&modeout,&modein);
	lineButtons = CreateButtonArray(linlabels,nlinlabels,1,nlinlabels,MODEBTNWID,0,0,&modeout,&modein);
	sideButtons = CreateButtonArray(sidlabels,nsidlabels,1,nsidlabels,MODEBTNWID,0,0,&modeout,&modein);
	sectButtons = CreateButtonArray(seclabels,nseclabels,1,nseclabels,MODEBTNWID,0,0,&modeout,&modein);

	// filter buttons
	if (ViewOnly)
	{
		strcpy(VertClassName[3],VertClassName[5]);
		NVertClassNames = 4;
	}
	fvrtButtons = CreateButtonArray(VertClassName,NVertClassNames,NVertClassNames,1,0,20,0,&filtout,&filtin);
	for (i=1;i<fvrtButtons->nbtns-1;i++)
		fvrtButtons->btn[i].stt=0;
	fvrtButtons->btn[1].stt = 1;
	fthgButtons = CreateButtonArray(ThingClassName,NThingClassNames,NThingClassNames,1,0,20,0,&filtout,&filtin);
	for (i=1;i<fthgButtons->nbtns-1;i++)
		fthgButtons->btn[i].stt=1;
	fdifButtons = CreateButtonArray(DiffClassName,NDiffClassNames,1,NDiffClassNames,25,0,0,&flinout,&flinin2);
	fdifButtons->btn[4].stt=1; fdifButtons->btn[5].stt=1;
	fsecButtons = CreateButtonArray(SectorClassName,NSectorClassNames,NSectorClassNames,1,0,20,0,&filtout,&filtin);
	for (i=1;i<fsecButtons->nbtns-1;i++)
		fsecButtons->btn[i].stt=1;
	flinButtons = CreateButtonArray(LineClassName,NLineClassNames,(NLineClassNames+1)/2,2,64,20,0,&flinout,&flinin);
	for (i=2;i<flinButtons->nbtns-2;i++)
		flinButtons->btn[i].stt=1;

	// next/prev level buttons
	fprvButtons = CreateButtonArray(PrvNxtName,NPrvNxtNames,1,NPrvNxtNames,24,0,0,&flinout,&flinin2);
}

