#include "patch.h"

int LastSyntax=-1;
int predicate=0;
int substitution=0;
char substr[32];

long fldcnts[8];
char **keys[]=
{
	thing_flds,
	vertex_flds,
	linedef_flds,
	sidedef_flds,
	sector_flds,
};

WadDir dir;
char objspec[256];

char *ThingFlagStr(int attr)
{
	static char attrstr[256];

	*attrstr=0;
	strcat(attrstr,(attr&1)? "Ez" : "");
	strcat(attrstr,(attr&2)? "Md" : "");
	strcat(attrstr,(attr&4)? "Hd" : "");
	strcat(attrstr,(attr&8)? "Df" : "");
	strcat(attrstr,(attr&16)? "~S" : "");
	strcat(attrstr,(attr&32)? "~D" : "");
	strcat(attrstr,(attr&64)? "~C" : "");
	return attrstr;
}

char *ShortThingFlagStr(int attr)
{
	return ThingFlagStr(attr);
}

char *LineFlagStr(int attr)
{
	static char attrstr[256];

	sprintf
	(
		attrstr,
		"%3s%3s%3s%3s%3s%3s%3s%3s%3s%3s",
		(attr&1)? "Im" : "  ",
		(attr&2)? "Mo" : "  ",
		(attr&4)? "2S" : "  ",
		(attr&8)? "Up" : "  ",
		(attr&16)? "Lw" : "  ",
		(attr&32)? "Se" : "  ",
		(attr&64)? "So" : "  ",
		(attr&128)? "In" : "  ",
		(attr&256)? "Ma" : "  ",
		(attr&512)? "Pt" : "  "
	);
	return attrstr;
}

char *ShortLineFlagStr(int attr)
{
	static char attrstr[256];

	sprintf
	(
		attrstr,
		"%s%s%s%s%s%s%s%s%s%s",
		(attr&1)? "Im" : "",
		(attr&2)? "Mo" : "",
		(attr&4)? "2S" : "",
		(attr&8)? "Up" : "",
		(attr&16)? "Lw" : "",
		(attr&32)? "Se" : "",
		(attr&64)? "So" : "",
		(attr&128)? "In" : "",
		(attr&256)? "Ma" : "",
		(attr&256)? "Pt" : ""
	);
	return attrstr;
}

int ThingNumber(char *tname)
{
	int i;

	for (i=0;i<ntassoc && strnicmp(tname,thingassoc[i].name,strlen(tname));i++)
		;
	if (i<ntassoc)
		return thingassoc[i].typ;
	if (stricmp(tname,"MT_PUSH")==0)
		return 5001;
	else if (stricmp(tname,"MT_PULL")==0)
		return 5002;
	return 0;
}

char *ThingName(int typ)
{
	int i;

	for (i=0;i<ntassoc && thingassoc[i].typ!=typ;i++)
		;
	if (i<ntassoc)
		return thingassoc[i].name;
	if (typ==5001)
		return "MT_PUSH";
	else if (typ==5002)
		return "MT_PULL";
	return NULL;
}

int ctoi(char *s)
{
	char *endp;

	while(*s==' ' || *s=='\t')
		s++;
	return strtol(s,&endp,0);
}

int IsHexNumeric(char *s)
{
	while (*s && isxdigit(*s++))
		;
	return *s? 0 : 1;
}

int IsANumber(char *s)
{
	if (s[0]=='0' && tolower(s[1])=='x')
	{
		s+=2;
		return IsHexNumeric(s);
	}
	else
		return IsNumeric(s);
}

//LEVOBJ[(m,n)][.FIELD=[value]]

int ParseObjectTransaction(char *objspec,int *objt,int *levn,int *objn,int *fldn,char *valstr)
{
	char buff[256];
	char fldstr[32];
	char objstr[32];
	int i,j,syntax;

	char *p = strdup(objspec);	// p is beginning of copy of objspec
	char *q = strchr(p,'(');	// q is first occurrence of ( if any
	char *s = strchr(p,',');	// s is first occurrence of , if any
	char *t = strchr(p,')');	// t is first occurrence of ) if any
	char *u = strchr(p,'.');	// u is first occurrence of . if any
	char *v = strchr(p,'=');	// v is first occurrence of = if any
	char *w = strchr(p,'#');	// w is first occurrence of # if any

	if (v && v[1]=='=') predicate=1;

	fldcnts[O_THING]=nthingflds;	// set counts of fields for object types
	fldcnts[O_VERTEX]=nvertexflds;
	fldcnts[O_LINEDEF]=nlinedefflds;
	fldcnts[O_SIDEDEF]=nsidedefflds;
	fldcnts[O_SECTOR]=nsectorflds;

	*objt = *levn = *fldn = -1;
	strcpy(valstr,"");
	strcpy(substr,"");
												 
	if (!q && !s && !t && !u)
		syntax=STX_ALLOALLP;
	else if (u && !q && !s && !t)
		syntax=STX_ALLOONEP;
	else if (q && t && !s && !u)
		syntax=STX_LEVOALLP;
	else if (q && t && u && !s)
		syntax=STX_LEVOONEP;
	else if (q && s && t && !u)
		syntax=STX_ONEOALLP;
	else if (q && s && t && u)
		syntax=STX_ONEOONEP;
	else 
	{
	 	dprintf(logfile,"Syntax error, use: LEVOBJ[(m[,n])][.FIELD][=VAL]\n");
	 	xlibfree(p);
	 	return 1;
	}
	LastSyntax=syntax;

	if (v && !predicate)
	{
		if (!q)
		{
			dprintf(logfile,"Syntax ERROR: level number not found, missing (\n");
			xlibfree(p);
			return 1;
		}

		if (!s)
		{
			dprintf(logfile,"Syntax ERROR: object number not found, missing ','\n");
			xlibfree(p);
			return 1;
		}

		if (!t)
		{
			dprintf(logfile,"Syntax ERROR: field not found, missing ')'\n");
			xlibfree(p);
			return 1;
		}

		if (t<s)
		{
			dprintf(logfile,"Syntax ERROR: found ')' before ','\n");
			xlibfree(p);
			return 1;
		}

		if (u!=t+1)
		{
			dprintf(logfile,"Syntax ERROR: missing '.' after ')'\n");
			xlibfree(p);
			return 1;
		}

		if (w && (v || w<v))
		{
			dprintf(logfile,"Syntax ERROR: found # before =\n");
			xlibfree(p);
			return 1;
		}

		if (w && !predicate)
		{
			dprintf(logfile,"Syntax ERROR: substitution with # only usable with predicate: ==\n");
			xlibfree(p);
			return 1;
		}
	}

	if (q) *q='\0';
	if (u) *u='\0';
	if (t) *t='\0';
	if (s) *s='\0';
	if (v) *v='\0';
	if (w) *w='\0';

	strcpy(objstr,p);

	for (i=0;i<nlevobjs && strnicmp(objstr,levobjstrs[i],strlen(objstr));i++)
		;

	if (i<nlevobjs)
	{
		*objt = i;
		*levn = (q && t)? ctoi(q+1) : -1;
		*objn = (q && s && t)? ctoi(s+1) : -1;
		strcpy(fldstr,u? u+1 : "");
		if (v && v[1] && (!predicate || v[2]))
			strcpy(valstr,predicate? v+2 : v+1);
		if (w)
		{
			substitution=1;
			strcpy(substr,w+1);
		}

		xlibfree(p);

		if (*fldstr)
		{
			for (j=0;j<fldcnts[*objt] && strnicmp(fldstr,keys[*objt][j],strlen(fldstr));j++)
				;
			if (j<fldcnts[*objt])
				*fldn=j;
			else
			{
				dprintf(logfile,"Syntax ERROR: %s field %s not understood\n",levobjstrs[*objt],fldstr);
				return 1;
			}
		}
		else *fldn=-1;
	}
	else
	{
		dprintf(logfile,"Syntax ERROR: level object %s not understood\n",buff);
		xlibfree(p);
		return 1;
	}

	if (Verbose)
	{
		dprintf(logfile,"\nObject specification: %s\n",objspec);
		dprintf(logfile,"Type: %d, %s\n",*objt,*objt>=0? levobjstrs[*objt] : "All objects");
		if (*levn>=1)
			dprintf(logfile,"Map: Map%02d\n",*levn);
		else
			dprintf(logfile,"All Maps\n");
		dprintf(logfile,"Obj: %d\n",*objn);
		dprintf(logfile,"Fld: %d, %s\n",*fldn,*fldn>=0? keys[*objt][*fldn] : "All fields");
		dprintf(logfile,"Val: %s\n",valstr);
	}
	return 0;
}

void GetThingFld(Thing *t,int fldn,char *valstr)
{
	switch(fldn)
	{
		case TH_X:
			sprintf(valstr,"%0d",t->x);
			break;
		case TH_Y:
			sprintf(valstr,"%0d",t->y);
			break;
		case TH_TYPE:
			sprintf(valstr,"%s(%0d)",ThingName(t->typ),t->typ);
			break;
		case TH_ANGLE:
			sprintf(valstr,"%s",angles[t->ang/45]);
			break;
		case TH_FLAGS:
			sprintf(valstr,"%s",ThingFlagStr(t->attr));
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

int ThingFieldNumeric(int fldn)
{
		if (fldn>=TH_X && fldn<=TH_FLAGS)
			return 1;
		else
			return 0;
}

Value GetThingVal(Thing *t,int fldn)
{
	Value v;

	strcpy(v.str,"");
	switch(fldn)
	{
		case TH_X:
			v.n = t->x;
			break;
		case TH_Y:
			v.n = t->y;
			break;
		case TH_TYPE:
			v.n = t->typ;
			break;
		case TH_ANGLE:
			v.n = t->ang;
			break;
		case TH_FLAGS:
			v.n = t->attr;
			break;
		default:
			v.n = -1;
			sprintf(v.str,"Error");
			break;
	}
	return v;
}

int CmpThingFld(Thing *t,int fldn,char *valstr)
{
	Value v;
	int i,attr,bit;

	v = GetThingVal(t,fldn);
	Trim(valstr);
	if (IsANumber(valstr))
		return v.n - ctoi(valstr);
	switch(fldn)
	{
		case TH_X:
		case TH_Y:
			return -1;
		case TH_TYPE:
			return v.n - ThingNumber(valstr);
		case TH_ANGLE:
			for (i=0;i<nangles;i++)
				if (stricmp(angles[i],valstr)==0)
					break;
			return v.n-45*i;
		case TH_FLAGS:
			for (i=0,attr=0,bit=1;i<nthgflags;i++,bit<<=1)
				if (stristr(valstr,thingflags[i]))
					attr |= bit;
			return v.n-attr;
		default:
			return -1;
	}
}

void SetThingFld(Thing *t,int fldn,char *valstr)
{
	int i,bit;

	switch(fldn)
	{
		case TH_X:
			t->x = ctoi(valstr);
			break;
		case TH_Y:
			t->y = ctoi(valstr);
			break;
		case TH_TYPE:
			t->typ = IsANumber(valstr)? ctoi(valstr) : ThingNumber(valstr);
			break;
		case TH_ANGLE:
			if (IsANumber(valstr))
				t->ang = ctoi(valstr);
			for (i=0;i<nangles;i++)
				if (stricmp(valstr,angles[i])==0)
					t->ang =i*45;
			break;
		case TH_FLAGS:
			if (IsANumber(valstr))
				t->attr = ctoi(valstr);
			else
			{
				for (i=0,t->attr=0,bit=1;i<nthgflags;i++,bit<<=1)
					if (stristr(valstr,thingflags[i]))
						t->attr |= bit;
			}
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

void GetVertexFld(Vertex *v,int fldn,char *valstr)
{
	switch(fldn)
	{
		case VE_X:
			sprintf(valstr,"%0d",v->x);
			break;
		case VE_Y:
			sprintf(valstr,"%0d",v->y);
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

int VertexFieldNumeric(int fldn)
{
		if (fldn>=VE_X && fldn<=VE_Y)
			return 1;
		else
			return 0;
}

Value GetVertexVal(Vertex *v,int fldn)
{
	Value val;

	strcpy(val.str,"");
	switch(fldn)
	{
		case VE_X:
			val.n = v->x;
			break;
		case VE_Y:
			val.n = v->y;
			break;
		default:
			val.n = -1;
			strcpy(val.str,"Error");
			break;
	}
	return val;
}

int CmpVertexFld(Vertex *v,int fldn,char *valstr)
{
	Value val;

	val = GetVertexVal(v,fldn);
	Trim(valstr);
	if (IsANumber(valstr))
		return val.n - ctoi(valstr);
	else return -1;
	return -1;
}

void SetVertexFld(Vertex *v,int fldn,char *valstr)
{
	switch(fldn)
	{
		case VE_X:
			v->x = ctoi(valstr);
			break;
			break;
		case VE_Y:
			v->y = ctoi(valstr);
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

void GetLinedefFld(Linedef *l,int fldn,char *valstr)
{
	switch(fldn)
	{
		case LI_FROM:
			sprintf(valstr,"%0d",l->from);
			break;
		case LI_TO:
			sprintf(valstr,"%0d",l->to);
			break;
		case LI_FLAGS:
			sprintf(valstr,"%s",LineFlagStr(l->attr));
			break;
		case LI_TYPE:
			sprintf(valstr,"%0d",l->typ);
			break;
		case LI_TAG:
			sprintf(valstr,"%0d",l->tag);
			break;
		case LI_RSID:
			sprintf(valstr,"%0d",l->sides[0]);
			break;
		case LI_LSID:
			sprintf(valstr,"%0d",l->sides[1]);
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

int LinedefFieldNumeric(int fldn)
{
		if (fldn>=LI_FROM && fldn<=LI_LSID)
			return 1;
		else
			return 0;
}

Value GetLinedefVal(Linedef *l,int fldn)
{
	Value v;

	v.n = 0;
	strcpy(v.str,"");
	switch(fldn)
	{
		case LI_FROM:
			v.n = l->from;
			break;
		case LI_TO:
			v.n = l->to;
			break;
		case LI_FLAGS:
			v.n = l->attr;
			break;
		case LI_TYPE:
			v.n = l->typ;
			break;
		case LI_TAG:
			v.n = l->tag;
			break;
		case LI_RSID:
			v.n = l->sides[0];
			break;
		case LI_LSID:
			v.n = l->sides[1];
			break;
		default:
			v.n = -1;
			strcpy(v.str,"Error");
			break;
	}
	return v;
}

int CmpLinedefFld(Linedef *l,int fldn,char *valstr)
{
	int i,bit,attr;
	Value v;


	v = GetLinedefVal(l,fldn);
	Trim(valstr);
	if (IsANumber(valstr))
		return v.n - ctoi(valstr);

	switch(fldn)
	{
		case LI_FROM:
		case LI_TO:
		case LI_TYPE:
		case LI_TAG:
		case LI_RSID:
		case LI_LSID:
			return -1;
		case LI_FLAGS:
			for (i=0,attr=0,bit=1;i<nlinflags;i++,bit<<=1)
				if (stristr(valstr,lineflags[i]))
					attr |= bit;
			return v.n - attr;
			break;
		default:
			return -1;
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

void SetLinedefFld(Linedef *l,int fldn,char *valstr)
{
	int i,bit;

	switch(fldn)
	{
		case LI_FROM:
			l->from = ctoi(valstr);
			break;
		case LI_TO:
			l->to = ctoi(valstr);
			break;
		case LI_FLAGS:
			if (IsANumber(valstr))
				l->attr = ctoi(valstr);
			else
			{
				for (i=0,l->attr=0,bit=1;i<nlinflags;i++,bit<<=1)
					if (stristr(valstr,lineflags[i]))
						l->attr |= bit;
			}
			break;
		case LI_TYPE:
			l->typ = ctoi(valstr);
			break;
		case LI_TAG:
			l->tag = ctoi(valstr);
			break;
		case LI_RSID:
			l->sides[0] = ctoi(valstr);
			break;
		case LI_LSID:
			l->sides[1] = ctoi(valstr);
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

void GetSidedefFld(Sidedef *s,int fldn,char *valstr)
{
	strcpy(valstr,"");
	switch(fldn)
	{
		case SI_XOFF:
			sprintf(valstr,"%0d",s->xoff);
			break;
		case SI_YOFF:
			sprintf(valstr,"%0d",s->yoff);
			break;
		case SI_UPPER:
			sprintf(valstr,"%s",Str8(s->utxt));
			break;
		case SI_LOWER:
			sprintf(valstr,"%s",Str8(s->ltxt));
			break;
		case SI_NORMAL:
			sprintf(valstr,"%s",Str8(s->ntxt));
			break;
		case SI_SECT:
			sprintf(valstr,"%0d",s->sect);
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

int SidedefFieldNumeric(int fldn)
{
		if (fldn==SI_XOFF || fldn==SI_YOFF || fldn==SI_SECT)
			return 1;
		else
			return 0;
}

Value GetSidedefVal(Sidedef *s,int fldn)
{
	Value v;

	v.n = 0;
	strcpy(v.str,"");
	switch(fldn)
	{
		case SI_XOFF:
			v.n = s->xoff;
			break;
		case SI_YOFF:
			v.n = s->yoff;
			break;
		case SI_UPPER:
			strcpy(v.str,Str8(s->utxt));
			break;
		case SI_LOWER:
			strcpy(v.str,Str8(s->ltxt));
			break;
		case SI_NORMAL:
			strcpy(v.str,Str8(s->ntxt));
			break;
		case SI_SECT:
			v.n = s->sect;
			break;
		default:
			v.n = -1;
			strcpy(v.str,"Error");
			break;
	}
	return v;
}

int CmpSidedefFld(Sidedef *s,int fldn,char *valstr)
{
	Value v;

	v = GetSidedefVal(s,fldn);
	Trim(valstr);
	switch(fldn)
	{
		case SI_XOFF:
		case SI_SECT:
		case SI_YOFF:
			if (IsANumber(valstr))
				return v.n-ctoi(valstr);
			return -1;
		case SI_UPPER:
		case SI_LOWER:
		case SI_NORMAL:
			return strnicmp(v.str,valstr,8);
		default:
			return -1;
	}
	return -1;
}


void SetSidedefFld(Sidedef *s,int fldn,char *valstr)
{
	switch(fldn)
	{
		case SI_XOFF:
			s->xoff = ctoi(valstr);
			break;
		case SI_YOFF:
			s->yoff = ctoi(valstr);
			break;
		case SI_UPPER:
			strncpy(s->utxt,valstr,8);
			break;
		case SI_LOWER:
			strncpy(s->ltxt,valstr,8);
			break;
		case SI_NORMAL:
			strncpy(s->ntxt,valstr,8);
			break;
		case SI_SECT:
			s->sect = ctoi(valstr);
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

void GetSectorFld(Sector *s,int fldn,char *valstr)
{
	switch(fldn)
	{
		case SE_FHGT:
			sprintf(valstr,"%0d",s->fhgt);
			break;
		case SE_CHGT:
			sprintf(valstr,"%0d",s->chgt);
			break;
		case SE_FLOOR:
			sprintf(valstr,"%s",Str8(s->ftxt));
			break;
		case SE_CEILING:
			sprintf(valstr,"%s",Str8(s->ctxt));
			break;
		case SE_LIGHT:
			sprintf(valstr,"%0d",s->lite);
			break;
		case SE_TYPE:
			if (s->typ<nsectypes)
				sprintf(valstr,"%s",sectortypes[s->typ]);
			else
				sprintf(valstr,"%d",s->typ);
			break;
		case SE_TAG:
			sprintf(valstr,"%0d",s->tag);
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

int SectorFieldNumeric(int fldn)
{
		if ((fldn>=SE_FHGT && fldn<=SE_CHGT) || (fldn>=SE_LIGHT && fldn<=SE_TAG))
			return 1;
		else
			return 0;
}

Value GetSectorVal(Sector *s,int fldn)
{
	Value v;

	v.n = 0;
	strcpy(v.str,"");
	switch(fldn)
	{
		case SE_FHGT:
			v.n = s->fhgt;
			break;
		case SE_CHGT:
			v.n = s->chgt;
			break;
		case SE_FLOOR:
			strcpy(v.str,Str8(s->ftxt));
			break;
		case SE_CEILING:
			strcpy(v.str,Str8(s->ctxt));
			break;
		case SE_LIGHT:
			v.n = s->lite;
			break;
		case SE_TYPE:
			v.n = s->typ;
			break;
		case SE_TAG:
			v.n = s->tag;
			break;
		default:
			v.n = -1;
			strcpy(v.str,"Error");
			break;
	}
	return v;
}

int CmpSectorFld(Sector *s,int fldn,char *valstr)
{
	int i,typ=-1;
	Value v;

	v = GetSectorVal(s,fldn);
	Trim(valstr);

	switch(fldn)
	{
		case SE_FHGT:
		case SE_CHGT:
		case SE_LIGHT:
		case SE_TAG:
			if (IsANumber(valstr))
				return v.n - ctoi(valstr);
			return -1;
		case SE_FLOOR:
		case SE_CEILING:
			return strnicmp(v.str,valstr,8);
		case SE_TYPE:
			if (IsANumber(valstr))
				return v.n - ctoi(valstr);
			else
			{
				for (i=0;i<nsectypes;i++)
					if (strnicmp(valstr,sectortypes[i],strlen(valstr))==0)
					{
						typ = i;
						break;
					}
				return v.n - typ;
			}
			break;
		default:
			return -1;
	}
	return -1;
}

void SetSectorFld(Sector *s,int fldn,char *valstr)
{
	int i;

	switch(fldn)
	{
		case SE_FHGT:
			s->fhgt = ctoi(valstr);
			break;
		case SE_CHGT:
			s->chgt = ctoi(valstr);
			break;
		case SE_FLOOR:
			strncpy(s->ftxt,valstr,8);
			break;
		case SE_CEILING:
			strncpy(s->ctxt,valstr,8);
			break;
		case SE_LIGHT:
			s->lite = ctoi(valstr);
			break;
		case SE_TYPE:
			if (IsANumber(valstr))
				s->typ = ctoi(valstr);
			else
			{
				for (i=0;i<nsectypes;i++)
					if (strnicmp(valstr,sectortypes[i],strlen(valstr))==0)
					{
						s->typ = i;
						break;
					}
				if (i==nsectypes)
				{
					dprintf(logfile,"Syntax ERROR: unknown sector type: %s\n",valstr);
					break;
				}
			}
			break;
		case SE_TAG:
			s->tag = ctoi(valstr);
			break;
		default:
			sprintf(valstr,"ERROR: Bad Field number");
			break;
	}
}

void ParseAtomicPredicate(char *buffer,char *fldname,int *op,char *valstr)
{
	char *p;
	int s,e;

	strcpy(valstr,"");
	strcpy(fldname,"");
	*op=-1;

	s=0; e=0;
	p = strchr(buffer,'=');
	if (p)
	{
		if (p>buffer && p[-1]=='<')
		{
			s = -1;
			*op = OP_LTE;
		}
		else if (p>buffer && p[-1]=='>')
		{
			s = -1;
			*op = OP_GTE;
		}
		else if (p[1]=='=')
		{
			e = 1;
			*op = OP_EQL;
		}
		else if (p>buffer && p[-1]=='!')
		{
			s = -1;
			*op = OP_NEQ;
		}
		else if (p>buffer && p[-1]=='+')
		{
			s = -1;
			*op = OP_AAS;
		}
		else if (p>buffer && p[-1]=='-')
		{
			s = -1;
			*op = OP_SAS;
		}
		else
			*op = OP_ASS;
	}
	else
	{
		p = strchr(buffer,'<');
		if (p) *op = OP_LT;
		else
		{
			p = strchr(buffer,'>');
			if (p) *op = OP_GT;
		}
	}
	if (p)
	{
		if (p[e]) strcpy(valstr,p+e+1);
		Trim(valstr);
		p[s]='\0';
		strcpy(fldname,buffer);
		Trim(fldname);
	}
}

void AssignField(int objtype,char *fldname,char *valstr,int *objs,int nobjs)
{
	int i,j;
	int s1,s2;
	int first=1;

	for (j=0;j<nobjs;j++)
	{
		switch(objtype)
		{
			case O_THING:
				for (i=0;i<nthingflds;i++)
					if (stricmp(thing_flds[i],fldname)==0)
						break;
				if (i<nthingflds)
					SetThingFld(Things+objs[j],i,valstr);
				break;
			case O_LINEDEF:
				for (i=0;i<nlinedefflds;i++)
					if (stricmp(linedef_flds[i],fldname)==0)
						break;
				if (i<nlinedefflds)
					SetLinedefFld(Linedefs+objs[j],i,valstr);
				break;
			case O_SIDEDEF:
				for (i=0;i<nsidedefflds;i++)
					if (stricmp(sidedef_flds[i],fldname)==0)
						break;
				if (i<nsidedefflds)
				{
					s1 = Linedefs[objs[j]].sides[0];
					s2 = Linedefs[objs[j]].sides[1];
					if (s1>=0 && first)
						SetSidedefFld(Sidedefs+s1,i,valstr);
					if (s2>=0 && !first)
						SetSidedefFld(Sidedefs+s2,i,valstr);
				}
				break;
			case O_VERTEX:
				for (i=0;i<nvertexflds;i++)
					if (stricmp(vertex_flds[i],fldname)==0)
						break;
				if (i<nvertexflds)
					SetVertexFld(Vertexes+objs[j],i,valstr);
				break;
			case O_SECTOR:
				for (i=0;i<nsectorflds;i++)
					if (stricmp(sector_flds[i],fldname)==0)
						break;
				if (i<nsectorflds)
					SetSectorFld(Sectors+objs[j],i,valstr);
				break;
		}
	}
}

int cledmain(int argc,char **argv)
{
	int i,j,k,ln;
	long n,m;
	int cleared=0;
	long upr;
	int objt,levn,objn,fldn;
	long obj0,objz,fld0,fldz;
	char valstr[256],befstr[256];
	char resn[16],mapn[16];
	Thing thng;
	Vertex vert;
	Linedef ldef;
	Sidedef sdef;
	Sector sect;
	Value bv,av;

	CopyEnvironmentCLArgs("CLED",argc,argv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;
	}

  if (nclfiles<2)
  {
      dprintf(logfile,"\nCLED.EXE %s by Jim Flynn, Copyright 1997,8 -- Freeware --\n",PATCHER_VERSION);
      dprintf(logfile,"Report or change a level object's properties from the DOS command line\n");
      dprintf(logfile,"\nUsage: CLED wadname.wad LEVOBJ[(m[,n])][.FIELD][=value|==find|==find#subs])\n");
      dprintf(logfile,"Changes a LEVOBJ = THING,VERTEX,LINEDEF,SIDEDEF,SECTOR\n");
      dprintf(logfile,"The one numbered n in Map m, is affected\n");
      dprintf(logfile,"FIELD depends on which LEVOBJ. FIELDS for:\n");
      dprintf(logfile,"-------------------------------------------------------------------------------\n");
      dprintf(logfile,"THING VERTEX LINEDEF SIDEDEF SECTOR |          Values\n");
      dprintf(logfile,"-------------------------------------------------------------------------------\n");
      dprintf(logfile,"X      X     FROM    UPPER   FHGT   | FLAGS:\n");
      dprintf(logfile,"Y      Y     TO      LOWER   CHGT   |  THING   = EzMdHdDf~S~D~C\n");
      dprintf(logfile,"TYPE         FLAGS   NORMAL  FLOOR  |  LINEDEF = ImMo2sUpLwSeSoInMaPt\n");
      dprintf(logfile,"ANGLE        TYPE    XOFF    CEILING| THING ANGLES: N NE E SE S SW W NW\n");
      dprintf(logfile,"FLAGS        TAG     YOFF    TAG    | SECTOR TYPES: Normal, Random, FBlnk\n");
      dprintf(logfile,"             RSID    SECT    LIGHT  |  SBlnk, Osc, Open, FSync, SSync, Flicker\n");
      dprintf(logfile,"             LSID            TYPE   |  DamBlk, Close, Dam5, Dam10, Dam20\n");
      dprintf(logfile,"                                    |  DamExit, Secret\n");
      dprintf(logfile,"\nIf value or parts of input are missing, the current values are reported\n");
		  dprintf(logfile,"Example: CLED EPISODE.WAD SECTOR(3,25).TYPE=Flicker\n");
      dprintf(logfile,"Syntax of value depends on FIELD type, see CLED.TXT for details\n");

			FreeCLStrs();
      return 1;
  }

	if (nclopts>0 && (stricmp(cloptions[0],"DOOM")==0 || stricmp(cloptions[0],"DOOM2")==0))
		strcpy(defname,cloptions[0]);

	cleared=0;
	strcpy(outname,"");

	strcpy(pwadname,clfiles[0]);
 	strcpy(objspec,clfiles[1]);

	FreeCLStrs();

	AddExt(pwadname,".WAD");

	if (ParseObjectTransaction(objspec,&objt,&levn,&objn,&fldn,valstr))
	{
		return 3;
	}

	if (objt<0)
	{
		dprintf(logfile,"Syntax error parsing object specifier\n");
		dprintf(logfile,"Please use Thing, Vertex, Linedef, SIdedef, or SEctor\n");
		return 3;
	}

	fld0=fldz=fldn;
	if (fldn<0)
	{
		fld0=0;
		fldz=fldcnts[objt]-1;
	}

	obj0=objz=objn;
	if (objn<0)
	{
		obj0=0;
		objz=0;
	}

	AddExt(defname,".DEF");
	InitCategories(defname,&ResDef);

	pst = fopen(pwadname,"r+b");
	if (pst!=NULL)
	{
		fread(&phdr,sizeof(WadHeader),1,pst);
		ReadDir(&phdr,&pdir,pst);

		for (i=0;i<pdir.nentries;i++)
		{
			if (IsMapName(pdir.dirp+i) && (levn<0 || LevelNumber(pdir.dirp[i].resname)==levn))
			{
				strcpy(mapn,Str8(pdir.dirp[i].resname));
				ln = LevelNumber(mapn);

				for (j=i+1;j<i+11;j++)
				{
					 strcpy(resn,Str8(pdir.dirp[j].resname));
					 if (stricmp(resn,levobjstrs[objt])==0)
						break;
				}
				if (j<i+11)
				{
					if (objn<0) objz = pdir.dirp[j].reslen;
					upr = (objn<0? (objz/objsize[objt])-1 : objz);
					for (m=obj0;m<=upr;m++)
					{
						for (k=fld0;k<=fldz;k++)  // do one or all fields
						{
							switch (objt)
							{
								case O_THING:
									// seek to resource array
									fseek(pst,pdir.dirp[j].resptr+sizeof(Thing)*m,SEEK_SET);
									n=ftell(pst); // record position
									fread(&thng,sizeof(Thing),1,pst);
									GetThingFld(&thng,k,befstr);
									bv=GetThingVal(&thng,k);
									if (valstr && *valstr) // if value specified
									{
										if (predicate)
										{
											if (CmpThingFld(&thng,k,valstr)==0)
											{
												if (ThingFieldNumeric(k))
													dprintf(logfile,"Map%02d Thing before: %4ld %s: %s[0x%x]\n",
														ln,m,keys[objt][k],befstr,bv.n);
												else
													dprintf(logfile,"Map%02d Thing before: %4ld %s: %s\n",
														ln,m,keys[objt][k],befstr);
												if (substitution)
												{
													SetThingFld(&thng,k,substr);
													fseek(pst,n,SEEK_SET); 
													SafeWrite(&thng,sizeof(Thing),1,pst);
													GetThingFld(&thng,k,befstr);
													bv=GetThingVal(&thng,k);
													if (ThingFieldNumeric(k))
														dprintf(logfile,"Map%02d Thing after: %4ld %s: %s[0x%x]\n",
															ln,m,keys[objt][k],befstr,bv.n);
													else
														dprintf(logfile,"Map%02d Thing after: %4ld %s: %s\n",
															ln,m,keys[objt][k],befstr);
												}
											}
										}
										else
										{
											SetThingFld(&thng,k,valstr); // set thing = val
											fseek(pst,n,SEEK_SET); 
											SafeWrite(&thng,sizeof(Thing),1,pst);
											if (ThingFieldNumeric(k))
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s[0x%x])",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr,bv.n);
											else
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s)",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr);
											GetThingFld(&thng,k,befstr);
											av=GetThingVal(&thng,k);
											if (ThingFieldNumeric(k))
												dprintf(logfile,"%s[0x%x]\n",befstr,av.n);
											else
												dprintf(logfile,"%s\n",befstr);
										}
									}
									else
									{
										if (ThingFieldNumeric(k))
											dprintf(logfile,"Map%02d Thing %4ld %s: %s[0x%x]\n",
												ln,m,keys[objt][k],befstr,bv.n);
										else
											dprintf(logfile,"Map%02d Thing %4ld %s: %s\n",
												ln,m,keys[objt][k],befstr);
									}
									break;
								case O_VERTEX:
									// seek to resource array
									fseek(pst,pdir.dirp[j].resptr+sizeof(Vertex)*m,SEEK_SET);
									n=ftell(pst); // record position
									fread(&vert,sizeof(Vertex),1,pst);
									GetVertexFld(&vert,k,befstr);
									bv=GetVertexVal(&vert,k);
									if (valstr && *valstr) // if value specified
									{
										if (predicate)
										{
											if (CmpVertexFld(&vert,k,valstr)==0)
											{
												if (VertexFieldNumeric(k))
													dprintf(logfile,"Map%02d Vertex %4ld %s: %s[0x%x]\n",
														ln,m,keys[objt][k],befstr,bv.n);
												else
													dprintf(logfile,"Map%02d Vertex %4ld %s: %s\n",
														ln,m,keys[objt][k],befstr);
												if (substitution)
												{
													SetVertexFld(&vert,k,substr);
													fseek(pst,n,SEEK_SET); 
													SafeWrite(&vert,sizeof(Vertex),1,pst);
													GetVertexFld(&vert,k,befstr);
													bv=GetVertexVal(&vert,k);
													if (VertexFieldNumeric(k))
														dprintf(logfile,"Map%02d Vertex after: %4ld %s: %s[0x%x]\n",
															ln,m,keys[objt][k],befstr,bv.n);
													else
														dprintf(logfile,"Map%02d Vertex after: %4ld %s: %s\n",
															ln,m,keys[objt][k],befstr);
												}
											}
										}
										else
										{
											SetVertexFld(&vert,k,valstr); // set Vertex = val
											fseek(pst,n,SEEK_SET); 
											SafeWrite(&vert,sizeof(Vertex),1,pst);
											if (VertexFieldNumeric(k))
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s[0x%x])",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr,bv.n);
											else
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s)",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr);
											GetVertexFld(&vert,k,befstr);
											av=GetVertexVal(&vert,k);
											if (VertexFieldNumeric(k))
												dprintf(logfile,"%s[0x%x]\n",befstr,av.n);
											else
												dprintf(logfile,"%s\n",befstr);
										}
									}
									else
									{
										if (VertexFieldNumeric(k))
											dprintf(logfile,"Map%02d Vertex %4ld %s: %s[0x%x]\n",
												ln,m,keys[objt][k],befstr,bv.n);
										else
											dprintf(logfile,"Map%02d Vertex %4ld %s: %s\n",
												ln,m,keys[objt][k],befstr);
									}
									break;
								case O_LINEDEF:
									// seek to resource array
									fseek(pst,pdir.dirp[j].resptr+sizeof(Linedef)*m,SEEK_SET);
									n=ftell(pst); // record position
									fread(&ldef,sizeof(Linedef),1,pst);
									GetLinedefFld(&ldef,k,befstr);
									bv=GetLinedefVal(&ldef,k);
									if (valstr && *valstr) // if value specified
									{
										if (predicate)
										{
											if (CmpLinedefFld(&ldef,k,valstr)==0)
											{
												if (LinedefFieldNumeric(k))
													dprintf(logfile,"Map%02d Linedef %4ld %s: %s[0x%x]\n",
														ln,m,keys[objt][k],befstr,bv.n);
												else
													dprintf(logfile,"Map%02d Linedef %4ld %s: %s\n",
														ln,m,keys[objt][k],befstr);
												if (substitution)
												{
													SetLinedefFld(&ldef,k,substr);
													fseek(pst,n,SEEK_SET); 
													SafeWrite(&ldef,sizeof(Linedef),1,pst);
													GetLinedefFld(&ldef,k,befstr);
													bv=GetLinedefVal(&ldef,k);
													if (LinedefFieldNumeric(k))
														dprintf(logfile,"Map%02d Linedef after: %4ld %s: %s[0x%x]\n",
															ln,m,keys[objt][k],befstr,bv.n);
													else
														dprintf(logfile,"Map%02d Linedef after: %4ld %s: %s\n",
															ln,m,keys[objt][k],befstr);
												}
											}
										}
										else
										{
											SetLinedefFld(&ldef,k,valstr); // set Linedef = val
											fseek(pst,n,SEEK_SET); 
											SafeWrite(&ldef,sizeof(Linedef),1,pst);
											if (LinedefFieldNumeric(k))
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s[0x%x])",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr,bv.n);
											else
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s)",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr);
											GetLinedefFld(&ldef,k,befstr);
											av=GetLinedefVal(&ldef,k);
											if (LinedefFieldNumeric(k))
												dprintf(logfile,"%s[0x%x]\n",befstr,av.n);
											else
												dprintf(logfile,"%s\n",befstr);
										}
									}
									else
									{
										if (LinedefFieldNumeric(k))
											dprintf(logfile,"Map%02d Linedef %4ld %s: %s[0x%x]\n",
												ln,m,keys[objt][k],befstr,bv.n);
										else
											dprintf(logfile,"Map%02d Linedef %4ld %s: %s\n",
												ln,m,keys[objt][k],befstr);
									}
									break;
								case O_SIDEDEF:
									// seek to resource array
									fseek(pst,pdir.dirp[j].resptr+sizeof(Sidedef)*m,SEEK_SET);
									n=ftell(pst); // record position
									fread(&sdef,sizeof(Sidedef),1,pst);
									GetSidedefFld(&sdef,k,befstr);
									bv=GetSidedefVal(&sdef,k);
									if (valstr && *valstr) // if value specified
									{
										if (predicate)
										{
											if (CmpSidedefFld(&sdef,k,valstr)==0)
											{
												if (SidedefFieldNumeric(k))
													dprintf(logfile,"Map%02d Sidedef %4ld %s: %s[0x%x]\n",
														ln,m,keys[objt][k],befstr,bv.n);
												else
													dprintf(logfile,"Map%02d Sidedef %4ld %s: %s\n",
														ln,m,keys[objt][k],befstr);
												if (substitution)
												{
													SetSidedefFld(&sdef,k,substr);
													fseek(pst,n,SEEK_SET); 
													SafeWrite(&sdef,sizeof(Sidedef),1,pst);
													GetSidedefFld(&sdef,k,befstr);
													bv=GetSidedefVal(&sdef,k);
													if (SidedefFieldNumeric(k))
														dprintf(logfile,"Map%02d Sidedef after: %4ld %s: %s[0x%x]\n",
															ln,m,keys[objt][k],befstr,bv.n);
													else
														dprintf(logfile,"Map%02d Sidedef after: %4ld %s: %s\n",
															ln,m,keys[objt][k],befstr);
												}
											}
										}
										else
										{
											SetSidedefFld(&sdef,k,valstr); // set Sidedef = val
											fseek(pst,n,SEEK_SET); 
											SafeWrite(&sdef,sizeof(Sidedef),1,pst);
											if (SidedefFieldNumeric(k))
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s)",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr);
											else
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s[0x%x])",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr,bv,n);
											GetSidedefFld(&sdef,k,befstr);
											av=GetSidedefVal(&sdef,k);
											if (SidedefFieldNumeric(k))
												dprintf(logfile,"%s\n",befstr);
											else
												dprintf(logfile,"%s[0x%x]\n",befstr,av.n);
										}
									}
									else
									{
										if (SidedefFieldNumeric(k))
											dprintf(logfile,"Map%02d Sidedef %4ld %s: %s[0x%x]\n",
												ln,m,keys[objt][k],befstr,bv.n);
										else
											dprintf(logfile,"Map%02d Sidedef %4ld %s: %s\n",
												ln,m,keys[objt][k],befstr);
									}
									break;
								case O_SECTOR:
									// seek to resource array
									fseek(pst,pdir.dirp[j].resptr+sizeof(Sector)*m,SEEK_SET);
									n=ftell(pst); // record position
									fread(&sect,sizeof(Sector),1,pst);
									GetSectorFld(&sect,k,befstr);
									bv=GetSectorVal(&sect,k);
									if (valstr && *valstr) // if value specified
									{
										if (predicate)
										{
											if (CmpSectorFld(&sect,k,valstr)==0)
											{
												if (SectorFieldNumeric(k))
													dprintf(logfile,"Map%02d Sector %4ld %s: %s[0x%x]\n",
														ln,m,keys[objt][k],befstr,bv.n);
												else
													dprintf(logfile,"Map%02d Sector %4ld %s: %s\n",
														ln,m,keys[objt][k],befstr);
												if (substitution)
												{
													SetSectorFld(&sect,k,substr);
													fseek(pst,n,SEEK_SET); 
													SafeWrite(&sect,sizeof(Sector),1,pst);
													GetSectorFld(&sect,k,befstr);
													bv=GetSectorVal(&sect,k);
													if (SectorFieldNumeric(k))
														dprintf(logfile,"Map%02d Sector after: %4ld %s: %s[0x%x]\n",
															ln,m,keys[objt][k],befstr,bv.n);
													else
														dprintf(logfile,"Map%02d Sector after: %4ld %s: %s\n",
															ln,m,keys[objt][k],befstr);
												}
											}
										}
										else
										{
											SetSectorFld(&sect,k,valstr); // set Sector = val
											fseek(pst,n,SEEK_SET); 
											SafeWrite(&sect,sizeof(Sector),1,pst);
											if (SectorFieldNumeric(k))
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s[0x%x])",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr,bv.n);
											else
												dprintf(logfile,"%s, Map%02d, %s, #%0d, %s=(%s)",
													pwadname,levn,levobjstrs[objt],objn,keys[objt][k],befstr);
											GetSectorFld(&sect,k,befstr); // set Sector = val
											av=GetSectorVal(&sect,k);
											if (SectorFieldNumeric(k))
												dprintf(logfile,"%s[0x%x]\n",befstr,av.n);
											else
												dprintf(logfile,"%s\n",befstr);
										}
									}
									else
									{
										if (SectorFieldNumeric(k))
											dprintf(logfile,"Map%02d Sector %4ld %s: %s[0x%x]\n",
												ln,m,keys[objt][k],befstr,bv.n);
										else
											dprintf(logfile,"Map%02d Sector %4ld %s: %s\n",
												ln,m,keys[objt][k],befstr);
									}
									break;
								default:
									break;
							}
						}
					}
				}
				else
				{
					dprintf(logfile,"Level %d resource %s not found\n",ln,levobjstrs[objt]);
					return 3;
				}
			}
		}
	}
	else dprintf(logfile,"Can't open %s for modification\n",pwadname);

	return 0;
}
