#include <grx20.h>
#include <mousex.h>
#include "patch.h"
#include "go32.h"
#include "pc.h"

static int hscroll=0;
static int curhist=0;
static int currow=0;
static int curpos=0;
static int curattr;
static int screenrows=25;
static int screencols=80;
static char tail[512];
static int bufpos=0;
static short blank_row_buf[256];
static int cursortype=_NORMALCURSOR;
static int insmode=1;
static int changed=0;
static int gfxmode=0;

static int wmnx,wmxx,wmny,wmxy;
static int chw,chh;
static GrTextOption *topt;

static char ***linesp=NULL;
static int *nlinesp=NULL;

void InitEditLines(char ***linp,int *nlinp)
{
	*nlinp = 0;
	*linp = NULL;
}

void SetEditLines(char ***linp,int *nlinp)
{
	linesp = linp;
	nlinesp = nlinp;
}

void FreeEditLines(char ***linp,int *nlinp)
{
	int i;

	for (i=0;i<*nlinp;i++)
		free((*linp)[i]);
	free(*linp);
	*nlinp = 0;
	*linp = NULL;
}

void InitGraphicData(int mnx,int mny,int mxx,int mxy,GrTextOption *top)
{
	topt = top;
	chw = GrCharWidth('Q',topt);
	chh = GrCharHeight('Q',topt);
	wmnx = mnx; wmny = mny;
	wmxx = mxx; wmxy = mxy;
	screenrows=(mxy-mny)/chh;
	screencols=(mxx-mnx)/chw;
    changed=0;
    curpos=bufpos=0;
    hscroll=0;
    insmode = InsOption;
//	cursortype = insmode? _NORMALCURSOR : _SOLIDCURSOR;
//	_setcursortype(cursortype);
	gfxmode=1;

}

void InitTextData()
{
    int i;

   	screenrows=ScreenRows();
   	screencols=ScreenCols();
   	ScreenGetCursor(&currow,&curpos);
   	ScreenSetCursor(currow,0);
   	ScreenGetCursor(&currow,&curpos);
    changed=0;
    curpos=bufpos=0;
    hscroll=0;
    curattr = ScreenAttrib;
    for (i=0;i<screencols;i++)
        blank_row_buf[i] = (curattr<<8) | ' ';
    insmode = InsOption;
	cursortype = insmode? _NORMALCURSOR : _SOLIDCURSOR;
	_setcursortype(cursortype);
}

void ScrollScreen()
{
  /* scroll screen */
  movedata(_go32_conventional_mem_selector(), 0xb8000 + ScreenCols()*2,
          _go32_conventional_mem_selector(), 0xb8000,
          ScreenCols() * (ScreenRows()-1) * 2);
  /* fill last row */
  movedata(_go32_my_ds(), (int)blank_row_buf,
          _go32_conventional_mem_selector(), 0xb8000 + ScreenCols() * (ScreenRows()-1) * 2,
          ScreenCols() * 2);
}

void DisplayString(char *q,int curattr,int col,int row)
{
   ScreenPutString(q,curattr,col,row);
}

void ClearRight(char *buffer,int hscroll)
{
  if (strlen(buffer+hscroll)<screencols-1)
 	{
   	gotoxy(strlen(buffer+hscroll)+1,currow+1);
   	clreol();
 	}
}

void DisplayCursor(int row,int col)
{
  ScreenSetCursor(row,col);
}

// This displays the line under edit so that the current cursor position
// is visible
//
// Assumptions: - the current display is screenrowsXscreencols
//              - the current line is displayed with prompt at (currow,0)
//              - the command line starts at (currow,startpos)
//              - the portion of the command line
//                  from hscroll to hscroll+screencols-startpos
//                    displays to right of prompt

void DisplayLine(char *buffer)
{
    int ch;
    char *p=buffer,*q=NULL;

    if (!p) return;

    if (strlen(p+hscroll)>screencols-1)
    {
        ch = p[hscroll+screencols-1];
        p[hscroll+screencols-1] ='\0';
        q = strdup(p+hscroll);
        p[hscroll+screencols-1]=ch;
    }
    else q = strdup(p+hscroll);

    DisplayString(q,curattr,0,currow);
	ClearRight(buffer,hscroll);
    DisplayCursor(currow,curpos);
    xlibfree(q);
}

void CommandLineEditHelp()
{
    dprintf(logfile,"\n\nPatcher Command Line Editor Help\n");
    dprintf(logfile,"\nCursor Motion:                       History:                                 \n");
    dprintf(logfile," Left      Move left                  Up     Get older command line           \n");
    dprintf(logfile," Right     Move right                 Down   Get newer command line           \n");
    dprintf(logfile," Home      Move before first char                                             \n");
    dprintf(logfile," End       Move after last char      Exiting:                                 \n");
    dprintf(logfile," CtrlLeft  Move left word             Enter  Execute line as edited           \n");
    dprintf(logfile," CtrlRight Move right word            ESC    Abandon line edit                \n");
    dprintf(logfile,"                                                                              \n");
    dprintf(logfile,"Deletion:                            Insertion:                               \n");
    dprintf(logfile," BS        Delete previous char       Ins     Toggle Ins on or off            \n");
    dprintf(logfile," Del       Delete current char        Ins On  Insert before current char      \n");
    dprintf(logfile," CtrlDel   Delete to end of line      Ins Off Insert over current char, if any\n");
    dprintf(logfile,"\n");
}

int NextWord(char *buf,int pos)
{
	int n;

	if (buf[pos]==32 || buf[pos]=='\t')
		return pos+1+strspn(buf+pos+1," \t");
	else
	{
		n = pos+1+strcspn(buf+pos+1," \t");
		if (n<strlen(buf))
			return n+1+strspn(buf+n+1," \t");
		else return n;
	}
}

int LastWord(char *buf,int pos)
{
	if (buf[pos]==32 || buf[pos]=='\t')
	{
		while (pos>0 && (buf[pos]==32 || buf[pos]=='\t')) pos--;
		while (pos>0 && (buf[pos]!=32 && buf[pos]!='\t')) pos--;
		return pos;
	}
	else
	{
		while (pos>0 && (buf[pos]!=32 && buf[pos]!='\t')) pos--;
		return pos;
	}
}

char *EditCommand(char *buffer)
{
    int ch=0,i=0,k;
	int newpos;
    
    strcpy(buffer,"");
    changed=0;
	curhist=0;
    while (1)
    {
        ch=WaitKey();
        switch(ch)
        {
            case LF:
                break;
            case K_F1:
                i = curpos;
								CommandLineEditHelp();
								PatcherPrompt();
                ScreenGetCursor(&currow,&curpos);
                curpos=i;
                DisplayLine(buffer);
                break;
            case K_UP:
                if (changed)
                    AddDynStr(linesp,nlinesp,buffer);
                if ((*nlinesp)>0)
                {
                    if (curhist>0) curhist--;
                    else curhist=(*nlinesp)-1;
                    strcpy(buffer,(*linesp)[curhist]);
                    if (strlen(buffer)<screencols-1)
                        curpos=bufpos=strlen(buffer);
                    else
                    {
                        hscroll=strlen(buffer)-screencols+1;
                        curpos=screencols-1;
                        bufpos=strlen(buffer);
                    }
                    DisplayLine(buffer);
                }
                changed=0;
                break;
            case K_DN:
                if (changed)
                    AddDynStr(linesp,nlinesp,buffer);
                if ((*nlinesp)>0)
                {
                    if (curhist<(*nlinesp)-1) curhist++;
                    else curhist=0;
                    strcpy(buffer,(*linesp)[curhist]);
                    if (strlen(buffer)<screencols-1)
                        curpos=bufpos=strlen(buffer);
                    else
                    {
                        hscroll=strlen(buffer)-screencols+1;
                        curpos=screencols-1;
                        bufpos=strlen(buffer);
                    }
                    DisplayLine(buffer);
                }
                changed=0;
                break;
            case K_LT:
                if (bufpos>0)
                {
                    bufpos--;
                    curpos--;
                    if (curpos<0)
                    {
                        hscroll += curpos;
                        curpos=0;
                        DisplayLine(buffer);
                    }
                    else DisplayCursor(currow,curpos);
                }
                break;
            case K_RT:
                if (bufpos<strlen(buffer))
                {
                    bufpos++;
                    curpos++;
                    if (curpos>=screencols-1)
                    {
                        hscroll += curpos-screencols+1;
                        curpos = screencols-1;
                        DisplayLine(buffer);
                    }
                    else DisplayCursor(currow,curpos);
                }
                break;
			case CTL_LT:
				newpos = LastWord(buffer,bufpos);
				if (newpos>0)
				{
					curpos -= bufpos-newpos;
					bufpos = newpos;
                   	if (curpos<0)
                   	{
                       	hscroll += curpos;
                       	curpos=0;
                       	DisplayLine(buffer);
                   	}
                   	else DisplayCursor(currow,curpos);
				}
				break;
			case CTL_RT:
				newpos = NextWord(buffer,bufpos);
				if (newpos>0)
				{
					curpos += newpos-bufpos;
					bufpos = newpos;
                    if (curpos>=screencols-1)
                    {
                        hscroll += curpos-screencols+1;
                        curpos = screencols-1;
                        DisplayLine(buffer);
                    }
                    else DisplayCursor(currow,curpos);
				}
				break;
            case K_HOME:
                hscroll=0;
                curpos=0;
                bufpos=0;
                DisplayLine(buffer);
                break;
            case K_END:
                if (strlen(buffer)<screencols-1)
                    curpos=bufpos=strlen(buffer);
                else
                {
                    hscroll=strlen(buffer)-screencols+1;
                    curpos=screencols-1;
                    bufpos=strlen(buffer);
                }
                DisplayLine(buffer);
                break;
            case K_INS:
                insmode = 1-insmode;
								_setcursortype(insmode? _NORMALCURSOR : _SOLIDCURSOR);
                break;
            case K_DEL:
                if (bufpos<strlen(buffer))
                {
                    strcpy(tail,buffer+bufpos+1);
                    strcpy(buffer+bufpos,tail);
                }
                changed=1;
                DisplayLine(buffer);
                break;
            case CTL_DEL:
                buffer[bufpos]='\0';
                changed=1;
                DisplayLine(buffer);
				break;
            case ESC:
                strcpy(buffer,"");
                bufpos=0;
                curpos=0;
								_setcursortype(_NORMALCURSOR);
                return NULL;
            case CR:
	              currow++;
    	          curpos=0;
                if (currow>screenrows-1)
                {
                    ScrollScreen();
                    currow--;
                }
                DisplayCursor(currow,curpos);
                if (changed)
                    AddDynStr(linesp,nlinesp,buffer);
								_setcursortype(_NORMALCURSOR);
                return buffer;
            case BS:
                if (bufpos>0)
                {
                    strcpy(tail,buffer+bufpos);
                    bufpos--;
                    curpos--;
                    strcpy(buffer+bufpos,tail);
                    if (curpos<0)
                    {
                        hscroll+=curpos;
                        curpos=0;
                    }
                    DisplayLine(buffer);
                    changed=1;
                }
                break;
            default:
                if (32<=ch && ch<127)
                {
					k = (insmode || bufpos==strlen(buffer))? 0 : 1;
                    strcpy(tail,buffer+bufpos+k);
                    buffer[bufpos++]=ch;
                    strcpy(buffer+bufpos,tail);
                    curpos++;
                    if (curpos>=screencols-1)
                    {
                        hscroll += curpos-screencols+1;
                        curpos = screencols-1;
                    }
                    DisplayLine(buffer);
                    changed=1;
                }
                break;

        }
    }
		_setcursortype(_NORMALCURSOR);
    return NULL;
}



