#include "patch.h"

void FreeCmpStructures()
{
	FreeVTex();
	FreePalette();
	FreeTextures(&pwt);
	FreeTextures(&bwt);
	FreeDir(&bdir);
	FreeDir(&pdir);
	FreeResourceDefinition(&ResDef);
	FreeResourceDefinition(&bResDef);
	FreeResourceDefinition(&pResDef);
	CloseStreams();
}

int cmpmain(int argc,char **argv)
{
	int i,n,j;
	int cleared=0;
	char *p,*q;

	dprintf(logfile,"\n");

	CopyEnvironmentCLArgs("CMP",argc,argv);
	if (ParseCommandLine(-1,2))
	{
		FreeCLStrs();
		return 3;
	}

	if (nclfiles!=2)
	{
		dprintf(logfile,"\nCMP.EXE %s by Jim Flynn, Copyright 1997 -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"Report resource differences between base and patch wads\n");
		dprintf(logfile,"\nUsage: CMP basewad[.WAD] patchwad[.WAD]\n");
		dprintf(logfile,"\n Resource selection option:\n");
		dprintf(logfile," +XYZ compares only XYZ resources, -XYZ compares all BUT XYZ resources.\n");
		dprintf(logfile,"  L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
		dprintf(logfile,"  E EFFECTS    M MUSICS     G GRAPHICS\n");
		dprintf(logfile,"  S SPRITES    P PATCHES    F FLATS\n");
		dprintf(logfile,"\n General option, use SET CMP=options to set defaults:\n");
		dprintf(logfile," +Q quicker picture compare\n");
		dprintf(logfile," /I:name specify wad containing a PLAYPAL for palette\n");
		dprintf(logfile,"\n Game definition options, use SET CMP=options to set defaults:\n");
		dprintf(logfile," /D:name[.DEF] specify resource definitions\n");
		dprintf(logfile," /DOOM2 /DOOM /HERETIC /HEXEN specify game\n");
		dprintf(logfile,"\n See CMP.TXT for details.\n");

		FreeCLStrs();
		return(1);
	}

	cleared=0;
	strcpy(outname,"");
	cleared=0;
	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfLWTEMGSPF");
		if (n==strlen(cloptions[i]))
		{
			if (!cleared)	// if first selection option
			{
				if (cloptsense[i])	// clear all if it adds types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=0;
					DoGraphics=DoSprites=DoPatches=DoFlats=DoNonStandard=0;
				}
				else if (!cloptsense[i]) // set all if it subtracts types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=1;
					DoGraphics=DoSprites=DoPatches=DoFlats=DoNonStandard=1;
				}
				cleared=1;	// but don't do it for following selection opts
			}
			for (j=0;j<strlen(cloptions[i]);j++)
			{
				switch(tolower(cloptions[i][j]))
				{
					case 'l':
						DoLumps = cloptsense[i];
						break;
					case 'w':
						DoWads = cloptsense[i];
						break;
					case 't':
						DoTextures = cloptsense[i];
						break;
					case 'e':
						DoEffects = cloptsense[i];
						break;
					case 'm':
						DoMusics = cloptsense[i];
						break;
					case 'g':
						DoGraphics = cloptsense[i];
						break;
					case 's':
						DoSprites = cloptsense[i];
						break;
					case 'p':
						DoPatches = cloptsense[i];
						break;
					case 'f':
						DoFlats = cloptsense[i];
						break;
					case 'u':
						DoNonStandard = cloptsense[i];
						break;
				}
			}
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
			strcpy(defname,cloptions[i]);
		else if (tolower(cloptions[i][0])=='d' && cloptions[i][1]==':')
			strcpy(defname,cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='i' && cloptions[i][1]==':')
			strcpy(iwadname,cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='q')
			ComparePicsGood=!cloptsense[i];
		else dprintf(logfile,"Warn: Command line option %s not understood\n",cloptions[i]);
	}

	strcpy(bwadname,clfiles[0]);
	strcpy(pwadname,clfiles[1]);

	FreeDynStr(&clstrs,&nclstrs);
	FreeDynStr(&clfiles,&nclfiles);
	FreeDynStr(&cloptions,&nclopts);

	AddExt(defname,".DEF");
	InitCategories(defname,&ResDef);
	AddExt(iwadname,".WAD");

	if (*iwadname)						// then the IWAD, if spec'd
	{
		FILE *ist;
		WadHeader ihdr;
		WadDir idir;

		InitDir(&idir);
		ist = fopen(iwadname,"rb");
		if (ist!=NULL)
		{
			dprintf(logfile,"Using PLAYPAL from IWAD %s for palette\n",iwadname);
			fread(&ihdr,sizeof(WadHeader),1,ist);
			ReadDir(&ihdr,&idir,ist);
			n = IsIn("PLAYPAL",&idir);
			if (n>=0)
				palette = GetPalette(&idir,ist,n);
			fclose(ist);
			free(idir.dirp);
		}
	}
	else palette = GetPalette(NULL,NULL,-1);

	q=strrchr(bwadname,'\\');
	p=strrchr(bwadname,'.');
    if (p && (!q || q<p))
		if (strnicmp(p+1,"RES",3)==0 || strnicmp(p+1,"SGN",3)==0)
		{
			ReadResourceControl(bwadname,bwadname);
			if (errorlev<=1)
			{
				AddExt(bwadname,".WAD");
				WriteControlWad(bwadname);
				dprintf(logfile,"Control file converted to wad: %s\n",bwadname);
			}
			else
			{
				return errorlev;
			}
		}

	q=strrchr(pwadname,'\\');
	p=strrchr(pwadname,'.');
    if (p && (!q || q<p))
		if (strnicmp(p+1,"RES",3)==0 || strnicmp(p+1,"SGN",3)==0)
		{
			ReadResourceControl(pwadname,pwadname);
			if (errorlev<=1)
			{
				AddExt(pwadname,".WAD");
				WriteControlWad(pwadname);
				dprintf(logfile,"Control file converted to wad: %s\n",pwadname);
			}
			else
			{
				return errorlev;
			}
		}

	AddExt(bwadname,".WAD");
	AddExt(pwadname,".WAD");

	bst = fopen(bwadname,"rb");
	if (bst)
	{
		fread(&bhdr,sizeof(WadHeader),1,bst);
		ReadDir(&bhdr,&bdir,bst);
		LocateCategories(&bdir,&ResDef,&bResDef);

		pst = fopen(pwadname,"rb");
		if (pst)
		{
			fread(&phdr,sizeof(WadHeader),1,pst);
			ReadDir(&phdr,&pdir,pst);
			LocateCategories(&pdir,&ResDef,&pResDef);

			ReadPNames(&bdir,bst,&bwt);
			ReadTextures(&bdir,bst,&bwt);

			ReadPNames(&pdir,pst,&pwt);
			ReadTextures(&pdir,pst,&pwt);

			CompareWads();
		}
		else
		{
			dprintf(logfile,"Patch wad: %s not found, exiting\n",pwadname);
			dprintf(logfile,"\n");
			return (3);
		}
	}
	else
	{
		dprintf(logfile,"Base wad: %s not found, exiting\n",bwadname);
		dprintf(logfile,"\n");
		return (3);
	}

	dprintf(logfile,"\n");

	return errorlev;
}


