#include "patch.h"

/************************************************************
 CMPSUBS.C
			 /-------------------------\
			||    CMP subroutines     ||
			 \-------------------------/
									
************************************************************/

// Compare an entry name in the basewad and curwad
// -5 entry exists in neither base nor cur
// -4 entry exists in basewad only
// -3 entry exists in curwad only
// 	0 entry exists in base and cur and is the same
//  1 entry exists in base and cur but data is different
//  2 entry exists in base and cur but lens differ by > 3

int CompareEntry(char *resn)
{
	char *buff1=NULL,*buff2=NULL;
	int b,p,bn,pn,bo,po,r;

	b = IsIn(resn,&bdir);
	p = IsIn(resn,&pdir);
	if (b>=0 && p>=0)
	{
		bn = bdir.dirp[b].reslen;
		pn = pdir.dirp[p].reslen;
		if (abs(bn-pn)>=4) return 2;

		if (bn==pn && pn==0) return 0;
		if (bn==0 || pn==0) return 2;

		bo = bdir.dirp[b].resptr;
		buff1 = (char *)malloc(bn);
		fseek(bst,bo,SEEK_SET);
		fread(buff1,bn,1,bst);

		po = pdir.dirp[p].resptr;
		buff2 = (char *)malloc(pn);
		fseek(pst,po,SEEK_SET);
		fread(buff2,pn,1,pst);

		r = memcmp(buff1,buff2,min(bn,pn))? 1 : 0;
		free(buff1);
		free(buff2);
		return r;
	}
	else if (p>=0)
		return -3;
	else if (b>=0)
		return -4;
	else
		return -5;

	return 0;
}

int CompareEntryNums(int b, int p)
{
	char *buff1=NULL,*buff2=NULL;
	int bn,pn,bo,po,r;

	bn = bdir.dirp[b].reslen;
	bo = bdir.dirp[b].resptr;
	pn = pdir.dirp[p].reslen;
	po = pdir.dirp[p].resptr;

	if (abs(bn-pn)>=4) return 2;
	if (bn==pn && pn==0) return 0;
	if (bn==0 || pn==0) return 2;

	buff1 = (char *)malloc(bn);
	fseek(bst,bo,SEEK_SET);
	fread(buff1,bn,1,bst);

	buff2 = (char *)malloc(pn);
	fseek(pst,po,SEEK_SET);
	fread(buff2,pn,1,pst);

	r = memcmp(buff1,buff2,min(bn,pn))? 1 : 0;
	free(buff1);
	free(buff2);
	return r;
}




// Compare an entry name in the basewad and curwad
// -5 a patch exists in neither base nor cur
// -4 a patch exists in basewad only
// -3 a patch exists in curwad only
// -2 texture exists in base only
// -1 texture exists in cur only
// 	0 texture exists in base and cur and is the same and so are the patches
//  1 texture exists in base and cur but a patch's data is different
//  2 texture exists in base and cur but a patch's size differs by > 3
//  3 texture exists in base and cur but patch structure differs

int CmpTextures(TextureDesc *tdb, TextureDesc *tdp)
{
	int i,b,p,r;
	char buff1[16],buff2[16];

	if (tdb->twid != tdp->twid) return 3;		
	if (tdb->thgt != tdp->thgt) return 3;

	if (tdb->npatches != tdp->npatches) return 3;
	for (i=0;i<tdb->npatches;i++)
	{
		if (tdb->patches[i].xoff!=tdp->patches[i].xoff) return 3;
		if (tdb->patches[i].yoff!=tdp->patches[i].yoff) return 3;

		strcpy(buff1,GetPName(&bwt.pnames,tdb->patches[i].pidx));
		strcpy(buff2,GetPName(&pwt.pnames,tdp->patches[i].pidx));
		if (stricmp(buff1,buff2)) return 3;

		b = IsIn(buff1,&bdir);
		p = IsIn(buff1,&pdir);
		if ((r=ComparePicNums(b,p)))
			return r;
	}
	return 0;
}

int CmpPNames(char *pnam)
{
	int b,p;

	p = GetPIdx(&pwt.pnames,pnam);
	b = GetPIdx(&bwt.pnames,pnam);
	
	if (p>=0 && b>=0) return 0;
	if (p<0 && b>=0) return 1;
	if (p>=0 && b<0) return 2;
	return 3;
}

static int totcompare=0;
static int header1=0,header2=0,header3=0;

void CmpHeader1(char *name)
{
	dprintf(logfile,"\n====== %s in %s but not %s\n",name,bwadname,pwadname);
	header1=1;
}

void CmpHeader2(char *name)
{
	dprintf(logfile,"\n====== %s in both wads that differ:\n",name);
	header2=1;
}

void CmpHeader3(char *name)
{
	dprintf(logfile,"\n====== %s in %s but not %s\n",name,pwadname,bwadname);
	header3=1;
}

void ClrHeaders()
{
	header1=header2=header3=0;
}

void ComparePNames(int c)
{
	int i,p;
	static int doonce=1;

	if (!doonce) return;
	doonce=0;

	if (!IsSectionEnabled(ResDef.cat[c].secname)) return;

	ClrHeaders();
	for (i=0;i<bwt.pnames.npnames;i++)
		if ((p=(GetPIdx(&pwt.pnames,GetPName(&bwt.pnames,i))))<0)
		{
			if (!header1) CmpHeader1("PNAMES");
			dprintf(logfile,"T102: %-8s was not defined in second wad\n",GetPName(&bwt.pnames,i));
		}
	if (header1) dprintf(logfile,"\n");
	
	for (i=0;i<pwt.pnames.npnames;i++)
		if ((p=(GetPIdx(&bwt.pnames,GetPName(&pwt.pnames,i))))<0)
		{
			if (!header3) CmpHeader3("PNAMES");
			dprintf(logfile,"T101: %-8s was not defined in first wad\n",GetPName(&pwt.pnames,i));
		}
	if (header3) dprintf(logfile,"\n");
	if (!header1 && !header3) dprintf(logfile,"PNAMES identical\n");
	


}

// Copy each texture in bwad that does not occur in pwad to wts.t1.
// Copy each texture in pwad not identical to a bwad texture to wta.t1
//
// CompareTexture result codes:
// -5 a patch exists in neither base nor cur
// -4 a patch exists in basewad only
// -3 a patch exists in curwad only
// -2 texture exists in base only
// -1 texture exists in cur only
// 	0 texture exists in base and cur and is the same and so are the patches
//  1 texture exists in base and cur but a patch's data is different
//  2 texture exists in base and cur but a patch's size differs by > 3
//  3 texture exists in base and cur but patch structure differs

void CompareTextures(int c)
{
	int p,b,e,l,m;
	char tname[16];
	Category *ctgy,*ctg1,*ctg2;
	TextureLump *tlp[3],*tlb[3];
	static int doonce=1;

	if (!doonce) return;
	doonce=0;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;
	if (!IsSectionEnabled(ctgy->secname)) return;

	tlb[0] = &bwt.t1;
	tlp[0] = &pwt.t1;
	tlb[1] = &bwt.t2;
	tlp[1] = &pwt.t2;
	tlb[2] = &bwt.t3;
	tlp[2] = &pwt.t3;

	ClrHeaders();
	for (l=0;l<3;l++)
		for (b=0;b<tlb[l]->N;b++)
		{
			strcpy(tname,Str8(tlb[l]->textures[b].textname));
			for (m=0;m<3 && (p=FindTexture(tlp[m],tname))<0;m++)
				;
			if (p<0)
			{
				if (!header1) CmpHeader1("TEXTURES");
				dprintf(logfile,"T102: %-8s was not defined in second wad\n",Str8(tlb[l]->textures[b].textname));
			}
		}
	if (header1) dprintf(logfile,"\n");

	for (l=0;l<3;l++)
		for (p=0;p<tlp[l]->N;p++)
		{
			strcpy(tname,Str8(tlp[l]->textures[p].textname));
			for (m=0;m<3 && (b=FindTexture(tlb[m],tname))<0;m++)
				;
			if (b>=0 && (e=CmpTextures(tlb[m]->textures+b,tlp[l]->textures+p)))
			{
				if (!header2) CmpHeader2("TEXTURES");
				switch(e)
				{
					case -5:
						dprintf(logfile,"T203: %-8s A patch was not defined in either wad\n",Str8(tlp[l]->textures[p].textname));
						break;
					case -4:
						dprintf(logfile,"T202: %-8s A patch was not defined in second wad\n",Str8(tlp[l]->textures[p].textname));
						break;
					case -3:
						dprintf(logfile,"T201: %-8s A patch was not defined in first wad\n",Str8(tlp[l]->textures[p].textname));
						break;
					case -2:
						dprintf(logfile,"T102: %-8s was not defined in second wad\n",Str8(tlp[l]->textures[p].textname));
						break;
					case -1:
						dprintf(logfile,"T101: %-8s was not defined in first wad\n",Str8(tlp[l]->textures[p].textname));
						break;
					case 1:
						dprintf(logfile,"T204: %-8s A patch had differing data\n",Str8(tlp[l]->textures[p].textname));
						break;
					case 2:
						dprintf(logfile,"T205: %-8s A patch had a different size\n",Str8(tlp[l]->textures[p].textname));
						break;
					case 3:
						dprintf(logfile,"T103: %-8s Different patch structures\n",Str8(tlp[l]->textures[p].textname));
						break;
				}
			}
		}
	if (header2) dprintf(logfile,"\n");

	for (l=0;l<3;l++)
		for (p=0;p<tlp[l]->N;p++)
		{
			strcpy(tname,Str8(tlp[l]->textures[p].textname));
			for (m=0;m<3 && (b=FindTexture(tlb[m],tname))<0;m++)
				;
			if (b<0)
			{
				if (!header3) CmpHeader3("TEXTURES");
				dprintf(logfile,"T101: %-8s was not defined in first wad\n",Str8(tlp[l]->textures[p].textname));
			}
		}
	if (header3) dprintf(logfile,"\n");
	if (!header1 && !header2 && !header3) dprintf(logfile,"%s identical\n",ctgy->secname);
}

void CompareMarked(int c)
{
	int k,m,e=0;
	Category *ctgy,*ctg1,*ctg2;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;

	if (!IsSectionEnabled(ctgy->secname)) return;

	if (ctg1->nnames<=0 && ctg2->nnames<=0) return;

	ClrHeaders();
	for (k=0;k<ctg1->nnames;k++)
		if (StrIdxFind(ctg2->Names,ctg2->nnames,ctg1->Names[k].str)<0)
		{
			if (!header1) CmpHeader1(ctgy->secname);
			dprintf(logfile,"%c102: %-8s was not defined in second wad\n",ctg1->secname[0],ctg1->Names[k].str);
		}
	if (header1) dprintf(logfile,"\n");

	for (k=0;k<ctg1->nnames;k++)
	{
		if ((m=StrIdxFind(ctg2->Names,ctg2->nnames,ctg1->Names[k].str))>=0)
		{
			if (ctg1->Extns[0].didx==DT_PICS)
				e = ComparePicNums(ctg1->Names[k].didx,ctg2->Names[m].didx);
			else
				e = CompareEntryNums(ctg1->Names[k].didx,ctg2->Names[m].didx);
		
			if (e)
			{
				if (!header2) CmpHeader2(ctgy->secname);
				switch(e)
				{
					case 2:
						dprintf(logfile,"%c104: %s had differing sizes\n",ctg1->secname[0],ctg1->Names[k].str);
						break;
					case 1:
						dprintf(logfile,"%c103: %s had differing data\n",ctg1->secname[0],ctg1->Names[k].str);
						break;
					default:
						dprintf(logfile,"Unknown ERROR: %d\n",e);
						break;
				}
			}
		}
	}
	if (header2) dprintf(logfile,"\n");


	for (k=0;k<ctg2->nnames;k++)
		if (StrIdxFind(ctg1->Names,ctg1->nnames,ctg2->Names[k].str)<0)
		{
			if (!header3) CmpHeader3(ctgy->secname);
			dprintf(logfile,"%c101: %-8s was not defined in first wad\n",ctg1->secname[0],ctg2->Names[k].str);
		}
	if (header3) dprintf(logfile,"\n");
	if (!header1 && !header2 && !header3) dprintf(logfile,"%s identical\n",ctgy->secname);
}

void CompareListed(int c)
{
	int i,e;
	Category *ctgy,*ctg1,*ctg2;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;

	if (!IsSectionEnabled(ctgy->secname)) return;
	if (ctg1->nnames==0 && ctg2->nnames==0) return;

	ClrHeaders();
	for (i=0;i<ctg1->nnames;i++)
	    if (ctg1->Names[i].didx>=0 && ctg2->Names[i].didx<0)
		{
			if (!header1) CmpHeader1(ctgy->secname);
			dprintf(logfile,"%c102: %-8s was not defined in second wad\n",ctgy->secname[0],ctg1->Names[i].str);
		}
	if (header1) dprintf(logfile,"\n");

	for (i=0;i<ctg1->nnames;i++)
	{
	    if (ctg1->Names[i].didx>=0 && ctg2->Names[i].didx>=0)
		{
			if (ctg1->Extns[0].didx==DT_PICS)
				e = ComparePicNums(ctg1->Names[i].didx,ctg2->Names[i].didx);
			else
				e = CompareEntryNums(ctg1->Names[i].didx,ctg2->Names[i].didx);
			if (e)
			{
				if (!header2) CmpHeader2(ctgy->secname);
				switch(e)
				{
					case 2:
						dprintf(logfile,"%c104: %s had differing sizes\n",ctgy->secname[0],ctg1->Names[i].str);
						break;
					case 1:
						dprintf(logfile,"%c103: %s had differing data\n",ctgy->secname[0],ctg1->Names[i].str);
						break;
					default:
						dprintf(logfile,"Unknown ERROR: %d\n",e);
						break;
				}
			}
		}
	}
	if (header2) dprintf(logfile,"\n");

	for (i=0;i<ctg2->nnames;i++)
	    if (ctg1->Names[i].didx<0 && ctg2->Names[i].didx>=0)
		{
			if (!header3) CmpHeader3(ctgy->secname);
			dprintf(logfile,"%c101: %-8s was not defined in first wad\n",ctgy->secname[0],ctg2->Names[i].str);
		}
	if (header3) dprintf(logfile,"\n");
	if (!header1 && !header2 && !header3) dprintf(logfile,"%s identical\n",ctgy->secname);
}

int CmpLevels(int b, int p)
{
	int i,j,c;
	Category *ctgy;

	assert(b>=0 && p>=0);

	c = CategoryOf("LEVOBJ",&ResDef);
	ctgy = ResDef.cat+c;

	for (i=0;i<ctgy->nnames;i++)
		if (strnicmp(bdir.dirp[b+1+i].resname,ctgy->Names[i].str,strlen(ctgy->Names[i].str)) ||
		    strnicmp(pdir.dirp[p+1+i].resname,ctgy->Names[i].str,strlen(ctgy->Names[i].str)))
		{
			dprintf(logfile,"L104: %8s has bad structure\n",ctgy->Names[i].str);
			return -1;
		}

	for (i=1;i<1+ctgy->nnames;i++)
		if ((j=CompareEntryNums(b+i,p+i)))
			return j;
	return 0;
}

void CompareLevels(int c)
{
	int i,e;
	Category *ctgy,*ctg1,*ctg2;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;

	if (!IsSectionEnabled(ctgy->secname)) return;

	if (ctg1->nnames<=0 && ctg2->nnames<=0) return;

	ClrHeaders();
	for (i=0;i<ctg1->nnames;i++)
		if (ctg1->Names[i].didx>=0 && ctg2->Names[i].didx<0)
		{
			if (!header1) CmpHeader1(ctgy->secname);
		  	dprintf(logfile,"W102: %-8s was not defined in second wad\n",ctg1->Names[i].str);
		}
	if (header1) dprintf(logfile,"\n");

	for (i=0;i<ctg1->nnames;i++)
		if (ctg1->Names[i].didx>=0 && ctg2->Names[i].didx>=0)
			if ((e=CmpLevels(ctg1->Names[i].didx,ctg2->Names[i].didx)))
			{
				if (!header2) CmpHeader2(ctgy->secname);
				switch(e)
				{
					case 2:
						dprintf(logfile,"W104: %-8s had differing sizes\n",ctg1->Names[i].str);
						break;
					case 1:
						dprintf(logfile,"W103: %-8s had differing data\n",ctg1->Names[i].str);
						break;
				}
			}
	if (header2) dprintf(logfile,"\n");

	for (i=0;i<ctg2->nnames;i++)
		if (ctg1->Names[i].didx<0 && ctg2->Names[i].didx>=0)
		{
			if (!header3) CmpHeader3(ctgy->secname);
			dprintf(logfile,"W101: %-8s was not defined in first wad\n",ctg2->Names[i].str);
		}
	if (header3) dprintf(logfile,"\n");
	if (!header1 && !header2 && !header3) dprintf(logfile,"%s identical\n",ctgy->secname);
}

int CompareWads()
{
	int c;
	Category *ctgy;

	totcompare=0;
	for (c=0;c<ResDef.ncats;c++)
	{
		ctgy = ResDef.cat+c;
		if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
		{
			CompareTextures(c);
			ComparePNames(c);
		}
		else if (stricmp(ctgy->secname,"LEVELS")==0)
			CompareLevels(c);
		else if (ctgy->idm==ID_INFO || ctgy->idm==ID_NONE)
			continue;
		else if (stricmp(ctgy->secname,"HISTORY")==0)
			continue;
		else if (ctgy->idm == ID_LISTED)
			CompareListed(c);
		else if (ctgy->idm == ID_MARKED)
			CompareMarked(c);

		totcompare |= header1|header2|header3;
	}
	if (!totcompare) dprintf(logfile,"\nWads were identical\n");
	return 0;
}

