#include "patch.h"


void LoadPalette(char *wadname)
{
	// Locate a palette to be inserted in BMPs output

	AddExt(wadname,".WAD");
	if (*wadname)						// then the IWAD, if spec'd
	{
		FILE *ist;
		WadHeader ihdr;
		WadDir idir;

		InitDir(&idir);
		ist = fopen(wadname,"rb");
		if (ist!=NULL)
		{
			int n;

			fread(&ihdr,sizeof(WadHeader),1,ist);
			ReadDir(&ihdr,&idir,ist);
			n = IsIn("PLAYPAL",&idir);
			if (n>=0)
			{
				dprintf(logfile,"Using PLAYPAL from %s for palette\n",iwadname);
				fseek(ist,idir.dirp[n].resptr,SEEK_SET);
				fread(ipal,768,1,ist);
				for (n=0;n<768;n+=3)
				{
					PaletteUsed[n/3].rgbRed = ipal[n+0];
					PaletteUsed[n/3].rgbGreen = ipal[n+1];
					PaletteUsed[n/3].rgbBlue = ipal[n+2];
					PaletteUsed[n/3].rgbReserved = 0;
				}
			}
			fclose(ist);
			FreeDir(&idir);
		}
	}
}

void DoCpCommandLineOpts()
{
	int i;

	for (i=0;i<nclopts;i++)
	{
 		if (tolower(cloptions[i][0])=='w' && cloptions[i][1]==':')
		{
			strcpy(iwadname,cloptions[i]+2);
			AddExt(iwadname,".WAD");
			AddColorTranMaps(iwadname,NULL,NULL);
			LoadPalette(iwadname);
		}
		else dprintf(logfile,"Warn: command line option: %s not understood\n",cloptions[i]);
	}
}

int cpmain(int argc,char **argv)
{
	char cpname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("CP",argc,argv);
	if (ParseCommandLine(-1,-1))
	{
		dprintf(logfile,"Error parsing command line\n");
		FreeCLStrs();
		return 3;
	}

	if (nclstrs<1)
	{
		dprintf(logfile,"\nCP.EXE by Jim Flynn, Copyright 1998 %s -- Freeware --\n\n",PATCHER_VERSION);
		dprintf(logfile,"Usage: CP cutpaste[.CP]\n");
		dprintf(logfile,"Options: use SET CP=options to set defaults\n");
		dprintf(logfile," /W:wadname[.WAD] load COLMAPS, PLAYPAL\n");
		dprintf(logfile,"\n cutpaste spec file line formats:\n");
		dprintf(logfile,"\n 1) output BMP specification -- Specifies name and size of output\n");
		dprintf(logfile,"   BMPNAME width height [(path)] [; comment]\n");
		dprintf(logfile,"\n 2) input BMP patch -- Specifies output placement and input section of BMP:\n");
		dprintf(logfile,"   * BMPNAME xoff yoff [lite xorg yorg xsiz ysiz] [(path)] [=sign] [; comment]\n");
		dprintf(logfile,"\n 3) input WAD patch -- Specfies a wad to render as a BMP\n");
		dprintf(logfile,"   @ WADNAME xoff yoff [lite xorg yorg xsiz ysiz] [(path)] [; comment]\n");
		dprintf(logfile,"\n FIELDS:  xoff,yoff: offsets of patch within output BMP    Default: 0,0\n");
		dprintf(logfile,"          lite:      light level 0-255 to render patch     Default: 255\n");
		dprintf(logfile,"          xorg,yorg: offsets of patch within input BMP     Default: 0,0\n");
		dprintf(logfile,"          xsiz,ysiz: size of patch                         Default: full patch\n");
		dprintf(logfile," See CP.TXT for details\n");
		exit(3);
	}

	InitDoomFont();
	NoPatchWarnings=0;
	FollowCase = 0;

	memcpy(PaletteUsed,DoomPalette,256*sizeof(RGBQUAD));
	memcpy(ColorMapUsed[0],ColorMap[0],34*256);
	DoCpCommandLineOpts();

	strcpy(cpname,clfiles[0]);
	AddExt(cpname,".CP");
	dprintf(logfile,"Reading CP file\n");
	ReadCPControl(cpname);
	dprintf(logfile,"Writing BMPs\n");
	WriteCpBMPs();
	dprintf(logfile,"Done\n");

	FreeCLStrs();
	return 0;
}

