#include "patch.h"

/************************************************************
 CPSUBS.C
			 /-------------------------\
			||  CUTPASTE subroutines   ||
			 \-------------------------/
									
************************************************************/

#define INIT_PATCHALLOC 256
static int patchalloc=0;


void CreateCpSignBitmap(int i,int ltwid,Bitmap *bm)
{
	// Create the bitmap from the sign's text description
	RenderString(ltwid,mtxres[0][i].sign,bm);
	dprintf(logfile,"Info: Created %s as %d X %d patch resource\n",mtxres[0][i].name,bm->realw,bm->logh);
}

void ReadCpBMP(int i,Bitmap *bm,RGBQUAD *pal)
{
	char resfilename[256];

	// Read a bitmap from a BMP file

	if (mtxres[0][i].path && *mtxres[0][i].path)
	{
		strcpy(resfilename,mtxres[0][i].path);
		if (IsDir(mtxres[0][i].path))
		{
			AddTrailingBack(resfilename);
		  strcat(resfilename,mtxres[0][i].name);
		}
	}
	else
	{
		strcpy(resfilename,"PATCHES\\");
	  strcat(resfilename,mtxres[0][i].name);
	}

	AddExt(resfilename,".BMP");
	ReadBMP(bm,resfilename,pal);
}


// This subroutine actually writes the CUTPASTE output BMPs
// memory balanced

void WriteCpBMPs(void)
{
	int i,j;
	char *sgn;
	Bitmap out;
	Bitmap *patchbitmaps=NULL;
	int npatchbitmaps=0;
	RGBQUAD palette[256];
	char bmpname[256];

	for (i=0;i<mtxcnt[0];i++)
	{
		if (stricmp(mtxres[0][i].name,".CMAP")==0)
		{
			if (SyntaxType(mtxres[0][i].id)!=ST_RESDEFN)
			{
				if (ncmaps)
				{
					if (mtxres[0][i].path && *mtxres[0][i].path)
					{
						for (j=0;j<ncmaps;j++)
							if (stricmp(cmaps[j].name,mtxres[0][i].path)==0)
								memcpy(ColorMapUsed[0],cmaps[j].cmap[0],34*256);
					}
					else
						memcpy(ColorMapUsed[0],ColorMap[0],34*256);
				}
				else
				{
					if (!mtxres[0][i].path || !*mtxres[0][i].path)
						memcpy(ColorMapUsed[0],ColorMap[0],34*256);
					dprintf(logfile,"Warning: .COLORMAP directive, but no colormaps loaded\n");
				}
			}
		}
		else if (SyntaxType(mtxres[0][i].id)==ST_RESDEFN)		// next cutpaste definition
		{
			// write last cutpaste out

			if (i)
			{
				WriteBMP(&out,bmpname,PaletteUsed);	

				npatchbitmaps=0;							// reinitialize bitmap array
				free(patchbitmaps);
				patchbitmaps=NULL;
				free(out.bmp);
				out.bmp=NULL;
				out.logw=0;
			}

			if (mtxres[0][i].nargs==2)
			{
				out.realw = atoi(mtxres[0][i].args[0]);			 // set up output
				out.logh = atoi(mtxres[0][i].args[1]);			 	 // Bitmap for this
				out.logw = ((out.realw+3)>>2)<<2;								 // new cutpaste
				out.bmp = malloc(out.logw*out.logh*sizeof(UBYTE));
				memset(out.bmp,xparent,out.logw*out.logh);
				if (mtxres[0][i].path && *mtxres[0][i].path)
				{
					strcpy(bmpname,mtxres[0][i].path);
					if (IsDir(bmpname))
					{
						strcat(bmpname,"\\");
					  strcat(bmpname,mtxres[0][i].name);
					}
				}
				else
				{
					strcpy(bmpname,"PATCHES\\");
					strcat(bmpname,mtxres[0][i].name);
				}
				AddExt(bmpname,".BMP");
			}
			else
				dprintf(logfile,"Error: Cutpaste definition must have 2 args\n");
		}
		else if (out.logw)              // else its a non-empty patch definition
		{
			int xoff,yoff,light,xorg,yorg,xsiz,ysiz;
			int id = mtxres[0][i].id;

			if (npatchbitmaps>=patchalloc)
				patchbitmaps = realloc(patchbitmaps,(patchalloc=(patchalloc? 2*patchalloc : INIT_PATCHALLOC))*sizeof(Bitmap));
			sgn = mtxres[0][i].sign;
			if (sgn && *sgn && SyntaxType(id)==ST_RESINPUT && InputType(id)==IT_SIGN)
			{
				int wid = (mtxres[0][i].nargs>5)? atoi(mtxres[0][i].args[5]) : out.realw;
				CreateCpSignBitmap(i,wid,patchbitmaps+npatchbitmaps);
			}
			else if (SyntaxType(id)==ST_RESINPUT && InputType(id)==IT_BMP)
				ReadCpBMP(i,patchbitmaps+npatchbitmaps,palette);
			else if (SyntaxType(id)==ST_RESINPUT && InputType(id)==IT_WAD)
;//				WadToBitmap(i,patchbitmaps+npatchbitmaps);
			npatchbitmaps++;

			if (mtxres[0][i].nargs>=2)
			{
				xoff  = atoi(mtxres[0][i].args[0]);
				yoff  = atoi(mtxres[0][i].args[1]);
				light = (mtxres[0][i].nargs>2)? atoi(mtxres[0][i].args[2]) : 255;
				xorg  = (mtxres[0][i].nargs>3)? atoi(mtxres[0][i].args[3]) : 0;
				yorg  = (mtxres[0][i].nargs>4)? atoi(mtxres[0][i].args[4]) : 0;
				xsiz  = (mtxres[0][i].nargs>5)? atoi(mtxres[0][i].args[5]) : patchbitmaps[npatchbitmaps-1].realw;
				ysiz  = (mtxres[0][i].nargs>6)? atoi(mtxres[0][i].args[6]) : patchbitmaps[npatchbitmaps-1].logh;

				OverlayBitmaps(&out,patchbitmaps+npatchbitmaps-1,xoff,yoff,light,0,0,xorg,yorg,xsiz,ysiz);
			}
			else
				dprintf(logfile,"Error: Cutpaste definition must have 2 to 7 args\n");
		}
	}
	WriteBMP(&out,bmpname,PaletteUsed);	

	npatchbitmaps=0;							// reinitialize bitmap array
	free(patchbitmaps);
	patchbitmaps=NULL;
	free(out.bmp);
	out.bmp=NULL;
	out.logw=0;
}

// This routine implements reading the CUTPASTE specification file

int ReadCPControl(char *ctlname)
{
	static char buffer[4096];
	static char resname[32],bpname[256],bsign[4096];
	char *pname,*sign;
	char *p,*q;
	static char **nums=NULL;
	int nnums=0;
	int ispatch=0;
	FILE *st;
	int lineno;
	int nlines=0;
	int cond;

	MaxParseError = NoError;
	InitVTex();

	// Open the base control file, st[0], note redirection level = 0 

	lineno = 0;

	st = fopen(ctlname,"r");

	// If open successful parse the file, else report not found

	if (st!=NULL)
	{
		// get a line

		while ((p = fgets(buffer,4096,st)))		// On end of file, exit
		{
			nlines++;
			q = strchr(buffer,'\n');
			if (q) *q=0;
			lineno++;

			// Parse the line read

			pname = bpname;
			sign = bsign;
			nums=NULL; nnums=0;
			ParseCp(p,lineno,&ispatch,resname,pname,sign,&nnums,&nums,&cond);
			if (*sign=='\0') sign=NULL;
			if (*pname=='\0') pname=NULL;

			// Deal with line read

			if (*p && *p!='.' && cond)	// ignore comments, directives, blank lines	
			{
				// significant data line in cp file
				// record the line's data in the cutpiece lists

				if (ispatch!=0)
					if (resname[0]=='+')		// make patches with + indicating tall 2S
					{
						ispatch = 2;
						memmove(resname,resname+1,strlen(resname));
					}

				AddVTex
				(
					&mtxres[0],
					&mtxcnt[0],
					ispatch,
					resname,
					pname,
					sign,
					nnums,
					nums
				);
			}
		}
		fclose(st);
	}
	else
	{
		dprintf(logfile,"Fatal: Cannot open input file: %s\n",ctlname);
		errorlev=3;
		FreeParser();
		return 1;
	}

	if (MaxParseError >= PathOverflow)
	{
		dprintf(logfile,"Errors in input file, aborting CUTPASTE\n");
		errorlev=3;
		FreeParser();
		return 1;
	}
	else
		dprintf(logfile,"Cut/paste lines read sucessfully: %d\n",nlines);

	FreeParser();
	return 0;
}

