#include "patch.h"

/************************************************************
 DIFSUBS.C
			 /-------------------------\
			||    DIF subroutines     ||
			 \-------------------------/
									
************************************************************/

static int NDifs=0;

void ZeroDifCalls(void)
{
	NDifs=0;
}

int NDifCalls(void)
{
	return NDifs;
}

void WriteDifHistory(int bidx,int pidx,long *offset)
{
	StrIdxAry *sa=NULL;
	int num=0;
	char *buffer = NULL;
	char line[256],*tok;
	int i,n=0,o=0;

	ListStrIdx(&sa,&num,stringf("DIF %s [%s]%s",cmdline,GetDateTime(),LINETERM));
	if (bidx>=0)
	{
		o = bdir.dirp[bidx].resptr;
		n = bdir.dirp[bidx].reslen;
		if (n>=0)
		{
			buffer = (char *)realloc(buffer,n+1);
			fseek(bst,o,SEEK_SET);
			fread(buffer,n,1,bst);
			buffer[n]='\0';

			tok = strtok(buffer,"\r\n");
			while (tok)
			{
				strcpy(line,"  ");
				strcat(line,tok);
				strcat(line,LINETERM);
				ListStrIdx(&sa,&num,line);
				tok = strtok(NULL,"\r\n");
			}
		}
	}
	else ListStrIdx(&sa,&num,stringf("  %s: No History%s",bwadname,LINETERM));

	if (pidx>=0)
	{
		o = pdir.dirp[pidx].resptr;
		n = pdir.dirp[pidx].reslen;
		if (n>=0)
		{
			buffer = (char *)realloc(buffer,n+1);
			fseek(pst,o,SEEK_SET);
			fread(buffer,n,1,pst);
			buffer[n]='\0';

			tok = strtok(buffer,"\r\n");
			while (tok)
			{
				strcpy(line,"    ");
				strcat(line,tok);
				strcat(line,"LINETERM");
				ListStrIdx(&sa,&num,line);
				tok = strtok(NULL,"\r\n");
			}
		}
	}
	else ListStrIdx(&sa,&num,stringf("    %s: No History%s",pwadname,LINETERM));

	fseek(ost,*offset,SEEK_SET);
	n = 0;
	for (i=0;i<num;i++)
	{
		fwrite(sa[i].str,strlen(sa[i].str),1,ost);
		n+= strlen(sa[i].str);
	}
	AddDirEntry(&odir,*offset,n,"HISTORY");
	*offset += n;

	for (i=0;i<num;i++)
		free(sa[i].str);
	free(sa);
	free(buffer);
}

// Transfer the idx-th directory entry of a wad to the output wad

void WriteDifEntry(WadDir *dir,FILE *st,int idx,unsigned long *offset,WadDir *odir,FILE *ost)
{
	int o,n,offs=*offset;
	char *buffer=NULL;

	o = dir->dirp[idx].resptr;
	n = dir->dirp[idx].reslen;
    if (n>0)
    {
		buffer = (char *)realloc(buffer,n);
		fseek(st,o,SEEK_SET);
		fread(buffer,n,1,st);
		fseek(ost,offs,SEEK_SET);
		SafeWrite(buffer,n,1,ost);
		free(buffer);
    }
	AddDirEntry(odir,offs,n,Str8(dir->dirp[idx].resname));
	offs += n;

	*offset = offs;
	NDifs++;
}

void DifferencePNames()
{
	int i;
	static int doonce=1;

	if (!doonce) return;
	doonce=0;

	if (!IsSectionEnabled("TEXTURES")) return;

	for (i=0;i<bwt.pnames.npnames;i++)
		AddPName(&owt.pnames,GetPName(&bwt.pnames,i));
}

// Copy each texture in bwad that does not occur in pwad to owt.t1
//

void DifferenceTextures(int c,long *offs)
{
	int p,b,l,m;
	char tname[16];
	Category *ctgy,*ctg1,*ctg2;
	TextureLump *tlp[3],*tlb[3];
	static int doonce=1;

	if (!doonce) return;
	doonce=0;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;
	if (!IsSectionEnabled(ctgy->secname)) return;

	tlb[0] = &bwt.t1;
	tlp[0] = &pwt.t1;
	tlb[1] = &bwt.t2;
	tlp[1] = &pwt.t2;
	tlb[2] = &bwt.t3;
	tlp[2] = &pwt.t3;

	for (l=0;l<3;l++)
		for (b=0;b<tlb[l]->N;b++)
		{
			strcpy(tname,Str8(tlb[l]->textures[b].textname));
			for (m=0;m<3 && (p=FindTexture(tlp[m],tname))<0;m++)
				;
			if (p<0)
				AddTexture(&owt.t1,tlb[l]->textures+b,1);
		}

	for (l=0;l<3;l++)
		for (p=0;p<tlp[l]->N;p++)
		{
			strcpy(tname,Str8(tlp[l]->textures[p].textname));
			for (m=0;m<3 && (b=FindTexture(tlb[m],tname))<0;m++)
				;
			if (b>=0 && CmpTextures(tlb[m]->textures+b,tlp[l]->textures+p))
				AddTexture(&owt.t1,tlb[m]->textures+b,1);
		}

	if (owt.t1.N+owt.t2.N+owt.t3.N)
	{
		WriteTextures(ost,&owt,offs);
		WritePNames(ost,&owt,offs);
	}
	dprintf(logfile,"Info:  Differenced %5ld %-8s from %5ld\n",owt.t1.N,ctgy->secname,tlb[0]->N+tlb[1]->N+tlb[2]->N);
}

void WriteMarker(char *base,WadDir *dir,long offs,int mark)
{
	if (SingleMarker)
		AddDirEntry(dir,offs,0,stringf("%c_%s",mark,base));
	else
		AddDirEntry(dir,offs,0,stringf("%c%c_%s",mark,mark,base));
}

void DifferenceMarked(int c,unsigned long *offs)
{
	int k,p,m=0,e,on;
	Category *ctgy,*ctg1,*ctg2;
	int marked = 0;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;

	if (!IsSectionEnabled(ctgy->secname)) return;
	if (ctg1->nnames<=0 && ctg2->nnames<=0) return;

	ZeroDifCalls();
	on = odir.nentries;

	for (k=0;k<ctg1->nnames;k++)
	{
		m++;
		if ((p=StrIdxFind(ctg2->Names,ctg2->nnames,ctg1->Names[k].str))<0)
		{
			if (!marked)
				WriteMarker("START",&odir,*offs,ctgy->mark);
			marked=1;
			WriteDifEntry(&bdir,bst,ctg1->Names[k].didx,offs,&odir,ost);
		}
	}

	for (k=0;k<ctg2->nnames;k++)
		if ((p=StrIdxFind(ctg1->Names,ctg1->nnames,ctg2->Names[k].str))>=0)
		{
			if (ctg2->Extns[0].didx==DT_PICS)
				e = ComparePicNums(ctg1->Names[p].didx,ctg2->Names[k].didx);
			else
				e = CompareEntryNums(ctg1->Names[p].didx,ctg2->Names[k].didx);
			if (e)
			{
				if (!marked)
					WriteMarker("START",&odir,*offs,ctgy->mark);
				marked=1;
				WriteDifEntry(&bdir,bst,ctg1->Names[p].didx,offs,&odir,ost);
			}
		}
	if (marked)
		WriteMarker("END",&odir,*offs,ctgy->mark);

	dprintf(logfile,"Info:  Differenced %5d %-8s from %5d\n",NDifCalls(),ctg1->secname,m);
}

void DifferenceListed(int c,unsigned long *offs)
{
	int k,m=0,e=0;
	Category *ctgy,*ctg1,*ctg2;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;

	if (!IsSectionEnabled(ctgy->secname)) return;

	ZeroDifCalls();
	for (k=0;k<ctg1->nnames;k++)
	{
		if (ctg1->Names[k].didx>=0)
		{
			m++;
			if (ctg2->Names[k].didx<0)
				WriteDifEntry(&bdir,bst,ctg1->Names[k].didx,offs,&odir,ost);
		}
	}

	for (k=0;k<ctg2->nnames;k++)
		if (ctg1->Names[k].didx>=0 && ctg2->Names[k].didx>=0)
		{
			if (ctg2->Extns[0].didx==DT_PICS)
				e = ComparePicNums(ctg1->Names[k].didx,ctg2->Names[k].didx);
			else
				e = CompareEntryNums(ctg1->Names[k].didx,ctg2->Names[k].didx);
			if (e) WriteDifEntry(&bdir,bst,ctg1->Names[k].didx,offs,&odir,ost);
		}
	dprintf(logfile,"Info:  Differenced %5d %-8s from %5d\n",NDifCalls(),ctg1->secname,m);
}

void DifferenceLevels(int c,unsigned long *offs)
{
	int i,j,k,m=0;
	Category *ctgy,*ctg1,*ctg2;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;


	if (!IsSectionEnabled(ctgy->secname)) return;
	i = CategoryOf("LEVOBJ",&ResDef);
	if (i>=0) i = ResDef.cat[i].nnames;	// is is number of level resources

	ZeroDifCalls();
	for (k=0;k<ctg1->nnames;k++)
	{
		m++;
		if (ctg1->Names[k].didx>=0 && ctg2->Names[k].didx<0)
			for (j=ctg1->Names[k].didx;j<ctg1->Names[k].didx+1+i;j++)
				WriteDifEntry(&bdir,bst,j,offs,&odir,ost);
	}

	for (k=0;k<ctg2->nnames;k++)
		if (ctg1->Names[k].didx>=0 && ctg2->Names[k].didx>=0)
		{
			if (CmpLevels(ctg1->Names[k].didx,ctg2->Names[k].didx))
				for (j=ctg1->Names[k].didx;j<ctg1->Names[k].didx+1+i;j++)
					WriteDifEntry(&bdir,bst,j,offs,&odir,ost);
		}
	dprintf(logfile,"Info:  Differenced %5d %-8s from %5d\n",NDifCalls()/10,ctg1->secname,m);
}

int DifferenceWads(char *fname)
{
	long offs = 16;
	int c;
	Category *ctgy,*ctg1,*ctg2;

	ost = fopen(fname,"wb");
	if (ost)
	{
 		dprintf(logfile,"\nDifferencing %s and %s to %s\n",bwadname,pwadname,outname);
		for (c=0;c<ResDef.ncats;c++)
		{
			ctgy = ResDef.cat+c;
			ctg1 = bResDef.cat+c;
			ctg2 = pResDef.cat+c;

			if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
			{
				DifferencePNames();
				DifferenceTextures(c,&offs);
			}
			else if (ctgy->idm==ID_INFO || ctgy->idm==ID_NONE)
				continue;
			else if (stricmp(ctgy->secname,"HISTORY")==0)
			{
				if (IsSectionEnabled(ctgy->secname))
					WriteDifHistory(ctg1->Names[0].didx,ctg2->Names[0].didx,&offs);
			}
			else if (stricmp(ctgy->secname,"LEVELS")==0)
				DifferenceLevels(c,&offs);
			else if (ctgy->idm==ID_MARKED)
				DifferenceMarked(c,&offs);
			else if (ctgy->idm==ID_LISTED)
				DifferenceListed(c,&offs);
		}
		SafeWrite(odir.dirp,sizeof(WadDirEntry),odir.nentries,ost);
		ohdr.waddirlen = odir.nentries;
		ohdr.waddiraddr = offs;
		strncpy(ohdr.wadtag,"PWAD",4);
		fseek(ost,0,SEEK_SET);
		SafeWrite(&ohdr,sizeof(WadHeader),1,ost);

		fclose(ost);
		dprintf(logfile,"\nInfo: %s written with difference of %s and %s\n",outname,bwadname,pwadname);
	}
	return 0;
}
