#include "patch.h"

void FreeDupStructures()
{
	FreePalette();
	FreeTextures(&pwt);
	FreeVTex();
	FreeDir(&pdir);
	FreeResourceDefinition(&ResDef);
	FreeResourceDefinition(&pResDef);
	CloseStreams();
}

// Open the control files, make the output wad

int dupmain(int argc,char **argv)
{
	int i,j,n;
	int cleared=0;

	dprintf(logfile,"\n");

	CopyEnvironmentCLArgs("DUP",argc,argv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;
	}

	if (nclfiles!=1)
	{
		dprintf(logfile,"\nDUP.EXE %s by Jim Flynn, Copyright 1996-7  -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"\nCheck lump and texture names in a wad for duplicates\n");
		dprintf(logfile,"\nUsage: DUP [options] wadfile[.wad]\n");
		dprintf(logfile,"\n Resource selection option:\n");
		dprintf(logfile," +XYZ checks only XYZ resources, -XYZ checks all BUT XYZ resources.\n");
		dprintf(logfile," L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
		dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
		dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
		dprintf(logfile,"\n General options, use SET DUP=options to set defaults:\n");
		dprintf(logfile," /A Additional unmarked patches in PNAMES checked\n");
		dprintf(logfile," /X Cross check different resources against each other\n");
		dprintf(logfile,"\n Game definition options, use SET DUP=options to set defaults:\n");
		dprintf(logfile," /D:name[.DEF] Specify resource definitions file\n");
		dprintf(logfile," /DOOM2 /DOOM /HERETIC /HEXEN specify game\n");
		dprintf(logfile,"\n See DUP.TXT for details.\n");
						 
		FreeCLStrs();
		return(1);
	}

	cleared=0;
	strcpy(outname,"");
	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfLWTEMGSPF");
		if (n==strlen(cloptions[i]))
		{
			if (!cleared)	// if first selection option
			{
				if (cloptsense[i])	// clear all if it adds types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=0;
					DoGraphics=DoSprites=DoPatches=DoFlats=DoNonStandard=0;
				}
				else if (!cloptsense[i]) // set all if it subtracts types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=1;
					DoGraphics=DoSprites=DoPatches=DoFlats=DoNonStandard=1;
				}
				cleared=1;	// but don't do it for following selection opts
			}
			for (j=0;j<strlen(cloptions[i]);j++)
			{
				switch(tolower(cloptions[i][j]))
				{
					case 'l':
						DoLumps = cloptsense[i];
						break;
					case 'w':
						DoWads = cloptsense[i];
						break;
					case 't':
						DoTextures = cloptsense[i];
						break;
					case 'e':
						DoEffects = cloptsense[i];
						break;
					case 'm':
						DoMusics = cloptsense[i];
						break;
					case 'g':
						DoGraphics = cloptsense[i];
						break;
					case 's':
						DoSprites = cloptsense[i];
						break;
					case 'p':
						DoPatches = cloptsense[i];
						break;
					case 'f':
						DoFlats = cloptsense[i];
						break;
					case 'u':
						DoNonStandard = cloptsense[i];
						break;
				}
			}
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
			strcpy(defname,cloptions[i]);
		else if (tolower(cloptions[i][0])=='v')
			Verbose = 1;
		else if (tolower(cloptions[i][0])=='a')
			AdditionalPatches = 1;
		else if (tolower(cloptions[i][0])=='x')
			CrossCheckDups = 1;
		else if (tolower(cloptions[i][0])=='d' && cloptions[i][1]==':')
			strcpy(defname,cloptions[i]+2);
		else dprintf(logfile,"Warn: command line option: %s not understood\n",cloptions[i]);
	}
	strcpy(pwadname,clfiles[0]);
	FreeCLStrs();

	AddExt(pwadname,".WAD");
	AddExt(defname,".DEF");

	InitCategories(defname,&ResDef);

	pst = fopen(pwadname,"rb");
	if (pst!=NULL)
	{
		fread(&phdr,sizeof(WadHeader),1,pst);
		ReadDir(&phdr,&pdir,pst);
		LocateCategories(&pdir,&ResDef,&pResDef);

		// Locate a palette to be inserted in BMPs output

		palette = GetPalette(NULL,NULL,-1);

		ReadPNames(&pdir,pst,&pwt);
		ReadTextures(&pdir,pst,&pwt);

		return CheckDuplicates();
	}

//	FreeDupStructures();
	return errorlev;
}

