#include "patch.h"

/************************************************************
 DUPSUBS.C
			 /-------------------------\
			||     DUP subroutines     ||
			 \-------------------------/
									
************************************************************/


int CheckDuplicates()
{
	char patn[16];
	int i,j,k,c,lno=1;
	TextureDesc *td;
	Category *ctgy=NULL;
	PicData pic;
	TextureLump *tl;
	int first=1,ok=1;

	//
	// Make a list of each type of resource by category
	//
	dprintf(logfile,"\nScanning resources from %s\n",pwadname);
	for (c=0;c<pResDef.ncats;c++)
	{
		ctgy = pResDef.cat+c;
		if (IsSectionEnabled(ctgy->secname))
		{
			if (ctgy->idm==ID_INFO || ctgy->idm==ID_NONE)
				;
			else if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
			{
				tl = GetTextureLump(c,&pwt,&lno);
				if (tl)
				{
					for (j=0;j<tl->N;j++)
					{
						td = tl->textures+j;
						AddVTex2Num(&mtxres[c],&mtxcnt[c],0,
							Str8(td->textname),
							NULL,
							NULL,
							td->twid,td->thgt);
						for (k=0;k<td->npatches;k++)
						{
							strcpy(patn,GetPName(&pwt.pnames,td->patches[k].pidx));
							AddVTex2Num(&mtxres[c],&mtxcnt[c],1,patn,NULL,NULL,
								td->patches[k].xoff,td->patches[k].yoff);
						}
					}
					if (first)
						dprintf(logfile,"Info: Scanned %5ld PNAMES\n",pwt.pnames.npnames);
					first=0;
					dprintf(logfile,"Info: Scanned %5ld TEXTURE%d\n",tl->N,lno);
				}
			}
			else if (stricmp(ctgy->secname,"SOUNDS")==0)
			{
				for (i=0;i<ctgy->nnames;i++)
					if (ctgy->Names[i].didx>=0)
						AddVTex2Num(&mtxres[c],&mtxcnt[c],0,ctgy->Names[i].str,NULL,NULL,0,0);
				dprintf(logfile,"Info: Scanned %5d %-8s\n",mtxcnt[c],ctgy->secname);
			}
			else
			{
				for (i=0;i<ctgy->nnames;i++)
				{
					j = ctgy->Names[i].didx;
					if (j<0) continue;

					switch(ctgy->Extns[0].didx)
					{
						case DT_GENERIC:
							AddVTex2Num(&mtxres[c],&mtxcnt[c],0,ctgy->Names[i].str,NULL,NULL,0,0);
							break;
						case DT_LEVELS:
							AddVTex2Num(&mtxres[c],&mtxcnt[c],0,ctgy->Names[i].str,NULL,NULL,0,0);
							  break;
						case DT_PICS:
							if (pdir.dirp[j].reslen)
							{
								fseek(pst,pdir.dirp[j].resptr,SEEK_SET);
								fread(&pic,sizeof(short),4,pst);
							}
							else pic.pxoff=pic.pyoff=0;
							AddVTex2Num(&mtxres[c],&mtxcnt[c],0,ctgy->Names[i].str,NULL,NULL,pic.pxoff,pic.pyoff);
							break;
						case DT_FLATS:
							AddVTex2Num(&mtxres[c],&mtxcnt[c],0,ctgy->Names[i].str,NULL,NULL,0,0);
							break;
					}
				}
				if (stricmp(ctgy->secname,"PATCHES")==0 && AdditionalPatches)
				{
					for (i=0;i<pwt.pnames.npnames;i++)
					{
						if (IsInVTex(GetPName(&pwt.pnames,i),mtxres[c],mtxcnt[c])<0)
							if ((j=IsIn(GetPName(&pwt.pnames,i),&pdir))>=0)
							{
								if (pdir.dirp[j].reslen)
								{
									fseek(pst,pdir.dirp[j].resptr,SEEK_SET);
									fread(&pic,sizeof(short),4,pst);
								}
								else pic.pxoff=pic.pyoff=0;
								AddVTex2Num(&mtxres[c],&mtxcnt[c],0,ctgy->Names[i].str,NULL,NULL,pic.pxoff,pic.pyoff);
							}
					}
				}
	  		dprintf(logfile,"Info: Scanned %5d %-8s\n",mtxcnt[c],ctgy->secname);
			}
		}
	}

	// First check each category for duplicates within itself
	ok=1;
	for (c=0;c<pResDef.ncats;c++)
	{
		ctgy = pResDef.cat+c;
		if (IsSectionEnabled(ctgy->secname))
		{
			int intextures = strnicmp(ctgy->secname,"TEXTURE",7)==0;
			if (ctgy->idm!=ID_INFO && ctgy->idm!=ID_NONE)
			{
				for (i=0;i<mtxcnt[c];i++)
					if (!intextures || SyntaxType(mtxres[c][i].id)==ST_RESDEFN)
						for (j=i+1;j<mtxcnt[c];j++)
							if (!intextures || SyntaxType(mtxres[c][i].id)==ST_RESDEFN)
								if (stricmp(mtxres[c][i].name,mtxres[c][j].name)==0)
								{
									if (ok) dprintf(logfile,"\n");
								 	dprintf(logfile,"ERROR: Duplicate %s found: %s\n",
										              ctgy->secname,mtxres[c][i].name);
								  ok=0;
								}
			}
		}
	}
	if (ok) dprintf(logfile,"\nNo resource duplicates found\n");

	if (CrossCheckDups)
	{
		int ncats=0,cn[MAXCAT];
		int k,m;

		dprintf(logfile,"\n");

		// Check for patches/flats/sprites/graphics with the same name

		cn[ncats++] = CategoryOf("PATCHES",&pResDef);
		cn[ncats++] = CategoryOf("FLATS",&pResDef);
		cn[ncats++] = CategoryOf("SPRITES",&pResDef);
		cn[ncats++] = CategoryOf("GRAPHICS",&pResDef);
		cn[ncats++] = CategoryOf("MUSICS",&pResDef);
		cn[ncats++] = CategoryOf("SOUNDS",&pResDef);
		cn[ncats++] = CategoryOf("TRANMAPS",&pResDef);
		cn[ncats++] = CategoryOf("COLMAPS",&pResDef);

		ok=1;
		for (i=0;i<ncats-1;i++)
			for (j=i+1;j<ncats;j++)
			{
				if (cn[i]>=0 && cn[j]>=0 &&
					  IsSectionEnabled(pResDef.cat[cn[i]].secname) &&
					  IsSectionEnabled(pResDef.cat[cn[j]].secname) &&
						mtxcnt[cn[i]] && mtxcnt[cn[j]])
				{
					dprintf(logfile,"Checking %s against %s\n",
													pResDef.cat[cn[i]].secname,
													pResDef.cat[cn[j]].secname);
					for (k=0;k<mtxcnt[cn[i]];k++)
						for (m=0;m<mtxcnt[cn[j]];m++)
							if (stricmp(mtxres[cn[i]][k].name,mtxres[cn[j]][m].name)==0)
							{
								if (ok) dprintf(logfile,"\n");
								dprintf(logfile,"WARNING: %s Duplicates %s: %s\n",
																pResDef.cat[cn[i]].secname,
																pResDef.cat[cn[j]].secname,
																mtxres[cn[i]][k].name);
								ok=0;
							}
				}
			}
		if (ok) dprintf(logfile,"\nNo cross-resource duplicates found\n");
	}

	return ok;
}

