#include "patch.h"

// return the length of an opened binary file

int filelen(FILE *st)
{
	long p,l1,l2;

	p = ftell(st);
	fseek(st,0,SEEK_SET);
	l1 = ftell(st);
	fseek(st,0,SEEK_END);
	l2 = ftell(st);
	fseek(st,p,SEEK_SET);
	return l2-l1;
}

// trim spaces off left of string

void LeftTrim(char *str)
{
	int k,n;

	k = strlen(str);
	if (k)
	{
		n = strspn(str," \t");
		memmove(str,str+n, k-n+1);
	}
}

// trim spaces off right of string

void RightTrim(char *str)
{
	int k,n;

	n = k = strlen(str);
	if (k)
	{
		n--;
		while (n>=0 && (str[n]==' ' || str[n]=='\t')) n--;
		str[n+1]='\0';
	}
}

// trim spaces off both ends of a string

void Trim(char *str)
{
	RightTrim(str);
	LeftTrim(str);
}

// remove a trailing backslash, unless it starts the string or follows :

void RemoveTrailingBack(char *str)
{
	int n;
	char *p;

	if (!strlen(str)) return;
	n = strspn(str," \t");
	p = strrchr(str+n,'\\');
	if (p)
	{
		if (p==str+n) return;
		if (p[-1]==':') return;
		*p='\0';	// remove backslash
	}
}

// Right trim and add a trailing backslash, unless one present already

void AddTrailingBack(char *str)
{
	if (!str) return;
	if (strlen(str)==0)
	{
		strcpy(str,"\\");
		return;
	}

	RightTrim(str);
	if (str[strlen(str)-1]!='\\')
		strcat(str,"\\");
}
int Exists(char *filnam)
{
	FILE *st;

	if ((st=fopen(filnam,"r")))
	{
		fclose(st);
		return 1;
	}
	return 0;
}

// If the extension is missing, add the extension ext
// If the extension is present, replace it with ext

void AddExt(char *str,char *ext)
{
	char *p,*q;

	if (!str || !*str)
		return;

	p = strrchr(str,'\\');		// last backslash
	q = strrchr(str,'.');		// last period
	if (p && q && q<p)			// if is a backslash, and the . is before it
		strcat(str,ext);		// add an extension, since there is none
	else if (q)					// else if is a period, replace the extension
	{							
		*q='\0';
		strcat(str,ext);
	}
	else if (!q)				// else if no period, default the extension
		strcat(str,ext);
}

// If the extension is missing, add the extension ext
// If the extension is present, leave it be

void SupplyExt(char *str,char *ext)
{
	char *p,*q;

	if (!str || !*str)
		return;

	p = strrchr(str,'\\');		// last backslash
	q = strrchr(str,'.');		// last period
	if (p && q && q<p)			// if is a backslash, and the . is before it
		strcat(str,ext);		// add an extension, since there is none
	else if (!q)
		strcat(str,ext);		// if no period, add the extension
}

// wildcard matching *, ?, # supported, use \ to literalize, \\ for literal "\"
// note: fully embedded matching - does not work like DOS

int match(char *wild,char *cand)
{
	if (!wild) return 1;	// if wild spec is NULL, anything matches
	if (!cand) return 0;	// if cand spec is NULL, matches nothing (but NULL)

	switch(*wild)
	{
		case '\0':			// if wild spec exhausted, match is true
			return 1;

		case '*':			// if first char wild spec is *
			while (wild[1]=='*') wild++;	// skip redundant * chars
			for (;*cand;cand++)				// match rmndr w/ final substrings
			   if (match(wild+1,cand))		// does wild after * match 
			   		return 1;				// anywhere in cand?
			return 0;

		case '?':			// if first char wild spec is ?
			if (!*cand) return 0;			// if cand is empty cannot match ?
			return match(wild+1,cand+1);	// else will do, match rest

		case '#':			// if first char wild spec is #
			if ('0'>*cand || '9'<*cand)
				return 0;					// *cand not digit no match for #
			return match(wild+1,cand+1);	// else  match rest

		case '\\':			// if first char wild spec is "\" 
			if (wild[1]) wild++;			// anything following "\" is normal
											// so fall thru with char after "\" 
		default:			// any other char or any char following "\" 
			if (*wild!=*cand) return 0;		// normal chars must correspond
			return match(wild+1,cand+1);	// and then the rest must match
	}
}

int IsDir(char *pathname)
{
	unsigned attr;

	if (!_dos_getfileattr(pathname,&attr))
		return (attr&_A_SUBDIR)? 1 : 0;
	else
		return 0;
}

char *DriveOf(char *pathname)
{
	static char drive[MAXDRIVE];

	fnsplit(pathname,drive,NULL,NULL,NULL);
	return drive;
}

char *DirOf(char *pathname)
{
	static char out[MAXDIR+MAXFILE];
	char directory[MAXDIR];
	char filename[MAXFILE];

	fnsplit(pathname,NULL,directory,filename,NULL);

	if (IsDir(pathname) && *filename)
	{
		if (directory[strlen(directory)-1]!='\\')
			strcat(directory,"\\");
		strcpy(out,directory);
		strcat(out,filename);
	}
	else strcpy(out,directory);

	return out;
}

char *DriveDirOf(char *pathname)
{
	static char out[MAXDRIVE+MAXDIR+MAXFILE];

	strcpy(out,DriveOf(pathname));
	strcat(out,DirOf(pathname));

	return out;
}

char *FilenameOf(char *pathname)
{
	static char filename[MAXFILE+MAXEXT];
	char ext[MAXEXT];

	strcpy(filename,"");
	if (IsDir(pathname))
		return filename;

	fnsplit(pathname,NULL,NULL,filename,ext);
	if (ext && *ext)
		strcat(filename,ext);

	return filename;
}

char *BaseOf(char *pathname)
{
	static char filename[MAXFILE];

	strcpy(filename,"");
	if (IsDir(pathname))
		return filename;

	fnsplit(pathname,NULL,NULL,filename,NULL);

	return filename;
}

char *ExtOf(char *pathname)
{
	static char extension[MAXEXT];

	fnsplit(pathname,NULL,NULL,NULL,extension);
	return extension;
}

// Makes a directory from any path or returns error

int MakeDir(char *path,unsigned mask)
{
	char *p;
	static err=0;

	if (Exists(path)) return 0;	// if path exists, done

	if ((p=strrchr(path,'\\'))) // get path before last '\'
	{
		*p='\0';				// strip last dir component
		err=MakeDir(path,0777);	// recursively make path before last '\'
		*p='\\';				// restore last dir component
		if (err) return err;	// on error from mkdir, return err
	}
	return mkdir(path,0777);	// make the directory for last component
}

void getfilename(char *path,char *fnam)
{
	char nam[256],ext[256];

	fnsplit(path,NULL,NULL,nam,ext);
	strcpy(fnam,nam);
	strcat(fnam,ext);
}


char *NextFileName(char *base,char *ext)
{
	static char nextout[256];
	static char nbase[256];
	static char ddir[256];
	int i;

	strcpy(nbase,BaseOf(base));
	if (strlen(nbase)>5) nbase[5]='\0';
	strcpy(ddir,DriveDirOf(base));

	for (i=0;i<1000;i++)
	{
		if (*ddir)
			sprintf(nextout,"%s\\%s%03d",ddir,nbase,i);
		else
			sprintf(nextout,"%s%03d",nbase,i);
		AddExt(nextout,ext);
		if (!Exists(nextout))
			break;
	}
	if (i==1000) strcpy(nextout,"");

	return nextout;
}

// Initialize the range globals for base and patch wad sprites, flats
// levels and patches. All are set empty.

char *DOSName(char *inname)
{
	static char buffer[256];
	char *p;

	strcpy(buffer,inname);
	strupr(buffer);

	if ((p=strstr(buffer,"VILE\\")))
		p[4]='#';
	if ((p=strstr(buffer,"CLER\\")))
		p[4]='#';

	while ((p = strchr(buffer,'[')))
		*p = '@';
	while ((p = strchr(buffer,']')))
		*p = '$';
	return buffer;
}


// Convert a spritename to a valid DOS filename
// This routine is used in ReadBMP and WriteBMP for file system names only, 
// the remainder of the program uses actual sprite names.

void DOSTranslate(char *q,char *p)
{
	for (;*q;p++,q++)		// for each char
	{
		switch(*q)			// translate bad chars to something ok
		{
			case '[':
				*p = '@';
				break;
			case ']':
				*p = '$';
				break;
			default:
				*p = *q;
				break;
		}
	}
	*p = '\0';				// terminate output string
}

// Convert a sprite filename that was converted to become a legal DOS filename
// back to the actual sprite name. Not used at this time.

void DOSUnTranslate(char *q,char *p)
{
	for (;*q;p++,q++)		// for each char
	{
		switch(*q)			// translate subst'd chars back to sprite chars
		{
			case '@':
				*p = '[';
				break;
			case '#':
				*p = '\\';
				break;
			case '$':
				*p = ']';
				break;
			default:
				*p = *q;
				break;
		}
	}
	*p = '\0';				// terminate output string
}

// Safe write routine - exits on an I/O error

void SafeWrite(const void *buffer, int siz, int n, FILE *st)
{
	clearerr(st);
	if (fwrite(buffer,siz,n,st)<n)
	{
		if (ferror(st))
		{
			perror("I/O error during file write, out of space?");
			exit(2);
		}
	}
}

// Move file from a source path to a
// destination path on another drive
// overwriting an existing file by that
// name if it exists.

// dstpath must be right trimmed

int MoveFile(char *srcp, char *dstp,int mv)
{
	FILE *src,*dst;
	int err,ch;
	struct stat sfs,dfs;
	char p1[256],p2[256],*srcpath,*dstpath,fname[256];
	struct utimbuf utbf;

	if (stat(srcp,&sfs)) return SRCNFND;		// get source existence, name
	if (S_ISDIR(sfs.st_mode)) return SRCISDIR; 	// don't move a dir
	utbf.modtime = sfs.st_mtime;
	utbf.actime	= sfs.st_atime;

	srcpath = (char *)malloc(256);
	strncpy(srcpath,srcp,256);	// allocate name buffers with room
	srcpath[255]='\0';

	dstpath = (char *)malloc(256);
	strncpy(dstpath,dstp,256);
	dstpath[255]='\0';

	getfilename(srcpath,fname);
	src = fopen(srcpath,"rb");
	if (src)		   							// source open
	{
		// if dstpath is a dir, append filename of source to it

		if (stat(dstpath,&dfs)==0 && S_ISDIR(dfs.st_mode))
		{
			if (dstpath[strlen(dstpath)-1]=='\\')
				strcat(dstpath,fname);	// trailing back or root case
			else
			{
				strcat(dstpath,"\\");			// fill out the dstpath
				strcat(dstpath,fname);
			}
		} 	// (dstpath not found trapped by dst=fopen below)

		_fixpath(srcpath,p1);			// handle relative paths
		_fixpath(dstpath,p2);
		if (stricmp(p1,p2)==0)					// src==dst trap
		{
			free(srcpath); free(dstpath);
			fclose(src);
			return SRCISDST;
		}

		// open destination and perform copy

		dst = fopen(dstpath,"wb");				// truncate destination
		if (dst)								// destination open
		{
			while ((ch=fgetc(src))!=EOF)		// while not error or eof
				fputc(ch,dst);					// transfer chars

			if ((err=ferror(src)))				// if an error
			{
				fclose(src);
				fclose(dst);					// close and
				free(srcpath); free(dstpath);
				return err;						// pass error back
			}
			fclose(dst);						// close destination
			utime(dstpath,&utbf);
		}
		else
		{
			free(srcpath); free(dstpath);
			return DSTNFND;
		}
		fclose(src);							// close source
		// remove the source file if no errors and mv is true
		if (mv)
			remove(srcpath);	// delete source

	}
	else
	{
		free(srcpath); free(dstpath);
		return SRCNFND;
	}

	free(srcpath); free(dstpath);
	return 0;
}

