// Global Data Declarations
// See GLOBAL.C for definitions

#define PATCHER_VERSION "v4.98b"
#define LINETERM "\r\n"

// Constants

/*#define XPARENT 0xf7*/
#define XPARENT xparent
#define max(a,b) (((a)<(b))? (b) : (a))
#define min(a,b) (((a)<(b))? (a) : (b))
#define nextmultfour(a) ((((a)+3)>>2)<<2)

// Vertex macros -- slp is pointer to Linedef or Seg

#define V1Of(slp)     (Vertexes+(slp)->from)
#define V2Of(slp)     (Vertexes+(slp)->to)
#define VOf(slp,wch)	((wch)? V2Of(slp) : V1Of(slp))

// Coordinate macros -- slp is pointer to Linedef or Seg

#define X1Of(slp)     (Vertexes[(slp)->from].x)
#define Y1Of(slp)     (Vertexes[(slp)->from].y)
#define X2Of(slp)     (Vertexes[(slp)->to].x)
#define Y2Of(slp)     (Vertexes[(slp)->to].y)
#define DXOf(slp)     (Vertexes[(slp)->to].x-Vertexes[(slp)->from].x)
#define DYOf(slp)     (Vertexes[(slp)->to].y-Vertexes[(slp)->from].y)
#define XOf(slp,wch)	((wch)? X2Of(slp) : X1Of(slp))
#define YOf(slp,wch)	((wch)? Y2Of(slp) : Y1Of(slp))

// Linedef/Sidedef/Sector/Seg/Ssector macros
// all are safe with null first argument, so are nestable

#define LinSid(lp,sd)	((lp)->sides[(sd)]>=0? Sidedefs+(lp)->sides[(sd)] : NULL)
#define SidSec(si)    ((si)? Sectors+(si)->sect : NULL)
#define LinSec(lp,sd) ((lp)? SidSec(LinSid(lp,sd)) : NULL)
#define SegLin(sg)    ((sg)? Linedefs+(sg)->ldef : NULL)
#define SegSid(sg)    ((sg)? LinSid(SegLin(sg),sg->dir) : NULL)
#define SegSec(sg)    ((sg)? LinSec(SegLin(sg),sg->dir) : NULL)
#define SscSeg(ss)    ((ss)? Segs+(ss)->seg0 : NULL)
#define SscSec(ss)    ((ss)? SegSec(SscSeg(ss)) : NULL)

#define MAXCAT 256

#define MAXUNSINT 0xffffffff
#define MAXPOSINT 0x7fffffff
#define MAXNEGINT 0x80000000
#define MAXUNSSH 0xffff
#define MAXPOSSH 0x7fff
#define MAXNEGSH 0x8000
#define NF_SUBSECTOR 0x8000

// font color names

#define TEXTRED 0
#define TEXTBLUE 1
#define TEXTGREEN 2
#define TEXTWHITE 3
#define TEXTGRAY 4
#define TEXTYELLOW 5

// stream open function error codes

#define NOERROR 0			
#define BASEWONTOPEN 1
#define PATCHWONTOPEN  2
#define OUTWADWONTOPEN 3
#define ERRBASEDIR 4
#define ERRPATCHDIR 5

// error codes to patcher shell
//
// EXESUCCESS - exe closed normally, intended function performed
// EXEQUIT - exe closed normally, intended function not performed
// EXETERMED - exe was stopped with Ctrl-C or other supervisory function
// EXEABORT - exe was aborted due to error, patcher shell continues
// EXEFATAL	- exe was aborted and patcher shell must be too
// 
#define EXESUCCESS 0
#define EXEQUIT 1
#define EXETERMED 2
#define EXEABORT 3
#define EXEFATAL 4

extern const double pi;

extern char *LevelTag;
extern Sidedef *Sidedefs;
extern int NSidedefs;
extern Linedef *Linedefs;
extern int NLinedefs;
extern Vertex  *Vertexes;
extern int NVertexes;
extern Sector  *Sectors;
extern int NSectors;
extern Thing   *Things;
extern int NThings;
extern Seg     *Segs;
extern int NSegs;
extern Ssector *Ssectors;
extern int NSsectors;
extern Node    *Nodes;
extern int NNodes;
extern unsigned char *Reject;
extern int NReject;
extern unsigned short *Blockmap;
extern int NBlockmap;
extern int map_minx,map_miny,map_maxx,map_maxy;

extern int xparent;
extern char **resources;
extern int nresources;

extern CMap *cmaps;
extern int ncmaps;
extern TMap *tmaps;
extern int ntmaps;
extern UBYTE ipal[];
extern RGBQUAD PaletteUsed[];
extern UBYTE ColorMapUsed[][256];
extern UBYTE TranMapUsed[][256];

extern Symbol *symbols;
extern int nsymbols;
extern int symbolsalloc;

extern IfElt *ifstack;
extern int ifstackalloc;
extern int ifstacklen;

extern char **plines,**vlines,**elines;
extern int nplines,nvlines,nelines;

extern char *tagnames[];
extern char *switchnames[];
extern char *animnames[];
extern int nanimnames,nswitchnames;
extern char *fanimnames[];
extern int nfanimnames;

extern int errorlev;
extern FILE *logfile;
extern FILE *bst,*pst,*ost;
extern char iwadname[],outname[];
extern char bwadname[],pwadname[],owadname[];
extern char levmapfil[],extname[];
extern char cmdline[],resdefpath[],promptstr[];
extern RGBQUAD *palette;
extern WadHeader bhdr,phdr,ohdr;
extern WadDir bdir,pdir,odir;
extern WadTextures bwt,pwt,owt;
extern ResourceDefinition bResDef,pResDef,oResDef;
extern int *XIdx;
extern VTex *ctexture,*clevel,*csprite,*cflat,*cpatch,*cgraphic,*cmusic,*csound,*clump;
extern int nctext,ncleve,ncspri,ncflat,ncpatc,ncgrap,ncmusi,ncsoun,nclump;
extern VTex *mtxres[];
extern int mtxcnt[];
extern int ParseError, MaxParseError;

extern ResourceDefinition ResDef;
extern char defname[];

extern char *resstr[];
extern char *musstr[];
extern char *d1musstr[];

extern unsigned char ColorMap[][256];
extern unsigned char WaterMap[][256];
extern unsigned char *FontData[];
extern PicData DoomFont[];
extern unsigned char *BigData[];
extern PicData BigFont[];
extern RGBQUAD DoomPalette[];
extern int colidx[];
extern int redidx[];
extern int blueidx[];
extern int greenidx[];
extern int whiteidx[];
extern int grayidx[];
extern int yellowidx[];

extern int XParencyAdjust;
extern int CrossCheckDups;

extern int Alphabetize;
extern int PatchFormat;
extern int Xlation;
extern int DoubleSize;
extern int SingleMap;

extern int Verbose;
extern int NewOnly;
extern int MakeIwad;
extern int MakeRwad;
extern int SingleMarker;
extern int DoubleMarker;
extern int RenameControl;
extern int AdditionalPatches;
extern int FollowCase;
extern int ExtendTextures;
extern int QuietMode;
extern int NoPaletteMatching;

extern int DoLumps;			
extern int DoWads;			
extern int DoTextures;		
extern int DoEffects;		    
extern int DoMusics;			
extern int DoGraphics;		
extern int DoSprites;			
extern int DoPatches;			
extern int DoFlats;
extern int DoNonStandard;

extern int SuppressLumps;			// Disable missing lump warnings
extern int SuppressWads;			  // Disable missing level warnings
extern int SuppressTextures;		// Disable missing texture warnings
extern int SuppressEffects;		// Disable missing sound effect warnings
extern int SuppressMusics;			// Disable missing music warnings
extern int SuppressGraphics;		// Disable missing graphics warnings
extern int SuppressSprites;		// Disable missing sprites warnings
extern int SuppressPatches;		// Disable missing patches warnings
extern int SuppressFlats;			// Disable missing flats warnings
extern int SuppressNonStandard;// Disable missing non-standard resource warnings

extern int ReplaceLevels;
extern int BaseFirst;
extern int MapLevels;
extern int ReadDefinition;
extern int KeepUnusedPatches;
extern int WadSpec;
extern int JustRes;
extern int ClearHistory;
extern int InsOption;
extern int OnePageForm;
extern int PauseOutput;
extern int MultiPlayer;
extern int SaveInfoOnly;
extern int DoHists;
extern int CheckOnly;
extern int PromptForEm;
extern int MonstersOnly;
extern int TogetherTest;
extern int WallTest;
extern int EdgeTest;
extern int CeilingTest;
extern int ExternalRadii;
extern int ComparePicsGood;
extern int ViewOnly;
extern int NoPatchWarnings;

extern int Brightness;
extern int XlateTextColor;
extern int CenterText;

extern int FillEnable;		
extern char FillWad[];

extern int rad_dat[];
extern int hgt_dat[];

extern char **clfiles;	
extern int nclfiles;	
extern char **cloptions;
extern int *cloptsense;	
extern int nclopts;		
extern char **clstrs;	
extern int nclstrs;

extern char *thingflags[];
extern int nthgflags;
extern char *lineflags[];
extern int nlinflags;
extern char *sectortypes[];
extern int nsectypes;

extern THINGPAIR thingassoc[];
extern int ntassoc;
extern char *skill_strings[];
extern char *presence_strings[];
extern unsigned mplyrmask;
extern unsigned deafmask;
extern unsigned hardmask;
extern unsigned mediummask;
extern unsigned easymask;
extern THINGDATA thingdata1[];
extern THINGDATA thingdata2[];
extern THINGDATA thingdata3[];
extern THINGDATA thingdata4[];
extern THINGDATA thingdata5[];
extern int ldtrigs[];
extern int monst_iter[];
extern int ammo_iter[];
extern int weap_iter[];
extern int health_iter[];
extern int armour_iter[];

extern char *cmdnames[];
extern int (*cmd[])(int arc,char **arv);
extern int ncmds;
extern int (*cmdhelp[])(int arc,char **arv);
extern int ncmdhelps;

extern char *D2Names[];
extern int ND2Names;
extern char *D1Names[];
extern int ND1Names;

extern unsigned bmsk[];
extern int bitcnt[];

extern char *angles[];
extern int nangles;
extern char *thingflags[];
extern int nthgflags;
extern char *lineflags[];
extern int nlinflags;
extern char *sectortypes[];
extern int nsectypes;
extern char *levobjstrs[];
extern int nlevobjs;
extern long objsize[];
extern char *thing_flds[];
extern int nthingflds;
extern char *vertex_flds[];
extern int nvertexflds;
extern char *linedef_flds[];
extern int nlinedefflds;
extern char *sidedef_flds[];
extern int nsidedefflds;
extern char *sector_flds[];
extern int nsectorflds;

extern LineData linetypdata[];
extern int nlinetypes,nldtrigs;

extern ButtonArray *modeButtons,*vertButtons,*thngButtons;
extern ButtonArray *lineButtons,*sideButtons,*sectButtons;
extern ButtonArray *fthgButtons,*fsecButtons,*flinButtons;
extern ButtonArray *fdifButtons,*fprvButtons,*fvrtButtons;

// Stat monster types

#define TRO 0
#define SAR 1
#define COM 2
#define WLF 3
#define IMP 4
#define DEM 5
#define SPE 6
#define SOU 7
#define CAC 8
#define ELE 9
#define KNI 10
#define BAR 11
#define MAN 12
#define REV 13
#define ARA 14
#define VIL 15
#define CYB 16
#define SPI 17
#define BRA 18
#define SSH 19
#define SPN 20
#define KEE 21

// Stat ammo types

#define CLP 0
#define BOX 1
#define SHL 2
#define BSH 3
#define ROC 4
#define BRO 5
#define CEL 6
#define BCE 7
#define BAC 8

// Stat weapon types

#define SHO 0
#define CHG 1
#define DBS 2
#define RPG 3
#define PLR 4
#define BFG 5
#define CHS 6
#define BRL 7

// Stat health types

#define BOT 0
#define STI 1
#define MED 2
#define BES 3
#define BLU 4
#define MEG 5

// Stat armor types

#define HLM 0
#define GRA 1
#define BLA 2

// Thing type symbols

#define NOTHING 0
#define PLAYER1 1
#define PLAYER2 2
#define PLAYER3 3
#define PLAYER4 4
#define PLAYER5 4001
#define PLAYER6 4002
#define PLAYER7 4003
#define PLAYER8 4004
#define BLUECARD 5
#define YELLOWCARD 6
#define SPIDERDEMON 7
#define BACKPACK 8
#define SARGEANT 9
#define EXPLPLAYER1 10
#define DMSTART 11
#define EXPLPLAYER2	12
#define REDCARD 13
#define TPDEST 14
#define DEADPLAYER 15
#define CYBERDEMON 16
#define CELLPACK 17
#define DEADTROOPER 18
#define DEADSARGE 19
#define DEADIMP 20
#define DEADDEMON 21
#define DEADCACO 22
#define DEADSOUL 23
#define BLOODPOOL 24
#define IMPHUMAN 25
#define TWITCHHUMAN 26
#define SKULLPOLE 27
#define SKULLABOB 28
#define SKULLCANDLE 29
#define TALLGRNPILL 30
#define LOWGRNPILL 31
#define TALLREDPILL 32
#define LOWREDPILL 33
#define CANDLE 34
#define CANDELABRA 35
#define LOWGRNHEART 36
#define LOWREDSKULL 37
#define REDSKULL 38
#define YELLOWSKULL 39
#define BLUESKULL 40
#define EYESYMBOL 41
#define FLAMESKULL 42
#define GRAYTREE 43
#define HIBLUSTICK 44
#define HIGRNSTICK 45
#define HIREDSTICK 46
#define LOWBRNSCRUB 47
#define HITECHCOL 48
#define HANGNOLEG1 49
#define HANGARMOUT1 50
#define HANGONELEG1 51
#define HANGUPSIDE1 52
#define HANGCUTLEG1 53
#define LARGEBRNTREE 54
#define LOBLUSTICK 55
#define LOGRNSTICK 56
#define LOREDSTICK 57
#define SPECTRE 58
#define HANGARMOUT2 59
#define HANGONELEG2 60
#define HANGUPSIDE2 61
#define HANGCUTLEG2 62
#define HANGNOLEG2 63
#define ARCHVILE 64
#define COMMANDO 65
#define REVENANT 66
#define MANCUBUS 67
#define ARACHNOTRON 68
#define HELLKNIGHT 69
#define BURNINGBARREL 70
#define PAINELEMENTAL 71
#define HANGINGKEEN 72
#define HANGAVNOGUT 73
#define HANGAVGUTBRN 74
#define HANGAVTORSODN 75
#define HANGAVOPNSKULL 76
#define HANGAVTORSOUP 77
#define HANGAVNOBRN 78
#define COLONGIBS 79
#define SMALLBLOODPOOL 80
#define BRAINSTEM 81
#define SUPERSHOTGUN 82
#define MEGASPHERE 83
#define WOLFSOLDIER 84
#define TALLLAMP 85
#define SHORTLAMP 86
#define SPAWNSPOT 87
#define BOSSBRAIN 88
#define BOSSSHOOTER 89

#define SHOTGUN 2001
#define CHAINGUN 2002
#define ROCKETLAUNCHER 2003
#define PLASMARIFLE 2004
#define CHAINSAW 2005
#define BFG9000 2006
#define AMMOCLIP 2007
#define SHELLS 2008

#define ROCKET 2010
#define STIMPACK 2011
#define MEDIKIT 2012
#define BLUESPHERE 2013
#define BLUEBOTTLE 2014

#define ARMORHELMET 2015
#define GREENARMOR 2018
#define BLUEARMOR 2019

#define INVULNERABILITY 2022
#define BERSERKPACK 2023
#define BLURSPHERE 2024
#define RADIATIONSUIT 2025
#define COMPUTERMAP 2026

#define FLOORLAMP 2028

#define BARREL 2035

#define GOGGLES 2045
#define BOXROCKETS 2046
#define CELLBATTERY 2047
#define BOXAMMO 2048
#define BOXSHELLS 2049

#define IMPS 3001
#define DEMON 3002
#define BARON 3003
#define TROOPER 3004
#define CACODEMON 3005
#define LOSTSOUL 3006

#define PUSHER 5001
#define PULLER 5002

