#include "patch.h"

int MakHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return makmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad specification file parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies a text file containing\n",clfiles[0]);
		dprintf(logfile,"instructions for building a wad. It must have either\n");
		dprintf(logfile,".SGN, or .RES for extension. Both will be looked for if\n");
		dprintf(logfile,"not specified. The syntax of a .SGN or .RES file is\n");
		dprintf(logfile,"detailed in MAK.TXT\n");
	}
	if (nclfiles>1)
		for (i=1;i<nclfiles;i++)
			dprintf(logfile,"\nMAK only requires one parameter, %s would not be allowed\n",clfiles[i]);

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfuLWTEMGSPFU"); // its a sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			dprintf(logfile,"\nResource Selection option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"Default is to do all resources, this option allows control.\n");
			dprintf(logfile,"Letters correspond to resources according to:\n");
			dprintf(logfile,"\n L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
			dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
			dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
			dprintf(logfile,"[Note E=SOUNDS(EFFECTS), and W=LEVELS(WADS)]\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			dprintf(logfile,"\nGame Selection option:\n");
			dprintf(logfile,"-----------------------\n");
			dprintf(logfile,"Select the game definition file %s[.DEF]\n",cloptions[i]);
			dprintf(logfile,"This partially defines the structure of the game wad\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='d')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.DEF]");
			}
			else strcpy(optname,"name[.DEF]");
			dprintf(logfile,"\nCustom Game Definition option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"Select the game definition file %s\n",optname);
			dprintf(logfile,"Allows specification of a custom game definition file\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='i' && cloptions[i][1]==':')
		{
			strcpy(optname,cloptions[i]+2);
			AddExt(optname,"[.WAD]");
			dprintf(logfile,"\nInclude PLAYPAL option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The option /I:%s is used to specify a wad that has the\n",optname);
			dprintf(logfile,"PLAYPAL to be used by the wad being constructed. The .BMP's with\n");
			dprintf(logfile,"palettes differing from this PLAYPAL will be converted to it.\n");
			dprintf(logfile,"The extension on %s must be .WAD and is provided if missing\n",optname);
		}
		else if (tolower(cloptions[i][0])=='i')
		{
			dprintf(logfile,"\nIWAD output option:\n");
			dprintf(logfile,"--------------------\n");
			dprintf(logfile,"The option /I or +I causes the output wad to be given\n");
			dprintf(logfile,"the IWAD tag in its header. Whether the WAD generated has\n");
			dprintf(logfile,"the necessary resources and structure to be an IWAD is\n");
			dprintf(logfile,"not checked.\n");
		}
		else if (tolower(cloptions[i][0])=='1')
		{
			dprintf(logfile,"\nSingle Letter Marker option:\n");
			dprintf(logfile,"-----------------------------\n");
			dprintf(logfile,"The /1 or +1 option forces the directory markers used in\n");
			dprintf(logfile,"the output wads directory to be of the single letter style.\n");
			dprintf(logfile,"For example F_END, P_START as opposed to FF_START\n");
		}
		else if (tolower(cloptions[i][0])=='2')
		{
			dprintf(logfile,"\nDouble Letter Marker option:\n");
			dprintf(logfile,"-----------------------------\n");
			dprintf(logfile,"The /2 or +2 option forces the directory markers used in\n");
			dprintf(logfile,"the output wads directory to be the double letter style.\n");
			dprintf(logfile,"For example FF_END, PP_START as opposed to F_START\n");
		}
		else if (tolower(cloptions[i][0])=='r')
		{
			dprintf(logfile,"\nResource Wad Marker option:\n");
			dprintf(logfile,"----------------------------\n");
			dprintf(logfile,"The /R or +R option forces the use of numbered directory\n");
			dprintf(logfile,"markers like F1_START, P2_END, etc. creating subsections.\n");
		}
		else if (tolower(cloptions[i][0])=='c')
		{
			dprintf(logfile,"\nSign Fonts Follow Case option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"The /C or +C option causes any signs generated to use the case of\n");
			dprintf(logfile,"input text to control the sign font size selection. Uppercase\n");
			dprintf(logfile,"maps onto the larger font, lowercase onto the smaller.\n");
		}
		else if (tolower(cloptions[i][0])=='k')
		{
			dprintf(logfile,"\nKeep Unreferenced Patches option:\n");
			dprintf(logfile,"----------------------------------\n");
			dprintf(logfile,"The -K option allows MAK to ignore patches that are not referenced\n");
			dprintf(logfile,"by any texture. The default, +K is to keep all patches specified in the\n");
			dprintf(logfile,"input .SGN or .RES file\n");
		}
		else if (tolower(cloptions[i][0])=='o')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nOutput Wad Name option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The /O:%s or +O:%s option causes the output wad to be given\n",optname,optname);
			dprintf(logfile,"the name %s. The extension on %s must be .WAD and is provided\n",optname,optname);
			dprintf(logfile,"if missing\n");
		}
		else if (tolower(cloptions[i][0])=='x')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nTexture Extension option:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The /X:%s or +X:%s option will extend the texture set of the\n",optname,optname);
			dprintf(logfile,"output wad, so that it contains all textures in the wad named\n");
			dprintf(logfile,"(often the game's IWAD.) The extension on %s must be .WAD and\n",optname);
			dprintf(logfile,"is provided if missing\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}


int MerHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return mermain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nBase Wad parameter:\n");
		dprintf(logfile,"--------------------\n");
		dprintf(logfile,"The base parameter, %s, specifies a wad that you\n",clfiles[0]);
		dprintf(logfile,"wish to add another wad's resources to. It must have\n");
		dprintf(logfile,"a .WAD extension. It will be provided if missing.\n");
		if (nclfiles>1)
		{
			dprintf(logfile,"\nPatch Wad parameter:\n");
			dprintf(logfile,"---------------------\n");
			dprintf(logfile,"The patch parameter, %s, specifies the wad containing\n",clfiles[1]);
			dprintf(logfile,"resources to be added to the base wad. It must have\n");
			dprintf(logfile,"a .WAD extension. It will be provided if missing.\n");
			if (nclfiles>2)
			{
				dprintf(logfile,"\nOutput Wad parameter:\n");
				dprintf(logfile,"----------------------\n");
				dprintf(logfile,"The output parameter, %s, specifies the name the merged\n",clfiles[2]);
				dprintf(logfile,"output is to be written to. It must be different from\n");
				dprintf(logfile,"both the base and patch wad names. It must also have the\n");
				dprintf(logfile,"extension .WAD. It will be provided if missing\n");
				if (nclfiles>3)
					for (i=3;i<nclfiles;i++)
						dprintf(logfile,"\nMER only requires three parameters, %s would not be allowed\n",clfiles[i]);
			}
			else
			{
				dprintf(logfile,"\nMER requires three parameters, the parameter for\n");
				dprintf(logfile,"the output wad was missing\n");
			}
		}
		else
		{
			dprintf(logfile,"\nMER requires three parameters, the parameters for\n");
			dprintf(logfile,"the wad to be merged in, and the output wad were missing\n");
		}
	}
	else
	{
		dprintf(logfile,"\nMER requires three parameters, the parameters for\n");
		dprintf(logfile,"the base wad, the wad to be merged in, and the\n");
		dprintf(logfile,"output wad were missing\n");
	}

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfuLWTEMGSPFU"); // its a sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			dprintf(logfile,"\nResource Selection option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"Default is to do all resources, this option allows control.\n");
			dprintf(logfile,"Letters correspond to resources according to:\n");
			dprintf(logfile,"\n L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
			dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
			dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
			dprintf(logfile,"[Note E=SOUNDS(EFFECTS), and W=LEVELS(WADS)]\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			dprintf(logfile,"\nGame Selection option:\n");
			dprintf(logfile,"-----------------------\n");
			dprintf(logfile,"Select the game definition file %s[.DEF]\n",cloptions[i]);
			dprintf(logfile,"This partially defines the structure of the game wad\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='d')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.DEF]");
			}
			else strcpy(optname,"name[.DEF]");
			strcpy(optname,cloptions[i]+2);
			AddExt(optname,"[.DEF]");
			dprintf(logfile,"\nCustom Game Definition option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"Select the game definition file %s\n",optname);
			dprintf(logfile,"Allows specification of a custom game definition file\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='i' && cloptions[i][1]==':')
		{
			strcpy(optname,cloptions[i]+2);
			AddExt(optname,"[.WAD]");
			dprintf(logfile,"\nInclude PLAYPAL option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The option /I:%s is used to specify a wad that has the\n",optname);
			dprintf(logfile,"PLAYPAL to be used by the wad being constructed. The .BMP's with\n");
			dprintf(logfile,"palettes differing from this PLAYPAL will be converted to it.\n");
			dprintf(logfile,"The extension on %s must be .WAD and is provided if missing\n",optname);
		}
		else if (tolower(cloptions[i][0])=='i')
		{
			dprintf(logfile,"\nIWAD output option:\n");
			dprintf(logfile,"--------------------\n");
			dprintf(logfile,"The option /I or +I causes the output wad to be given\n");
			dprintf(logfile,"the IWAD tag in its header. Whether the WAD generated has\n");
			dprintf(logfile,"the necessary resources and structure to be an IWAD is\n");
			dprintf(logfile,"not checked.\n");
		}
		else if (tolower(cloptions[i][0])=='1')
		{
			dprintf(logfile,"\nSingle Letter Marker option:\n");
			dprintf(logfile,"-----------------------------\n");
			dprintf(logfile,"The /1 or +1 option forces the directory markers used in\n");
			dprintf(logfile,"the output wads directory to be of the single letter style.\n");
			dprintf(logfile,"For example F_END, P_START as opposed to FF_START\n");
		}
		else if (tolower(cloptions[i][0])=='2')
		{
			dprintf(logfile,"\nDouble Letter Marker option:\n");
			dprintf(logfile,"-----------------------------\n");
			dprintf(logfile,"The /2 or +2 option forces the directory markers used in\n");
			dprintf(logfile,"the output wads directory to be the double letter style.\n");
			dprintf(logfile,"For example FF_END, PP_START as opposed to F_START\n");
		}
		else if (tolower(cloptions[i][0])=='n')
		{
			dprintf(logfile,"\nNew Only or No Replacement option:\n");
			dprintf(logfile,"-----------------------------------\n");
			dprintf(logfile,"The option /N or +N will only use the resources\n");
			dprintf(logfile,"in the patch wad that do not occur in the base wad\n");
			dprintf(logfile,"Normally resources in the base wad also in the patch\n");
			dprintf(logfile,"wad are replaced by those resources in the output\n");
		}
		else if (tolower(cloptions[i][0])=='x')
		{
			dprintf(logfile,"\nExtend Textures/Sprites/Flats option:\n");
			dprintf(logfile,"--------------------------------------\n");
			dprintf(logfile,"The /X or +X option is equivalent to /N /1 +TSF.\n");
			dprintf(logfile,"It works on just textures, sprites, and flats, and\n");
			dprintf(logfile,"only copies new resources from the patch wad to the\n");
			dprintf(logfile,"output, not modified ones.\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int BrkHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return brkmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies a text file containing\n",clfiles[0]);
		dprintf(logfile,"instructions for building a wad. It must have either\n");
		dprintf(logfile,".SGN, or .RES for extension. Both will be looked for if\n");
		dprintf(logfile,"not specified. The syntax of a .SGN or .RES file is\n");
		dprintf(logfile,"detailed in MAK.TXT\n");
	}
	if (nclfiles>1)
		for (i=1;i<nclfiles;i++)
			dprintf(logfile,"\nMAK only requires one parameter, %s would not be allowed\n",clfiles[i]);

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfuLWTEMGSPFU"); // its a sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			dprintf(logfile,"\nResource Selection option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"Default is to do all resources, this option allows control.\n");
			dprintf(logfile,"Letters correspond to resources according to:\n");
			dprintf(logfile,"\n L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
			dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
			dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
			dprintf(logfile,"[Note E=SOUNDS(EFFECTS), and W=LEVELS(WADS)]\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			dprintf(logfile,"\nGame Selection option:\n");
			dprintf(logfile,"-----------------------\n");
			dprintf(logfile,"Select the game definition file %s[.DEF]\n",cloptions[i]);
			dprintf(logfile,"This partially defines the structure of the game wad\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='d')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.DEF]");
			}
			else strcpy(optname,"name[.DEF]");
			dprintf(logfile,"\nCustom Game Definition option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"Select the game definition file %s\n",optname);
			dprintf(logfile,"Allows specification of a custom game definition file\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='i' && cloptions[i][1]==':')
		{
			strcpy(optname,cloptions[i]+2);
			AddExt(optname,"[.WAD]");
			dprintf(logfile,"\nInclude PLAYPAL option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The option /I:%s is used to specify a wad that has the\n",optname);
			dprintf(logfile,"PLAYPAL to be used by the wad being constructed. The .BMP's with\n");
			dprintf(logfile,"palettes differing from this PLAYPAL will be converted to it.\n");
			dprintf(logfile,"The extension on %s must be .WAD and is provided if missing\n",optname);
		}
		else if (tolower(cloptions[i][0])=='i')
		{
			dprintf(logfile,"\nIWAD output option:\n");
			dprintf(logfile,"--------------------\n");
			dprintf(logfile,"The option /I or +I causes the output wad to be given\n");
			dprintf(logfile,"the IWAD tag in its header. Whether the WAD generated has\n");
			dprintf(logfile,"the necessary resources and structure to be an IWAD is\n");
			dprintf(logfile,"not checked.\n");
		}
		else if (tolower(cloptions[i][0])=='a')
		{
			dprintf(logfile,"\nAdditional Patches option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The option /A or +A will extract patches that are not\n");
			dprintf(logfile,"inside the normal P_START, P_END markers, or variants\n");
			dprintf(logfile,"as long as the	patch occurs in PNAMES. This is needed\n");
			dprintf(logfile,"to extract patches from wads built by some utilities\n");
		}
		else if (tolower(cloptions[i][0])=='o')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.RES]");
			}
			else strcpy(optname,"name[.RES]");
			dprintf(logfile,"\nResource Specification List Name option:\n");
			dprintf(logfile,"-----------------------------------------\n");
			dprintf(logfile,"The /O:%s or +O:%s option causes the output .RES file to be\n",optname,optname);
			dprintf(logfile,"given the name %s. The extension on %s must be .RES and is provided\n",optname,optname);
			dprintf(logfile,"if missing\n");
		}
		else if (tolower(cloptions[i][0])=='j')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.RES]");
			}
			else strcpy(optname,"name[.RES]");
			dprintf(logfile,"\nJust Write .RES File, Do Not Extract option\n");
			dprintf(logfile,"--------------------------------------------\n");
			dprintf(logfile,"The /J or +J option disables extraction only writing\n");
			dprintf(logfile,"the RES file. The output may be optionally named with\n");
			dprintf(logfile,"/J:%s or with the /O:%s option otherwise it keeps\n",optname,optname);
			dprintf(logfile,"the name of the input wad\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}


int CmpHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return cmpmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nFirst Wad parameter:\n");
		dprintf(logfile,"--------------------\n");
		dprintf(logfile,"The first parameter, %s, specifies a wad that you\n",clfiles[0]);
		dprintf(logfile,"wish to compare another wad's resources to. It must have\n");
		dprintf(logfile,"a .WAD extension. It will be provided if missing.\n");
		if (nclfiles>1)
		{
			dprintf(logfile,"\nSecond Wad parameter:\n");
			dprintf(logfile,"---------------------\n");
			dprintf(logfile,"The second parameter, %s, specifies the wad containing\n",clfiles[1]);
			dprintf(logfile,"resources to be compared with. It must have a .WAD\n");
			dprintf(logfile,"extension. It will be provided if missing.\n");
			 if (nclfiles>2)
			 	for (i=2;i<nclfiles;i++)
			 		dprintf(logfile,"\nCMP only requires two parameters, %s would not be allowed\n",clfiles[i]);
		}
		else
		{
			dprintf(logfile,"\nCMP requires two parameters, the parameter for\n");
			dprintf(logfile,"the second wad was missing\n");
		}
	}
	else
	{
		dprintf(logfile,"\nCMP requires two parameters, the parameters for\n");
		dprintf(logfile,"both wads to be compared were missing\n");
	}

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfuLWTEMGSPFU"); // its a sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			dprintf(logfile,"\nResource Selection option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"Default is to do all resources, this option allows control.\n");
			dprintf(logfile,"Letters correspond to resources according to:\n");
			dprintf(logfile,"\n L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
			dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
			dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
			dprintf(logfile,"[Note E=SOUNDS(EFFECTS), and W=LEVELS(WADS)]\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			dprintf(logfile,"\nGame Selection option:\n");
			dprintf(logfile,"-----------------------\n");
			dprintf(logfile,"Select the game definition file %s[.DEF]\n",cloptions[i]);
			dprintf(logfile,"This partially defines the structure of the game wad\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='d')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.DEF]");
			}
			else strcpy(optname,"name[.DEF]");
			dprintf(logfile,"\nCustom Game Definition option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"Select the game definition file %s\n",optname);
			dprintf(logfile,"Allows specification of a custom game definition file\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='i')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nInclude PLAYPAL option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The option /I:%s is used to specify a wad that has the\n",optname);
			dprintf(logfile,"PLAYPAL to be used by the wad being constructed. The .BMP's with\n");
			dprintf(logfile,"palettes differing from this PLAYPAL will be converted to it.\n");
			dprintf(logfile,"The extension on %s must be .WAD and is provided if missing\n",optname);
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}


int DifHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return difmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nRevised Wad parameter:\n");
		dprintf(logfile,"-----------------------\n");
		dprintf(logfile,"The revised wad parameter, %s, specifies a wad that you\n",clfiles[0]);
		dprintf(logfile,"wish to create a patch for, relative to an original wad.\n");
		dprintf(logfile,"It must have a .WAD extension. It will be provided if missing.\n");
		if (nclfiles>1)
		{
			dprintf(logfile,"\nOriginal Wad parameter:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The original wad parameter, %s, specifies the wad\n",clfiles[1]);
			dprintf(logfile,"that will be patched to create the revised wad. It must have\n");
			dprintf(logfile,"a .WAD extension. It will be provided if missing.\n");
			if (nclfiles>2)
			{
				dprintf(logfile,"\nOutput Wad parameter:\n");
				dprintf(logfile,"----------------------\n");
				dprintf(logfile,"The output parameter, %s, specifies the name patch\n",clfiles[2]);
				dprintf(logfile,"output is to be written to. It must be different from\n");
				dprintf(logfile,"both the revised and original wad names. It must also have\n");
				dprintf(logfile,"the extension .WAD. It will be provided if missing\n");
				if (nclfiles>3)
					for (i=3;i<nclfiles;i++)
						dprintf(logfile,"\nDIF only requires three parameters, %s would not be allowed\n",clfiles[i]);
			}
			else
			{
				dprintf(logfile,"\nDIF requires three parameters, the parameter for\n");
				dprintf(logfile,"the output wad was missing\n");
			}
		}
		else
		{
			dprintf(logfile,"\nDIF requires three parameters, the parameters for\n");
			dprintf(logfile,"the wad to be based on, and the output patch wad were\n");
			dprintf(logfile,"missing\n");
		}
	}
	else
	{
		dprintf(logfile,"\nDIF requires three parameters, the parameters for\n");
		dprintf(logfile,"the revised wad, the wad to be based on, and the\n");
		dprintf(logfile,"output wad were all missing\n");
	}

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfuLWTEMGSPFU"); // its a sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			dprintf(logfile,"\nResource Selection option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"Default is to do all resources, this option allows control.\n");
			dprintf(logfile,"Letters correspond to resources according to:\n");
			dprintf(logfile,"\n L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
			dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
			dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
			dprintf(logfile,"[Note E=SOUNDS(EFFECTS), and W=LEVELS(WADS)]\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			dprintf(logfile,"\nGame Selection option:\n");
			dprintf(logfile,"-----------------------\n");
			dprintf(logfile,"Select the game definition file %s[.DEF]\n",cloptions[i]);
			dprintf(logfile,"This partially defines the structure of the game wad\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='d')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.DEF]");
			}
			else strcpy(optname,"name[.DEF]");
			dprintf(logfile,"\nCustom Game Definition option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"Select the game definition file %s\n",optname);
			dprintf(logfile,"Allows specification of a custom game definition file\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='i')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nInclude PLAYPAL option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The option /I:%s is used to specify a wad that has the\n",optname);
			dprintf(logfile,"PLAYPAL to be used by the wad being constructed. The .BMP's with\n");
			dprintf(logfile,"palettes differing from this PLAYPAL will be converted to it.\n");
			dprintf(logfile,"The extension on %s must be .WAD and is provided if missing\n",optname);
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int KilHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return kilmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nInput Wad parameter:\n");
		dprintf(logfile,"-----------------------\n");
		dprintf(logfile,"The input wad parameter, %s, specifies a wad that you\n",clfiles[0]);
		dprintf(logfile,"wish to delete resources from. It must have a .WAD\n");
		dprintf(logfile,"extension. It will be provided if missing.\n");
		if (nclfiles>1)
		{
			dprintf(logfile,"\nKil specification parameter:\n");
			dprintf(logfile,"-----------------------------\n");
			dprintf(logfile,"The second parameter, %s specifies which resources to delete.\n",clfiles[1]);
			dprintf(logfile,"It may be a wad, in which case any resource listed in its\n");
			dprintf(logfile,"directory are deleted. Or it may be a .RES or .SGN file\n");
			dprintf(logfile,"in which case all the resources listed in its sections\n");
			dprintf(logfile,"are deleted.\n");
			if (nclfiles>2)
			{
				dprintf(logfile,"\nOutput Wad parameter:\n");
				dprintf(logfile,"----------------------\n");
				dprintf(logfile,"The output parameter, %s, specifies the name the\n",clfiles[2]);
				dprintf(logfile,"undeleted resources are written to. It must be different from\n");
				dprintf(logfile,"both the input and kilspec wad names. It must also have\n");
				dprintf(logfile,"the extension .WAD. It will be provided if missing\n");
				if (nclfiles>3)
					for (i=3;i<nclfiles;i++)
						dprintf(logfile,"\nKIL only requires three parameters, %s would not be allowed\n",clfiles[i]);
			}
			else
			{
				dprintf(logfile,"\nKIL requires three parameters, the parameter for\n");
				dprintf(logfile,"the output wad was missing\n");
			}
		}
		else
		{
			dprintf(logfile,"\nKIL requires three parameters, the parameters for\n");
			dprintf(logfile,"the resources to delete, and the output wad were\n");
			dprintf(logfile,"missing\n");
		}
	}
	else
	{
		dprintf(logfile,"\nKIL requires three parameters, the parameters for\n");
		dprintf(logfile,"the input wad, the resources to delete, and the\n");
		dprintf(logfile,"output wad were all missing\n");
	}

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfuLWTEMGSPFU"); // its a sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			dprintf(logfile,"\nResource Selection option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"Default is to do all resources, this option allows control.\n");
			dprintf(logfile,"Letters correspond to resources according to:\n");
			dprintf(logfile,"\n L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
			dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
			dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
			dprintf(logfile,"[Note E=SOUNDS(EFFECTS), and W=LEVELS(WADS)]\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			dprintf(logfile,"\nGame Selection option:\n");
			dprintf(logfile,"-----------------------\n");
			dprintf(logfile,"Select the game definition file %s[.DEF]\n",cloptions[i]);
			dprintf(logfile,"This partially defines the structure of the game wad\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='d')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.DEF]");
			}
			else strcpy(optname,"name[.DEF]");
			dprintf(logfile,"\nCustom Game Definition option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"Select the game definition file %s\n",optname);
			dprintf(logfile,"Allows specification of a custom game definition file\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='1')
		{
			dprintf(logfile,"\nSingle Letter Marker option:\n");
			dprintf(logfile,"-----------------------------\n");
			dprintf(logfile,"The /1 or +1 option forces the directory markers used in\n");
			dprintf(logfile,"the output wads directory to be of the single letter style.\n");
			dprintf(logfile,"For example F_END, P_START as opposed to FF_START\n");
		}
		else if (tolower(cloptions[i][0])=='2')
		{
			dprintf(logfile,"\nDouble Letter Marker option:\n");
			dprintf(logfile,"-----------------------------\n");
			dprintf(logfile,"The /2 or +2 option forces the directory markers used in\n");
			dprintf(logfile,"the output wads directory to be the double letter style.\n");
			dprintf(logfile,"For example FF_END, PP_START as opposed to F_START\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int CvtHelp(int arc,char **arv)
{
	int i;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return cvtmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\n.BMP parameters:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameters specify a collection of .BMP files\n");
		dprintf(logfile,"for palette conversion. Each parameter may contain\n");
		dprintf(logfile,"wildcards. An option specifies the palette. They\n");
		dprintf(logfile,"must have extension .BMP. It will be provided if missing\n");
	}

	for (i=0;i<nclopts;i++)
	{
	    if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			dprintf(logfile,"\nGame Selection option:\n");
			dprintf(logfile,"-----------------------\n");
			dprintf(logfile,"Select the game definition file %s[.DEF]\n",cloptions[i]);
			dprintf(logfile,"This partially defines the structure of the game wad\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='d')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.DEF]");
			}
			else strcpy(optname,"name[.DEF]");
			dprintf(logfile,"\nCustom Game Definition option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"Select the game definition file %s\n",optname);
			dprintf(logfile,"Allows specification of a custom game definition file\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='i')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nInclude PLAYPAL option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The option /I:%s is used to specify a wad that has the\n",optname);
			dprintf(logfile,"PLAYPAL to be used by the wad being constructed. The .BMP's with\n");
			dprintf(logfile,"palettes differing from this PLAYPAL will be converted to it.\n");
			dprintf(logfile,"The extension on %s must be .WAD and is provided if missing\n",optname);
		}
		else if (strchr("rgb",tolower(cloptions[i][0])))
		{
			dprintf(logfile,"\nColor Weights option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"When matching palettes, the red, green and blue components\n");
			dprintf(logfile,"can be weighted with the /R:n /G:n, and /B:n options\n");
			dprintf(logfile,"Their relative value controls the importance of the color in\n");
			dprintf(logfile,"the matching algorithm.\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}


int SetPalHelp(int arc,char **arv)
{
	int i;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return spalmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nGame Definition parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The game definition parameter is the definition file to be\n");
		dprintf(logfile,"be modified with a new palette. If you have manually edited\n");
		dprintf(logfile,"the definition palette or sign lookup you should delete those\n");
		dprintf(logfile,"sections before running this\n");

	}

	for (i=0;i<nclopts;i++)
	{
	    if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			dprintf(logfile,"\nGame Palette option:\n");
			dprintf(logfile,"-----------------------\n");
			dprintf(logfile,"Select the game definition file %s[.DEF]\n",cloptions[i]);
			dprintf(logfile,"This selects the palette to be written to the definition\n");
		}
		else if (tolower(cloptions[i][0])=='d')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.DEF]");
			}
			else strcpy(optname,"name[.DEF]");
			dprintf(logfile,"\nCustom Game Palette option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"Select the game definition file %s\n",optname);
			dprintf(logfile,"Allows specification of a custom game definition file\n");
			dprintf(logfile,"containing the palette to be written to the output\n");
			dprintf(logfile,"definition file\n");
		}
		else if (tolower(cloptions[i][0])=='i')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nInclude PLAYPAL option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The option /I:%s is used to specify a wad that has the\n",optname);
			dprintf(logfile,"PLAYPAL to be used by the game definition being output. The\n");
			dprintf(logfile,"extension on %s must be .WAD and is provided if missing\n",optname);
		}
		else if (tolower(cloptions[i][0])=='l')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.LMP]");
			}
			else strcpy(optname,"name[.LMP]");
			dprintf(logfile,"\nLump Palette option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The file %s is a binary image of the palette to be used\n",optname);
			dprintf(logfile,"PLAYPAL.LMP can be used, and is the default if the name\n");
			dprintf(logfile,"is not specified. Only the first 768 bytes of the file are\n");
			dprintf(logfile,"accessed (the first palette.)\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}


int HistHelp(int arc,char **arv)
{
	int i;

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return histmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nHistory Wad parameter:\n");
		dprintf(logfile,"-----------------------\n");
		dprintf(logfile,"The wad from which the history is to be displayed. The\n");
		dprintf(logfile,"history is an outline, showing timestamped Patcher\n");
		dprintf(logfile,"constructions used to build the wad\n");
	}
	else
	{
		dprintf(logfile,"\nHIST requires one parameter, the name of the wad\n");
		dprintf(logfile,"for which the Patcher history is displayed\n");
	}

	for (i=0;i<nclopts;i++)
	{
		dprintf(logfile,"HIST does not have any options. The option %s\n",cloptions[i]);
		dprintf(logfile,"would be ignored\n");
	}

	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int ReplHelp(int arc,char **arv)
{
	int i;

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return replmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nSubstitution List / Map Selection parameters:\n");
		dprintf(logfile,"----------------------------------------------\n");
		dprintf(logfile,"There may be any number of parameters, all but one\n");
		dprintf(logfile,"of which must be a map name to substitute in. The optional\n");
		dprintf(logfile,"non-mapname parameter is the file that contains the\n");
		dprintf(logfile,"substitutions to be made. The substitution file must\n");
		dprintf(logfile,"have extenstion .DAT, it will be supplied if not. If\n");
		dprintf(logfile,"no mapnames are specified all maps are done. If no\n");
		dprintf(logfile,"substitution file is specified REPLACE.DAT is used.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		dprintf(logfile,"\nREPL does not have any options. The option %s\n",cloptions[i]);
		dprintf(logfile,"would be ignored\n");
	}

	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int TxtUseHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return txtumain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nTxtUse Wad parameter:\n");
		dprintf(logfile,"----------------------\n");
		dprintf(logfile,"The initial parameter, %s, specifies a wad file containing\n",clfiles[0]);
		dprintf(logfile,"levels whose texture usage is to be analyzed. It is always first\n");
		dprintf(logfile,"and it is required. A .WAD extension is supplied if not present\n");
		if (nclfiles>1)
		{
			dprintf(logfile,"\nTexture and Map Selection parameters:\n");
			dprintf(logfile,"--------------------------------------\n");
			dprintf(logfile,"After the wad parameter come optional map name and\n");
			dprintf(logfile,"texture selection parameters. If any map name is listed\n");
			dprintf(logfile,"only the listed levels are processed. Map names may include\n");
			dprintf(logfile,"hyphenated ranges, Mapnn[-mm]. A texture may be specified\n");
			dprintf(logfile,"by name, possibly with wildcards, or by keywords ANIMATIONS\n");
			dprintf(logfile,"or SWITCHES. levels whose texture usage is to be analyzed. It\n");
			dprintf(logfile,"is always first and it is required. A .WAD extension is supplied\n");
			dprintf(logfile,"if not present\n");
		}
	}
	else
	{
		dprintf(logfile,"TXTUSE requires one parameter, the wad whose levels are\n");
		dprintf(logfile,"to be analyzed. This parameter was missing\n");
	}

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"dstDST"); // its a output sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			dprintf(logfile,"\nOutput Selection option:\n");
			dprintf(logfile,"-------------------------\n");
			dprintf(logfile,"Default is to write all outputs, this option allows control.\n");
			dprintf(logfile,"Letters correspond to outputs according to:\n");
			dprintf(logfile,"\n  D DETAILS    S SUMMARY    T TOTALS\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
		}
		else if (tolower(cloptions[i][0])=='i')
		{
			dprintf(logfile,"\nScan textures from TEXTUREn option:\n");
			dprintf(logfile,"------------------------------------\n");
			dprintf(logfile,"The /I or +I option takes the textures to scan from the\n");
			dprintf(logfile,"TEXTUREn resources of the wad being scanned, instead of\n");
			dprintf(logfile,"the sidedefs of the selected levels within the wad.\n");
		}
		else if (tolower(cloptions[i][0])=='r')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nResolve Textures option:\n");
			dprintf(logfile,"----------------------------\n");
			dprintf(logfile,"The /R:%s or +R:%s option loads resource wads to check\n",optname,optname);
			dprintf(logfile,"the textures in the levels against. If any textures in the levels\n");
			dprintf(logfile,"fail to occur in at least one resource wad (loaded with either\n");
			dprintf(logfile,"/W or /R) they are listed, and analyzed as usual. Missing\n");
			dprintf(logfile,"patches are also reported\n");
		}
		else if (tolower(cloptions[i][0])=='w')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nLoad Textures and Maps option:\n");
			dprintf(logfile,"---------------------------------\n");
			dprintf(logfile,"The /W:%s or +W:%s option loads resource wads to but\n",optname,optname);
			dprintf(logfile,"does not cause a check for missing textures. This can be\n");
			dprintf(logfile,"used with /U /I to check the wad loaded for unused textures.\n");
			dprintf(logfile,"It can also be used to load colormaps and tranmaps so that\n");
			dprintf(logfile,"they are not reported as missing textures on sidedefs\n");
		}
		else if (tolower(cloptions[i][0])=='u')
		{
			dprintf(logfile,"\nUnused Textures option:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The /U or +U option allows display of textures with no\n");
			dprintf(logfile,"occurrences. Useful for finding unused textures\n");
		}
		else if (tolower(cloptions[i][0])=='m')
		{
			dprintf(logfile,"\nSubstitute Missing Textures option:\n");
			dprintf(logfile,"--------------------------------------\n");
			dprintf(logfile,"The /M:textname option changes the levelwad, replacing\n");
			dprintf(logfile,"all missing textures with the one specified. This allows a\n");
		  dprintf(logfile,"level to be played anyway, or to mark the missing textures.\n");
		}
		else if (tolower(cloptions[i][0])=='c')
		{
			dprintf(logfile,"\nClear one-sided upper and lowers\n");
			dprintf(logfile,"-----------------------------------\n");
			dprintf(logfile,"The /C option substitutes all upper and lower textures\n");
			dprintf(logfile,"on one-sided (one sidedef) lines with \"-\"\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int FltUseHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return txtumain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nFltUse Wad parameter:\n");
		dprintf(logfile,"----------------------\n");
		dprintf(logfile,"The initial parameter, %s, specifies a wad file containing\n",clfiles[0]);
		dprintf(logfile,"levels whose flat usage is to be analyzed. It is always first\n");
		dprintf(logfile,"and it is required. A .WAD extension is supplied if not present\n");
		if (nclfiles>1)
		{
			dprintf(logfile,"\nTexture and Map Selection parameters:\n");
			dprintf(logfile,"--------------------------------------\n");
			dprintf(logfile,"After the wad parameter come optional map name and\n");
			dprintf(logfile,"flat selection parameters. If any map name is listed\n");
			dprintf(logfile,"only the listed levels are processed. Map names may include\n");
			dprintf(logfile,"hyphenated ranges, Mapnn[-mm]. A flat may be specified\n");
			dprintf(logfile,"by name, possibly with wildcards, or by keywords ANIMATIONS\n");
			dprintf(logfile,"or SWITCHES. levels whose flat usage is to be analyzed. It\n");
			dprintf(logfile,"is always first and it is required. A .WAD extension is supplied\n");
			dprintf(logfile,"if not present\n");
		}
	}
	else
	{
		dprintf(logfile,"FLTUSE requires one parameter, the wad whose levels are\n");
		dprintf(logfile,"to be analyzed. This parameter was missing\n");
	}

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"dstDST"); // its a output sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			dprintf(logfile,"\nOutput Selection option:\n");
			dprintf(logfile,"-------------------------\n");
			dprintf(logfile,"Default is to write all outputs, this option allows control.\n");
			dprintf(logfile,"Letters correspond to outputs according to:\n");
			dprintf(logfile,"\n  D DETAILS    S SUMMARY    T TOTALS\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
		}
		else if (tolower(cloptions[i][0])=='i')
		{
			dprintf(logfile,"\nScan flats from all provided option:\n");
			dprintf(logfile,"------------------------------------\n");
			dprintf(logfile,"The /I or +I option takes the flats to scan from the\n");
			dprintf(logfile,"set of all flats in all /R or /W specified wads.\n");
			dprintf(logfile,"Normally just the flats occuring in levels are.\n");
		}
		else if (tolower(cloptions[i][0])=='r')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nResolve Flats option:\n");
			dprintf(logfile,"----------------------------\n");
			dprintf(logfile,"The /R:%s or +R:%s option loads resource wads to check\n",optname,optname);
			dprintf(logfile,"the flats in the levels against. If any flats in the levels\n");
			dprintf(logfile,"fail to occur in at least one resource wad (loaded with either\n");
			dprintf(logfile,"/W or /R) they are listed, and analyzed as usual.\n");
		}
		else if (tolower(cloptions[i][0])=='w')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nLoad Flats option:\n");
			dprintf(logfile,"----------------------------\n");
			dprintf(logfile,"The /W:%s or +W:%s option loads resource wads to but\n",optname,optname);
			dprintf(logfile,"does not cause a check for missing flats. This is mainly\n");
			dprintf(logfile,"used with /U /I to check the wad loaded for unused flats\n");
		}
		else if (tolower(cloptions[i][0])=='u')
		{
			dprintf(logfile,"\nUnused Flats option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"The /U or +U option allows display of flats with no\n");
			dprintf(logfile,"occurrences. Useful for finding unused flats in a\n");
			dprintf(logfile,"resource wad\n");
		}
		else if (tolower(cloptions[i][0])=='m' && tolower(cloptions[i][1])==':')
		{
			dprintf(logfile,"\nSubstitute Missing Flats option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"The /M:flatname option changes the levelwad, replacing\n");
			dprintf(logfile,"all missing flats with the one specified. This allows a\n");
		  dprintf(logfile,"level to be played anyway, or to mark the missing flats.\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int DemonHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return demmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad or Lmp parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies a .WAD containing demos\n",clfiles[0]);
		dprintf(logfile,"or a demo .LMP file, as determined by extension. This\n");
		dprintf(logfile,"parameter specifies the demo(s) to be worked on.\n");
		if (nclfiles>1)
		{
			dprintf(logfile,"\nDEMO selection parameter:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The parameter, %s, specifies DEMO1, DEMO2, DEMO3,\n",clfiles[1]);
			dprintf(logfile,"or DEMO4. Only useful when working on a wads built\n");
			dprintf(logfile,"in demos to select some but not all.\n");
		}
	}
	else
	{
		dprintf(logfile,"\nDEMON requires one parameter, the name of a .WAD\n");
		dprintf(logfile,"containing demos or a .LMP file for an external demo.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='h')
		{
			dprintf(logfile,"\nDisplay Demo Header option:\n");
			dprintf(logfile,"----------------------------\n");
			dprintf(logfile,"The option -H can be used to supress the demo header display\n");
			dprintf(logfile,"which is on by default\n");
		}
		else if (tolower(cloptions[i][0])=='m')
		{
			dprintf(logfile,"\nDisplay Demo Moves option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The option /M or +M causes the moves contained in the\n");
			dprintf(logfile,"demo to be displayed, they are not shown by default\n");
		}
		else if (tolower(cloptions[i][0])=='v')
		{
			if (cloptions[i][1])
				strcpy(optname,cloptions[i]+1);
			else
				strcpy(optname,"ver");
			dprintf(logfile,"\nChange Demo Version option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The option /V%s will change the demos\n");
			dprintf(logfile,"version to the specified one. You can\n");
			dprintf(logfile,"specify 1.6, 1.7, 1.8, or 1.9.\n");
		}
		else if (tolower(cloptions[i][0])=='w')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.WAD]");
			}
			else strcpy(optname,"name[.WAD]");
			dprintf(logfile,"\nChange Demo Version to that in Wad option:\n");
			dprintf(logfile,"-------------------------------------------\n");
			dprintf(logfile,"The /W:%s or +W:%s option causes the demos version\n",optname,optname);
			dprintf(logfile,"to be changed to match the DEMO1 version in the\n");
			dprintf(logfile,"specified wad. The .WAD extension is supplied if missing.\n");
		}
		else if (tolower(cloptions[i][0])=='l')
		{
			if (cloptions[i][1])
				strcpy(optname,cloptions[i]+1);
			else
				strcpy(optname,"nn");
			dprintf(logfile,"\nChange Demo Level number:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The /L:%s or +L:%s option causes the demos level number\n",optname,optname);
			dprintf(logfile,"to be changed to that specified.\n");
		}
		else if (tolower(cloptions[i][0])=='t' && cloptions[i][1]==':')
		{
			strcpy(optname,cloptions[i]+2);
			n = atoi(optname);
			dprintf(logfile,"\nTruncate Demo option:\n");
			dprintf(logfile,"-------------------------------------------\n");
			dprintf(logfile,"The /T:%d or +T:%d option truncates the demo to\n",n,n);
			dprintf(logfile,"%d gametics.\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int CledHelp(int arc,char **arv)
{
	int i;

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return cledmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad to Edit parameter:\n");
		dprintf(logfile,"--------------------\n");
		dprintf(logfile,"The first parameter, %s, specifies a wad that you\n",clfiles[0]);
		dprintf(logfile,"wish to examine with CLED. It must have a .WAD\n");
		dprintf(logfile,"extension. It will be supplied if missing.\n");
		if (nclfiles>1)
		{
			dprintf(logfile,"\nObjects to Edit parameter:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The second parameter, %s, specifies the objects\n",clfiles[1]);
			dprintf(logfile,"that are displayed or edited. It must contain no\n");
			dprintf(logfile,"spaces or tabs. The syntax is shown on the line below:\n");
			dprintf(logfile,"\n LEVOBJ[(mapn[,objn])][.FIELD][=value]\n");
        	dprintf(logfile,"\nChanges a LEVOBJ = THING,VERTEX,LINEDEF,SIDEDEF,SECTOR\n");
        	dprintf(logfile,"The object numbered n in Map m, is affected\n");
        	dprintf(logfile,"FIELD depends on which LEVOBJ. FIELDS for:\n");
        	dprintf(logfile,"-------------------------------------------------------------------------------\n");
        	dprintf(logfile,"THING VERTEX LINEDEF SIDEDEF SECTOR |          Values\n");
        	dprintf(logfile,"-------------------------------------------------------------------------------\n");
        	dprintf(logfile,"X      X     FROM    UPPER   FHGT   | FLAGS:\n");
        	dprintf(logfile,"Y      Y     TO      LOWER   CHGT   |  THING   = Ez+Md+Hd+Df+Mp\n");
        	dprintf(logfile,"TYPE         FLAGS   NORMAL  FLOOR  |  LINEDEF = Im+Mo+2s+Up+Lw+Se+So+In+Ma\n");
        	dprintf(logfile,"ANGLE        TYPE    XOFF    CEILING| THING ANGLES: N NE E SE S SW W NW\n");
        	dprintf(logfile,"FLAGS        TAG     YOFF    TAG    | SECTOR TYPES\n");
        	dprintf(logfile,"             RSID    SECT    LIGHT  |  Normal, Random, FBlnk, SBlnk, Osc, Open\n");
        	dprintf(logfile,"             LSID            TYPE   |  FSync, SSync, Flicker, DamBlk, Close\n");
        	dprintf(logfile,"                                    |  Dam5, Dam10, Dam20, DamExit, Secret\n");
			 if (nclfiles>2)
			 	for (i=2;i<nclfiles;i++)
			 		dprintf(logfile,"\nCLED only requires two parameters, %s would not be allowed\n",clfiles[i]);
		}
		else
		{
			dprintf(logfile,"\nCLED requires two parameters, the second\n");
			dprintf(logfile,"parameter for the level objects to display/change\n");
			dprintf(logfile,"was missing.\n");
		}
	}
	else
	{
		dprintf(logfile,"\nCLED requires two parameters, the parameters for\n");
		dprintf(logfile,"the wad file being edited and the level objects to\n");
		dprintf(logfile,"edit were missing\n");
	}

	for (i=0;i<nclopts;i++)
	{
		dprintf(logfile,"CLED does not have any options. The option %s\n",cloptions[i]);
		dprintf(logfile,"would be ignored\n");
	}

	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int StatHelp(int arc,char **arv)
{
	int i;

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return statmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies a .WAD which is to be analyzed\n",clfiles[0]);
		if (nclfiles>1)
		{
			dprintf(logfile,"\nMAP selection parameter:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The parameter, %s, specifies a level within the wad\n",clfiles[1]);
			dprintf(logfile,"to be analyzed. If missing, all levels are analyzed.\n");
		}
	}
	else
	{
		dprintf(logfile,"\nSTAT requires one parameter, the name of a .WAD\n");
		dprintf(logfile,"to be analyzed.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='m')
		{
			dprintf(logfile,"\nMultiplayer things option:\n");
			dprintf(logfile,"----------------------------\n");
			dprintf(logfile,"The option /M or +M can be used to include multiplayer only\n");
			dprintf(logfile,"things in the analysis. Normally they are not.\n");
		}
		else if (tolower(cloptions[i][0])=='p')
		{
			dprintf(logfile,"\nPause after main display option:\n");
			dprintf(logfile,"---------------------------------\n");
			dprintf(logfile,"The option /P or +P causes the display to pause after\n");
			dprintf(logfile,"each level's main screen.\n");
		}
		else if (tolower(cloptions[i][0])=='f')
		{
			dprintf(logfile,"\nFull display option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The option /F or +F will output more data after\n");
			dprintf(logfile,"the main screen detailing secrets, weapons, monsters\n");
			dprintf(logfile,"power-ups etc, and their locations\n");
		}
		else if (tolower(cloptions[i][0])=='s')
		{
			dprintf(logfile,"\nSave estimate option:\n");
			dprintf(logfile,"----------------------\n");
			dprintf(logfile,"The /S option outputs only the information pertinent to\n");
			dprintf(logfile,"estimation of the savegame size.\n");
		}
		else if (tolower(cloptions[i][0])=='c')
		{
			dprintf(logfile,"\nCheck option:\n");
			dprintf(logfile,"--------------\n");
			dprintf(logfile,"The /C or +C option outputs the number of player starts,\n");
			dprintf(logfile,"the estimated savegame size, and the REJECT percentage as\n");
			dprintf(logfile,"a quick check on the level's completeness.\n");
		}
		else if (tolower(cloptions[i][0])=='h')
		{
			dprintf(logfile,"\nThing histogram option:\n");
			dprintf(logfile,"------------------------\n");
			dprintf(logfile,"The /H or +H option lists all things occuring in each level\n");
			dprintf(logfile,"and the totals across all levels analyzed if more than one.\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int NbldHelp(int arc,char **arv)
{
	int i;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

 	if (!nclfiles && !nclopts) return nbldmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nInput wad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies a .WAD which is to be built\n",clfiles[0]);
		if (nclfiles>1)
		{
			dprintf(logfile,"\nMAP selection parameter:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The parameter, %s, specifies a level within the wad\n",clfiles[1]);
			dprintf(logfile,"to be built. If missing, all levels are built.\n");
		}
	}
	else
	{
		dprintf(logfile,"\nNBLD requires one parameter, the name of a .WAD\n");
		dprintf(logfile,"to be analyzed.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='v')
		{
			dprintf(logfile,"\nVerbose output option:\n");
			dprintf(logfile,"----------------------------\n");
			dprintf(logfile,"The option /V or +V can be used to enable detailed\n");
			dprintf(logfile,"printouts during build - normally a debugging aid.\n");
		}
		else if (tolower(cloptions[i][0])=='o')
		{
			if (cloptions[i][1]==':')
			{
				strcpy(optname,cloptions[i]+2);
				if (!*optname)
					strcpy(optname,"output[.WAD]");
				dprintf(logfile,"\nOutput wad name option:\n");
				dprintf(logfile,"---------------------------------\n");
				dprintf(logfile,"The option /O:%s or +O:%s writes the built\n",optname,optname);
				dprintf(logfile,"wad to the file name %s.\n",optname);
			}
			else
			{
				dprintf(logfile,"\nOverwrite input wad option:\n");
				dprintf(logfile,"---------------------------------\n");
				dprintf(logfile,"The option /O or +O writes the built\n");
				dprintf(logfile,"wad over the input wad.\n");
			}
		}
		else if (tolower(cloptions[i][0])=='b')
		{
			if (cloptions[i][1]==':')
				strcpy(optname,cloptions[i]+2);
			if (!*optname)
				strcpy(optname,"balance-weight");
			dprintf(logfile,"\nSet balance weight option:\n");
			dprintf(logfile,"-----------------------------\n");
			dprintf(logfile,"The option /B:%s or +B:%s sets the importance\n",optname,optname);
			dprintf(logfile,"of balancing the BSP tree when picking nodes\n");
			dprintf(logfile,"Default is 1.\n");
		}
		else if (tolower(cloptions[i][0])=='s')
		{
			if (cloptions[i][1]==':')
				strcpy(optname,cloptions[i]+2);
			if (!*optname)
				strcpy(optname,"split-weight");
			dprintf(logfile,"\nSet split weight option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The option /S:%s or +S:%s sets the importance\n",optname,optname);
			dprintf(logfile,"of avoiding seg splits when picking nodes\n");
			dprintf(logfile,"Default is 17.\n");
		}
		else if (tolower(cloptions[i][0])=='e')
		{
			if (cloptions[i][1]==':')
				strcpy(optname,cloptions[i]+2);
			if (!*optname)
				strcpy(optname,"effect-weight");
			dprintf(logfile,"\nSet effect weight option:\n");
			dprintf(logfile,"----------------------------\n");
			dprintf(logfile,"The option /E:%s or +E:%s sets the importance\n",optname,optname);
			dprintf(logfile,"of not splitting lines marked with tags 900 or bigger\n");
			dprintf(logfile,"Default is 100.\n");
		}
		else if (tolower(cloptions[i][0])=='t')
		{
			if (cloptions[i][1]==':')
				strcpy(optname,cloptions[i]+2);
			if (!*optname)
				strcpy(optname,"threshold");
			dprintf(logfile,"\nSet optimize threshold option:\n");
			dprintf(logfile,"---------------------------------\n");
			dprintf(logfile,"The option /T:%s or +T:%s will try all possible\n",optname,optname);
			dprintf(logfile,"splits if there are less than %s segs to split\n",optname);
			dprintf(logfile,"Builds are slower, but more responsive to weights when large.\n");
			dprintf(logfile,"Default is 10. 0 tries all splits always.\n");
		}
		else if (tolower(cloptions[i][0])=='r')
		{
			dprintf(logfile,"\nReject option:\n");
			dprintf(logfile,"-----------------\n");
			dprintf(logfile,"The /R or +R option causes the reject to be built (NIY)\n");
			dprintf(logfile,"The -R option causes the reject to be preserved\n");
			dprintf(logfile,"The default, with no R option is to zero the reject\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}


int GammaHelp(int arc,char **arv)
{
	int i;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return gammain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies the .WAD in which lighting\n",clfiles[0]);
		dprintf(logfile,"is to be modified\n",clfiles[0]);
		if (nclfiles>1)
		{
			dprintf(logfile,"\nMAP selection parameter:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The parameter, %s, specifies a level within the wad\n",clfiles[1]);
			dprintf(logfile,"that you wish to modify lighting in. If missing, all\n");
			dprintf(logfile,"levels are modified.\n");
		}
	}
	else
	{
		dprintf(logfile,"\nGAMMA requires one parameter, the name of a .WAD\n");
		dprintf(logfile,"to be analyzed.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='l')
		{
			if (cloptions[i][1]==':')
				strcpy(optname,cloptions[i]+2);
			else
				strcpy(optname,"nn");
			dprintf(logfile,"\nLower light bound option:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The option /L:%s or +L:%s is used to prevent\n",optname,optname);
			dprintf(logfile,"any light level from being set lower than the\n");
			dprintf(logfile,"number specified.\n");
		}
		else if (tolower(cloptions[i][0])=='u')
		{
			if (cloptions[i][1]==':')
				strcpy(optname,cloptions[i]+2);
			else
				strcpy(optname,"nn");
			dprintf(logfile,"\nUpper light bound option:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The option /U:%s or +U:%s is used to prevent\n",optname,optname);
			dprintf(logfile,"any light level from being set higher than the\n");
			dprintf(logfile,"number specified.\n");
		}
		else if (IsNumeric(cloptions[i]+2))
		{
			dprintf(logfile,"\nLighting change specification option:\n");
			dprintf(logfile,"--------------------------------------\n");
			dprintf(logfile,"The option /nnn or +nnn causes the lighting to be\n");
			dprintf(logfile,"incremented by nnn in each sector of the wad.\n");
			dprintf(logfile,"\nThe option -nnn causes the lighting to be\n");
			dprintf(logfile,"decremented by nnn in each sector of the wad.\n");
			dprintf(logfile,"\nThe option /nn%% or +nn%% causes the lighting to be\n");
			dprintf(logfile,"incremented by nn percent in each sector of the wad.\n");
			dprintf(logfile,"\nThe option -nn%% causes the lighting to be\n");
			dprintf(logfile,"decremented by nn percent in each sector of the wad.\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int RenlinHelp(int arc,char **arv)
{
	int i;

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return renmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies the .WAD in which lines\n");
		dprintf(logfile,"are to be renumbered.\n",clfiles[0]);
		if (nclfiles>1)
		{
			if (IsNumeric(clfiles[1]))
			{
				dprintf(logfile,"\nLine number parameters:\n");
				dprintf(logfile,"--------------------------\n");
				dprintf(logfile,"You may list as many line numbers after the\n");
				dprintf(logfile,"wadname as you wish to renumber the lowest in the wad\n");
				if (nclfiles>2)
				{
					if (!IsNumeric(clfiles[2]))
					{
						dprintf(logfile,"\nMAP selection or line number parameters:\n");
						dprintf(logfile,"--------------------------\n");
						dprintf(logfile,"The parameter, %s, specifies a level within the wad\n",clfiles[2]);
						dprintf(logfile,"that you wish to renumber lines in. If missing, all\n");
						dprintf(logfile,"levels are modified.\n");
					}
				}
			}
			else
			{
				dprintf(logfile,"\nMAP selection or line number parameters:\n");
				dprintf(logfile,"--------------------------\n");
				dprintf(logfile,"The parameter, %s, specifies a level within the wad\n",clfiles[1]);
				dprintf(logfile,"that you wish to renumber lines in. If missing, all\n");
				dprintf(logfile,"levels are modified.\n");
				if (nclfiles>2)
				{
					if (IsNumeric(clfiles[2]))
					{
						dprintf(logfile,"\nLine number parameters:\n");
						dprintf(logfile,"--------------------------\n");
						dprintf(logfile,"You may list as many line numbers after the\n");
						dprintf(logfile,"wadname as you wish to renumber the lowest in the wad\n");
					}
				}
			}
		}
	}
	else
	{
		dprintf(logfile,"\nRENLIN requires one parameter, the name of a .WAD\n");
		dprintf(logfile,"in which linedefs are to be renumbered.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='p')
		{
			dprintf(logfile,"\nPrompt for lines option:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The option /P or +P causes RENLIN to prompt for\n");
			dprintf(logfile,"line numbers to be renumbered lowest. These are added\n");
			dprintf(logfile,"after any specified on the command line.\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int MoveitHelp(int arc,char **arv)
{
	int i;

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

 	if (!nclfiles && !nclopts) return movemain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies the .WAD in which levels\n",clfiles[0]);
		dprintf(logfile,"are to be moved\n",clfiles[0]);
		if (nclfiles>1)
		{
			dprintf(logfile,"\nX movement parameter:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The parameter, %s, specifies how coordinates\n",clfiles[1]);
			dprintf(logfile,"are to change. If just a number it specifies the\n");
			dprintf(logfile,"x coordinate motion. If of the form y:13 or z=C it can\n");
			dprintf(logfile,"specify any coordinate. When the letter C is specified\n");
			dprintf(logfile,"it centers the coordinate insted of moving it.\n");
			if (nclfiles>2)
			{
				dprintf(logfile,"\nY movement parameter:\n");
				dprintf(logfile,"--------------------------\n");
				dprintf(logfile,"The parameter, %s, specifies how coordinates\n",clfiles[2]);
				dprintf(logfile,"are to change. If just a number it specifies the\n");
				dprintf(logfile,"y coordinate motion. If of the form x:13 or z=C it can\n");
				dprintf(logfile,"specify any coordinate. When the letter C is specified\n");
				dprintf(logfile,"it centers the coordinate insted of moving it.\n");
				if (nclfiles>3)
				{
					dprintf(logfile,"\nZ movement parameter:\n");
					dprintf(logfile,"--------------------------\n");
					dprintf(logfile,"The parameter, %s, specifies how coordinates\n",clfiles[3]);
					dprintf(logfile,"are to change. If just a number it specifies the\n");
					dprintf(logfile,"z coordinate motion. If of the form y:13 or x=C it can\n");
					dprintf(logfile,"specify any coordinate. When the letter C is specified\n");
					dprintf(logfile,"it centers the coordinate insted of moving it.\n");
				}
			}
		}
	}
	else
	{
		dprintf(logfile,"\nMOVEIT requires one parameter, the name of a .WAD\n");
		dprintf(logfile,"to be moved.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='l' && cloptions[i][1]==':')
		{
			dprintf(logfile,"\nLevel selection option:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The option %s is used to restrict\n",cloptions[i]);
			dprintf(logfile,"MOVEIT to the one level named. Default is to do all levels\n");
		}
		else if (tolower(cloptions[i][0])=='g')
		{
			dprintf(logfile,"\nPreserve 64X64 grid option:\n");
			dprintf(logfile,"------------------------------\n");
			dprintf(logfile,"The option /G is used to keep all vertices in the\n");
			dprintf(logfile,"same relationship to the 64X64 grid after the move\n");
			dprintf(logfile,"that they had prior to the move\n");
		}
		else if (stricmp(cloptions[i],"doom")==0)
		{
			dprintf(logfile,"\nDOOM 1 option:\n");
			dprintf(logfile,"-----------------\n");
			dprintf(logfile,"The option /DOOM allows DOOM 1 wads to be moved\n");
			dprintf(logfile,"Use /L:ExMy instead of /L:MAPnn if you use /DOOM\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}


int NosecretHelp(int arc,char **arv)
{
	int i;

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return nosmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies the .WAD in which secrets\n",clfiles[0]);
		dprintf(logfile,"are to be removed\n",clfiles[0]);
		for (i=1;i<nclfiles;i++)
		{
			dprintf(logfile,"\nMAP selection parameter:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The parameter, %s, specifies a level that \n",clfiles[i]);
			dprintf(logfile,"you wish to clear secrets in. If missing, all\n");
			dprintf(logfile,"levels' secrets are cleared.\n");
		}
	}
	else
	{
		dprintf(logfile,"\nNOSECRET requires one parameter, the name of a .WAD\n");
		dprintf(logfile,"to clear all secrets from.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		dprintf(logfile,"NOSECRET does not have any options. The option %s\n",cloptions[i]);
		dprintf(logfile,"would be ignored\n");
	}

	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int StuckHelp(int arc,char **arv)
{
	int i,n;

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return stucmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies the .WAD that thing placement\n");
		dprintf(logfile,"is to be checked in\n",clfiles[0]);
		if (nclfiles>1)
		{
			dprintf(logfile,"\nMAP selection parameter:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The parameter, %s, specifies a level or range of levels\n",clfiles[1]);
			dprintf(logfile,"that are to be checked for thing placement. If missing, all\n");
			dprintf(logfile,"all levels are checked.\n");
		}
	}
	else
	{
		dprintf(logfile,"\nSTUCK requires one parameter, the name of a .WAD\n");
		dprintf(logfile,"to be checked for thing placement.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"twecTWEC"); // its a test select option
		if (n==strlen(cloptions[i]))	// if all are test selection chars
		{
			dprintf(logfile,"\nTest selection option:\n");
			dprintf(logfile,"-------------------------\n");
			dprintf(logfile,"Default is to do all tests, this option allows control.\n");
			dprintf(logfile,"Letters correspond to tests according to:\n");
			dprintf(logfile,"\n  T TOGETHER     W WALL    E EDGE    C CEILING\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
			dprintf(logfile,"\nTOGETHER tests for things stuck together\n");
			dprintf(logfile,"WALL tests for things stuck in walls\n");
			dprintf(logfile,"EDGE tests for things placed on edges with drops>24\n");
			dprintf(logfile,"CEILING tests for things too tall for their ceiling\n");
		}
		else if (tolower(cloptions[i][0])=='m')
		{
			dprintf(logfile,"\nMultiplayer things option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The option /M or +M can be used to include multiplayer only\n");
			dprintf(logfile,"things in the tests. Normally they are not.\n");
		}
		else if (tolower(cloptions[i][0])=='a')
		{
			dprintf(logfile,"\nAll things option:\n");
			dprintf(logfile,"-------------------\n");
			dprintf(logfile,"The option /A or +A will cause all things to be tested.\n");
			dprintf(logfile,"Normally, only monsters are tested.\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int MusputHelp(int arc,char **arv)
{
	int i;

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return muspmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nMusic source parameter:\n");
		dprintf(logfile,"------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies the source for the music\n",clfiles[0]);
		dprintf(logfile,"to be put in the destination wad. It can be a .MUS file, a .MID\n");
		dprintf(logfile,"file, or a .WAD file, with @nn added to specify level nn's music\n");
		if (nclfiles>1)
		{
			dprintf(logfile,"\nDestination wad parameter:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The parameter, %s, specifies the destination wad for the music\n",clfiles[1]);
			dprintf(logfile,"with @nn appended to indicate that the music is for level nn.\n");
		}
		else
		{
			dprintf(logfile,"\nMUSPUT requires two parameters, the music source\n");
			dprintf(logfile,"and the destination .WAD. The destination was missing.\n");
		}
	}
	else
	{
		dprintf(logfile,"\nMUSPUT requires two parameters, the music source\n");
		dprintf(logfile,"and the destination .WAD.\n");
	}

	for (i=0;i<nclopts;i++)
	{
		dprintf(logfile,"MUSPUT does not have any options. The option %s\n",cloptions[i]);
		dprintf(logfile,"would be ignored\n");
	}

	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int DupHelp(int arc,char **arv)
{
	int i,n;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return dupmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies a wad that you want to\n",clfiles[0]);
		dprintf(logfile,"check for duplicate resource names.\n");
	}
	if (nclfiles>1)
		for (i=1;i<nclfiles;i++)
			dprintf(logfile,"\nDUP only requires one parameter, %s would not be allowed\n",clfiles[i]);

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfuLWTEMGSPFU"); // its a sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			dprintf(logfile,"\nResource Selection option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"Default is to do all resources, this option allows control.\n");
			dprintf(logfile,"Letters correspond to resources according to:\n");
			dprintf(logfile,"\n L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
			dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
			dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
			dprintf(logfile,"[Note E=SOUNDS(EFFECTS), and W=LEVELS(WADS)]\n");
			dprintf(logfile,"\nSelect just X and Y with +XY\n");
			dprintf(logfile,"Select all but X, Y and Z with -XYZ\n");
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			dprintf(logfile,"\nGame Selection option:\n");
			dprintf(logfile,"-----------------------\n");
			dprintf(logfile,"Select the game definition file %s[.DEF]\n",cloptions[i]);
			dprintf(logfile,"This partially defines the structure of the game wad\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='d')
		{
			if (cloptions[i][1]==':' && cloptions[i][2])
			{
				strcpy(optname,cloptions[i]+2);
				AddExt(optname,"[.DEF]");
			}
			else strcpy(optname,"name[.DEF]");
			dprintf(logfile,"\nCustom Game Definition option:\n");
			dprintf(logfile,"-------------------------------\n");
			dprintf(logfile,"Select the game definition file %s\n",optname);
			dprintf(logfile,"Allows specification of a custom game definition file\n");
			dprintf(logfile,"If the /I:wad option was not used it can also provide the\n");
			dprintf(logfile,"palette.\n");
		}
		else if (tolower(cloptions[i][0])=='a')
		{
			dprintf(logfile,"\nAdditional Patches option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The option /A or +A will extract patches that are not\n");
			dprintf(logfile,"inside the normal P_START, P_END markers, or variants\n");
			dprintf(logfile,"as long as the	patch occurs in PNAMES. This is needed\n");
			dprintf(logfile,"to see patches from wads built by some utilities\n");
		}
		else if (tolower(cloptions[i][0])=='x')
		{
			dprintf(logfile,"\nCross Resource option:\n");
			dprintf(logfile,"---------------------------\n");
			dprintf(logfile,"The option /X or +X will check the names used for one\n");
			dprintf(logfile,"kind of resource for duplicating names used for other\n");
			dprintf(logfile,"kinds of resource. The default is just to check for\n");
			dprintf(logfile,"duplicates within each resource type\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}
	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}

int ViewHelp(int arc,char **arv)
{
	int i;
	char optname[256];

	InitCLStrs();
	CopyEnvironmentCLArgs("_NOT!DEFINED_",arc,arv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

   	if (!nclfiles && !nclopts) return viewmain(0,NULL);

	if (nclfiles>0)
	{
		dprintf(logfile,"\nWad parameter:\n");
		dprintf(logfile,"---------------------------------\n");
		dprintf(logfile,"The parameter, %s, specifies the .WAD to view\n",clfiles[0]);
		dprintf(logfile,"are to be removed\n",clfiles[0]);
		for (i=1;i<nclfiles;i++)
		{
			dprintf(logfile,"\nMAP selection parameter:\n");
			dprintf(logfile,"--------------------------\n");
			dprintf(logfile,"The optional parameter, %s, specifies a level within\n",clfiles[i]);
			dprintf(logfile,"the wad to view. The first level is viewed by default\n");
		}
	}
	else
	{
		dprintf(logfile,"\nVIEW requires one parameter, the name of a .WAD\n");
	}

	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='r')
		{
			strcpy(optname,"nnnn");
			if (cloptions[i][1]==':')
				strcpy(optname,cloptions[i]+2);
			dprintf(logfile,"\nHorizontal Resolution option:\n");
			dprintf(logfile,"-------------------------------------------\n");
			dprintf(logfile,"The /R:%s or +R:%s option sets the horizontal screen\n");
			dprintf(logfile,"resolution to one of 640,800,1024,1280, or 1600\n");
			dprintf(logfile,"\nNote: Win'95 requires that resolutions higher than 640 be\n");
			dprintf(logfile,"run in a full-screen session. Alt-Enter will toggle a DOS Window.\n");
		}
		else if (tolower(cloptions[i][0])=='f')
		{
			dprintf(logfile,"\nFull View option:\n");
			dprintf(logfile,"-------------------------------------------\n");
			dprintf(logfile,"The /F option makes the initial view mode fill\n");
			dprintf(logfile,"the screen, hiding data displays and filter buttons.\n");
			dprintf(logfile,"CTRL-F within the viewer will toggle this state\n");
		}
		else if (tolower(cloptions[i][0])=='v')
		{
			dprintf(logfile,"\nView Only option:\n");
			dprintf(logfile,"-------------------------------------------\n");
			dprintf(logfile,"The /V option restricts program operations to\n");
			dprintf(logfile,"strictly those of a viewer.\n");
		}
		else if (tolower(cloptions[i][0])=='t')
		{
			if (cloptions[i][1]==':')
				strcpy(optname,cloptions[i]+2);
			else
				strcpy(optname,"textures[.WAD]");
			dprintf(logfile,"\nTexture wad option:\n");
			dprintf(logfile,"---------------------\n");
			dprintf(logfile,"The option /T:%s is used to specify the texture\n",optname);
			dprintf(logfile,"set that alignment will be determined with respect to.\n");
		}
		else
		{
			dprintf(logfile,"\nCommand line option: %s not understood\n",cloptions[i]);
		}
	}

	dprintf(logfile,"\n");
	FreeCLStrs();
	return 0;
}


