#include "patch.h"

void FreeKilStructures()
{
	FreeVTex();
	FreePalette();
	FreeDir(&bdir);
	FreeDir(&pdir);
	FreeDir(&odir);
	FreeTextures(&bwt);
	FreeTextures(&pwt);
	FreeTextures(&owt);
	FreeResourceDefinition(&ResDef);
	FreeResourceDefinition(&bResDef);
	FreeResourceDefinition(&pResDef);
	CloseStreams();
}

int kilmain(int argc,char **argv)
{
	int i,n,j;
	int cleared=0;
	char *p;

	dprintf(logfile,"\n");

	CopyEnvironmentCLArgs("KIL",argc,argv);
	if (ParseCommandLine(-1,3))
	{
		FreeCLStrs();
		return 3;
	}

	if (nclfiles!=3)
	{
		dprintf(logfile,"\nKIL.EXE %s by Jim Flynn, Copyright 1997 -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"\nCreate an output wad that is the input wad with any resources\n");
		dprintf(logfile,"in the kilspec deleted. The kilspec is a .SGN or .RES file listing\n");
		dprintf(logfile,"the resources to delete in the usual sections\n");
		dprintf(logfile,"\nUsage: KIL [options] input[.WAD] kilspec[.SGN|.WAD] output[.WAD]\n");
		dprintf(logfile,"\n Resource selection option:\n");
		dprintf(logfile," +XYZ deletes only XYZ resources, -XYZ deletes all BUT XYZ resources.\n");
		dprintf(logfile,"  L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
		dprintf(logfile,"  E EFFECTS    M MUSICS     G GRAPHICS\n");
		dprintf(logfile,"  S SPRITES    P PATCHES    F FLATS\n");
		dprintf(logfile,"\n General options, use SET KIL=options to set defaults:\n");
		dprintf(logfile," /1 Convert FF_END to F_END, etc.\n");
		dprintf(logfile," /2 Convert F_END to FF_END, etc.\n");
		dprintf(logfile,"\n Game definition options, use SET KIL=options to set defaults:\n");
		dprintf(logfile," /D:name[.DEF] specify resource definitions\n");
		dprintf(logfile," /DOOM2 /DOOM /HERETIC /HEXEN specify game\n");
		dprintf(logfile,"\n See KIL.TXT for details.\n");

		FreeCLStrs();
		return(1);
	}

	cleared=0;
	strcpy(outname,"");
	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspufLWTEMGSPFU");
		if (n==strlen(cloptions[i]))
		{
			if (!cleared)	// if first selection option
			{
				if (cloptsense[i])	// clear all if it adds types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=0;
					DoGraphics=DoSprites=DoPatches=DoFlats=0;
				}
				else if (!cloptsense[i]) // set all if it subtracts types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=1;
					DoGraphics=DoSprites=DoPatches=DoFlats=1;
				}
				cleared=1;	// but don't do it for following selection opts
			}
			for (j=0;j<strlen(cloptions[i]);j++)
			{
				switch(tolower(cloptions[i][j]))
				{
					case 'l':
						DoLumps = cloptsense[i];
						break;
					case 'w':
						DoWads = cloptsense[i];
						break;
					case 't':
						DoTextures = cloptsense[i];
						break;
					case 'e':
						DoEffects = cloptsense[i];
						break;
					case 'm':
						DoMusics = cloptsense[i];
						break;
					case 'g':
						DoGraphics = cloptsense[i];
						break;
					case 's':
						DoSprites = cloptsense[i];
						break;
					case 'p':
						DoPatches = cloptsense[i];
						break;
					case 'f':
						DoFlats = cloptsense[i];
						break;
					case 'u':
						DoNonStandard = cloptsense[i];
						break;
				}
			}
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
			strcpy(defname,cloptions[i]);
		else if (tolower(cloptions[i][0])=='d' && cloptions[i][1]==':')
			strcpy(defname,cloptions[i]+2);
 		else if (cloptions[i][0]=='1')
		{
			SingleMarker = cloptsense[i];
			DoubleMarker = !cloptsense[i];
		}
 		else if (cloptions[i][0]=='2')
		{

			DoubleMarker = cloptsense[i];
			SingleMarker = !cloptsense[i];
		}
		else dprintf(logfile,"Warn: Command line option %s not understood\n",cloptions[i]);
	}

	strcpy(bwadname,clfiles[0]);
	strcpy(pwadname,clfiles[1]);
	strcpy(outname,clfiles[2]);

	FreeCLStrs();

	AddExt(defname,".DEF");
	InitCategories(defname,&ResDef);
	palette = GetPalette(NULL,NULL,-1);
	AddExt(bwadname,".WAD");

	if (!Exists(pwadname))
	{
		AddExt(pwadname,".SGN");
		if (!Exists(pwadname))
		{
			AddExt(pwadname,".RES");
			if (!Exists(pwadname))
				AddExt(pwadname,".WAD");
		}
	}
	p = strrchr(pwadname,'\\');
	if (!p) p = pwadname;
	p = strrchr(p,'.');
	if (p && strnicmp(p,".WAD",4)==0)
		WadSpec=1;

	if (WadSpec)
	{
		pst = fopen(pwadname,"rb");
		if (pst)
		{
			fread(&phdr,sizeof(WadHeader),1,pst);
			ReadDir(&phdr,&pdir,pst);
			LocateCategories(&pdir,&ResDef,&pResDef);

			ReadPNames(&pdir,pst,&pwt);
			ReadTextures(&pdir,pst,&pwt);
		}
	}
	else
	{
		//
		// Get the resource names to delete into mtxres[c] lists for each
		// category index c
		//
		ReadResourceControl(pwadname,pwadname);
		if (errorlev<=1)
			dprintf(logfile,"KIL specification read: %s\n",pwadname);
		else
		{
			//FreeKilStructures();
			return errorlev;
		}
	}

	AddExt(outname,".WAD");

	bst = fopen(bwadname,"rb");
	if (bst)
	{
		fread(&bhdr,sizeof(WadHeader),1,bst);
		ReadDir(&bhdr,&bdir,bst);
		LocateCategories(&bdir,&ResDef,&bResDef);

		ReadPNames(&bdir,bst,&bwt);
		ReadTextures(&bdir,bst,&bwt);

		KillWad(outname);
	}
	else
	{
		dprintf(logfile,"Input wad: %s not found, exiting\n",bwadname);
		dprintf(logfile,"\n");
		//FreeKilStructures();
		return (3);
	}

	dprintf(logfile,"\n");
	//FreeKilStructures();
	return errorlev;
}


