#include "patch.h"

/************************************************************
 KILSUBS.C
			 /-------------------------\
			||    KIL subroutines     ||
			 \-------------------------/
									
************************************************************/

static int NKils=0;

void ZeroKilCalls(void)
{
	NKils=0;
}

int NKilCalls(void)
{
	return NKils;
}

void WriteKilHistory(int bidx,long *offset)
{
	StrIdxAry *sa=NULL;
	int num=0;
	char *buffer = NULL;
	char line[256],*tok;
	int i,n=0,o=0;

	ListStrIdx(&sa,&num,stringf("KIL %s [%s]%s",cmdline,GetDateTime(),LINETERM));
	if (bidx>=0)
	{
		o = bdir.dirp[bidx].resptr;
		n = bdir.dirp[bidx].reslen;
		if (n>=0)
		{
			buffer = (char *)realloc(buffer,n+1);
			fseek(bst,o,SEEK_SET);
			fread(buffer,n,1,bst);
			buffer[n]='\0';

			tok = strtok(buffer,"\r\n");
			while (tok)
			{
				strcpy(line,"  ");
				strcat(line,tok);
				strcat(line,LINETERM);
				ListStrIdx(&sa,&num,line);
				tok = strtok(NULL,"\r\n");
			}
		}
	}
	else ListStrIdx(&sa,&num,stringf("  %s: No History%s",bwadname,LINETERM));

	ListStrIdx(&sa,&num,stringf("  %s: KIL specification%s",pwadname,LINETERM));

	fseek(ost,*offset,SEEK_SET);
	n = 0;
	for (i=0;i<num;i++)
	{
		fwrite(sa[i].str,strlen(sa[i].str),1,ost);
		n+= strlen(sa[i].str);
	}
	AddDirEntry(&odir,*offset,n,"HISTORY");
	*offset += n;

	for (i=0;i<num;i++)
		free(sa[i].str);
	free(sa);
	free(buffer);
}

// Transfer the idx-th directory entry of a wad to the output wad

void WriteKilEntry(WadDir *dir,FILE *st,int idx,unsigned long *offset,WadDir *odir,FILE *ost)
{
	int o,n,offs=*offset;
	char *buffer=NULL;

	o = dir->dirp[idx].resptr;
	n = dir->dirp[idx].reslen;
    if (n>0)
    {
		buffer = (char *)realloc(buffer,n);
		fseek(st,o,SEEK_SET);
		fread(buffer,n,1,st);
		fseek(ost,offs,SEEK_SET);
		SafeWrite(buffer,n,1,ost);
		free(buffer);
    }
	AddDirEntry(odir,offs,n,Str8(dir->dirp[idx].resname));
	offs += n;

	*offset = offs;
	NKils++;
}

void KillPNames()
{
	int i,c;
	static int doonce=1;

	if (!doonce) return;
	doonce=0;

	if (!IsSectionEnabled("TEXTURES")) return;
	c = CategoryOf("PATCHES",&ResDef);

	for (i=0;i<bwt.pnames.npnames;i++)
		if
		(
			(WadSpec && IsIn(GetPName(&bwt.pnames,i),&pdir)<0)
			||
			(
				!WadSpec
				&&
				(c<0 || IsInVTex(GetPName(&bwt.pnames,i),mtxres[c],mtxcnt[c])<0)
			)
		)
			AddPName(&owt.pnames,GetPName(&bwt.pnames,i));
	dprintf(logfile,"Info:  Kept %5ld %-8s from %5ld, deleted: %5d\n",owt.pnames.npnames,"PNAMES",bwt.pnames.npnames,bwt.pnames.npnames-owt.pnames.npnames);
}

// Copy each texture in bwad that does not occur in pwad to owt.t1
//

void KillTextures(int c,long *offs)
{
	int p,b,l,i,m;
	char tname[16];
	Category *ctgy,*ctg1,*ctg2;
	TextureLump *tlp[3],*tlb[3];
	TextureDesc td;
	static int doonce=1;

	if (!doonce) return;
	doonce=0;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;

	tlb[0] = &bwt.t1;
	tlp[0] = &pwt.t1;
	tlb[1] = &bwt.t2;
	tlp[1] = &pwt.t2;
	tlb[2] = &bwt.t3;
	tlp[2] = &pwt.t3;

	for (l=0;l<3;l++)
		for (b=0;b<tlb[l]->N;b++)
		{
			strcpy(tname,Str8(tlb[l]->textures[b].textname));
			if (WadSpec)
				for (m=0;m<3 && (p=FindTexture(tlp[m],tname))<0;m++)
					;
			else
			{
				p = IsInVTex(tname,mtxres[c],mtxcnt[c]);
				if (p>=0 && SyntaxType(mtxres[c][p].id)==ST_RESINPUT) p=-1;
			}

			if (p<0 || !IsSectionEnabled(ctgy->secname))
			{
				td = tlb[l]->textures[b];
				if (td.npatches)
				{
					td.patches = (Patch *)malloc(td.npatches*sizeof(Patch));
					for (i=0;i<td.npatches;i++)
					{
						td.patches[i] = tlb[l]->textures[b].patches[i];
						td.patches[i].pidx = GetPIdx(&owt.pnames,GetPName(&bwt.pnames,tlb[l]->textures[b].patches[i].pidx));
						if (td.patches[i].pidx<0)
							dprintf(logfile,"ERROR: Texture %s requires killed patch %s\n",tname,GetPName(&bwt.pnames,tlb[l]->textures[b].patches[i].pidx)); 
					}
					AddTexture(&owt.t1,&td,1);
					free(td.patches);
					td.patches=NULL;
				}
			}
		}
	WriteTextures(ost,&owt,offs);
	WritePNames(ost,&owt,offs);
	dprintf(logfile,"Info:  Kept %5ld %-8s from %5ld, deleted: %5d\n",owt.t1.N,ctgy->secname,tlb[0]->N+tlb[1]->N+tlb[2]->N,tlb[0]->N-tlb[1]->N-tlb[2]->N-owt.t1.N);
}

void KillMarked(int c,unsigned long *offs)
{
	int k,m=0;
	Category *ctgy,*ctg1,*ctg2;
	int marked=0;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	ctg2 = pResDef.cat+c;

	if (ctg1->nnames<=0) return;

	ZeroKilCalls();
	for (k=0;k<ctg1->nnames;k++)
	{
		m++;
		if
		(
			!IsSectionEnabled(ctgy->secname)
			||
			(WadSpec && StrIdxFind(ctg2->Names,ctg2->nnames,ctg1->Names[k].str)<0)
			||
			(!WadSpec && IsInVTex(ctg1->Names[k].str,mtxres[c],mtxcnt[c])<0)
		)
		{
			if (!marked)
				WriteMarker("START",&odir,*offs,ctgy->mark);
			marked=1;
			WriteKilEntry(&bdir,bst,ctg1->Names[k].didx,offs,&odir,ost);
		}
	}
	if (marked) WriteMarker("END",&odir,*offs,ctgy->mark);
	dprintf(logfile,"Info:  Kept %5d %-8s from %5d, deleted: %5d\n",NKilCalls(),ctg1->secname,m,m-NKilCalls());
}

void KillListed(int c,unsigned long *offs)
{
	int k,m=0;
	Category *ctgy,*ctg1,*ctg2=NULL;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	if (WadSpec)
		ctg2 = pResDef.cat+c;

	ZeroKilCalls();
	for (k=0;k<ctg1->nnames;k++)
	{
		if (ctg1->Names[k].didx>=0)
		{
			m++;
			if
			(
				!IsSectionEnabled(ctgy->secname) ||
				(WadSpec && ctg2->Names[k].didx<0) ||
				(!WadSpec && IsInVTex(ctg1->Names[k].str,mtxres[c],mtxcnt[c])<0)
			)
				WriteKilEntry(&bdir,bst,ctg1->Names[k].didx,offs,&odir,ost);
		}
	}
	dprintf(logfile,"Info:  Kept %5d %-8s from %5d, deleted: %5d\n",NKilCalls(),ctg1->secname,m,m-NKilCalls());
}

void KillLevels(int c,unsigned long *offs)
{
	int i,j,k,m=0;
	Category *ctgy,*ctg1,*ctg2=NULL;

	ctgy = ResDef.cat+c;
	ctg1 = bResDef.cat+c;
	if (WadSpec)
		ctg2 = pResDef.cat+c;

	i = NLevObj()+1;

	if (i>0)
	{
		ZeroKilCalls();
		for (k=0;k<ctg1->nnames;k++)
		{
			if (ctg1->Names[k].didx>=0)
			{
				m++;
				if
				(
					!IsSectionEnabled(ctgy->secname) ||
					(WadSpec && ctg2->Names[k].didx<0) ||
					(!WadSpec && IsInVTex(ctg1->Names[k].str,mtxres[c],mtxcnt[c])<0)
				)
				for (j=ctg1->Names[k].didx;j<ctg1->Names[k].didx+i;j++)
					WriteKilEntry(&bdir,bst,j,offs,&odir,ost);
			}
		}
		dprintf(logfile,"Info:  Kept %5d %-8s from %5d, deleted: %5d\n",NKilCalls()/i,ctg1->secname,m,m-NKilCalls()/i);
	}
}

int KillWad(char *fname)
{
	long offs = 16;
	int c;
	Category *ctgy,*ctg1;

	ost = fopen(fname,"wb");
	if (ost)
	{
 		dprintf(logfile,"\nRemoving resources in %s listed in %s leaving %s\n",bwadname,pwadname,outname);
		for (c=0;c<ResDef.ncats;c++)
		{
			ctgy = ResDef.cat+c;
			ctg1 = bResDef.cat+c;

			if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
			{
				KillPNames();
				KillTextures(c,&offs);
			}
			else if (ctgy->idm==ID_INFO || ctgy->idm==ID_NONE)
				continue;
			else if (stricmp(ctgy->secname,"HISTORY")==0)
				WriteKilHistory(ctg1->Names[0].didx,&offs);
			else if (stricmp(ctgy->secname,"LEVELS")==0)
				KillLevels(c,&offs);
			else if (ctgy->idm==ID_MARKED)
				KillMarked(c,&offs);
			else if (ctgy->idm==ID_LISTED)
				KillListed(c,&offs);
		}
		SafeWrite(odir.dirp,sizeof(WadDirEntry),odir.nentries,ost);
		ohdr.waddirlen = odir.nentries;
		ohdr.waddiraddr = offs;
		strncpy(ohdr.wadtag,"PWAD",4);
		fseek(ost,0,SEEK_SET);
		SafeWrite(&ohdr,sizeof(WadHeader),1,ost);

		fclose(ost);
		dprintf(logfile,"\nInfo: %s written from %s without resources in %s\n",outname,bwadname,pwadname);
	}
	return 0;
}
