#include "patch.h"

/************************************************************
 LSUBS.C
			 /-------------------------\
			||   Level Subroutnes      ||
			 \-------------------------/
									
************************************************************/

static int *usedvert=NULL;

int VertUsed(int vn)
{
	if (0<=vn && vn<NVertexes)
		return usedvert[vn];
	return -1;
}

int ReadLevelData(char *maptag, WadDir *dir, FILE *st,int nlevobj)
{
	int i,j,n,o,f,t;

	i = IsIn(maptag,dir);
	if (i>=0)
	{
		ResetBlockmapValid();
		LevelTag = realloc(LevelTag,sizeof(maptag)+1);
		strcpy(LevelTag,maptag);
		j = IsAfter(i,"THINGS",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Things = realloc(Things,n);
			fseek(st,o,SEEK_SET);
			fread(Things,n,1,st);
			NThings = n/sizeof(Thing);
		}
		j = IsAfter(i,"LINEDEFS",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Linedefs = realloc(Linedefs,n);
			fseek(st,o,SEEK_SET);
			fread(Linedefs,n,1,st);
			NLinedefs = n/sizeof(Linedef);
		}
		j = IsAfter(i,"SIDEDEFS",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Sidedefs = realloc(Sidedefs,n);
			fseek(st,o,SEEK_SET);
			fread(Sidedefs,n,1,st);
			NSidedefs = n/sizeof(Sidedef);
		}
		j = IsAfter(i,"VERTEXES",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Vertexes = realloc(Vertexes,n);
			fseek(st,o,SEEK_SET);
			fread(Vertexes,n,1,st);
			NVertexes = n/sizeof(Vertex);
			if (NVertexes>0)
			{
				usedvert = realloc(usedvert,NVertexes*sizeof(int));
			}
		}
		j = IsAfter(i,"SEGS",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Segs = realloc(Segs,n);
			fseek(st,o,SEEK_SET);
			fread(Segs,n,1,st);
			NSegs = n/sizeof(Seg);
		}
		j = IsAfter(i,"SSECTORS",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Ssectors = realloc(Ssectors,n);
			fseek(st,o,SEEK_SET);
			fread(Ssectors,n,1,st);
			NSsectors = n/sizeof(Ssector);
		}
		j = IsAfter(i,"NODES",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Nodes = realloc(Nodes,n);
			fseek(st,o,SEEK_SET);
			fread(Nodes,n,1,st);
			NNodes = n/sizeof(Node);
		}

		j = IsAfter(i,"SECTORS",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Sectors = realloc(Sectors,n);
			fseek(st,o,SEEK_SET);
			fread(Sectors,n,1,st);
			NSectors = n/sizeof(Sector);
		}
		j = IsAfter(i,"REJECT",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Reject = realloc(Reject,n);
			fseek(st,o,SEEK_SET);
			fread(Reject,n,1,st);
			NReject = n;
		}
		j = IsAfter(i,"BLOCKMAP",dir);
		if (j<=i+nlevobj)
		{
			n = dir->dirp[j].reslen;
			o = dir->dirp[j].resptr;
			Blockmap = realloc(Blockmap,n);
			fseek(st,o,SEEK_SET);
			fread(Blockmap,n,1,st);
			NBlockmap = n;
		}
	}
	else return 1;

	if (NVertexes>0)
		memset(usedvert,0,NVertexes*sizeof(int));
	for (i=0;i<NLinedefs;i++)
	{
		f = Linedefs[i].from;
		t = Linedefs[i].to;
		if (0<=f && f<NVertexes)
			usedvert[f] = 1;
		if (0<=t && t<NVertexes)
			usedvert[t] = 1;
	}

	GetMapLimits();
	return 0;
}


int AddThing(short x,short y,short ang,unsh typ,unsh attr)
{
	Things = realloc(Things,(NThings+1)*sizeof(Thing));
	Things[NThings].x = x;
	Things[NThings].y = y;
	Things[NThings].ang = ang;
	Things[NThings].typ = typ;
	Things[NThings].attr = attr;
	return NThings++;
}

int AddLinedef(unsh from,unsh to,unsh attr,unsh typ,unsh tag,short side0,short side1)
{
	Linedefs = realloc(Linedefs,(NLinedefs+1)*sizeof(Linedef));
	Linedefs[NLinedefs].from = from;
	Linedefs[NLinedefs].to = to;
	Linedefs[NLinedefs].attr = attr;
	Linedefs[NLinedefs].typ = typ;
	Linedefs[NLinedefs].tag = tag;
	Linedefs[NLinedefs].sides[0] = side0;
	Linedefs[NLinedefs].sides[1] = side1;
	return NLinedefs++;
}

int AddSidedef(short xoff,short yoff,char *utxt,char *ltxt,char *ntxt,unsh sect)
{
	Sidedefs = realloc(Sidedefs,(NSidedefs+1)*sizeof(Sidedef));
	Sidedefs[NSidedefs].xoff = xoff;
	Sidedefs[NSidedefs].yoff = yoff;
	strcpy(Sidedefs[NSidedefs].utxt,utxt);
	strcpy(Sidedefs[NSidedefs].ltxt,ltxt);
	strcpy(Sidedefs[NSidedefs].ntxt,ntxt);
	Sidedefs[NSidedefs].sect = sect;
	return NSidedefs++;
}

int AddVertex(short x,short y)
{
	Vertexes = realloc(Vertexes,(NVertexes+1)*sizeof(Vertex));
	Vertexes[NVertexes].x = x;
	Vertexes[NVertexes].y = y;
	return NVertexes++;
}

int AddNewVertex(short x,short y)
{
	int i;

	for (i=0;i<NVertexes;i++)
		if (x==Vertexes[i].x && y==Vertexes[i].y)
			return i;

	Vertexes = realloc(Vertexes,(NVertexes+1)*sizeof(Vertex));
	Vertexes[NVertexes].x = x;
	Vertexes[NVertexes].y = y;
	return NVertexes++;
}

int AddSeg(unsh from,unsh to,unsh ang,unsh ldef,unsh dir,unsh offs)
{
	Segs = realloc(Segs,(NSegs+1)*sizeof(Seg));
	Segs[NSegs].from = from;
	Segs[NSegs].to = to;
	Segs[NSegs].ang = ang;
	Segs[NSegs].ldef = ldef;
	Segs[NSegs].dir = dir;
	Segs[NSegs].offs = offs;
	return NSegs++;
}

int AddNode(short x,short y,short dx,short dy,
            short lmxy,short lmny,short lmnx,short lmxx,
						short rmxy,short rmny,short rmnx,short rmxx,
						unsh child0,unsh child1)
{
	Nodes = realloc(Nodes,(NNodes+1)*sizeof(Node));
	Nodes[NNodes].x = x;
	Nodes[NNodes].y = y;
	Nodes[NNodes].dx = dx;
	Nodes[NNodes].dy = dy;
	Nodes[NNodes].lmxy = lmxy;
	Nodes[NNodes].lmny = lmny;
	Nodes[NNodes].lmnx = lmnx;
	Nodes[NNodes].lmxx = lmxx;
	Nodes[NNodes].rmxy = rmxy;
	Nodes[NNodes].rmny = rmny;
	Nodes[NNodes].rmnx = rmnx;
	Nodes[NNodes].rmxx = rmxx;
	Nodes[NNodes].children[0]=child0;
	Nodes[NNodes].children[1]=child1;
	return NNodes++;
}

void DumpSeg(unsh segnum)
{
		Seg *sg=Segs+segnum;

		printf("#:%5u Fr:%5u To:%5u An:%4x Ld:%5u Di:%4d Of:%5u\n",
				    segnum,sg->from,sg->to,sg->ang,sg->ldef,sg->dir,sg->offs);
}

void DumpSegs()
{
	unsh i;

	for (i=0;i<NSegs;i++)
		DumpSeg(i);
}

int AddSsector(unsh nsegs,unsh seg0)
{
	Ssectors = realloc(Ssectors,(NSsectors+1)*sizeof(Ssector));
	Ssectors[NSsectors].nsegs = nsegs;
	Ssectors[NSsectors].seg0 = seg0;
	return NSsectors++;
}

void DumpSsector(unsh subsecnum)
{
	Ssector *ss=Ssectors+subsecnum;
	printf("#:%5u Ns:%5u S0:%5u\n",subsecnum,ss->nsegs,ss->seg0);
}

void DumpSsectors()
{
	unsh i;

	for (i=0;i<NSsectors;i++)
		DumpSsector(i);
}

char *FormatSeg(Seg *sg)
{
	char *segstr = malloc(32);
	sprintf(segstr,"%5ld:%u->%u ",sg-Segs,sg->from,sg->to);
	return segstr;
}

void DumpNode(unsh nodenum)
{
		Node *nd=Nodes+nodenum;

		printf("#:%5u X:%5d Y:%5d Dx:%5d Dy:%5d C0:%4x C1:%4x\n",
				    nodenum,nd->x,nd->y,nd->dx,nd->dy,nd->children[0],nd->children[1]);
		printf("LBBX: %d %d %d %d  RBBX: %d %d %d %d\n",
			      nd->lmnx,nd->lmny,nd->lmxx,nd->lmxy,
			      nd->rmnx,nd->rmny,nd->rmxx,nd->rmxy);
		printf("\n");
}

void DumpNodes()
{
	unsh i;

	for (i=0;i<NNodes;i++)
		DumpNode(i);
}

int AddSector(short fhgt,short chgt,char *ftxt,char *ctxt,unsh lite,unsh typ,unsh tag)
{
	Sectors = realloc(Sectors,(NSectors+1)*sizeof(Sector));
	Sectors[NSectors].fhgt = fhgt;
	Sectors[NSectors].chgt = chgt;
	strcpy(Sectors[NSectors].ftxt,ftxt);
	strcpy(Sectors[NSectors].ctxt,ctxt);
	Sectors[NSectors].lite = lite;
	Sectors[NSectors].typ = typ;
	Sectors[NSectors].tag = tag;
	return NSectors++;
}

unsigned short *BlockmapPtr(int col, int row)
{
	int nrows,ncols,offs;

	ncols = Blockmap[2];
	nrows = Blockmap[3];

	offs = Blockmap[4+row*ncols+col];
	return Blockmap+offs+1;
}

static int blkmok=-1;

/* Reset Blockmap Valid */

void ResetBlockmapValid()
{
	blkmok = -1;
}

int BlockmapValid()
{
	int i,ncols,nrows;

	if (!Blockmap) return 0;
	if (blkmok>=0) return blkmok;
	ncols = Blockmap[2];
	nrows = Blockmap[3];
	if (nrows<=0 || ncols<=0) return 0;
	for (i=1;i<nrows*ncols;i++)
		if (Blockmap[4+i-1]>=Blockmap[4+i])
			return blkmok=0;
	return blkmok=1;
}

void InitLevelData()
{
	LevelTag=NULL;
	Things=NULL;
	NThings=0;
	Linedefs=NULL;
	NLinedefs=0;
	Sidedefs=NULL;
	NSidedefs=0;
	Vertexes=NULL;
	NVertexes=0;
	Segs=NULL;
	NSegs=0;
	Ssectors=NULL;
	NSsectors=0;
	Nodes=NULL;
	NNodes=0;
	Sectors=NULL;
	NSectors=0;
	Reject=NULL;
	NReject=0;
	Blockmap=NULL;
	NBlockmap=0;
	usedvert=NULL;
}

void FreeLevelData()
{
	free(LevelTag);
	free(Things);
	free(Linedefs);
	free(Sidedefs);
	free(Vertexes);
	free(Segs);
	free(Ssectors);
	free(Nodes);
	free(Sectors);
	free(Reject);
	free(Blockmap);
	free(usedvert);
	InitLevelData();
}

void GetMapLimits()
{
	int i,x,y;

	map_maxx = -32767; map_minx = 32767;
	map_maxy = -32767; map_miny = 32767;

	for (i=0;i<NThings;i++)
	{
		x = Things[i].x;
		y = Things[i].y;
		if (x<map_minx) map_minx=x;
		if (x>map_maxx) map_maxx=x;
		if (y<map_miny) map_miny=y;
		if (y>map_maxy) map_maxy=y;
	}
	for (i=0;i<NVertexes;i++)
	{
		x = Vertexes[i].x;
		y = Vertexes[i].y;
		if (x<map_minx) map_minx=x;
		if (x>map_maxx) map_maxx=x;
		if (y<map_miny) map_miny=y;
		if (y>map_maxy) map_maxy=y;
	}
}

// List all the standard maps found in a wad whose directory is passed

int ListMaps(char ***maplist,int *nmaps,WadDir *dir)
{
	int i,j;

	*nmaps = 0;
	*maplist = NULL;

	for (i=1;i<=32;i++)
		if (IsIn(stringf("MAP%02d",i),dir)>=0)
			AddDynStr(maplist,nmaps,stringf("MAP%02d",i));

	for (j=1;j<=4;j++)
		for (i=1;i<=9;i++)
		if (IsIn(stringf("E%dM%d",j,i),dir)>=0)
			AddDynStr(maplist,nmaps,stringf("E%dM%d",j,i));

	return *nmaps;
}

int linelen(int lineno)
{
	int v1,v2,y,x;

	v1 = Linedefs[lineno].from;
	v2 = Linedefs[lineno].to;
	x = Vertexes[v2].x;
	x -= Vertexes[v1].x;
	y = Vertexes[v2].y;
	y -= Vertexes[v1].y;

	return (int)sqrt((double)(y*y+x*x));
}

int uprhgt(int sideno,int lineno,int *t)
{
	int s1,s2,c1,c2;

	s1=Linedefs[lineno].sides[0];
	s2=Linedefs[lineno].sides[1];
	c1=Sectors[Sidedefs[s1].sect].chgt;
	c2=Sectors[Sidedefs[s2].sect].chgt;
	*t=Sectors[Sidedefs[s1].sect].tag;
	if (!*t) *t=Sectors[Sidedefs[s2].sect].tag;
	return (s1==sideno)? max(0,c1-c2) : max(0,c2-c1);
}

int no1hgt(int sideno,int lineno,int *t)
{
	int c1,c2;

	c1=Sectors[Sidedefs[sideno].sect].chgt;
	c2=Sectors[Sidedefs[sideno].sect].fhgt;
	*t=Sectors[Sidedefs[sideno].sect].tag;
	return max(0,c1-c2);
}

int no2hgt(int sideno,int lineno,int *t)
{
	int s1,s2,c1,c2;

	s1=Linedefs[lineno].sides[0];
	s2=Linedefs[lineno].sides[1];
	if (s2>=0)
	{
		c1=min(Sectors[Sidedefs[s1].sect].chgt,Sectors[Sidedefs[s2].sect].chgt);
		c2=max(Sectors[Sidedefs[s1].sect].fhgt,Sectors[Sidedefs[s2].sect].fhgt);
	}
	else
	{
		c1=Sectors[Sidedefs[s1].sect].chgt;
		c2=Sectors[Sidedefs[s1].sect].fhgt;
	}
	*t=Sectors[Sidedefs[s1].sect].tag;
	if (!*t) *t=Sectors[Sidedefs[s2].sect].tag;
	return max(0,c1-c2);
}

int lwrhgt(int sideno,int lineno,int *t)
{
	int s1,s2,c1,c2;

	s1=Linedefs[lineno].sides[0];
	s2=Linedefs[lineno].sides[1];
	c1=Sectors[Sidedefs[s1].sect].chgt;
	c2=Sectors[Sidedefs[s2].sect].chgt;
	*t=Sectors[Sidedefs[s1].sect].tag;
	if (!*t) *t=Sectors[Sidedefs[s2].sect].tag;
	return (s1==sideno)? max(0,c2-c1) : max(0,c1-c2);
}

#define blkshift 7							 /* places to shift rel position for cell num */
#define blkmask ((1<<7)-1)			 /* mask for rel position within cell */
#define blkmargin 0							 /* size guardband around map required */

void BuildBlockmap()
{
	BlockMapDesc bmd;
	int NBlocks;									 /* nrows*ncols */
	int i,j,k;
	int *blockcount=NULL;					 /* length of each blocklist incl 0,-1 */
	int blocklisttotal=0;					 /* total length of all blocklists */
	unsigned short *p;

	// scan for limits again, in case a node vertex increased them

	GetMapLimits();

	// set up blockmap area to enclose level plus margin
	// create the array of pointers on NBlocks to blocklists
	// create the array of blocklist counts

	bmd.xorg = map_minx-blkmargin;
	bmd.yorg = map_miny-blkmargin;
	bmd.ncols = (map_maxx+blkmargin-bmd.xorg+1+blkmask)>>blkshift;
	bmd.nrows = (map_maxy+blkmargin-bmd.yorg+1+blkmask)>>blkshift;
	NBlocks = bmd.ncols*bmd.nrows;

	bmd.blocklists = malloc(NBlocks*sizeof(unsigned short *));
	blockcount = malloc(NBlocks*sizeof(int));

	// allocate each blocklist, and enter the initial 0 in all blocklists
	// set all blockcounts to 1 correspondingly

	for (i=0;i<NBlocks;i++)
	{
		bmd.blocklists[i] = malloc(sizeof(unsigned short));
	  bmd.blocklists[i][0] = 0;
		blockcount[i] = 1;
	}

	// For each Linedef in the wad, determine all blockmap blocks it crosses,
	// and add the block number of each to a list, blocklist
	// a list of all blocks that Linedef i touches

	for (i=0;i<NLinedefs;i++)
	{
		Linedef *ld = Linedefs+i;
		short x1 = X1Of(ld);	 										 // Linedef[i] coords
		short y1 = Y1Of(ld);
		short x2 = X2Of(ld);
		short y2 = Y2Of(ld);
		int vert = x1==x2;												 // Linedef[i] slopetype
		int horiz = y1==y2;
		int spos = (x2-x1)*(y2-y1) > 0;
		int sneg = (x2-x1)*(y2-y1) < 0;
		int *blocklist=NULL;											 // list of blocks that touch it
		int nblocklist=0;													 // number blocks in it so far
		int bx,by;
		int minx = min(x1,x2);
		int maxx = max(x1,x2);
		int miny = min(y1,y2);
		int maxy = max(y1,y2);

		// The line always belongs to the blocks containing its endpoints

		bx = (x1-bmd.xorg)>>blkshift;
		by = (y1-bmd.yorg)>>blkshift;
		AddDynInt(&blocklist,&nblocklist,by*bmd.ncols+bx);
		bx = (x2-bmd.xorg)>>blkshift;
		by = (y2-bmd.yorg)>>blkshift;
		AddDynInt(&blocklist,&nblocklist,by*bmd.ncols+bx);


		// For each column, see where the line along its left edge, which 
		// it contains, intersects the Linedef i. Add all the blocks touching
		// the intersection point to the blocklist for i

		if (!vert)
		{
			for (j=0;j<bmd.ncols;j++)
			{
				// intersection of Linedef with x=bmd.xorg+(j<<blkshift)
				// (y-y1)*(x2-x1) = (y2-y1)*(x-x1)
				// y = (y2-y1)*(x-x1)+y1*(x2-x1);

				short x = bmd.xorg+(j<<blkshift);				// (x,y) is intersection
				short y = ((y2-y1)*(x-x1))/(x2-x1)+y1;
				short yb = (y-bmd.yorg)>>blkshift;			// block row number
				short yp = (y-bmd.yorg)&blkmask;				// y position within block

				if (yb<0 || yb>bmd.nrows-1)							// outside blockmap, continue
					continue;

				if (x<minx || x>maxx)				// line doesn't touch column
					continue;

				// The cell that contains the intersection point is always added

				AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*yb+j);

				// if the intersection is at a corner it depends on the slope
				// which blocks are hit

				if (yp==0)
				{
					if (sneg)				//   \ - blocks x,y x,y-, x-,y
					{
						if (yb>0 && miny<y)
							AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*(yb-1)+j);
						if (j>0 && minx<x)
							AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*yb+j-1);
					}
					else if (spos)  //   / - blocks x,y x-,y-
					{
						if (yb>0 && j>0 && minx<x)
							AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*(yb-1)+j-1);
					}
					else if (horiz)	//   - - blocks x,y x-,y
					{
						if (j>0 && minx<x)
							AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*yb+j-1);
					}
					//skip column intersections with vertical lines
				
				}
				else if (j>0 && minx<x)	// else its always the this one and the one to the left
					AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*yb+j-1);
			}
		}

		// For each row, see where the line along its bottom edge, which 
		// it contains, intersects the Linedef i. Add all the blocks touching
		// the intersection point to the blocklist for i

		if (!horiz)
		{
			for (j=0;j<bmd.nrows;j++)
			{
				// intersection of Linedef with x=bmd.xorg+(j<<blkshift)
				// (x,y) on Linedef i satisfies: (y-y1)*(x2-x1) = (y2-y1)*(x-x1)
				// y = (y2-y1)*(x-x1)+y1*(x2-x1);

				short y = bmd.yorg+(j<<blkshift);			// (x,y) is intersection
				short x = ((x2-x1)*(y-y1))/(y2-y1)+x1;
				short xb = (x-bmd.xorg)>>blkshift;		// block column number
				short xp = (x-bmd.xorg)&blkmask;			// x position within block

				if (xb<0 || xb>bmd.ncols-1)						// outside blockmap, continue
					continue;

				if (y<miny || y>maxy)			// line doesn't touch row
					continue;

				// The cell that contains the intersection point is always added

				AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*j+xb);

				// if the intersection is at a corner it depends on the slope
				// which blocks are hit

				if (xp==0)
				{
					if (sneg)				//   \ - blocks x,y x,y-, x-,y
					{
						if (j>0 && miny<y)
							AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*(j-1)+xb);
						if (xb>0 && minx<x)
							AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*j+xb-1);
					}
					else if (vert)  //   | - blocks x,y x,y-
					{
						if (j>0 && miny<y)
							AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*(j-1)+xb);
					}
					else if (spos)  //   / - blocks x,y x-,y-
					{
						if (xb>0 && j>0 && miny<y)
							AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*(j-1)+xb-1);
					}
					//skip row intersections with horizontal lines
				}
				else if (j>0 && miny<y)	// else its always the this one and the one below
					AddNewDynInt(&blocklist,&nblocklist,bmd.ncols*(j-1)+xb);
			}
		}

		// Add Linedef i to each block in blocklist

		for (j=0;j<nblocklist;j++)
		{
			if (blockcount[blocklist[j]]==1 && i==0)
				AddDynShort
				(
					(short **)(bmd.blocklists+blocklist[j]),
					blockcount+blocklist[j],
					(short)i
				);
			else
				AddNewDynShort
				(
					(short **)(bmd.blocklists+blocklist[j]),
					blockcount+blocklist[j],
					(short)i
				);
		}

		// Done with the blocklist, free it and reset its length

		FreeDynInt(&blocklist,&nblocklist);
	}

	// Add terminating 0xffff to all blocklists

	for (i=0,blocklisttotal=0;i<NBlocks;i++)
	{
		AddDynShort
		(
			(short **)(bmd.blocklists+i),
			blockcount+i,
			(short)0xffff
		);
		blocklisttotal += blockcount[i];
	}

	// Rewrite the Blockmap lump

	Blockmap = realloc(Blockmap,(NBlockmap=4*sizeof(short)+NBlocks*sizeof(short)+blocklisttotal*sizeof(short)));

	// write the header

	p = Blockmap;
	*p++ = bmd.xorg;					
	*p++ = bmd.yorg;
	*p++ = bmd.ncols;
	*p++ = bmd.nrows;

	// write the short offsets

	*p++ = i = p-Blockmap+NBlocks;
	for (j=1;j<NBlocks;j++)		
		*p++ = (i+=blockcount[j-1]);

	// write the block lists

	for (j=0;j<NBlocks;j++)
		for (k=0;k<blockcount[j];k++)
			*p++ = bmd.blocklists[j][k];

	// free the temporary storage we created

	for (j=0;j<NBlocks;j++)		
		FreeDynShort((short **)(bmd.blocklists+j),blockcount+j);
	free(bmd.blocklists);
	free(blockcount);
}

void BuildZeroReject()
{
	Reject = realloc(Reject,NReject=((NSectors*NSectors+7)>>3));
	memset(Reject,0,(NSectors*NSectors+7)>>3);
}

void WriteLevelWad(char *filename,char *maptag)
{
	WadHeader whdr;
	WadDir wdir;
	unsigned offs;
	FILE *st;

	// init header
	strncpy(whdr.wadtag,"PWAD",4);
	whdr.waddiraddr=0; // not known yet
	whdr.waddirlen=11; // 11 directory entries

	// init dir	to zero length

	InitDir(&wdir);
	AddDirEntry(&wdir,0,0,maptag);	// add the map tag as 0 length entry

	st = fopen(filename,"wb");			// open for binary write
	if (st)
	{
		int n = sizeof(WadHeader);

		SafeWrite(&whdr,n,1,st);			// write the header at offset 0
		offs = n;											// file offset now size of header

		n = sizeof(Thing)*NThings;
		SafeWrite(Things,n,1,st);
		AddDirEntry(&wdir,offs,n,"THINGS");
		offs += n;

		n = sizeof(Linedef)*NLinedefs;
		SafeWrite(Linedefs,n,1,st);
		AddDirEntry(&wdir,offs,n,"LINEDEFS");
		offs += n;

		n = sizeof(Sidedef)*NSidedefs;
		SafeWrite(Sidedefs,n,1,st);
		AddDirEntry(&wdir,offs,n,"SIDEDEFS");
		offs += n;

		n = sizeof(Vertex)*NVertexes;
		SafeWrite(Vertexes,n,1,st);
		AddDirEntry(&wdir,offs,n,"VERTEXES");
		offs += n;

		n = sizeof(Seg)*NSegs;
		SafeWrite(Segs,n,1,st);
		AddDirEntry(&wdir,offs,n,"SEGS");
		offs += n;

		n = sizeof(Ssector)*NSsectors;
		SafeWrite(Ssectors,n,1,st);
		AddDirEntry(&wdir,offs,n,"SSECTORS");
		offs += n;

		n = sizeof(Node)*NNodes;
		SafeWrite(Nodes,n,1,st);
		AddDirEntry(&wdir,offs,n,"NODES");
		offs += n;

		n = sizeof(Sector)*NSectors;
		SafeWrite(Sectors,n,1,st);
		AddDirEntry(&wdir,offs,n,"SECTORS");
		offs += n;

		n = NReject;
		SafeWrite(Reject,n,1,st);
		AddDirEntry(&wdir,offs,n,"REJECT");
		offs += n;

		n = NBlockmap;
		SafeWrite(Blockmap,n,1,st);
		AddDirEntry(&wdir,offs,n,"BLOCKMAP");
		offs += n;

		SafeWrite(wdir.dirp,sizeof(WadDirEntry),wdir.nentries,st);

		fseek(st,0,SEEK_SET);
		whdr.waddiraddr = offs;
		whdr.waddirlen = wdir.nentries;
		SafeWrite(&whdr,sizeof(WadHeader),1,st);

		fclose(st);
	}
	else printf("Can't open %s for writing level %s\n",filename,maptag);
	FreeDir(&wdir);
}

void *IsLevelResource(char *resname,int *size)
{
	if (stricmp(resname,"THINGS")==0)
		return *size=NThings*sizeof(Thing),Things;
	if (stricmp(resname,"LINEDEFS")==0)
		return *size=NLinedefs*sizeof(Linedef),Linedefs;
	if (stricmp(resname,"SIDEDEFS")==0)
		return *size=NSidedefs*sizeof(Sidedef),Sidedefs;
	if (stricmp(resname,"VERTEXES")==0)
		return *size=NVertexes*sizeof(Vertex),Vertexes;
	if (stricmp(resname,"SEGS")==0)
		return *size=NSegs*sizeof(Seg),Segs;
	if (stricmp(resname,"NODES")==0)
		return *size=NNodes*sizeof(Node),Nodes;
	if (stricmp(resname,"SSECTORS")==0)
		return *size=NSsectors*sizeof(Ssector),Ssectors;
	if (stricmp(resname,"SECTORS")==0)
		return *size=NSectors*sizeof(Sector),Sectors;
	if (stricmp(resname,"BLOCKMAP")==0)
		return *size=NBlockmap,Blockmap;
	if (stricmp(resname,"REJECT")==0)
		return *size=NReject,Reject;
	return NULL;
}

void WriteOpenLevelWad(FILE *pst,WadHeader *whdr,WadDir *wdir,char *maptag)
{
	if (pst)
	{
		int n;
		unsigned offs;

		offs = (whdr && whdr->waddirlen)? whdr->waddiraddr : sizeof(WadHeader);
		fseek(pst,offs,SEEK_SET);

		AddDirEntry(wdir,offs,0,maptag);	// add the map tag as 0 length entry

		n = sizeof(Thing)*NThings;
		SafeWrite(Things,n,1,pst);
		AddDirEntry(wdir,offs,n,"THINGS");
		offs += n;

		n = sizeof(Linedef)*NLinedefs;
		SafeWrite(Linedefs,n,1,pst);
		AddDirEntry(wdir,offs,n,"LINEDEFS");
		offs += n;

		n = sizeof(Sidedef)*NSidedefs;
		SafeWrite(Sidedefs,n,1,pst);
		AddDirEntry(wdir,offs,n,"SIDEDEFS");
		offs += n;

		n = sizeof(Vertex)*NVertexes;
		SafeWrite(Vertexes,n,1,pst);
		AddDirEntry(wdir,offs,n,"VERTEXES");
		offs += n;

		n = sizeof(Seg)*NSegs;
		SafeWrite(Segs,n,1,pst);
		AddDirEntry(wdir,offs,n,"SEGS");
		offs += n;

		n = sizeof(Ssector)*NSsectors;
		SafeWrite(Ssectors,n,1,pst);
		AddDirEntry(wdir,offs,n,"SSECTORS");
		offs += n;

		n = sizeof(Node)*NNodes;
		SafeWrite(Nodes,n,1,pst);
		AddDirEntry(wdir,offs,n,"NODES");
		offs += n;

		n = sizeof(Sector)*NSectors;
		SafeWrite(Sectors,n,1,pst);
		AddDirEntry(wdir,offs,n,"SECTORS");
		offs += n;

		n = NReject;
		SafeWrite(Reject,n,1,pst);
		AddDirEntry(wdir,offs,n,"REJECT");
		offs += n;

		n = NBlockmap;
		SafeWrite(Blockmap,n,1,pst);
		AddDirEntry(wdir,offs,n,"BLOCKMAP");
		offs += n;

		SafeWrite(wdir->dirp,sizeof(WadDirEntry),wdir->nentries,pst);

		fseek(pst,0,SEEK_SET);
		strncpy(whdr->wadtag,"PWAD",4);
		whdr->waddiraddr = offs;
		whdr->waddirlen = wdir->nentries;
		SafeWrite(whdr,sizeof(WadHeader),1,pst);
		fseek(pst,offs,SEEK_SET);
	}
}

int TackLevelToWad(char *levname,char *maptag)
{
	FILE *pst=NULL;
	WadHeader phdr={"PWAD",0,0};
	WadDir pdir={NULL,0,0};

	pst = fopen(levname,"r+b");
	if (pst)
	{
		fread(&phdr,sizeof(WadHeader),1,pst);
		ReadDir(&phdr,&pdir,pst);
	}
	else
		pst = fopen(levname,"w+b");

	if (pst)
	{
		WriteOpenLevelWad(pst,&phdr,&pdir,maptag);
		fclose(pst);
	}
	else dprintf(logfile,"Can't open %s for write\n",levname);

	return 0;
}

int WriteTotalWad(char *name,FILE *pst,WadHeader *phdr,WadDir *pdir,char *levname)
{
	WadDir odir,ldir;
	WadHeader ohdr,lhdr;
	FILE *lst = fopen(levname,"rb");
	FILE *ost = fopen("__nbld__.tmp","w+b");
	char *buffer=NULL;
	int buffermax=0;
	int offs=0,i;

	if (!lst || !pst || !ost) return 1;

	InitDir(&ldir);
	fread(&lhdr,sizeof(WadHeader),1,lst);
	ReadDir(&lhdr,&ldir,lst);

	InitDir(&odir);
	strncpy(ohdr.wadtag,"PWAD",4);
	ohdr.waddiraddr = sizeof(WadHeader);
	ohdr.waddirlen = 0;

	fwrite(&ohdr,sizeof(WadHeader),1,ost);
	offs = sizeof(WadHeader);

	for (i=0;i<pdir->nentries;i++)
	{
		int n;
		char resname[16];

		strcpy(resname,Str8(pdir->dirp[i].resname));
		if ((IsD1Map(resname) || IsD2Map(resname)) && IsIn(resname,&ldir)>=0)
		{
			FreeLevelData();
			if (!ReadLevelData(resname,&ldir,lst,11))
			{
				// Write the level data in memory, just read from the level wad,
				// to the output wad at ohdr.waddiraddr. On return ohdr.waddiraddr
				// is at the beginning of directory, as is the file position.

 				WriteOpenLevelWad(ost,&ohdr,&odir,resname);
				offs = ohdr.waddiraddr;

				// Advance the entry index in input pwad past the level entries
				// for the level we just read from the level wad

				while(++i<pdir->nentries && IsLevelResource(Str8(pdir->dirp[i].resname),&n))
					;
				i--;

				FreeLevelData();
			}
		}
		else
		{
			fseek(pst,pdir->dirp[i].resptr,SEEK_SET);
			if ((n=pdir->dirp[i].reslen)>buffermax)
				buffer=realloc(buffer,buffermax=n);

			if (n)
			{
				fread(buffer,n,1,pst);
				fwrite(buffer,n,1,ost);
			}
			AddDirEntry(&odir,offs,n,resname);
			offs += n;
			ohdr.waddirlen++;
			ohdr.waddiraddr = offs;
		}
	}
	free(buffer);
	fclose(lst);
	FreeDir(&ldir);
	remove(levname);

	fwrite(odir.dirp,odir.nentries*sizeof(WadDirEntry),1,ost);

	strncpy(ohdr.wadtag,phdr->wadtag,4);
	ohdr.waddirlen = odir.nentries;
	ohdr.waddiraddr = offs;
	fseek(ost,0,SEEK_SET);
	fwrite(&ohdr,sizeof(WadHeader),1,ost);

	FreeDir(&odir);
	fclose(ost);

	FreeDir(pdir);
	fclose(pst);

	if (rename("__nbld__.tmp",name))
	{
		perror("Error renaming __nbld__.tmp");
	  return 1;
	}

	return 0;
}

int CompactWad(char *name,FILE *pst,WadHeader *phdr,WadDir *pdir)
{
	WadDir odir;
	WadHeader ohdr;
	FILE *st = fopen("__nbld__.tmp","w+b");
	char *buffer=NULL;
	int buffermax=0;
	int offs=0,i;

	if (!st) return 1;

	InitDir(&odir);

	strncpy(ohdr.wadtag,"PWAD",4);

	fwrite(&ohdr,sizeof(WadHeader),1,st);
	offs = sizeof(WadHeader);

	for (i=0;i<pdir->nentries;i++)
	{
		int n;

		fseek(pst,pdir->dirp[i].resptr,SEEK_SET);
		if ((n=pdir->dirp[i].reslen)>buffermax)
			buffer=realloc(buffer,buffermax=n);

		if (n)
		{
			fread(buffer,n,1,pst);
			fwrite(buffer,n,1,st);
		}
		AddDirEntry(&odir,offs,n,Str8(pdir->dirp[i].resname));
		offs += n;
	}
	free(buffer);

	fwrite(odir.dirp,odir.nentries*sizeof(WadDirEntry),1,st);

	strncpy(ohdr.wadtag,phdr->wadtag,4);
	ohdr.waddirlen = odir.nentries;
	ohdr.waddiraddr = offs;
	fseek(st,0,SEEK_SET);
	fwrite(&ohdr,sizeof(WadHeader),1,st);

	fclose(st);

	if (rename("__nbld__.tmp",name))
	{
		perror("Error renaming __nbld__.tmp");
	  return 1;
	}

	return 0;
}

void PrintVertex(Vertex v)
{
	dprintf(logfile,"X:%5hd Y:%5hd\n",v.x,v.y);
}

static int *blocking=NULL;
static int nblocks=0;
static UBYTE *sreject=NULL;
static int nreject=0;

void SetSectorsVisible(int sec1,int sec2)
{
	int n1 = sec1*NSectors+sec2;
	int n2 = sec2*NSectors+sec1;

	sreject[n1>>3] &= ~(1<<(n1&7));
	sreject[n2>>3] &= ~(1<<(n2&7));
}

int SectorVisibleFromSector(int sec1,int sec2)
{
	int n1 = sec1*NSectors+sec2;
	return !(sreject[n1>>3] & (1<<(n1&7)));
}

int SubSectorVisibleFromSubSector(int ssec1,int ssec2)
{
	int s1 = SubSectorInSector(Ssectors+ssec1)-Sectors;
	int s2 = SubSectorInSector(Ssectors+ssec2)-Sectors;
	return SectorVisibleFromSector(s1,s2);
}

//
// This method of calculating reject is based on 2S lines. If two sectors
// are visible to each other then a 2S line in one sector is visible from
// a 2S line in the other.
//
// To speed things up we should remove all sectors with completely 2S 
// boundaries for which every vertex lies in another sector. These sectors
// are always mutually visible with the surrounding sector.
//
// Also for speed, if the four sectors on the sides of the two 2S lines
// are all known in visibility relative to each other, the 2S line pair
// can be skipped.
//

void BuildReject()
{
	int i;
	int rejected=0,rejtot=0;

	// allocate the reject and set it to all '1's - no visibility at all

	sreject = malloc(nreject=((NSectors*NSectors+7)>>3));
	memset(sreject,0xff,nreject);

	// clear unused bits in last byte for predictable compares (debugging)

	sreject[nreject-1] &= ~(((-1)<<(8*nreject-NSectors*NSectors))&255);

	// The first thing to do is some serious pruning. We need to cut down
	// the number of possible blocking linedefs, while at the same time
	// finding as many sector pairs that allow visibility as possible.

	// Remove all linedefs that are 2S and open, or have a moveable
	// floor or ceiling on the other side from the set of possible
	// blocking linedefs. At the same time, make any two sectors facing
	// each over a non-blocker visible from each other

  for (i=0;i<NLinedefs;i++)
	{
		Linedef *ld = Linedefs + i;

		if (ld->sides[1]<0)
			AddDynInt(&blocking,&nblocks,i);					// all 1S lines block	
		else
		{
			Sidedef *sd1 = LinSid(ld,0);
			Sidedef *sd2 = LinSid(ld,1);
			Sector *sec1 = SidSec(sd1);
			Sector *sec2 = SidSec(sd2);

			if
			(
				min(sec1->chgt,sec2->chgt) < max(sec1->fhgt,sec2->fhgt)
				&&
				!sec1->tag && !sec2->tag							// 2S lines between umovable
			)																				// sectors with chgt<fhgt block
				AddDynInt(&blocking,&nblocks,i);
			else
				SetSectorsVisible(sd1->sect,sd2->sect);
		}
	}
	printf("%d 2S open and openable lines removed\n",NLinedefs-nblocks);

	// Make all sectors visible from themselves
	
	for (i=0;i<NSectors;i++)
		SetSectorsVisible(i,i);

	printf("Lines: %d, Blocking: %d\n",NLinedefs,nblocks);

	rejected = 0;
	for (i=0;i<nreject;i++)
	{
		rejected += bitcnt[(unsigned)(sreject[i])];
	}
	rejtot = NSectors*NSectors;

	printf("Visibility check complete. Reject %d%%\n",(rejected*100)/rejtot);

	free(Reject);
	Reject=sreject;
	NReject=nreject;

	FreeDynInt(&blocking,&nblocks);
}
