#include "patch.h"

//
// MAK.EXE
//
// Convert a text description of a wad, in a .SGN or .RES file, and data
// files containing resources into a wad. The default operation of writing
// the output to the same name as the input, but with .WAD extension, can
// be overidden with the /O:name switch.
//
// The IWAD structure (or game) is specified by a resource definition file.
// See RESDEF.TXT for instructions on configuring Patcher to work with one
// or more structure definitions besides DOOM II's, which is default.
//
// The resource definition filters the resources that are built into the
// output wad. If a category of resources, or a specific name of a resource
// is missing from the definition file, such resources are ignored in the
// .SGN or .RES file, and not built into the output wad. 
//
//

void FreeMakStructures()
{
	FreePalette();
	FreeVTex();
	FreeTextures(&pwt);
	FreeResourceDefinition(&ResDef);
	FreeDir(&pdir);
}

void DoMakCommandLineOpts()
{
	int i,j,n;
	int cleared=0;
	int cleared2=0;

	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfuLWTEMGSPFU"); // its a sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			if (!cleared)	// if first selection option
			{
				if (cloptsense[i])	// clear all if it adds types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=0;
					DoGraphics=DoSprites=DoPatches=DoFlats=0;
				}
				else if (!cloptsense[i]) // set all if it subtracts types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=1;
					DoGraphics=DoSprites=DoPatches=DoFlats=1;
				}
				cleared=1;	// but don't do it for following selection opts
			}
			for (j=0;j<strlen(cloptions[i]);j++)	// enable or disable
			{										// each resource for
				switch(tolower(cloptions[i][j]))	// which a letter occurs
				{
					case 'l':
						DoLumps = cloptsense[i];
						break;
					case 'w':
						DoWads = cloptsense[i];
						break;
					case 't':
						DoTextures = cloptsense[i];
						break;
					case 'e':
						DoEffects = cloptsense[i];
						break;
					case 'm':
						DoMusics = cloptsense[i];
						break;
					case 'g':
						DoGraphics = cloptsense[i];
						break;
					case 's':
						DoSprites = cloptsense[i];
						break;
					case 'p':
						DoPatches = cloptsense[i];
						break;
					case 'f':
						DoFlats = cloptsense[i];
						break;
					case 'u':
						DoNonStandard = cloptsense[i];
						break;
				}
			}
		}
		else if (tolower(cloptions[i][0])=='n') // suppress warnings option
		{
			char *p = cloptions[i]+1;
			int sense = cloptsense[i];

			if (*p==':') p++;
			n = strspn(p,"lwtemgspfuLWTEMGSPFU"); // if all are res selection chars
			if (n==strlen(p))
			{
				if (!cleared2)	// if first selection option
				{
					SuppressLumps=SuppressWads=SuppressTextures=!sense;
					SuppressEffects=SuppressMusics=SuppressGraphics=!sense;
					SuppressSprites=SuppressPatches=SuppressFlats=!sense;
					cleared2=1;	// but don't do it for following suppress warning opts
				}
				for (j=0;j<strlen(p);j++)	// enable or disable
				{													// each missing resource warning
					switch(tolower(p[j]))		// for which a letter occurs
					{
						case 'l':
							SuppressLumps = sense;
							break;
						case 'w':
							SuppressWads = sense;
							break;
						case 't':
							SuppressTextures = sense;
							break;
						case 'e':
							SuppressEffects = sense;
							break;
						case 'm':
							SuppressMusics = sense;
							break;
						case 'g':
							SuppressGraphics = sense;
							break;
						case 's':
							SuppressSprites = sense;
							break;
						case 'p':
							SuppressPatches = sense;
							break;
						case 'f':
							SuppressFlats = sense;
							break;
						case 'u':
							SuppressNonStandard = sense;
							break;
					}
				}
			}
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
		{
			strcpy(defname,cloptions[i]);
			AddExt(defname,".DEF");
			FreeResourceDefinition(&ResDef);
			InitCategories(defname,&ResDef);
		}
 		else if (tolower(cloptions[i][0])=='i')
		{
			if (cloptions[i][1]==':')
				strcpy(iwadname,cloptions[i]+2);
			else
			{
				MakeIwad = 1;
				SingleMarker = 1;
				DoubleMarker = 0;
			}
		}
 		else if (tolower(cloptions[i][0])=='r')
			MakeRwad = cloptsense[i];
 		else if (cloptions[i][0]=='1')
		{
			SingleMarker = cloptsense[i];
			DoubleMarker = !cloptsense[i];
		}
 		else if (cloptions[i][0]=='2')
		{

			DoubleMarker = cloptsense[i];
			SingleMarker = !cloptsense[i];
		}
		else if (tolower(cloptions[i][0])=='c')
			FollowCase = cloptsense[i];
		else if (tolower(cloptions[i][0])=='z')
			NoPaletteMatching=1;
		else if (tolower(cloptions[i][0])=='k')
			KeepUnusedPatches = cloptsense[i];
		else if (tolower(cloptions[i][0])=='n')
			NoPatchWarnings = 1;
		else if (tolower(cloptions[i][0])=='x' && cloptions[i][1]==':')
		{
			strcpy(extname,cloptions[i]+2);
			AddExt(extname,".WAD");
			ExtendTextures = cloptsense[i];
		}
		else if (tolower(cloptions[i][0])=='o' && cloptions[i][1]==':')
			strcpy(outname,cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='d' && cloptions[i][1]==':')
		{
			strcpy(defname,cloptions[i]+2);
			AddExt(defname,".DEF");
			FreeResourceDefinition(&ResDef);
			InitCategories(defname,&ResDef);
		}
		else dprintf(logfile,"Warn: command line option: %s not understood\n",cloptions[i]);
	}

	// Locate a palette to be inserted in BMPs output
	// If color or translucency maps are available, load those too

	FreePalette();
	AddExt(iwadname,".WAD");
	if (*iwadname)						// then the IWAD, if spec'd
	{
		FILE *ist;
		WadHeader ihdr;
		WadDir idir;

		InitDir(&idir);
		ist = fopen(iwadname,"rb");
		if (ist!=NULL)
		{
			dprintf(logfile,"Using resources from %s for palette\n",iwadname);
			fread(&ihdr,sizeof(WadHeader),1,ist);
			ReadDir(&ihdr,&idir,ist);
			n = IsIn("PLAYPAL",&idir);
			if (n>=0)
				palette = GetPalette(&idir,ist,n);
			else
				palette = GetPalette(NULL,NULL,-1);

			AddColorTranMaps(NULL,&idir,ist);

			fclose(ist);
			FreeDir(&idir);
		}
	}
	else palette = GetPalette(NULL,NULL,-1);
}

// Open the control files, make the output wad

int makmain(int argc,char **argv)
{
	int err=0;

	CopyEnvironmentCLArgs("MAK",argc,argv);
	if (ParseCommandLine(-1,1))
	{
		FreeCLStrs();
		return 3;	
	}

	if (nclfiles!=1)
	{
		dprintf(logfile,"\nMAK.EXE %s by Jim Flynn, Copyright 1996-7  -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"Make a wad from a resource file and resource tree\n");
		dprintf(logfile,"\nUsage: MAK [options] resfile[.res]\n");
		dprintf(logfile,"\n Resource selection option:\n");
		dprintf(logfile," +XYZ creates only XYZ resources, -XYZ creates all BUT XYZ resources.\n");
		dprintf(logfile," +NXYZ No warning for missing X,Y,Z  -NXYZ Warning for X,Y,Z only\n");
		dprintf(logfile," L LUMPS      W WADS       T TEXTURES     U NON-STANDARD\n");
		dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
		dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
		dprintf(logfile,"\n General options, use SET MAK=options to set defaults:\n");
		dprintf(logfile," /I create IWAD            /1 Convert FF_END to F_END, etc.\n");
		dprintf(logfile," /I:wad use wad's palette  /2 Convert F_END to FF_END, etc.\n");
		dprintf(logfile," /R use F1,2,3 and P1,2,3  /O:name[.WAD]  Specify output wad name\n");
		dprintf(logfile," /C Signs follow case      /X:txtwad[.WAD] Base textures on those in txtwad.wad\n");
		dprintf(logfile," +/-K Keep unref'd patches /Z Do not convert palettes, ever\n");
		dprintf(logfile,"\n Game definition options, use SET MAK=options to set defaults:\n");
		dprintf(logfile," /DOOM2 /DOOM /HERETIC /HEXEN specify game\n");
		dprintf(logfile," /D:name[.DEF]  Specify resource definitions\n");
		dprintf(logfile,"\n See MAK.TXT for details.\n");
						  
		FreeCLStrs();
		return(1);
	}

	InitDoomFont();

	strcpy(outname,"");
	strcpy(iwadname,"");
	strcpy(extname,"");

	NoPatchWarnings = 0;
	FollowCase = 0;
	NoPaletteMatching = 0;
	KeepUnusedPatches = 1;

	DoMakCommandLineOpts();

	strcpy(pwadname,clfiles[0]);
	if (outname[0]=='\0')
		strcpy(outname,pwadname);
	AddExt(outname,".WAD");

	FreeCLStrs();


	if (!ResDef.cat)
	{
		AddExt(defname,".DEF");
		InitCategories(defname,&ResDef);
	}

	if ((err=ReadResourceControl(pwadname, outname)))
		if (err>1) return err;

	memcpy(PaletteUsed,palette,256*sizeof(RGBQUAD));
	memcpy(ColorMapUsed[0],ColorMap[0],34*256);

	if (errorlev<=1)
	{
		AddExt(outname,".WAD");
		err=WriteControlWad(outname);
		if (err) return err;
		dprintf(logfile,"Control file converted to wad: %s\n",outname);
	}

	printf("\n");
	return errorlev;
}

