#include "patch.h"

/************************************************************
 MAKSUBS.C
			 /-------------------------\
			||     MAK subroutines     ||
			 \-------------------------/
									
************************************************************/

static RGBQUAD pal[256];

// Advance a file pointer and the offset in the file
// to next longword alignment boundary

void Advance(int *off,int n,FILE *st)
{
	int m;

	m = ((n+3)>>2)<<2;
	*off += m;
	fseek(st,m-n,SEEK_CUR);
}

// Open a MAK description file, extension specified or .SGN or .RES

void OpenSgnRes(FILE **st,char *ctlname)
{
	*st = fopen(ctlname,"r");
	if (!*st)
	{
		AddExt(ctlname,".SGN");
		*st = fopen(ctlname,"r");
		if (!*st)
		{
			AddExt(ctlname,".RES");
			*st = fopen(ctlname,"r");
		}
	}
}

void CreateSignPatch(int c,int i,int ltwid)
{
	char buffer[256];
	Bitmap out;

	// Create the graphic from the sign's text description
	RenderString(ltwid,mtxres[c][i].sign,&out);
	dprintf(logfile,"Info: Created %s as %d X %d patch resource\n",mtxres[c][i].name,out.realw,out.logh);

	// Write the new patch to disk as specified
	ResourceFilename(buffer,c,i);
	MakeDir(DirOf(buffer),(mode_t)0777);

	AddExt(buffer,".BMP");
	WriteBMP(&out,buffer,palette);
	free(out.bmp);
}

// Read a resource file and embed it in a wad without translation.
// memory balanced except directory extension

void WriteGenControl(VTex *list,int len,FILE *tst,char *resn,char *path,int *offs,WadDir *dir)
{
	int n;
	FILE *st;
	char *buffer = NULL;

	if (path && *path)
	{
		st = fopen(path,"rb");
		if (st!=NULL)
		{
			n = filelen(st);
			if (n>0)
			{
				buffer = malloc(n);
				fread(buffer,n,1,st);
				fclose(st);

				SafeWrite(buffer,n,1,tst);
				free(buffer);

				AddDirEntry(dir,*offs,n,resn);
				Advance(offs,n,tst);
			}
		}
		else
		{
			dprintf(logfile,"ERROR: Can't open lump: %s\n",path);
			errorlev=3;
			return;
		}
	}
}

// Read a .BMP file and embed in a wad as a DOOM picture
// memory balanced except directory extension

void WritePicControl(VTex *list,int len,FILE *tst,char *resn,char *path,int p,int *offs,WadDir *dir,int pf)
{
	int n;
	PicData pic;
	Bitmap out;

	if (path && *path)
	{
		ReadBMP(&out,path,pal);	// allocs out.bmp
		if (!NoPaletteMatching && (memcmp(pal,palette,sizeof(RGBQUAD)*xparent) ||
			memcmp(pal+xparent+1,palette+xparent+1,sizeof(RGBQUAD)*(255-xparent))))
		{
			dprintf(logfile,"Info: Converting %s's palette\n",path);
			ConvertBMP(&out,pal,palette);
		}

		if (out.logw>0)
		{
			int tallposts = MultPost(list[p].id);
			int xoff,yoff;

			// default the internal offsets if not specified in MAK file

			if (list[p].nargs>0)
				xoff = atoi(list[p].args[0]);
			else if (ResourceTypeIs(list[p].id,"GRAPHICS"))
				xoff = 0;
			else if (ResourceTypeIs(list[p].id,"SPRITES"))
				xoff = out.realw/2 - 1;
			else 
				xoff = out.realw/2 - 1;

			if (list[p].nargs>1)
				yoff = atoi(list[p].args[1]);
			else if (ResourceTypeIs(list[p].id,"GRAPHICS"))
			  yoff = 0;
			else if (ResourceTypeIs(list[p].id,"SPRITES"))
				yoff = out.logh;
			else 
				yoff = out.logh-5;

			// Now actually convert BMP to pic and put it in the wad 

			ConvertToPicture(&out,xoff,yoff,tallposts,&pic);
			WritePictureImage(tst,ftell(tst),&pic,tallposts);
			n=GetPictureLen(&pic,tallposts);

			AddDirEntry(dir,*offs,n,resn);
			Advance(offs,n,tst);

			free(pic.coloffs);
			free(pic.posts);
		}
		else
		{
			dprintf(logfile,"ERROR: Can't open picture lump: %s\n",path);
			errorlev=3;
			return;
		}
		free(out.bmp);
	}
}

// Read a .WAV file and embed in a wad as a DOOM sound
// memory balanced except directory extension

void WriteSoundControl(VTex *list,int len,FILE *tst,char *resn,char *path,int *offs,WadDir *dir)
{
	int n,pos;
	FILE *st=NULL;
	char *buffer = NULL;
	WaveHdr whdr;
	FmtData fdata;
	CkHdr chdr;
	RawHeader rhdr;
	int ncks=0;

	if (path && *path)
	{
		st = fopen(path,"rb");
		if (st)
		{
			fread(&whdr,sizeof(WaveHdr),1,st);
			if (strnicmp(whdr.riff,"RIFF",4)!=0 || strnicmp(whdr.wave,"WAVE",4)!=0)
				dprintf(logfile,"WARNING: not a valid .WAV file: %s\n",path);

			fread(&chdr,sizeof(CkHdr),1,st);		// format header
			pos = ftell(st);						// save position after len

			if (strnicmp(chdr.ckid,"fmt ",4))
				dprintf(logfile,"ERROR: bad format header\n");

			fread(&fdata,sizeof(FmtData),1,st);
			if ((fdata.s1!=0x2b11 && fdata.s1!=0x5622) || (fdata.s2!=0x2b11 && fdata.s2!=0x5622))
				dprintf(logfile,"WARNING: bad sample rate %s: %ld %ld\n",path,fdata.s1,fdata.s2);
			if (fdata.a1!=0x0001 || fdata.a2!=1)
				dprintf(logfile,"WARNING: This wave format not supported\n");


			fseek(st,pos,SEEK_SET);		// seek to end of format chunk
			fseek(st,2*((chdr.cklen+1)/2), SEEK_CUR);

			fread(&chdr,sizeof(CkHdr),1,st);	// search for data chunk
			while (ncks<16 && strnicmp(chdr.ckid,"data",4))
			{
				fseek(st,2*((chdr.cklen+1)/2),SEEK_CUR);
				fread(&chdr,sizeof(CkHdr),1,st);
				ncks++;
			}

			n = chdr.cklen;

			if (n>0 && ncks<16)
			{
				buffer = malloc(n);
				fread(buffer,n,1,st);
				fclose(st);

				rhdr.three=3;
				rhdr.zero=0;
				rhdr.srate=fdata.s1;
				rhdr.nsamps=chdr.cklen;

				SafeWrite(&rhdr,sizeof(RawHeader),1,tst);
				SafeWrite(buffer,n,1,tst);
				free(buffer);

				AddDirEntry(dir,*offs,n+sizeof(RawHeader),resn);
				Advance(offs,n+sizeof(RawHeader),tst);
			}
			else
			{
				if (n<0)
					dprintf(logfile,"ERROR: .WAV %s has negative number of samples, skipping\n",resn);
				else if (ncks>=16)
					dprintf(logfile,"ERROR: .WAV %s has too many chunks, probably corrupt\n",resn);
			}
		}
		else
		{
			dprintf(logfile,"ERROR: Can't open sound lump: %s\n",path);
			errorlev=3;
			return;
		}
	}
}

// Read a .BMP file and embed in a wad as a DOOM flat
// memory balanced except directory extension

void WriteFlatControl(VTex *list,int len,FILE *tst,char *resn,char *path,int *offs,WadDir *dir)
{
	int n;
	Bitmap out;

	if (path && *path)
	{
		ReadBMP(&out,path,pal);	// allocs out
		if (!NoPaletteMatching && (memcmp(pal,palette,sizeof(RGBQUAD)*xparent) ||
			memcmp(pal+xparent+1,palette+xparent+1,sizeof(RGBQUAD)*(255-xparent))))
		{
			dprintf(logfile,"Info: Converting %s's palette\n",path);
			ConvertBMP(&out,pal,palette);
		}
		if (out.logw>0)
		{
			n = out.logw*out.logh;
			fseek(tst,*offs,SEEK_SET);
			SafeWrite(out.bmp,n,1,tst);
			AddDirEntry(dir,*offs,n,resn);
			Advance(offs,n,tst);
		}
		else
		{
			dprintf(logfile,"ERROR: Can't open flat lump: %s\n",path);
			errorlev=3;
			return;
		}
		free(out.bmp);
	}
}

// This routine embeds an external level in the MAK output wad
// memory balanced except for directory extension

void WriteLevelControl(VTex *list,int len,FILE *tst,char *resn,char *path,int *offs,WadDir *dir)
{
	int i,n;
	FILE *st;
	char *buffer = NULL;
	WadDir wd;
	WadHeader wh;
	int gotmap=0;

	if (path && *path)
	{
		AddExt(path,".WAD");
		st = fopen(path,"rb");
		if (st!=NULL)
		{
			fread(&wh,sizeof(WadHeader),1,st);
			InitDir(&wd);
			ReadDir(&wh,&wd,st);

			for (i=0;i<wd.nentries;i++)
			{
				if (IsMapResource(wd.dirp+i))
				{
					n = wd.dirp[i].reslen;
					if (n>0)
					{
						buffer = malloc(n);
						fseek(st,wd.dirp[i].resptr,SEEK_SET);
						fread(buffer,n,1,st);

						SafeWrite(buffer,n,1,tst);
						free(buffer);
					}

					AddDirEntry(dir,*offs,n,wd.dirp[i].resname);
					Advance(offs,n,tst);
				}
				else if (IsMapName(wd.dirp+i))
				{
					if (gotmap) break;
					AddDirEntry(dir,*offs,0,resn);
					gotmap=1;
				}
				else if (gotmap)
					break;
			}
			FreeDir(&wd);
		}
		else
		{
			dprintf(logfile,"ERROR: Can't open level wad: %s\n",path);
			errorlev=3;
			return;
		}
		fclose(st);
	}
}

int WarningSuppressed(int c)
{
	if (stricmp(ResDef.cat[c].secname,"LUMPS")==0)
		return SuppressLumps;
	else if (stricmp(ResDef.cat[c].secname,"TEXTURES")==0)
		return SuppressTextures;
	else if (stricmp(ResDef.cat[c].secname,"SOUNDS")==0)
		return SuppressEffects;
	else if (stricmp(ResDef.cat[c].secname,"MUSICS")==0)
		return SuppressMusics;
	else if (stricmp(ResDef.cat[c].secname,"LEVELS")==0)
		return SuppressWads;
	else if (stricmp(ResDef.cat[c].secname,"GRAPHICS")==0)
		return SuppressGraphics;
	else if (stricmp(ResDef.cat[c].secname,"PATCHES")==0)
		return SuppressPatches;
	else if (stricmp(ResDef.cat[c].secname,"SPRITES")==0)
		return SuppressSprites;
	else if (stricmp(ResDef.cat[c].secname,"FLATS")==0)
		return SuppressFlats;
	else
		return SuppressNonStandard;
	return 0;
}

char *ResourceFilename(char *resfilename,int c,int i)
{
	Category *ctgy,*ctgy2=NULL;
	char resname[16];
	int d;

	if (resfilename==NULL)
		resfilename = malloc(256);

	ctgy2 = ctgy = ResDef.cat+c;
	if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
	{
		d = CategoryOf("PATCHES",&ResDef);
		if (d>=0)
			ctgy2 = ResDef.cat+d;
	}

	strcpy(resname,mtxres[c][i].name);
	if (stricmp(ctgy->secname,"HISTORY")) // if not a HISTORY lump
	{
		if (mtxres[c][i].path && *mtxres[c][i].path)
		{
			strcpy(resfilename,mtxres[c][i].path);
			if (IsDir(resfilename))
			{
				strcat(resfilename,"\\");
		  	strcat(resfilename,resname);
			}
		}
		else
		{
			strcpy(resfilename,ctgy2->defaultdir);
			strcat(resfilename,"\\");
			strcat(resfilename,resname);
		}
	}
	else
	{
		strcpy(resname,"HISTORY");
		strcpy(resfilename,ctgy->defaultdir);
		strcat(resfilename,"\\");
		strcat(resfilename,resname);
	}
	return resfilename;
}

void WriteResource(int c, int i, int *offs, WadDir *dir, FILE *tst, WadTextures *xt)
{
	int j,d,k,c1,c2;
	char *resfilename;
	char resname[16];
	Category *ctgy,*ctgy1,*ctgy2;

	ctgy = ResDef.cat+c;

  resfilename=ResourceFilename(NULL,c,i);
	strcpy(resname,mtxres[c][i].name);

	// if the resource path is specified in the mak file entry, use
	// the method corresponding to the extension specified, if any.
	
	// Try opening each valid resource extension, in order
	// they are listed, and use the method corresponding to the first extension
	// that succeeds.
	for (j=0;j<ctgy->nextns;j++)
	{
		AddExt(resfilename,ctgy->Extns[j].str);
		if (Exists(DOSName(resfilename)))
		{
			d = ctgy->Extns[j].didx;
			switch(d)
			{
				case DT_GENERIC:
					// translate map names to music resource names in MUSICS section
					if (stricmp(ctgy->secname,"MUSICS")==0)
					{
						c1 = CategoryOf("LEVELS",&ResDef);
						c2 = CategoryOf("MUSICS",&ResDef);

						if (c1>=0 && c2>=0)
						{
							ctgy1 = ResDef.cat+c1;
							ctgy2 = ResDef.cat+c2;

							for (k=0;k<ctgy1->nnames && stricmp(ctgy1->Names[k].str,resname);k++)
								;
							// if found in level names use corresponding music name
							if (k<ctgy1->nnames) 
								strcpy(resname,ctgy2->Names[k%ctgy2->nnames].str);
						}
					}
					WriteGenControl(mtxres[c],mtxcnt[c],tst,resname,resfilename,offs,dir);
					break;
				case DT_LEVELS:
					WriteLevelControl(mtxres[c],mtxcnt[c],tst,resname,resfilename,offs,dir);
					break;
				case DT_SOUNDS:
					WriteSoundControl(mtxres[c],mtxcnt[c],tst,resname,resfilename,offs,dir);
					break;
				case DT_PICS:
					WritePicControl(mtxres[c],mtxcnt[c],tst,resname,resfilename,i,offs,dir,stricmp(ctgy->secname,"PATCHES")==0? 1 : 0);
					break;
				case DT_FLATS:
					WriteFlatControl(mtxres[c],mtxcnt[c],tst,resname,resfilename,offs,dir);
					break;
				default:
					break;
			}
			break;   // prevent both .MUS and .MID from being inserted
		}
		else if (strnicmp(resname,"FONTA_",6)==0 || strnicmp(resname,"FONTB_",6)==0 || strnicmp(resname,"FONTAY_",7)==0)
			AddDirEntry(dir,*offs,0,resname);	// patch for HERETIC fonts
	}
	if (j==ctgy->nextns)
		if (!WarningSuppressed(c))
		{
			if (!xt->pnames.npnames || (stricmp(ResDef.cat[c].secname,"PATCHES")==0 && GetPIdx(&(xt->pnames),resname)<0))
				dprintf(logfile,"WARNING: Resource file for %s (%s) not found\n",resname,resfilename);
		}
	free(resfilename);
}

int UsedByTexture(char *name)
{
	Category *ctgy;
	int i,j;

	for (i=0;i<ResDef.ncats;i++)
	{
		ctgy = ResDef.cat+i;
		if (strnicmp(ctgy->secname,"TEXTURE",7)==0)
			for (j=0;j<mtxcnt[i];j++)
				if (SyntaxType(mtxres[i][j].id)==ST_RESINPUT && stricmp(mtxres[i][j].name,name)==0)
					return 1;
	}
	return 0;
}

//
// Render a .WAD file as a Bitmap. Each sector is filled by the
// bitmap specified by the floor name. Sector lighting is implemented
// thru the colormap specified by the ceiling name, standard if not
// a custom colormap name. The tag is an x-offset into the sector floor
// bitmap, the type is a similar y-offset. To create transparencies use
// the floor name XPARENT.
//

void WadToBitmap(char *resfilename,char *path,Bitmap *out)
{
	FILE *st;
	int i,j,x,y;
	SectorData *sd=NULL;
	RGBQUAD pal[256];
	char resdir[256];

	AddExt(resfilename,".WAD");
	strcpy(resdir,DirOf(resfilename));

	st = fopen (resfilename,"rb");
	if (st)
	{
		WadHeader whdr;
		WadDir wdir;

		InitDir(&wdir);
		InitLevelData();
		fread(&whdr,sizeof(WadHeader),1,st);
		ReadDir(&whdr,&wdir,st);

		if (!ReadLevelData("MAP01",&wdir,st,10))
		{
			UBYTE *orp,*rp;

			out->realw = map_maxx - map_minx;
			out->logh = map_maxy - map_miny;
			out->logw = ((out->realw+3)<<2)>>2;
			out->bmp = malloc(out->logw*out->logh);

			// build the table of render properties from sector properties
			sd = malloc(NSectors*sizeof(SectorData));
			for (i=0;i<NSectors;i++)
			{
				sd[i].secnum = -1;     // flag that sector data is not correct/complete

				sd[i].fill = malloc(256);
				strcpy(sd[i].fill,"");
				if (stricmp(Str8(Sectors[i].ftxt),"XPARENT"))
				{
					if (path && path && IsDir(path))
					{
						strcpy(sd[i].fill,path);
						AddTrailingBack(sd[i].fill);
					}
					else
					{
						strcpy(sd[i].fill,resdir);
						AddTrailingBack(sd[i].fill);
					}
					strcat(sd[i].fill,Str8(Sectors[i].ftxt));
					AddExt(sd[i].fill,".BMP");
					ReadBMP(&sd[i].fillbmp,sd[i].fill,pal);
					if (sd[i].fillbmp.logw)
						sd[i].secnum = i;		 // flag that sector BMP data is read

					sd[i].cmap = -1;
					for (j=0;j<ncmaps;j++)
						if (stricmp(cmaps[j].name,Str8(Sectors[i].ctxt))==0)
							sd[i].cmap = j;

					sd[i].light = 31-(Sectors[i].lite>>3);
					sd[i].xoff = Sectors[i].tag;
					sd[i].yoff = Sectors[i].typ;
				}
			}

			dprintf(logfile,"Info: Rendering %s as %d X %d BMP\n",resfilename,out->realw,out->logh);

			orp = out->bmp+(out->logh-1)*out->logw;
			for (y=0; y<map_maxy-map_miny; y++,orp-=out->logw)
			{
				for (x=0,rp=orp; x<map_maxx-map_minx; x++)
				{
					Sector *sec=PointNearSector(x+map_minx,y+map_miny);
					int s=-1;

					if (sec)
						s = sec-Sectors;

					if (s>=0 && sd[s].secnum>=0)
					{
						Bitmap *b=&sd[s].fillbmp;
						int l=sd[s].light;
						UBYTE pxl;

						pxl = b->bmp[((y+sd[s].yoff)%b->logh)*b->logw+((x+sd[s].xoff)%b->realw)];
						*rp++ = sd[s].cmap>=0?
							        cmaps[sd[s].cmap].cmap[l][pxl] : ColorMap[l][pxl];
					}
					else
						*rp++ = xparent;
				}
			}
			free(sd);
			FreeLevelData();
		}
		FreeDir(&wdir);
	}
}

void GetLighting(VTex *v,int *lite,int *cmap,int *tmap)
{
	*lite=256;
	*cmap=0;
	*tmap=-1;

	if (v->nargs>2)
		if (strchr(v->args[2],':'))
		{
			char *lstr = strdup(v->args[2]);
			char *tok = strtok(lstr,":");
			if (tok)
			{
				*lite = atoi(tok);
				tok = strtok(NULL,":");
				if (tok)
				{
					int i;

					for (i=0;i<ncmaps;i++)
						if (stricmp(cmaps[i].name,tok)==0)
							break;
					*cmap = i<ncmaps? i : -1;
					tok = strtok(NULL,":");
					if (tok)
					{
						int i;

						for (i=0;i<ntmaps;i++)
							if (stricmp(tmaps[i].name,tok)==0)
								break;
						*tmap = i<ntmaps? i : -1;
					}
				}
			}
			xlibfree(lstr);
		}
		else *lite = atoi(v->args[2]);
}

// Given a pointer c,i to a resource definition in a VTex list
// Generate a Bitmap for the resource based on the following
// resource inputs

Bitmap *GenerateBitmap(int c,int i)
{
	int def=i,npatches=0;
	Bitmap *b=NULL,ptch,*p=&ptch;
	VTex *mtxd = mtxres[c]+def;
	int lite=256,cmap=-1,tmap=-1;

	// Count the number of resource inputs following this definition

	i++;
	while (i<mtxcnt[c] && SyntaxType(mtxres[c][i++].id)==ST_RESINPUT)
		npatches++;

	// Create a bitmap to hold the generated graphic
	// For any extended resource definition default internal patch offsets
	// must be used, as the two numbers following the resource name in the
	// definition are the width and height of the composed graphic patch

	b = malloc(sizeof(Bitmap));
	b->realw = mtxd->nargs>0? atoi(mtxd->args[0]) : 0;
	b->logh = b->realw? (mtxd->nargs>1? atoi(mtxd->args[1]) : 0) : 0;
	b->logw = ((b->realw+3)>>2)<<2;

	if (npatches)				// if there are resource inputs following
	{
		if (b->realw)			// and the height and width were specified
		{
			b->bmp = malloc(b->logw*b->logh);						// allocate and initialize
			memset(b->bmp,xparent,b->logw*b->logh);			// to transparent

			// merge each input patch into the output graphic

			for (i=def+1;i<def+1+npatches;i++)
			{
				VTex *mtxi = mtxres[c]+i;
				char resfilename[256];

				if (SyntaxType(mtxi->id)==ST_RESINPUT)
				{
					ResourceFilename(resfilename,c,i);
					p->logw = 0;

					// whatever it is, convert it to a Bitmap

					switch(InputType(mtxi->id))
					{
						case IT_BMP:
							AddExt(resfilename,".BMP");
							ReadBMP(p,resfilename,pal);
							break;
						case IT_WAD:
							AddExt(resfilename,".WAD");
							AddColorTranMaps(resfilename,NULL,NULL);
							WadToBitmap(resfilename,mtxi->path,p);
							break;
						case IT_SIGN:
							RenderString(b->realw,mtxi->sign,p);
							break;
						default:
							break;
					}

					// parse the lighting parameter (args[2]) to get the
					// lighting, colormap and translucency for the patch
					// Then overlay it on the output graphic

					GetLighting(mtxi,&lite,&cmap,&tmap);
					if (p->logw)
						OverlayBitmaps
						(
							b,
							p,
							mtxi->nargs>0? atoi(mtxi->args[0]) : 0,
							mtxi->nargs>1? atoi(mtxi->args[1]) : 0,
							lite,
							cmap,
							tmap,
							mtxi->nargs>3? atoi(mtxi->args[3]) : 0,
							mtxi->nargs>4? atoi(mtxi->args[4]) : 0,
							mtxi->nargs>5? atoi(mtxi->args[5]) : p->realw,
							mtxi->nargs>6? atoi(mtxi->args[6]) : p->logh
						);
			  	free(p->bmp);
				}
			}
		}
	}

	// If the definition itself specifies a sign
	// render it and overlay it on the output graphic last
	// Note it always overlays at 0,0, use an input patch if
	// more control is needed.

	if (InputType(mtxd->id)==IT_SIGN)
	{
		RenderString(b->realw,mtxd->sign,p);
		GetLighting(mtxd,&lite,&cmap,&tmap);
		if (p->logw)
			OverlayBitmaps
			(
				b,
				p,
				0,
				0,
				lite,
				cmap,
				tmap,
				mtxd->nargs>3? atoi(mtxd->args[3]) : 0,
				mtxd->nargs>4? atoi(mtxd->args[4]) : 0,
				mtxd->nargs>5? atoi(mtxd->args[5]) : p->realw,
				mtxd->nargs>6? atoi(mtxd->args[6]) : p->logh
			);
		free(p->bmp);
	}
	return b;
}

// Scan the MAK file lists, marking all normal resource definitions that
// the engine can handle, vs. all extended definitions for which we need
// to compose patches.

void ScanResourceLists()
{
	int c;

	for (c=0;c<ResDef.ncats;c++)
	{
		Category *ctgy = ResDef.cat+c;
		int N = mtxcnt[c];
		VTex *v = mtxres[c];
		int istexture = (strnicmp(ctgy->secname,"TEXTURE",7)==0);
		int i;

		for (i=0;i<N;i++)
		{
			SetNormDef(v[i].id,1);		// set all list entries to normal

			// When a definition is encountered. This is a line that directly
			// defines a wad lump or texture, it does not start with *,@ or +.

			if (SyntaxType(v[i].id)==ST_RESDEFN)	
			{
				int normal_definition=1;
				int def=i;
				int firstp=i+1;

				// go thru the patch definitions following a resource definition
				// If any are wad renderings or have extended argument lists
				// or the texture being composed is tall then during MAK
				// all patches are composed into a single Bitmap which is
				// embedded in the wad as a DOOM pic. The normal definitions
				// are those that do not involve extended features.

				while(++i<N && SyntaxType(v[i].id)==ST_RESINPUT)
					if (InputType(v[i].id)==IT_WAD || v[i].nargs>2)
						normal_definition=0;
				i--;

				// mark all resource definitions with more than one patch
				// that are used in tall resource compositions non-normal

//				if (i>firstp)
//					normal_definition &= v[def].nargs<2 || atoi(v[def].args[1])<=254;

				// mark all non-texture definitions with a patch definition
 				// as non-normal as well

				if (i>=firstp && !istexture)
					normal_definition = 0;

				// Note for later use whether the resource def is normal or not

				SetNormDef(v[def].id,normal_definition);
			}
		}
	}
}

// This subroutine actually writes the MAK output wad
// memory balanced except for directory extension

int WriteControlWad(char *tname)
{
	int i,j,m,n,p,c,d,err=0;
	int offs = 0,npats,npts;
	FILE *tst,*hst,*xst;	
	WadDir dir,xdir;		
	WadHeader hdr,xhdr;
	WadTextures wtxt,*wt=&wtxt,*wtp;
	WadTextures xtxt,*xt=&xtxt;
	TextureDesc td;
	TextureLump *tlp=NULL;
	PNamesLump *pnp=NULL;
	int lno=1,jnk=1,pnameswritten=0;
	int curtexture=0;
	Category *ctgy;
	int normtext=0,exttext=0;
	Bitmap *b;
	char resfilename[256];

	AddExt(tname,".WAD");

	// If a texture extension wad is specified via /X:
	// read the WadTextures structure xtxt=*xt from it
	// This will be used to extend the texture section and detect
	// missing patches that will be supplied by the IWAD at runtime

	InitTextures(wt);
	InitTextures(xt);
	if (ExtendTextures)
	{
		InitDir(&xdir);
		AddExt(extname,".WAD");
		xst = fopen(extname,"rb");
		if (xst)
		{
			fread(&xhdr,sizeof(WadHeader),1,xst);
			ReadDir(&xhdr,&xdir,xst);
			ReadPNames(&xdir,xst,xt);
			ReadTextures(&xdir,xst,xt);
			fclose(xst);
		}
		FreeDir(&xdir);
	}

	// open the output wad for reading and writing

	tst = fopen(tname,"w+b");
	strncpy(hdr.wadtag,"PWAD",4);
	hdr.waddirlen=0;
	hdr.waddiraddr=0;
	fseek(tst,0,SEEK_SET);
	SafeWrite(&hdr,sizeof(WadHeader),1,tst);
	offs = sizeof(WadHeader);

	InitDir(&dir);

	//////////////////////////////////////////////////////////////////
	//
	// Make the output PNAMES resource
	//
	// PNAMES contains all non-duplicate references to patches 
	// by normal texture definitions in all TEXTUREn sections, 
	// plus the names of all extended texture definitions. If 
	// KeepUnusedPatches is true all the names of patches in the 
	// PATCHES section as well.
	//
	// Do this without regard to whether TEXTURES are selected
	//
	//////////////////////////////////////////////////////////////////

	c = CategoryOf("TEXTURE1",&ResDef);
	if (c<0)
		c = CategoryOf("TEXTURES",&ResDef);
	d = CategoryOf("TEXTURE2",&ResDef);
	for (m=0;m<2;m++) // Do once for c, once for d
	{
		int lastdefnormal=0;  // do not add patches not preceded by text def

		for (i=0;c>=0 && i<mtxcnt[c];i++)
		{
			VTex *v=mtxres[c]+i;

			if (SyntaxType(v->id)==ST_RESDEFN)
			{
				if (!(lastdefnormal = NormDef(v->id)) || InputType(v->id)==IT_SIGN)
					AddPName(&(wt->pnames),v->name);
			}
			else if (lastdefnormal)
				AddPName(&(wt->pnames),v->name);
		}
		c = d;
	}

	// add the patch section's names if KeepUnusedPatches. By definition
	// any new ones here are not referenced by any textures

	d = wt->pnames.npnames;
	c = CategoryOf("PATCHES",&ResDef);
	if (KeepUnusedPatches && c>=0 && mtxcnt[c])
	{
		for (i=0;i<mtxcnt[c];i++)
			AddPName(&(wt->pnames),mtxres[c][i].name);
		dprintf(logfile,"%ld PNAMES, %ld from textures, %ld from patches\n",wt->pnames.npnames,d,wt->pnames.npnames-d);
	}
	else
		dprintf(logfile,"%ld PNAMES referenced by textures\n",wt->pnames.npnames);

	////////////////////////////////////////////////////////////////////
	//
	// Armed with a PNames resource, build the output TEXTUREn resources
	//
	////////////////////////////////////////////////////////////////////

	td.npatches = 0;
	td.patches = NULL;
	for (c=0;c<ResDef.ncats;c++)
	{
		if (DoTextures || DoPatches)	// patches require PNAMES from TEXTURES
		{
			if (mtxcnt[c]>0 && strnicmp(ResDef.cat[c].secname,"TEXTURE",7)==0) // TEXTUREn
			{
				tlp = GetTextureLump(c,wt,&lno);
				for (i=0;;i++)				  // iterate thru mtxres[c]
				{
					//
					// Handle a texture definition
					//
					if (i==mtxcnt[c] || SyntaxType(mtxres[c][i].id)==ST_RESDEFN)
					{
						// Add the last texture definition compiled
						// before starting a new one

						if (i) // the first definition hasn't been compiled yet
						{
							// never add duplicate textures
							if (FindTextureN(wt,td.textname,&jnk)==-1)
							{
								// if the definition has a sign add it as the last
								// patch with 0 offsets

								if (mtxres[c][curtexture].sign && mtxres[c][curtexture].sign[0])
								{
									if (NormDef(mtxres[c][curtexture].id))
										CreateSignPatch(c,curtexture,td.twid);
									td.patches = realloc(td.patches,(1+td.npatches)*sizeof(Patch));
									td.patches[td.npatches].xoff = 0;
									td.patches[td.npatches].yoff = 0;
									td.patches[td.npatches].pidx = GetPIdx(&wt->pnames,mtxres[c][curtexture].name);
									td.patches[td.npatches].cmap = 0;
									td.patches[td.npatches].stepdir = 1;
									td.npatches++;
								}

								if (NormDef(mtxres[c][curtexture].id))
								{
									AddTexture(tlp,&td,1);
									npts += td.npatches;
									normtext++;
								}
								else // if a composition just reference a single patch
								{
									td.npatches=1;
									td.patches=realloc(td.patches,sizeof(Patch));
									td.patches[0].pidx = GetPIdx(&wt->pnames,mtxres[c][curtexture].name);
									td.patches[0].xoff = 0;
									td.patches[0].yoff = 0;
									td.patches[0].cmap = 0;
									td.patches[0].stepdir = 1;
									AddTexture(tlp,&td,1);
									npts++;
									exttext++;

									// compose the patches
									b = GenerateBitmap(c,curtexture);
									ResourceFilename(resfilename,c,curtexture);
									AddExt(resfilename,".BMP");
									WriteBMP(b,resfilename,palette);
									dprintf(logfile,"Info: Creating composed texture %s, %d X %d\n",mtxres[c][curtexture].name,b->realw,b->logh);
								}
							}
							else
								dprintf(logfile,"ERROR: Duplicate texture definition for %s, first used\n",td.textname);

							// exit the for loop after adding the last texture definition

							if (i==mtxcnt[c])
								break;		 
						}

						// Start a new texture definition

						curtexture=i; // note the number so patches know what they define

						strncpy(td.textname,mtxres[c][i].name,8);
						td.res1 = td.res2 = td.res3 = td.res4 = 0;
						td.twid = atoi(mtxres[c][i].args[0]);
						td.thgt = atoi(mtxres[c][i].args[1]);
						free(td.patches);
						td.npatches = 0;
						td.patches = NULL;

						// add each implied patch name to the [PATCHES] section
						// if it defines a normal non-composed texture

						p = CategoryOf("PATCHES",&ResDef);
						if (p>=0)
						{
							if
							(
								!NormDef(mtxres[c][curtexture].id) ||
								InputType(mtxres[c][curtexture].id)==IT_SIGN
							)
							{
								int k = mtxres[c][i].id;
								SetResourceType(k,p); // change res type
								SetInputType(k,IT_BMP);
								SetNormDef(k,1);
								AddNewVTex(&mtxres[p],&mtxcnt[p],k,mtxres[c][i].name,mtxres[c][i].path,NULL,0,NULL);
							}
						}
					}
					//
					// Handle a patch definition for a texture
					//
					else if (SyntaxType(mtxres[c][i].id)==ST_RESINPUT)
					{
						// add the patch data

						td.patches = realloc(td.patches,(td.npatches+1)*sizeof(Patch));
						td.patches[td.npatches].xoff = mtxres[c][i].nargs>0? atoi(mtxres[c][i].args[0]) : 0;
						td.patches[td.npatches].yoff = mtxres[c][i].nargs>1? atoi(mtxres[c][i].args[1]) : 0;
						td.patches[td.npatches].pidx = GetPIdx(&wt->pnames,mtxres[c][i].name);
						td.patches[td.npatches].cmap = 0;
						td.patches[td.npatches].stepdir = 1;
						td.npatches++;

						// if it has a sign, create the BMP

						if (mtxres[c][i].sign && mtxres[c][i].sign[0])
							CreateSignPatch(c,i,td.twid);

						// add each implied patch name to the [PATCHES] section
						// if it defines a normal non-composed texture

						p = CategoryOf("PATCHES",&ResDef);
						if (p>=0)
						{
							if (NormDef(mtxres[c][curtexture].id))
							{
								int k = mtxres[c][i].id;
								SetResourceType(k,p); // change res type
								SetInputType(k,IT_BMP);
								SetNormDef(k,1);
								k=SetSyntaxType(k,ST_RESDEFN);
								AddNewVTex(&mtxres[p],&mtxcnt[p],k,mtxres[c][i].name,mtxres[c][i].path,NULL,0,NULL);
							}
						}
					}
				}
				free(td.patches);
				td.patches=NULL;
				td.npatches=0;
			}
		}
	}
	dprintf(logfile,"%ld normal, %ld extended textures built\n",normtext,exttext);

	// For each category in the Resource Definition, write the
	// resources present in the MAK file to the output wad.

	if (DoNonStandard && (c=CategoryOf("HISTORY",&ResDef))>=0)
	{
		mkdir("HISTORY",0777);
		hst = fopen("HISTORY\\HISTORY.TXT","w");
		if (hst)
		{
			fprintf(hst,"MAK %s [%s]%s",cmdline,GetDateTime(),LINETERM);
			fclose(hst);
		}
		else dprintf(logfile,"ERROR: Cannote create HISTORY file\n");
		AddVTex
		(
			&mtxres[c],
			&mtxcnt[c],
			ST_RESDEFN<<SyntaxTypeShift,
			"HISTORY",
			NULL,
			NULL,
			0,
			NULL
		);
	}

	dprintf(logfile,"\nCreating %s from resource file %s\n",tname,pwadname);
	for (c=0;c<ResDef.ncats;c++)
	{
		ctgy = ResDef.cat+c;
		m=dir.nentries;
		if (strnicmp("TEXTURE",ctgy->secname,7)==0)
		{
			if (DoTextures)
			{
				if (ExtendTextures)
				{
					MergePNames(xt,wt,&owt);
					MergeTextures(xt,wt,&owt);
					wtp = &owt;
					tlp = GetTextureLump(c,&owt,&lno);
					pnp = &owt.pnames;
				}
				else
				{
					wtp = wt;
					tlp = GetTextureLump(c,wt,&lno);
					pnp = &wt->pnames;
				}

				// if textures exist in the output WadTextures,
				// write the TEXTUREn resource
				if (tlp->N)
				{
					SafeWrite(&tlp->N,sizeof(int),1,tst);
					SafeWrite(tlp->offs,sizeof(int),tlp->N,tst);
					for (j=0,npats=0;j<tlp->N;j++)
					{
						SafeWrite(tlp->textures+j,sizeof(TextureDesc)-sizeof(Patch *),1,tst);
						SafeWrite(tlp->textures[j].patches,sizeof(Patch),tlp->textures[j].npatches,tst);
						npats += tlp->textures[j].npatches;
					}
					n = sizeof(int) + sizeof(int) * tlp->N
						+ (sizeof(TextureDesc)-sizeof(Patch *)) * tlp->N
						+ npats * sizeof(Patch);
					AddDirEntry(&dir,offs,n,stringf("TEXTURE%d",lno));
					Advance(&offs,n,tst);
					dprintf(logfile,"Info: Inserted %5ld %-8s\n",tlp->N,stringf("TEXTURE%d",lno));
				}

				// if pnames exist write the PNAMES too
				if (!pnameswritten && pnp->npnames)
				{
					SafeWrite(&pnp->npnames,sizeof(int),1,tst);
					SafeWrite(pnp->patchname[0],8,pnp->npnames,tst);
					n = sizeof(int) + 8 * pnp->npnames;
					AddDirEntry(&dir,offs,n,"PNAMES");
					Advance(&offs,n,tst);
					dprintf(logfile,"Info: Inserted %5ld PNAMES\n",pnp->npnames);
					pnameswritten=1;
				}
			}
		}
		else if (stricmp(ctgy->secname,"HISTORY")==0)
		{
			if (IsSectionEnabled(ctgy->secname))
			{
				WriteResource(c,0,&offs,&dir,tst,xt);
				fprintf(stderr,"Inserted %5d %-8s: %-8s\r",1,ctgy->secname,"HISTORY");
				dprintf(logfile,"Info: Inserted %5d %-8s  %s\n",dir.nentries-m,ctgy->secname,"        ");
			}
		}
		else
		{
			if (mtxcnt[c]>0 && IsSectionEnabled(ctgy->secname))
			{
				if (ctgy->idm==ID_MARKED && ctgy->mark)
				{
					if (MakeIwad || MakeRwad || SingleMarker)
						AddDirEntry(&dir,offs,0,stringf("%c_START",ctgy->mark));
					else
						AddDirEntry(&dir,offs,0,stringf("%c%c_START",ctgy->mark,ctgy->mark));
					if (MakeRwad)
						AddDirEntry(&dir,offs,0,stringf("%c1_START",ctgy->mark));
				}
				m = dir.nentries;
				for (i=0;i<mtxcnt[c];i++)
				{
					VTex *v=mtxres[c]+i;

					if (SyntaxType(v->id)==ST_RESDEFN)
					{
						if (!NormDef(v->id))
						{
							Bitmap *b = GenerateBitmap(c,i);
							ResourceFilename(resfilename,c,i);
							AddExt(resfilename,".BMP");
							WriteBMP(b,resfilename,palette);
							dprintf(logfile,"Info: Creating composite resource %s, %d X %d\n",v->name,b->realw,b->logh);

							if (KeepUnusedPatches || stricmp(ctgy->secname,"PATCHES") || UsedByTexture(v->name))
								WriteResource(c,i,&offs,&dir,tst,xt);
						}
						else if (KeepUnusedPatches || stricmp(ctgy->secname,"PATCHES") || UsedByTexture(v->name))
						{
							if (InputType(v->id)==IT_SIGN)
								CreateSignPatch(c,i,0);
							WriteResource(c,i,&offs,&dir,tst,xt);
						}
					}
					if (dir.nentries>m)
						if (stricmp(ctgy->secname,"LEVELS")==0)
							fprintf(stderr,"Inserted %5d %-8s: %-8s\r",(dir.nentries-m)/(NLevObj()+1),ctgy->secname,mtxres[c][i].name);
						else
							fprintf(stderr,"Inserted %5d %-8s: %-8s\r",dir.nentries-m,ctgy->secname,mtxres[c][i].name);
				}
				if (dir.nentries>m)
				{
					if (stricmp(ctgy->secname,"LEVELS")==0)
						dprintf(logfile,"Info: Inserted %5d %-8s  %-8s\n",(dir.nentries-m)/(NLevObj()+1),ctgy->secname,"        ");
					else
						dprintf(logfile,"Info: Inserted %5d %-8s  %s\n",dir.nentries-m,ctgy->secname,"        ");
				}
				if (ctgy->idm==ID_MARKED && ctgy->mark)
				{
					if (MakeRwad)
					{
						AddDirEntry(&dir,offs,0,stringf("%c1_END",ctgy->mark));
						AddDirEntry(&dir,offs,0,stringf("%c2_START",ctgy->mark));
						AddDirEntry(&dir,offs,0,stringf("%c2_END",ctgy->mark));
						AddDirEntry(&dir,offs,0,stringf("%c3_START",ctgy->mark));
						AddDirEntry(&dir,offs,0,stringf("%c3_END",ctgy->mark));
					}
					if (MakeIwad || MakeRwad || SingleMarker)
						AddDirEntry(&dir,offs,0,stringf("%c_END",ctgy->mark));
					else
						AddDirEntry(&dir,offs,0,stringf("%c%c_END",ctgy->mark,ctgy->mark));
				}
			}
		}
	}
	fprintf(stderr,"                                                 \n");
	err=0;
exitwrite:
	// write the output wad's directory
	SafeWrite(dir.dirp,sizeof(WadDirEntry),dir.nentries,tst);
	dprintf(logfile,"Info: Inserted %5d Directory Entries at %XH\n",dir.nentries,offs);

	// finally rewrite the output wad's header and close
	fseek(tst,0,SEEK_SET);
	if (MakeIwad) strncpy(hdr.wadtag,"IWAD",4);
	hdr.waddirlen = dir.nentries;
	hdr.waddiraddr = offs;
	SafeWrite(&hdr,sizeof(WadHeader),1,tst);
	fclose(tst);
	if (err) remove(tname);
	dprintf(logfile,"Rewrote Directory Header, file closed\n");
	FreeDir(&dir);
	FreeTextures(wt);
	if (ExtendTextures) FreeTextures(xt);
	return err;

	err=3;
	goto exitwrite;
}

// This routine implements MAK
// Read a control file, and create a wad containing the resources in it

#define INCLUDELEVELS 16

int ReadResourceControl(char *ctlname,char *wadname)
{
	static char buffer[4096];
	char *p,*q;
	int redir=0;
	static char secname[INCLUDELEVELS][128];
	FILE *st[INCLUDELEVELS];
	int lineno[INCLUDELEVELS];
	int i;
	int nlines=0;
	VTex info;
	int currentsection=-1;

	MaxParseError = NoError;
	InitVTex();
	info.nargs=0;
	info.args=NULL;
	info.path=info.sign=info.name=NULL;
	info.id=0;

	// Open the base control file, st[0], note redirection level = 0 

	redir = 0;
	lineno[redir] = 0;
	strcpy(secname[redir],"");

	OpenSgnRes(st+redir,ctlname);

	// If open successful parse the file, else report not found

	if (st[redir]!=NULL)
	{
		// outer loop always starts reading a section header line

		while (redir>=0)
		{
			// get a line
			p = fgets(buffer,4096,st[redir]);
			nlines++;
			q = strchr(buffer,'\n');
			if (q) *q=0;

			// On end of file, close current file
			// Decrement redirection level, if -1, done

			if (p==NULL)
			{
				fclose (st[redir--]);	// close file, redir to last
				if (redir>=0) continue; else break;	// restart loop or exit
			}
			lineno[redir]++;

			// Parse the line read

			FreeDynStr(&info.args,&info.nargs);
			SetResourceType(info.id,currentsection);
		 	ParseMake(p,lineno[redir],&info);

			if (0|Verbose)
			{
				char ST[9]=";.[DIRRR", IT[4][4]={"---","BMP","WAD","SGN"};
				int i;

				printf("%2d %4d %-8s %1c %-8s %3s %1d %1d %1d",
					     redir,
					     lineno[redir],
					     info.name,
							 ST[SyntaxType(info.id)],
							 ResDef.cat[ResourceType(info.id)].secname,
							 IT[InputType(info.id)],
							 MultPost(info.id),
							 CondState(info.id),
							 NormDef(info.id));

				printf(" %2d args:",info.nargs);
				for (i=0;i<info.nargs;i++)
					printf(" %s",info.args[i]);
				printf("\n");
				printf("path: %s\n",info.path? info.path : "None");
				printf("sign: %s\n",info.sign? info.sign : "None");
			}

			// Deal with type of line read

			switch(SyntaxType(info.id))
			{
				case ST_COMMENT:
					break;
				case ST_SECTION:
					if (CondState(info.id))
					{
						strcpy(secname[redir],info.sign);	// get new section name in sign
						i = CategoryOf(info.sign,&ResDef);	// get category number of sec
						currentsection = i;

						if (info.path && *info.path)			// if a path specified
						{
							if (strnicmp(secname[redir],"TEXTURE",7)==0) // if texture section
							{
								int j = CategoryOf("PATCHES",&ResDef);		 // set default dir
								if (j>=0)																		  // for patches
									strcpy(ResDef.cat[j].defaultdir,info.path);
							}
							else																				 // set default dir
								strcpy(ResDef.cat[i].defaultdir,info.path);	// for section
						}
					}
					break;
				case ST_DIRECTIVE:
					switch (ResourceType(info.id))
					{
						case RK_INCLUDE:
						case RK_INCLUD2:
							if (info.path && *info.path && CondState(info.id))
							{
								if (redir>=INCLUDELEVELS-1)
								{
									dprintf(logfile,"Fatal: Include levels too deep in line #%d\n",lineno[redir]);
									errorlev=3;
									FreeParser();
									return 1;
								}
								redir++;		// redirect to next file
								OpenSgnRes(st+redir,info.path);
								st[redir] = fopen(info.path,"r");	// attempt to open
								if (!st[redir])											// failure is fatal
								{
									dprintf(logfile,"Fatal: Include file not found in line #%d\n",lineno[redir-1]);
									errorlev=3;
									FreeParser();
									return 1;
								}
								strcpy(secname[redir],secname[redir-1]);
								lineno[redir]=0;
								dprintf(logfile,"Info: Reading from %s\n",info.path);
							}
							break;
						default:
							break;
					}
					break;
				case ST_RESDEFN:
				case ST_RESINPUT:
					if (CondState(info.id))
					{
						for (i=0;i<ResDef.ncats;i++)
						{
							if (stricmp(secname[redir],ResDef.cat[i].secname)==0)
							{
								if (strnicmp(secname[redir],"TEXTURE",7) && SyntaxType(info.id)==ST_RESDEFN)
								{
									if (!AddNewVT(&mtxres[i],&mtxcnt[i],&info))
										dprintf(logfile,"ERROR: Duplicate %s definition for %s, first used\n",ResDef.cat[i].secname,info.name);
								}
								else
									AddVT(&mtxres[i],&mtxcnt[i],&info);
								break;
							}
						}

						if (i==ResDef.ncats)
						{
							dprintf(logfile,"Fatal: %s has section %s not in resource file: %s, line: %d, redir: %d\n",ctlname,secname[redir],defname,lineno[redir], redir);
							errorlev=3;
							FreeParser();
							return 1;
						}
					}
					break;
				default:
					break;
			}
		}
	}
	else
	{
		dprintf(logfile,"Fatal: Cannot open input file: %s\n",ctlname);
		errorlev=3;
		FreeParser();
		return 1;
	}

	if (MaxParseError >= PathOverflow)
	{
		dprintf(logfile,"Errors in input file, aborting MAK\n");
		errorlev=3;
		FreeParser();
		return 1;
	}
	else
		dprintf(logfile,"Control lines read sucessfully: %d\n",nlines);

	FreeParser();

	// Mark the resource definitions as normal that don't require
	// us to compose patches to create the wad resource.

	ScanResourceLists();

	return 0;
}

