#include "patch.h"

//============================================================================
// This program opens two input wads and replaces the common resources
// with those in the second, adding the new ones, keeping all those
// only in the first wad, writing the output to a third wad.
//
// The process proceeds one resource type at a time in order to
// minimize memory allocation, and to properly group resources in the
// output wad. 
//============================================================================

int OpenStreams(void);

void FreeMerStructures()
{
	FreeVTex();
	FreePalette();
	FreeTextures(&owt);
	FreeTextures(&pwt);
	FreeTextures(&bwt);
	FreeDir(&bdir);
	FreeDir(&pdir);
	FreeDir(&odir);
	FreeResourceDefinition(&ResDef);
	FreeResourceDefinition(&pResDef);
	FreeResourceDefinition(&bResDef);
	CloseStreams();
}

// Open the files, merge their textures and patchnames,
// then write the output wad

int mermain(int argc,char **argv)
{
	int i,j,n;
	int err,cleared=0;

	dprintf(logfile,"\n");

	CopyEnvironmentCLArgs("MER",argc,argv);
	if (ParseCommandLine(-1,3))
	{
		FreeCLStrs();
		return 3;
	}

	if (nclfiles!=3)
	{
		dprintf(logfile,"\nMER.EXE %s by Jim Flynn, Copyright 1996-7 -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"Merge base and patch wads to form output wad with resources of both\n");
		dprintf(logfile,"\nUsage: MER [options] basefile[.res|.wad] patchfile[.res|.wad] outfile[.wad]\n");
		dprintf(logfile,"\n -Basefile is the wad to which resources are added            all three\n");	   
		dprintf(logfile," -Patchfile is the wad containing changed and new resources    must be\n");
		dprintf(logfile," -Outfile is the wad written with the resources of both       different!\n");
		dprintf(logfile,"\n Resource selection option:\n");
		dprintf(logfile," +XYZ combines only XYZ resources, -XYZ combines all BUT XYZ resources.\n");
		dprintf(logfile," L LUMPS      W WADS       T TEXTURES  U USER DEFINED\n");
		dprintf(logfile," E EFFECTS    M MUSICS     G GRAPHICS\n");
		dprintf(logfile," S SPRITES    P PATCHES    F FLATS\n");
		dprintf(logfile,"\n General options, use SET MER=options to set defaults:\n");
		dprintf(logfile," /1 Convert FF_END to F_END, etc.   /2 Convert F_END to FF_END, etc.\n");
		dprintf(logfile," /N New only - No replacements      /X extend (+TSF /N /1)\n");
		dprintf(logfile," /I create IWAD                     /I:wad use wad's palette\n");
		dprintf(logfile,"\n Game definition options, use SET MER=options to set defaults:\n");
		dprintf(logfile," /D:resdef[.DEF] specify resources\n");
		dprintf(logfile," /DOOM2 /DOOM /HERETIC /HEXEN specify game\n");
		dprintf(logfile,"\n See MER.TXT for details.\n");
						  
		FreeCLStrs();
		return(1);
	}

	InitDoomFont();

	strcpy(bwadname,clfiles[0]);			// recover the filenames
	strcpy(pwadname,clfiles[1]);
	strcpy(owadname,clfiles[2]);

	cleared=0;
	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfLWTEMGSPF");
		if (n==strlen(cloptions[i]))
		{
			if (!cleared)	// if first selection option
			{
				if (cloptsense[i])	// clear all if it adds types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=0;
					DoGraphics=DoSprites=DoPatches=DoFlats=0;
				}
				else if (!cloptsense[i]) // set all if it subtracts types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=1;
					DoGraphics=DoSprites=DoPatches=DoFlats=1;
				}
				cleared=1;	// but don't do it for following selection opts
			}
			for (j=0;j<strlen(cloptions[i]);j++)
			{
				switch(tolower(cloptions[i][j]))
				{
					case 'l':
						DoLumps = cloptsense[i];
						break;
					case 'w':
						DoWads = cloptsense[i];
						break;
					case 't':
						DoTextures = cloptsense[i];
						break;
					case 'e':
						DoEffects = cloptsense[i];
						break;
					case 'm':
						DoMusics = cloptsense[i];
						break;
					case 'g':
						DoGraphics = cloptsense[i];
						break;
					case 's':
						DoSprites = cloptsense[i];
						break;
					case 'p':
						DoPatches = cloptsense[i];
						break;
					case 'f':
						DoFlats = cloptsense[i];
						break;
					case 'u':
						DoNonStandard = cloptsense[i];
						break;
				}
			}
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
			strcpy(defname,cloptions[i]);
		else if (tolower(cloptions[i][0])=='v')
			Verbose = 1;
		else if (tolower(cloptions[i][0])=='i')
		{
			if (cloptions[i][1]==':')
				strcpy(iwadname,cloptions[i]+2);
			else
			{
				MakeIwad = 1;
				SingleMarker = 1;
				DoubleMarker = 0;
			}
		}
		else if (tolower(cloptions[i][0])=='n')
			NewOnly = 1;
		else if (tolower(cloptions[i][0])=='x')
		{
			NewOnly = 1;
			SingleMarker = 1;
			DoubleMarker = 0;
			DoLumps=DoWads=DoEffects=DoMusics=DoGraphics=DoPatches=0;
			DoTextures=DoSprites=DoFlats=1;
		}
		else if (cloptions[i][0]=='1')
		{
			SingleMarker = cloptsense[i];
			DoubleMarker = !cloptsense[i];
		}
 		else if (cloptions[i][0]=='2')
		{
			DoubleMarker = cloptsense[i];
			SingleMarker = !cloptsense[i];
		}
		else if (tolower(cloptions[i][0])=='d' && cloptions[i][1]==':')
			strcpy(defname,cloptions[i]+2);
		else dprintf(logfile,"WARNING: command line option: %s not understood\n",cloptions[i]);
	}
	FreeCLStrs();

	AddExt(defname,".DEF");
	FreeResourceDefinition(&ResDef);
	InitCategories(defname,&ResDef);

	palette=GetPalette(NULL,NULL,-1);
	if (*iwadname)						// then the IWAD, if spec'd
	{
		FILE *ist;
		WadHeader ihdr;
		WadDir idir;

		InitDir(&idir);
		ist = fopen(iwadname,"rb");
		if (ist!=NULL)
		{
			dprintf(logfile,"Using PLAYPAL from IWAD %s for palette\n",iwadname);

			fread(&ihdr,sizeof(WadHeader),1,ist);
			ReadDir(&ihdr,&idir,ist);

			n = IsIn("PLAYPAL",&idir);
			if (n>=0) palette = GetPalette(&idir,ist,n);

			fclose(ist);
			free(idir.dirp);
		}
	}

    if ((err=OpenStreams())!=NOERROR)	// open the files
    {
        switch(err)
        {
            case BASEWONTOPEN:
                dprintf(logfile,"Base wad: %s won't open\n",bwadname);
				dprintf(logfile,"\n");
				//FreeMerStructures();
                return(3);
            case PATCHWONTOPEN:
                dprintf(logfile,"Patch wad: %s won't open\n",pwadname);
				dprintf(logfile,"\n");
				//FreeMerStructures();
                return(3);
            case OUTWADWONTOPEN:
                dprintf(logfile,"Output wad: %s won't open\n",owadname);
				dprintf(logfile,"\n");
				//FreeMerStructures();
                return(3);
            case ERRBASEDIR:
                dprintf(logfile,"Error reading base wad:%s directory\n",bwadname);
				dprintf(logfile,"\n");
				//FreeMerStructures();
                return(3);
            case ERRPATCHDIR:
                dprintf(logfile,"Error reading patch wad:%s directory\n",pwadname);
				dprintf(logfile,"\n");
				//FreeMerStructures();
                return(3);
			default:
                dprintf(logfile,"Unknown error in opening streams for MER\n");
				dprintf(logfile,"\n");
				//FreeMerStructures();
                return(3);
        }
    }

	LocateCategories(&bdir,&ResDef,&bResDef);
	LocateCategories(&pdir,&ResDef,&pResDef);

	dprintf(logfile,"\nPatching %s with %s to %s\n",bwadname,pwadname,owadname);

	ReadPNames(&bdir,bst,&bwt);					// read base PNAMES TEXTURE1
	ReadTextures(&bdir,bst,&bwt);

	ReadPNames(&pdir,pst,&pwt);					// read patch PNAMES TEXTURE1
	ReadTextures(&pdir,pst,&pwt);

	MergePNames(&bwt,&pwt,&owt);				// merge the PNAMES
	MergeTextures(&bwt,&pwt,&owt);				// merge the TEXTURE1's
    if (Alphabetize)
        SortTextures(&odir,ost,&owt);

	if ((err=WriteOutputWad()))					// write base with additions
		return err;

	dprintf(logfile,"\n");
	//FreeMerStructures();
	return errorlev;
}

// Open base, patch, and output wads

int OpenStreams(void)
{
    char *p,*q;

    // force .WAD extension on all files passed
	// if patchwad has a .RES extension, convert control file to .WAD first

	q=strrchr(bwadname,'\\');
	p=strrchr(bwadname,'.');
    if (p && (!q || q<p))
		if (strnicmp(p+1,"RES",3)==0 || strnicmp(p+1,"SGN",3)==0)
		{
			ReadResourceControl(bwadname,bwadname);
			if (errorlev<=1)
			{
				AddExt(bwadname,".WAD");
				WriteControlWad(bwadname);
				dprintf(logfile,"Control file converted to wad: %s\n",bwadname);
			}
			else return errorlev;
		}

	q=strrchr(pwadname,'\\');
	p=strrchr(pwadname,'.');
    if (p && (!q || q<p))
		if (strnicmp(p+1,"RES",3)==0 || strnicmp(p+1,"SGN",3)==0)
		{
			ReadResourceControl(pwadname,pwadname);
			if (errorlev<=1)
			{
				AddExt(pwadname,".WAD");
				WriteControlWad(pwadname);
				dprintf(logfile,"Control file converted to wad: %s\n",pwadname);
			}
			else return errorlev;
		}

	AddExt(bwadname,".WAD");
	AddExt(pwadname,".WAD");
	AddExt(owadname,".WAD");

    // try to open the wad files
    bst = fopen(bwadname,"rb");
    if (bst==NULL)
    {
        pst=NULL;
        ost=NULL;
        return BASEWONTOPEN;
    }
	fread(&bhdr,sizeof(WadHeader),1,bst);
	if (strncmp(bhdr.wadtag,"IWAD",4)==0 || strncmp(bhdr.wadtag,"PWAD",4)==0)
	{
		if (ReadDir(&bhdr,&bdir,bst))
		{
			fclose(bst);
			bst=pst=ost=NULL;
			return ERRBASEDIR;
		}
	}

    pst = fopen(pwadname,"rb");
    if (pst==NULL)
    {
        fclose(bst);
        bst=NULL;
        ost=NULL;
        return PATCHWONTOPEN;
    }
	fread(&phdr,sizeof(WadHeader),1,pst);
	if (strncmp(phdr.wadtag,"IWAD",4)==0 || strncmp(phdr.wadtag,"PWAD",4)==0)
	{
		if (ReadDir(&phdr,&pdir,pst))
		{
			fclose(pst);
			fclose(bst);
			bst=pst=ost=NULL;
			return ERRBASEDIR;
		}
	}

    ost = fopen(owadname,"wb");
    if (ost==NULL)
    {
        fclose(bst);
        bst=NULL;
        fclose(pst);
        pst=NULL;
        return OUTWADWONTOPEN;
    }
    // all files opened correctly
    return NOERROR;
}


