#include "patch.h"

/************************************************************
 MERSUBS.C
			 /-------------------------\
			||     MER subroutines     ||
			 \-------------------------/
									
************************************************************/

void WriteMerHistory(int bidx,int pidx,int *offset)
{
	StrIdxAry *sa=NULL;
	int num=0;
	char *buffer = NULL;
	char line[256],*tok;
	int i,n=0,o=0;

	ListStrIdx(&sa,&num,stringf("MER %s [%s]%s",cmdline,GetDateTime(),LINETERM));
	if (bidx>=0)
	{
		o = bdir.dirp[bidx].resptr;
		n = bdir.dirp[bidx].reslen;
		if (n>=0)
		{
			buffer = (char *)realloc(buffer,n+1);
			fseek(bst,o,SEEK_SET);
			fread(buffer,n,1,bst);
			buffer[n]='\0';

			tok = strtok(buffer,"\n\r");
			while (tok)
			{
				strcpy(line,"    ");
				strcat(line,tok);
				strcat(line,LINETERM);
				ListStrIdx(&sa,&num,line);
				tok = strtok(NULL,"\n\r");
			}
		}
	}
	else ListStrIdx(&sa,&num,stringf("    %s: No History%s",bwadname,LINETERM));

	if (pidx>=0)
	{
		o = pdir.dirp[pidx].resptr;
		n = pdir.dirp[pidx].reslen;
		if (n>=0)
		{
			buffer = (char *)realloc(buffer,n+1);
			fseek(pst,o,SEEK_SET);
			fread(buffer,n,1,pst);
			buffer[n]='\0';

			tok = strtok(buffer,"\n\r");
			while (tok)
			{
				strcpy(line,"    ");
				strcat(line,tok);
				strcat(line,LINETERM);
				ListStrIdx(&sa,&num,line);
				tok = strtok(NULL,"\n\r");
			}
		}
	}
	else ListStrIdx(&sa,&num,stringf("    %s: No History%s",pwadname,LINETERM));

	fseek(ost,*offset,SEEK_SET);
	n = 0;
	for (i=0;i<num;i++)
	{
		fwrite(sa[i].str,strlen(sa[i].str),1,ost);
		n+= strlen(sa[i].str);
	}
	AddDirEntry(&odir,*offset,n,"HISTORY");
	*offset += n;

	for (i=0;i<num;i++)
		free(sa[i].str);
	free(sa);
	free(buffer);
}

// Transfer the idx-th directory entry of a wad to the output wad

void WriteEntry(WadDir *dir,FILE *st,int idx,int *offset)
{
	int o,n,offs=*offset;
	char *buffer=NULL,resn[12];

	o = dir->dirp[idx].resptr;
	n = dir->dirp[idx].reslen;
	strcpy(resn,Str8(dir->dirp[idx].resname));

	if (IsMarker(dir->dirp+idx))
		return;		// only map name tags are passed by this routine

    if (n>0)
    {
		buffer = (char *)realloc(buffer,n);
		fseek(st,o,SEEK_SET);
		fread(buffer,n,1,st);
		SafeWrite(buffer,n,1,ost);
		free(buffer);
    }
	AddDirEntry(&odir,offs,n,resn);
	offs += n;

	*offset = offs;
}

// Transfer a resource from patch wad or base wad to ouptput as appropriate

void WriteSimpleResource(char *resn,int *offset,int bidx,int pidx,int doit)
{
	int offs=*offset;

	if (NewOnly || !doit)
	{
		if (bidx>=0)
			WriteEntry(&bdir,bst,bidx,&offs);
		else if (doit && pidx>=0)
			WriteEntry(&pdir,pst,pidx,&offs);
	}
	else
	{
		if (doit && pidx>=0)
			WriteEntry(&pdir,pst,pidx,&offs);
		else if (bidx>=0)
			WriteEntry(&bdir,bst,bidx,&offs);
	}
	*offset = offs;
}

void AddEndN(int kind,int offs,int N)
{
    char marker[16];

	if (DoubleMarker)
	{
		marker[0] = kind;
		marker[1] = kind;
		strcpy(marker+2,"_END");
	}
	else
	{
		*marker = kind;
		if (!N)
			strcpy(marker+1,"_END");
		else
		{
			marker[1] = '0'+N;
			strcpy(marker+2,"_END");
		}
	}
	AddDirEntry(&odir,offs,0,marker);
}

void AddStartN(int kind,int offs,int N)
{
    char marker[16];

	if (DoubleMarker)
	{
		marker[0] = kind;
		marker[1] = kind;
		strcpy(marker+2,"_START");
	}
	else
	{
		*marker = kind;
		if (!N)
			strcpy(marker+1,"_START");
		else
		{
			marker[1] = '0'+N;
			strcpy(marker+2,"_START");
		}
	}
	AddDirEntry(&odir,offs,0,marker);
}

// The strategy for the output wad is to read each resource from the base
// wad and if it is not PNAMES, TEXTUREn, or a patch resource to write it
// to the output wad, and update the output directory. Memory is allocated
// and freed to read each resource to minimize requirements.
//
// If it is PNAMES or TEXTUREn, then the merged version is written to the
// output wad instead, and the output directory updated accordingly.
//
// If it is a patch, then if it is a patch in the patch wad's PNAMES the
// patchwad version is written, else the basewad version is used. In either
// case the offset and length is entered in the output directory.
// 
// After writing the last patch, and before writing the end patches marker
// all patches in the patch wad that do not occur in the base wad are appended
// with appropriate directory updates. Then the end patch marker is written
// and the file closed.

int WriteOutputWad()
{
	int i,j,k,m,n,p,c;
	int offs = 12,npats;
	Category *ctg1,*ctg2;
	int doonce=1;
	int marked=0;

	strncpy(ohdr.wadtag,"PWAD",4);
	ohdr.waddirlen=0;
	ohdr.waddiraddr=0;
	fseek(ost,0,SEEK_SET);
	SafeWrite(&ohdr,sizeof(WadHeader),1,ost);

	for (c=0;c<ResDef.ncats;c++)
	{
		m = odir.nentries;
		ctg1 = bResDef.cat+c;
		ctg2 = pResDef.cat+c;

		if (stricmp(ctg1->secname,"HISTORY")==0)
		{
			WriteMerHistory(ctg1->Names[0].didx,ctg2->Names[0].didx,&offs);
		}
		else if (ctg1->idm==ID_INFO || ctg1->idm==ID_NONE)
			continue;
		else if (stricmp(ctg1->secname,"LEVELS")==0)
		{
			n = 1+NLevObj();
			if (DoWads)
			{
				m = odir.nentries;
				if (NewOnly)
				{
					for (i=0;i<ctg1->nnames;i++)
						if (ctg1->Names[i].didx>=0)
							for (k=0;k<n;k++)
								WriteEntry(&bdir,bst,k+ctg1->Names[i].didx,&offs);

					for (i=0;i<ctg2->nnames;i++)
					{
						if (ctg1->Names[i].didx<0)
							if (ctg2->Names[i].didx>=0)
								for (k=0;k<n;k++)
									WriteEntry(&pdir,pst,k+ctg2->Names[i].didx,&offs);

						if (odir.nentries>m)
							fprintf(stderr,"Merged %5d LEVELS  : MAP%02d   \r",(odir.nentries-m)/n,i);
					}
					if (odir.nentries>m)
						dprintf(logfile,"Info: Merged %5d LEVELS               \n",(odir.nentries-m)/n);
				}
				else
				{
					for (i=0;i<ctg2->nnames;i++)
						if (ctg2->Names[i].didx>=0)
							for (k=0;k<n;k++)
								WriteEntry(&pdir,pst,k+ctg2->Names[i].didx,&offs);

					for (i=0;i<ctg1->nnames;i++)
					{
						if (ctg2->Names[i].didx<0)
							if (ctg1->Names[i].didx>=0)
								for (k=0;k<n;k++)
									WriteEntry(&bdir,bst,k+ctg1->Names[i].didx,&offs);

						if (odir.nentries>m)
							fprintf(stderr,"Merged %5d LEVELS  : MAP%02d   \r",(odir.nentries-m)/n,i);
					}
					if (odir.nentries>m)
						dprintf(logfile,"Info: Merged %5d LEVELS               \n",(odir.nentries-m)/n);
				}
			}
			else
			{
				for (i=0;i<ctg1->nnames;i++)
				{
					if (ctg1->Names[i].didx>=0)
						for (k=0;k<n;k++)
							WriteEntry(&bdir,bst,k+ctg1->Names[i].didx,&offs);	

					if (odir.nentries>m)
						fprintf(stderr,"Merged %5d LEVELS  : MAP%02d   \r",(odir.nentries-m)/n,i);
				}
				if (odir.nentries>m)
					dprintf(logfile,"Info: Merged %5d LEVELS             \n",(odir.nentries-m)/n);
			}
		}
		else if (strnicmp(ctg1->secname,"TEXTURE",7)==0)
		{
			if (doonce)
			{
				doonce=0;
				if (owt.t1.N)
				{
					SafeWrite(&owt.t1.N,sizeof(int),1,ost);
					SafeWrite(owt.t1.offs,sizeof(int),owt.t1.N,ost);
					for (j=0,npats=0;j<owt.t1.N;j++)
					{
						SafeWrite(owt.t1.textures+j,sizeof(TextureDesc)-sizeof(Patch *),1,ost);
						SafeWrite(owt.t1.textures[j].patches,sizeof(Patch),owt.t1.textures[j].npatches,ost);
						npats += owt.t1.textures[j].npatches;
					}
					n = sizeof(int) + sizeof(int) * owt.t1.N
						+ (sizeof(TextureDesc)-sizeof(Patch *)) * owt.t1.N
						+ npats * sizeof(Patch);
					AddDirEntry(&odir,offs,n,"TEXTURE1");
					offs += n;
					if (DoTextures)
						dprintf(logfile,"Info: Merged %5ld + %5ld ->%5ld TEXTURE1s                     \n",bwt.t1.N,pwt.t1.N,owt.t1.N);
				}

				if (owt.pnames.npnames)
				{
					SafeWrite(&owt.pnames.npnames,sizeof(int),1,ost);
					SafeWrite(owt.pnames.patchname[0],8,owt.pnames.npnames,ost);
					n = sizeof(int) + 8 * owt.pnames.npnames;
					AddDirEntry(&odir,offs,n,"PNAMES");
					offs += n;
					if (DoTextures)
						dprintf(logfile,"Info: Merged %5ld + %5ld -> %5ld PNAMES                      \n",bwt.pnames.npnames,pwt.pnames.npnames,owt.pnames.npnames);
				}
			}
		}
		else if (ctg1->idm == ID_LISTED)
		{
			for (i=0;i<ctg1->nnames;i++)
			{
				WriteSimpleResource(ctg1->Names[i].str,&offs,ctg1->Names[i].didx,ctg2->Names[i].didx,IsSectionEnabled(ctg1->secname));
				if (odir.nentries>m)
					fprintf(stderr,"Merged %5d %-8s: %-8s        \r",odir.nentries-m,ctg1->secname,ctg1->Names[i].str);
			}
			if (odir.nentries>m)
				dprintf(logfile,"Info: Merged %5d %-8s  %-8s         \n",odir.nentries-m,ctg1->secname,"        ");
		}
		else if (ctg1->idm == ID_MARKED)
		{
			marked=0;
			if (ctg1->nnames>0 || ctg2->nnames>0)
			{
				for (k=0;k<ctg1->nnames;k++)
				{
					if
					(
						!NewOnly &&
						IsSectionEnabled(ctg1->secname) &&
						(p=StrIdxFind(ctg2->Names,ctg2->nnames,ctg1->Names[k].str))>=0
					)
					{
						if (!marked) AddStartN(ctg1->mark,offs,0);
						marked=1;
						WriteEntry(&pdir,pst,ctg2->Names[p].didx,&offs);// if superseded, from patch
					}
					else
					{
						if (!marked) AddStartN(ctg1->mark,offs,0);
						marked=1;
						WriteEntry(&bdir,bst,ctg1->Names[k].didx,&offs);// else copy from base
					}
					if (odir.nentries>m)
						fprintf(stderr,"Merged %5d %-8s: %-8s          \r",odir.nentries-m,ctg1->secname,ctg1->Names[k].str);
				}

				if (IsSectionEnabled(ctg1->secname))
				{
					for (k=0;k<ctg2->nnames;k++)
					{
						if((p=StrIdxFind(ctg1->Names,ctg1->nnames,ctg2->Names[k].str))<0)
						{
							if (!marked) AddStartN(ctg1->mark,offs,0);
							marked=1;
							WriteEntry(&pdir,pst,ctg2->Names[k].didx,&offs);// if new, write it out
							if (odir.nentries>m)
								fprintf(stderr,"Merged %5d %-8s: %-8s         \r",odir.nentries-m,ctg1->secname,ctg2->Names[k].str);
						}
					}
				}
				if (odir.nentries>m)
					dprintf(logfile,"Info: Merged %5d %-8s  %-8s         \n",odir.nentries-m,ctg1->secname,"         ");
    			if (marked) AddEndN(ctg1->mark,offs,0);
			}
		}
	}
	fprintf(stderr,"                                                       \n");


	// write the output wad's directory
	SafeWrite(odir.dirp,sizeof(WadDirEntry),odir.nentries,ost);

	// finally rewrite the output wad's header and close
	fseek(ost,0,SEEK_SET);
	if (MakeIwad) strncpy(ohdr.wadtag,"IWAD",4);
	ohdr.waddirlen = odir.nentries;
	ohdr.waddiraddr = offs;
	SafeWrite(&ohdr,sizeof(WadHeader),1,ost);
	dprintf(logfile,"Wads merged successfully\n");
	return 0;
}


