#include "patch.h"

int movemain(int argc,char **argv)
{
	int i,c;
	int cleared=0;
	char mapsel[256],*p,*tok;
	int mvorctr[4];
	int amount[4];
	int minx,maxx,miny,maxy,minz,maxz;
	int XMove,YMove,ZMove;
	int StayOnGrid=0;
	Category *ctgy;

	dprintf(logfile,"\n");
	InitCLStrs();

	CopyEnvironmentCLArgs("MOVEIT",argc,argv);
	if (ParseSlashOptionLine(-1,-1))
	{
		dprintf(logfile,"Error parsing command line\n");
		FreeCLStrs();
		return 3;
	}

	if (nclfiles<1)
	{
		dprintf(logfile,"\nMOVEIT %s by Jim Flynn, Copyright 1998  -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"\nMove or center things and vertices in a DOOM/DOOM2 wad.\n");
		dprintf(logfile,"\nUsage: MOVEIT [/L:MAPnn] wadfile[.wad] x-move y-move [z-move]\n");
		dprintf(logfile,"       Parameters are units to move by or C for centering\n");
		dprintf(logfile,"       Specify in xyz order or use z=nn x=nn y=nn syntax\n");
		dprintf(logfile,"\n General options, use SET MOVEIT=options to set defaults:\n");
		dprintf(logfile,"  /G move by multiples of 64X64 grid\n");
		dprintf(logfile,"  /L:MAPnn set map to do - else all are done\n");
		dprintf(logfile,"  /DOOM to move a DOOM 1 wad (use /L:ExMy then)\n");
//		dprintf(logfile,"\n See MOVEIT.TXT for details.\n");
						 
		FreeCLStrs();
		return(1);
	}

	cleared=0;
	strcpy(mapsel,"");
	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='l' && cloptions[i][1]==':')
			strcpy(mapsel,cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='g')
			StayOnGrid=1;
		else if (stricmp(cloptions[i],"DOOM")==0)
			strcpy(defname,"DOOM");
		else dprintf(logfile,"Warn: command line option: %s not understood\n",cloptions[i]);
	}

	strcpy(pwadname,clfiles[0]);
	for (i=0;i<3;i++)
		mvorctr[i]=amount[i]=0;
	for (i=1;i<4;i++)
	{
		if (nclfiles>i)
		{
			if (tolower(clfiles[i][0]=='c'))
				mvorctr[i-1] = 1;
			else
			{
				p = strdup(clfiles[i]);
				tok = strtok(p,"xyzXYZ=:");
				if (tok && *tok)
				{
					int j = tolower(clfiles[i][0]) - 'x';
					if (j>=0 && j<=2)
					{
						if (tolower(tok[0])=='c')
							mvorctr[j] = 1;
						else
							amount[j] = atoi(tok);
					}
					else amount[i-1] = atoi(clfiles[i]);
				}
				free(p);
			}
		}
	}

	FreeCLStrs();

	AddExt(pwadname,".WAD");
	AddExt(defname,".DEF");

	InitCommonStructures();
	InitCategories(defname,&ResDef);

	pst = fopen(pwadname,"r+b");
	if (pst!=NULL)
	{
		fread(&phdr,sizeof(WadHeader),1,pst);
		ReadDir(&phdr,&pdir,pst);
		c = CategoryOf("LEVELS",&ResDef);
		if (c>=0)
		{
			ctgy = ResDef.cat+c;
			for (i=0;i<ctgy->nnames;i++)
			{
     		if (!*mapsel || stricmp(mapsel,ctgy->Names[i].str)==0)
     		{
					if (!ReadLevelData(ctgy->Names[i].str,&pdir,pst,NLevObj()))
					{
						int p = IsIn(ctgy->Names[i].str,&pdir);
						int t = IsAfter(p,"THINGS",&pdir);
						int v = IsAfter(p,"VERTEXES",&pdir);
						int s = IsAfter(p,"SECTORS",&pdir);
						int n = IsAfter(p,"NODES",&pdir);

						int j;
						if (p<0 || t<0 || v<0 || s<0)
						{
							dprintf(logfile,"Can't read wad\n");
							fclose(pst);
							exit(3);
						}
						minx=miny=minz=  32767;
						maxx=maxy=maxz= -32768;
						for (j=0;j<NThings;j++)
						{
							if (Things[j].x<minx)
								minx = Things[j].x;
							if (Things[j].x>maxx)
								maxx = Things[j].x;
							if (Things[j].y<miny)
								miny = Things[j].y;
							if (Things[j].y>maxy)
								maxy = Things[j].y;
						}
						for (j=0;j<NVertexes;j++)
						{
							if (Vertexes[j].x<minx)
								minx = Vertexes[j].x;
							if (Vertexes[j].x>maxx)
								maxx = Vertexes[j].x;
							if (Vertexes[j].y<miny)
								miny = Vertexes[j].y;
							if (Vertexes[j].y>maxy)
								maxy = Vertexes[j].y;
						}
						for (j=0;j<NSectors;j++)
						{
							if (Sectors[j].fhgt<minz)
								minz = Sectors[j].fhgt;
							if (Sectors[j].fhgt>maxz)
								maxz = Sectors[j].fhgt;
							if (Sectors[j].chgt<minz)
								minz = Sectors[j].chgt;
							if (Sectors[j].chgt>maxz)
								maxz = Sectors[j].chgt;
						}

						if (mvorctr[0])
							XMove = -(minx+maxx)/2;
						else
							XMove = amount[0];
						if (StayOnGrid)
							XMove = 64*((XMove+32)/64);

						if (mvorctr[1])
							YMove =  -(miny+maxy)/2;
						else
							YMove = amount[1];
						if (StayOnGrid)
							YMove = 64*((YMove+32)/64);

						if (mvorctr[2])
							ZMove = -(minz+maxz)/2;
						else
							ZMove = amount[2];


						for (j=0;j<NThings;j++)
						{
							Things[j].x += XMove;
							Things[j].y += YMove;
						}

						fseek(pst,pdir.dirp[t].resptr,SEEK_SET);
						fwrite(Things,pdir.dirp[t].reslen,1,pst);

						for (j=0;j<NVertexes;j++)
						{
							Vertexes[j].x += XMove;
							Vertexes[j].y += YMove;
						}

						fseek(pst,pdir.dirp[v].resptr,SEEK_SET);
						fwrite(Vertexes,pdir.dirp[v].reslen,1,pst);

						for (j=0;j<NSectors;j++)
						{
							Sectors[j].fhgt += ZMove;
							Sectors[j].chgt += ZMove;
						}

						fseek(pst,pdir.dirp[s].resptr,SEEK_SET);
						fwrite(Sectors,pdir.dirp[s].reslen,1,pst);

						if (n>=0)
						{
							for (j=0;j<NNodes;j++)
							{
								Nodes[j].x += XMove;
								Nodes[j].y += YMove;
								Nodes[j].lmxy += YMove;
								Nodes[j].lmny += YMove;
								Nodes[j].lmnx += XMove;
								Nodes[j].lmxx += XMove;
								Nodes[j].rmxy += YMove;
								Nodes[j].rmny += YMove;
								Nodes[j].rmnx += XMove;
								Nodes[j].rmxx += XMove;
							}
							fseek(pst,pdir.dirp[n].resptr,SEEK_SET);
							fwrite(Nodes,pdir.dirp[n].reslen,1,pst);
						}

						FreeLevelData();

						dprintf(logfile,"%s coordinates moved by: %d %d %d\n",
														ctgy->Names[i].str,XMove,YMove,ZMove);

						dprintf(logfile,"X was from %8d to %8d, now from %8d to %8d\n",
														minx,maxx,minx+XMove,maxx+XMove);

						dprintf(logfile,"Y was from %8d to %8d, now from %8d to %8d\n",
														miny,maxy,miny+YMove,maxy+YMove);

						dprintf(logfile,"Z was from %8d to %8d, now from %8d to %8d\n",
														minz,maxz,minz+ZMove,maxz+ZMove);
					}
				}
			}
		}
	}
	else
	{
		dprintf(logfile,"Wad not found: %s\n",pwadname);
  	exit(3);
	}

	FreeDir(&pdir);
  fclose(pst);
	exit(0);
}

