#include "patch.h"
#include "unistd.h"

static int isdoom=0; // whether working on DOOM or DOOM II wads

int MakeMus(char *srcfile,int srclevel,int dstlevel)
{
	char *p,*q;
	char *buffer=NULL;
	char musicname[32];
	FILE *st,*ost;
	WadDir dir;
	WadHeader hdr;
	int i;

	InitDir(&dir);
	q = strrchr(srcfile,'\\');
	p = q? strrchr(q+1,'.') : strrchr(srcfile,'.');
	if (!p)
	{
		dprintf(logfile,"Input file has unknown type, please specify .MID, .MUS or .WAD\n");
		return(3);
	}

	if (strnicmp(p+1,"MID",3)==0 && !p[4])
	{
		strcpy(musicname,isdoom? d1musstr[dstlevel] : musstr[dstlevel]);
		AddExt(musicname,".MUS");
		MoveFile(srcfile,musicname,0);
	}
	else if (strnicmp(p+1,"WAD",3)==0 && !p[4])
	{
		strcpy(musicname,srclevel? (isdoom? d1musstr[srclevel] : musstr[srclevel]) : "D_");
		st = fopen(srcfile,"rb");
		if (st)
		{
			fread(&hdr,sizeof(WadHeader),1,st);
			ReadDir(&hdr,&dir,st);
			for (i=0;i<dir.nentries;i++)
			{
				if(strnicmp(dir.dirp[i].resname,musicname,strlen(musicname))==0)
				{
					fseek(st,dir.dirp[i].resptr,SEEK_SET);
					buffer = malloc(dir.dirp[i].reslen);
					fread(buffer,dir.dirp[i].reslen,1,st);

					strcpy(musicname,isdoom? d1musstr[dstlevel] : musstr[dstlevel]);
					AddExt(musicname,".MUS");

					ost = fopen(musicname,"wb");
					if (ost)
					{
						fwrite(buffer,dir.dirp[i].reslen,1,st);
						fclose(ost);
					}
					else
					{
						dprintf(logfile,"Can't open %s for write\n",musicname);
						return(3);
					}
					free(buffer);
					break;
				}
				fclose(st);
			}
		}
		else
		{
			dprintf(logfile,"Can't open %s for reading\n",srcfile);
			return(3);
		}
		if (i==dir.nentries)
		{
			dprintf(logfile,"Can't find music resource in %s\n",musicname);
			return(3);
		}
	}
	else if (strnicmp(p+1,"MUS",3)==0 && !p[4])
	{
		strcpy(musicname,isdoom? d1musstr[dstlevel] : musstr[dstlevel]);
		AddExt(musicname,".MUS");
		MoveFile(srcfile,musicname,0);
	}
	FreeDir(&dir);
	return 0;
}


int MakMusWad(char *dstnam,int dstlevel)
{
	FILE *rst;
	char tmpnam[256];
	int arc=0,err=0;
	char **arv=NULL;

	strcpy(tmpnam,isdoom? d1musstr[dstlevel] : musstr[dstlevel]);

	rst = fopen("TEMP.RES","w");
	if (rst)
	{
		fprintf(rst,"[MUSICS]\n");
		fprintf(rst,"%s\n",tmpnam);
		fclose(rst);
	}
	else
	{
		dprintf(logfile,"Can't open %s for writing\n","TEMP.RES");
		return(3);
	}

	errorlev=0;
	mkdir("MUSICS",0777);
	AddExt(tmpnam,".MUS");
	MoveFile(tmpnam,"MUSICS",1);
	AddDynStr(&arv,&arc,"MAK");
	AddDynStr(&arv,&arc,isdoom? "/DOOM" : "/D:RESOURCE");
	AddDynStr(&arv,&arc,"+M");
	AddDynStr(&arv,&arc,"TEMP.RES");
	AddDynStr(&arv,&arc,"NULL");
	free(arv[4]);
	arv[4]=NULL;
	arc--;
	strcpy(cmdline,"");
	err=makmain(arc,arv);
	CloseStreams();

	remove(stringf("MUSICS\\%s",tmpnam));
	rmdir("MUSICS");
	FreeDynStr(&arv,&arc);
	if (err) return err;

	AddExt(dstnam,".WAD");
	if (Exists(dstnam))
	{
  	errorlev=0;
		AddDynStr(&arv,&arc,"MER");
	  AddDynStr(&arv,&arc,isdoom? "/DOOM" : "/D:RESOURCE");
		AddDynStr(&arv,&arc,"+M");
		AddDynStr(&arv,&arc,dstnam);
		AddDynStr(&arv,&arc,"TEMP");
		AddDynStr(&arv,&arc,"TEMP2");
		AddDynStr(&arv,&arc,"NULL");
		free(arv[6]);
		arv[6]=NULL;
		arc--;
		strcpy(cmdline,"");
		err=mermain(arc,arv);
	  CloseStreams();
		remove ("TEMP.WAD");
		if (err) return err;
		err=MoveFile("TEMP2.WAD",dstnam,1);
		FreeDynStr(&arv,&arc);
	}
	else MoveFile("TEMP.WAD",dstnam,1);
	return 0;
}

int muspmain(int argc,char **argv)
{
	char *p,*q,srcname[256],dstname[256];
	int srclevel,dstlevel,err=0;

	CopyEnvironmentCLArgs("MUSPUT",argc,argv);
	ParseCommandLine(-1,-1);	// Get any number of options, up to two parms

	if (nclfiles!=1 && nclfiles!=2)
	{
		dprintf(logfile,"\nMUSPUT.EXE %s by Jim Flynn, Copyright 1997,8 -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"\nInsert music from .MID, .MUS or .WAD into an existing wad\n");
		dprintf(logfile,"or create a new wad containing only that music\n");
		dprintf(logfile,"\nUsage: MUSPUT musfile{.MID|.MUS|.WAD@n} wadfile[.wad]@m\n");
		dprintf(logfile,"\nGet music from musfile.MID, musfile.MUS or level n of musfile.WAD\n");
		dprintf(logfile," if wadfile exists, insert as level m's music\n");
		dprintf(logfile," if wadfile does not exist, create it with music for level m\n");
		dprintf(logfile," use level 33 for D_READ_M, 34 for D_DM2TTL, and 35 for D_DM2INT\n");
		dprintf(logfile,"\nEx1. Insert SONG.MID as level 13 music in MYWAD.WAD:\n");
		dprintf(logfile," MUSPUT SONG.MID MYWAD@13\n");
		dprintf(logfile,"\nEx2. Level E1M8 music for AWAD.WAD inserted as E2M3 music in MYWAD.WAD:\n");
		dprintf(logfile," MUSPUT /DOOM AWAD.WAD@18 MYWAD@23\n");
		dprintf(logfile,"\nEx3. NEW.WAD containing music in SONG.MUS as level 5 music is created:\n");
		dprintf(logfile," MUSPUT SONG.MUS NEW@5\n");
		return(1);
	}

	strcpy(srcname,clfiles[0]);
	strcpy(dstname,clfiles[1]);

	if (nclopts && !stricmp(cloptions[0],"doom"))
	{
		strcpy(defname,"doom.def");
	  isdoom = 1;
	}
	FreeCLStrs();

	srclevel=dstlevel=0;
	p = strrchr(srcname,'@');
	if (p)
	{
		srclevel=min(max(0,atoi(p+1)),isdoom? 55 : 35);
		*p='\0';
		AddExt(srcname,".WAD");
		if (isdoom)
		{
			if (srclevel<11)
				srclevel=0;
			else if ((srclevel%10)==0)
				srclevel=0;
		}
	}
	q = strrchr(dstname,'@');
	if (q)
	{
		dstlevel=min(max(0,atoi(q+1)),isdoom? 55 : 35);
		*q='\0';
		if (isdoom)
		{
			if (dstlevel<11)
				dstlevel=0;
			else if ((dstlevel%10)==0)
				dstlevel=0;
		}
	}
	else
	{
		dprintf(logfile,"Destination level must be specified with @nn after wad name\n");
		return(1);
	}
	if (!Exists(srcname))
	{
		dprintf(logfile,"\nNot found: %s\n",srcname);
		return(3);
	}

	err=MakeMus(srcname,srclevel,dstlevel);
	if (!err) err=MakMusWad(dstname,dstlevel);
	return err;
}
