#include "patch.h"

/************************************************************
 PARSE.C
			 /-------------------------\
			||    MAK file parsing     ||
			 \-------------------------/
									
************************************************************/

char *reskeys[]=
{
	".ELSE",
	".ELSEIF",
	".ENDIF",
	".IF",
	".SET",
	".OPTION",
	".COLORMAP",
	".INCLUDE",
	"INCLUDE",
};
int nreskeys = sizeof(reskeys)/sizeof(reskeys[0]);

extern void DoMakCommandLineOpts();

void freeifstack()
{
	free(ifstack);
	ifstack=NULL;
	ifstacklen=ifstackalloc=0;
}

void freesymbols()
{
	int i;

	for (i=0;i<nsymbols;i++)
		free(symbols[i].sym);
	free(symbols);
	symbols=NULL;
	nsymbols=symbolsalloc=0;
}

void FreeParser()
{
	freeifstack();
	freesymbols();
}

// updates the if stack state when an IF is encountered
// newelt is the evaluation of the IF condition

void pushifstack(int newelt,int tag)
{
  if (ifstacklen>=ifstackalloc)
	{
		ifstackalloc = ifstackalloc? 2*ifstackalloc : 256;
    ifstack = realloc(ifstack,ifstackalloc*sizeof(IfElt));
	}
  ifstack[ifstacklen].path = newelt;
  ifstack[ifstacklen++].id = tag;
}

// updates the if stack when an ENDIF is encountered

void popifstack(void)
{
	while (ifstacklen && ifstack[ifstacklen-1].id!=RK_IF)
		--ifstacklen;
	if (ifstacklen)
		--ifstacklen;
	else ParseError = IfStackUnderflow;
}

// updates the if stack when an ELSE is encountered

void negifstack(void)
{
  if (ifstacklen)
	{
    int v = ifstack[ifstacklen-1].path = !ifstack[ifstacklen-1].path;
		pushifstack(v, RK_ELSE);
	}
	else ParseError = IfStackUnderflow;
}

// updates the if stack when an ELSEIF is encountered

void chgifstack(int newelt)
{
  if (ifstacklen)
	{
    ifstack[ifstacklen-1].path = !ifstack[ifstacklen-1].path;
		pushifstack(newelt, RK_ELSEIF);
	}
	else ParseError = IfStackUnderflow;
}

// evaluates the if stack and returns whether the current state is on or off

int ifcondition(void)
{
	int i;

  for (i=0;i<ifstacklen;i++)
    if (!ifstack[i].path) return 0;
  return 1;
}

int GetSymbolValue(char *symbol,int *value)
{
	int i;

	for (i=0;i<nsymbols;i++)
		if (stricmp(symbol,symbols[i].sym)==0)
			break;
	if (i<nsymbols)
	{
		*value = symbols[i].val;
	  return 1;
	}
	return 0;
}

void SetSymbolValue(char *symbol,int value)
{
	int i;

	for (i=0;i<nsymbols;i++)
		if (stricmp(symbol,symbols[i].sym)==0)
			break;
	if (i<nsymbols)
	{
		symbols[i].val = value;
	}
	else
	{
		if (nsymbols>=symbolsalloc)
		{
			symbolsalloc = symbolsalloc? 2*symbolsalloc : 256;
			symbols = realloc(symbols,symbolsalloc*sizeof(Symbol));
		}
		symbols[nsymbols].sym = malloc(strlen(symbol)+1);
		strcpy(symbols[nsymbols].sym,symbol);
		symbols[nsymbols].val=value;
		nsymbols++;
	}
}

void ParseIF(char *line)
{
	char *l,*p,*r,*tok;
	int v;

	l = strupr(r=strdup(line));
	p = strstr(l,".IF");
	tok = strtok(l+3,"() \t[]");

	if (tok && *tok)
	{
		if (IsNumeric(tok))
			pushifstack(atoi(tok), RK_IF);
		else
		{
			if (GetSymbolValue(tok,&v))
				pushifstack(v, RK_IF);
			else
				ParseError = UndefinedSymbol;
		}
	}
	else
		ParseError = NoIfCondition;

	xlibfree (r);
}

void ParseELSEIF(char *line)
{
	char *l,*p,*tok;
	int v;

	if (!ifstacklen)
	{
		ParseError = ElseWithoutIf;
	  return;
	}

	l = strupr(strdup(line));
	p = strstr(l,".ELSEIF");
	tok = strtok(l+7,"() \t[]");

	if (tok && *tok)
		if (IsNumeric(tok))
			chgifstack(atoi(tok));
		else
		{
			if (GetSymbolValue(tok,&v))
				chgifstack(v);
			else
				ParseError = UndefinedSymbol;
		}
	else
		ParseError = NoIfCondition;
}

void ParseELSE(char *line)
{
	if (!ifstacklen)
	{
		ParseError = ElseWithoutIf;
	  return;
	}

	negifstack();
}

void ParseENDIF(char *line)
{
	if (!ifstacklen)
	{
		ParseError = EndWithoutIf;
	  return;
	}

	popifstack();
}

void ParseSET(char *line)
{
	char *l,*p,*symtok,*valtok;
	int val;

	l = strupr(strdup(line));
	p = strstr(l,".SET");


	symtok = strtok(l+4,"() \t[]");
	if (!symtok || !*symtok)
	{
		ParseError = SetWithoutSymbol;
		return;
	}
	valtok = strtok(NULL,"() \t[]");
	if (!valtok || !*valtok)
	{
		ParseError = SetWithoutValue;
		return;
	}

	if (!IsNumeric(valtok))
	{
		if (!GetSymbolValue(valtok,&val))
		{
			ParseError = SetWithoutValue;
			return;
		}
	}
	else val = atoi(valtok);

	SetSymbolValue(symtok,val);
}

void ParseOPTION(char *line)
{
	char *l,*p;
	int i;

	l = strupr(strdup(line));
	p = strstr(l,".OPTION");

	FreeDynStr(&clstrs,&nclstrs);
	clstrs=NULL; nclstrs=0;
	Tokenize(l+7," \t",&clstrs,&nclstrs);
	ParseCommandLine(-1,-1);
	for (i=0;i<nclopts;i++)
	{
		strcat(cmdline," ");
		strcat(cmdline,cloptsense[i]? "+" : "-");
		strcat(cmdline,cloptions[i]);
	}
}

void ParseCMAP(char *line)
{
	char *p = strdup(line);
	char *tok = strtok(p+9," \t");
	if (tok && *tok)
		AddVTex
		(
			&mtxres[0],
			&mtxcnt[0],
			ifcondition(),
			".CMAP",
			tok,
			NULL,
			0,
			NULL
		);
	else
		AddVTex
		(
			&mtxres[0],
			&mtxcnt[0],
			ifcondition(),
			".CMAP",
			NULL,
			NULL,
			0,
			NULL
		);

	xlibfree(p);
}

// Find and remove the inline comment, if any
// Truncate the string to empty if its all comment

void StripComment(char *line)
{
	char *p,*q;

	// handle null, empty lines
	if (!line || !*line)
		return;

	// Allow whitespace at start of any line
	if (line && *line)
		LeftTrim(line);

	// handle lines beginning with semicolon or containing all whitespace
	if (!*line || *line==';')
	{
		*line='\0';
		return;
	}

	// find the inline comment and truncate it if present
	// skip semicolons preceeded by backslash
	q = line;
	while ((p=strchr(q,';')) && p!=q && p[-1]=='\\')
	{
		memmove(p-1,p,strlen(p)+1); // delete backslashes before semicolons
		q=p;
	}
	if (p) *p='\0';	// truncate the comment found eventually, if any
}

// Abstract the physical filename if present
// Done after comment removal and sign extraction

void ExtractPath(char *line,char *path,int maxlen)
{
 	char *p1=NULL,*p2=NULL,*p3=line;

	strcpy(path,"");											// default to empty path

	// if there is a left paren not preceded by backslash
	while ((p1=strchr(p3,'(')) && p1!=p3 && p1[-1]=='\\')
	{
		memmove(p1-1,p1,strlen(p1)+1); 			// delete backslashes before parens
		p3 = p1;
	}

	// and a following right paren not preceded by backslash
	if ((p3=p1))
		while ((p2=strchr(p3,')')) && p2!=p3 && p2[-1]=='\\')
		{
			memmove(p2-1,p2,strlen(p2)+1); 		// delete backslashes before parens
			p3 = p2;
		}

	if (p1 && p2)							// if balanced parens found
	{
		*(p1++) = '\0';					// p1 points to char after left paren
		*(p2++) = '\0';					// p2 points to char after right paren

		if (strlen(p1)<=maxlen)
			strcpy(path,p1);			// text up to following right paren is pathname
		else
		{
			strncpy(path,p1,maxlen-1);
			path[maxlen-1]='\0';
			ParseError = PathOverflow;
		}
		strcat(line,p2);				// delete pathname and parens from line
	}
	else if (p1 && !p2)
		ParseError = UnbalParen;
}

// Abstract the sign text from the line, if present
// Done after comment and path removal

void ExtractSign(char *line,char *sign,int maxlen)
{
	char *p1,*p2=line;
	int n=0;

	strcpy(sign,"");											// default to empty sign

	// if there is an "=" not preceded by backslash

	while ((p1=strchr(p2,'=')) && p1!=p2 && p1[-1]=='\\')
	{
		memmove(p1-1,p1,strlen(p1)+1); 			// delete backslashes before =
		p2 = p1;
	}

	// remove leading and trailing spaces, extract text after =

	if (p1)
	{
		Trim(p1+1);

		// handle normal and overlength strings
		if (strlen(p1+1)<maxlen)
			strcpy(sign,p1+1);	// copy after = to sign buffer
		else
		{
			strncpy(sign,p1+1,maxlen-1);
			sign[maxlen-1]='\0';
			ParseError = SignOverflow;
		}

		// if trailing space preceeded by \ removed, put it back
		n = strlen(sign);
		if (n>0 && n<maxlen-1 && sign[n-1]=='\\')
		{
			sign[n]=' ';
			sign[n+1]='\0';
		}

		*p1='\0';											// delete = and after in line
	}
}

// Retrieve the section name from a [secname] (path) ; comment line

int ExtractSection(char *line,char *secname)
{
	char *p;

 	strcpy(secname,"");
	if (*line != '[')
		return 0;

	if ((p=strchr(line,']')))
	{
		if (p) *p='\0';
		strcpy(secname,line+1);
		if (p) *p=']';
		Trim(secname);
		strcpy(line,p+1);
		return 1;
	}
	else
		ParseError = UnTermSection;
	return 0;
}

//
// Check for the beginning of a line being a KEYWORD
//
// Returns -1 if not a keyword line, else index of keyword in array

int GetKeyword(char *line,char **keys,int nkeys,int *kwd)
{
	int i;

	for (i=0;i<nkeys;i++)
		if (strnicmp(line,keys[i],strlen(keys[i]))==0)
			if (!isalnum(line[strlen(keys[i])]))
			{
				*kwd=i;
			  return 1;
			}
	return 0;
}


void GetResInfo(char *line,VTex *info)
{
	int done=0;

	info->name = malloc(1);
	strcpy(info->name,"");			// default to empty and zero
	Tokenize(line," \t",&info->args,&info->nargs);

	SetSyntaxType(info->id,ST_RESDEFN);

	if (info->nargs>0)
	{
		do
		{
			switch(**info->args)
			{
				case '+':
					SetMultPost(info->id,1);
					break;
				case '@':
					SetInputType(info->id,IT_WAD);
					break;
				default:
					break;
			}
			switch(**info->args)
			{
				case '*':
				case '+':
				case '@':
					SetSyntaxType(info->id,ST_RESINPUT);
					memmove(info->args[0],info->args[0]+1,strlen(info->args[0]));
					if (!*(info->args[0]))
					{
						free(info->args[0]);
						info->args[0]=NULL;
						info->args++;
						info->nargs--;
					}
					break;
				default:
					done=1;	// quit when you get a token not beginning with a
					break;	// special character
			}
		} while (!done && info->nargs>0);
	}

	// all sprites are always done with multiple posts, even tall ones

	if (ResourceTypeIs(info->id,"SPRITES"))
		SetMultPost(info->id,1);

	// the resource name is always the first argument
	// after special tokens have been removed. Copy and delete.

	if (info->nargs>0)
	{
		info->name = realloc(info->name,max(9,1+strlen(info->args[0])));
		if (strlen(info->args[0])>8)
			ParseError=ResnameOverflow;
		strncpy(info->name,info->args[0],8);
		info->name[8]='\0';
		free(info->args[0]);
		info->args[0]=NULL;
		info->args++;
		info->nargs--;
	}

	if
	(
		(ResourceTypeIs(info->id,"TEXTURES") ||
		 ResourceTypeIs(info->id,"TEXTURE1") ||
		 ResourceTypeIs(info->id,"TEXTURE2")) &&
		 SyntaxType(info->id)==ST_RESDEFN
	)
		if (info->nargs<2)
			ParseError = InsufficientInfo;
}

//
// Abstract the entities from a make line
// Syntax description:
//
// \[sectionname\][ ];comment 
//
//     or
//
// [*][ ]resname [([ ]pathname[ ])] [int1 [int2]][[ ]=signtext][;comment]
//
//     or
//
// INCLUDE ([ ]pathname[ ]) [;comment]
//
//     or
//
// { |;|\t}comment
//
//     or
//
//     \n
//
// \[ and \] are literal brackets - [ and ] are required parts of syntax
// [parm] represents an optional parm
// [ ] represents 0 or more whitespaces
// {x|y|...} represents a choice of strings
//
// Syntax summary. Each line is a blank line, a comment, or a section line
// or a header line. Section and header lines can have inline comments. 
//
// Blank lines and comment lines are ignored
// 
// For header and section lines, the inline comment is defined by the first
// semicolon in the line not preceded by a literalizing backslash, and this
// semicolon and all following characters in the line are removed before
// further parsing.
//
// A header line starts with [, then a section name, then ] and may optionally
// be followed with a pathname in parentheses. Any line not starting with [
// is treated as a section line. A line without a closing ] will cause an error.
//
// For both header and section lines, if a ( occurs it must be followed by
// a ), and the trimmed text between is a path, or filename. The path for a
// header line must be a filename, and redirects where the section lines
// for the that header are read from. A fatal error occurs if the redirected
// file does not exist - parsing stops. A path for a section line describes
// where the resource for that line is read, it may be a directory, or a full
// file name, either relative to the current directory, or a full path.
//
// A section line may start with an asterisk, then after an optional whitespace
// a required resource name followed by zero, one or two integers, whitespace
// delimited. Then may follow optional whitespace and an equal sign. If present
// the text from the equal sign to the end of line, or start of inline comment
// is a graphic sign described by the text.
//
// A section line may define a sign, by inserting the symbol = 
//
// Also for header and section lines any occurrence of = outside a comment
// introduces a sign specification which terminates with end of string or
// start of comment. This sign text is removed from the string if present
// after comment removal, and stored to the sign buffer passed.
//
// Simple rules
// 1) A line is a header line iff it starts with [ in the first column
// 2) A blank line or line starting with whitespace or ; is ignored
// 3) The inline comment begins with the first ; not preceeded by a \
// 4) The inline comment ends at end of line or end of file
// 5) A section line begins with *, or the first letter of the resource name
// 6) Whitespace between * and the resource name is optional
// 7) If the first parentheses in a section line is balanced


void ParseMake
(
	char *line,			// the line we are parsing
	int lineno,			// the line number we are parsing
	VTex *info			// information: id,path,sign,argument list
)
{
	int kwd;
	int issection=0;
	int isdirective=0;
	char section[16];

	ParseError = NoError;

	info->name=malloc(1);
	strcpy(info->name,"");		 // default all return parameters to blank or zero
	info->path=malloc(1);
	strcpy(info->path,"");    
	info->sign=malloc(1);
	strcpy(info->sign,"");
	info->nargs=0;
	info->args=NULL;

	SetSyntaxType(info->id,ST_COMMENT);		// default syntax information
	SetInputType(info->id,IT_BMP);
	SetMultPost(info->id,0);
	SetCondState(info->id,1);
	SetNormDef(info->id,1);

	// Process line
	if (line && *line) 															// anything to parse?
	{
		StripComment(line);			         							// remove comment
 		if (*line)																		// if anything left
		{
			// if a directive its syntax is specified by the directive parser

			if ((isdirective = GetKeyword(line,reskeys,nreskeys,&kwd)))
			{
				SetSyntaxType(info->id,ST_DIRECTIVE);					// returning directive
				SetResourceType(info->id,kwd);									// keyword in resource type
				switch (kwd)
				{
					case RK_ELSE:
						ParseELSE(line);
						break;
					case RK_ELSEIF:
						ParseELSEIF(line);
						break;
					case RK_ENDIF:
						ParseENDIF(line);
						break;
					case RK_IF:
						ParseIF(line);
						break;
					case RK_SET:
						ParseSET(line);
						break;
					case RK_OPTION:
						ParseOPTION(line);
						DoMakCommandLineOpts();
						break;
					case RK_INCLUDE:
					case RK_INCLUD2:
						info->path = malloc(256);
						*info->path = '\0';
						ExtractPath(line,info->path,256);			 				// pull out path 
						break;
					default:
						ParseError = UnrecognizedKeyword;
						break;
				}
				if (ParseError==NoIfCondition)
					dprintf(logfile,"ERROR: Missing condition in IF in line #%d\n",lineno);
				else if (ParseError==ElseWithoutIf)
					dprintf(logfile,"ERROR: ELSE without IF in line #%d\n",lineno);
				else if (ParseError==EndWithoutIf)
					dprintf(logfile,"ERROR: ENDIF without IF in line #%d\n",lineno);
				else if (ParseError==UndefinedSymbol)
					dprintf(logfile,"ERROR: Symbol in IF condition undefined in line #%d\n",lineno);
				else if (ParseError==SetWithoutSymbol)
					dprintf(logfile,"ERROR: Missing symbol, value in SET directive in line #%d\n",lineno);
				else if (ParseError==SetWithoutValue)														 
					dprintf(logfile,"ERROR: Missing value for symbol in SET directive in line #%d\n",lineno);
				else if (ParseError==ValueNotNumeric)														 
					dprintf(logfile,"ERROR: Value for symbol not numeric in SET directive in line #%d\n",lineno);
				else if (ParseError==UnrecognizedKeyword)														 
					dprintf(logfile,"ERROR: Directive keyword not recognized in line #%d\n",lineno);
				else if (ParseError==IfStackUnderflow)														 
					dprintf(logfile,"ERROR: Unbalanced ENDIF or ELSE in line #%d\n",lineno);
				// Keep track of largest ParseError since MaxParseError last reset
				if (ParseError > MaxParseError)
					MaxParseError = ParseError;
			}
			else // else its a standard make line
			{
				info->path = malloc(256);
				ExtractPath(line,info->path,256);			 						// pull out path

				if (ParseError==PathOverflow)
					dprintf(logfile,"ERROR: Pathname overflow in line #%d\n",lineno);
				if (ParseError==UnbalParen)
					dprintf(logfile,"ERROR: Unbalanced parentheses in line #%d\n",lineno);
				// Keep track of largest ParseError since MaxParseError last reset
				if (ParseError > MaxParseError)
					MaxParseError = ParseError;

				if (*line)
				{
					info->sign = malloc(4096);
					*info->sign = '\0';
					ExtractSign(line,info->sign,4096);		 					// pull out sign
					if (info->sign && *info->sign)
					{
						SetInputType(info->id,IT_SIGN);
						info->sign = realloc(info->sign,1+strlen(info->sign));
					}
					else
					{
						free(info->sign);
					  info->sign=NULL;
					}

					if (ParseError==SignOverflow)
						dprintf(logfile,"WARNING: Sign overflow in line #%d, truncating\n",lineno);
					// Keep track of largest ParseError since MaxParseError last reset
					if (ParseError > MaxParseError)
						MaxParseError = ParseError;

					if (*line)
					{
						// See if its a section header, remove if found

						if ((issection = ExtractSection(line,section)))	 
						{
							if (info->sign)
								free(info->sign);
							info->sign = malloc(1+strlen(section));
							strcpy(info->sign,section);
							SetSyntaxType(info->id,ST_SECTION);
							if (ParseError==UnTermSection)
								dprintf(logfile,"ERROR: Unterminated section name in line #%d\n",lineno);
						}
						else  // it must be a resource definition
						{
							SetSyntaxType(info->id,ST_RESDEFN);
							GetResInfo(line,info);		// recover RESNAM parm...

							if (ParseError==ResnameOverflow)
								dprintf(logfile,"ERROR: Resource name longer than 8 in line #%d\n",lineno);
							if (ParseError==ExtraData)
								dprintf(logfile,"WARNING: extra data following resource info in line #%d\n",lineno);
							if (ParseError==InsufficientInfo)
								dprintf(logfile,"FATAL: texture definition without size in line #%d\n",lineno);
						}

						// Keep track of largest ParseError since MaxParseError last reset
						if (ParseError > MaxParseError)
							MaxParseError = ParseError;
					}
				}
			}
		}
	}

	Trim(info->path);
	SetCondState(info->id,ifcondition());
	if (!info->sign || !*info->sign)
	{
		free(info->sign);
		info->sign=NULL;
	}
	if (!info->path || !*info->path)
	{
		free(info->path);
		info->path=NULL;
	}
}

