////////////////////////////////////////////////////////////////////
//
// Definition of Parser errors
//

typedef enum
{
	NoError,					  // sucess code

	SignOverflow,				// warnings
	ResnameOverflow,
	ExtraData,					

	PathOverflow,				// errors - cause exit of parser
	UnbalParen,
	NoInfo,
	InsufficientInfo,
	NoResource,
	OutOfMemory,
	UnTermSection,
	NoIfCondition,
	ElseWithoutIf,
	EndWithoutIf,
	UndefinedSymbol,
	SetWithoutSymbol,
	SetWithoutValue,
	ValueNotNumeric,
	UnrecognizedKeyword,
	IfStackUnderflow,

} ParseErrors;

////////////////////////////////////////////////////////////////////
//
// definition of integer id of parsed line
//
// You can change the number of bits allocated to any field without
// any other changes as long as the total sum of all Bits is 32 or
// less. 
//
// The first field represents the syntax type of the line
// 

typedef enum
{
	ST_COMMENT=0,		// blank or comment line - ignore sign, path, args
	ST_DIRECTIVE,		// directive line - sign,path,args depend on directive
	ST_SECTION,			// section line - sign is the section name, path is default
	ST_RESDEFN,			// resource definition: RESNAME arg0 arg1...
	ST_RESINPUT,		// resource input: {*|+|@} INPNAME arg0 arg1...
// future use
	ST_RESERVD5,
	ST_RESERVD6,
	ST_RESERVD7
}	SyntaxTypes;

//----------------------
#define NSyntaxBits 3
//----------------------
#define NSyntaxTypes (1<<NSyntaxBits)
#define SyntaxTypeMask (NSyntaxTypes-1)
#define SyntaxTypeShift 0
#define SyntaxType(id) (((id)&SyntaxTypeMask)>>SyntaxTypeShift)
#define SetSyntaxType(id,val) ((id)=(((id)&~SyntaxTypeMask)|((val<<SyntaxTypeShift)&SyntaxTypeMask)))

//
// The next field represents the category number of the MAK
// section currently in place. 
//

//----------------------
#define NResourceBits 8
//----------------------
#define NResourceTypes (1<<NResourceBits)
#define ResourceTypeMask (NSyntaxTypes*(NResourceTypes-1))
#define ResourceTypeShift NSyntaxBits
#define ResourceType(id) (((id)&ResourceTypeMask)>>ResourceTypeShift)
#define SetResourceType(id,val) ((id)=(((id)&~ResourceTypeMask)|((val<<ResourceTypeShift)&ResourceTypeMask)))
#define ResourceTypeIs(id,string) (ResourceType(id)<ResDef.ncats? strnicmp(ResDef.cat[ResourceType(id)].secname,(string),strlen(string))==0 : 0)

//
// The next field represents an input type identifier
//

typedef enum
{
	IT_NONE,
	IT_BMP,
	IT_WAD,
	IT_SIGN
}	InputTypes;

//----------------------
#define NInputBits 2
//----------------------
#define NInputTypes (1<<NInputBits)
#define InputTypeMask (NSyntaxTypes*NResourceTypes*(NInputTypes-1))
#define InputTypeShift (NSyntaxBits+NResourceBits)
#define InputType(id) (((id)&InputTypeMask)>>InputTypeShift)
#define SetInputType(id,val) ((id)=(((id)&~InputTypeMask)|((val<<InputTypeShift)&InputTypeMask)))

//
// The next field specifies whether single or multiple posts
// should be used if the graphic height is larger than 254. If the
// bit is one, and the height is larger than 254 multiple posts are used.
//

//----------------------
#define NMultPostBits 1
//----------------------
#define NMultPostTypes (1<<NMultPostBits)
#define MultPostMask (NSyntaxTypes*NResourceTypes*NInputTypes)
#define MultPostShift (NSyntaxBits+NResourceBits+NInputBits)
#define MultPost(id) (((id)&MultPostMask)>>MultPostShift)
#define SetMultPost(id,val) ((id)=(((id)&~MultPostMask)|((val<<MultPostShift)&MultPostMask)))

//
// The next field specifies whether conditional assembly state is 
// on or off at the time the line was parsed, after execution of
// the conditional directive if the line was one.
//

//----------------------
#define NCondStateBits 1
//----------------------
#define NCondStateTypes (1<<NCondStateBits)
#define CondStateMask (NSyntaxTypes*NResourceTypes*NInputTypes*NMultPostTypes)
#define CondStateShift (NSyntaxBits+NResourceBits+NInputBits+NMultPostBits)
#define CondState(id) (((id)&CondStateMask)>>CondStateShift)
#define SetCondState(id,val) ((id)=(((id)&~CondStateMask)|((val<<CondStateShift)&CondStateMask)))

//
// The next field specifies whether a resource is simply defined, or
// whether composition of bitmaps is required during assembly. 
//

//----------------------
#define NNormDefBits 1
//----------------------
#define NNormDefTypes (1<<NNormDefBits)
#define NormDefMask (NSyntaxTypes*NResourceTypes*NInputTypes*NMultPostTypes*NCondStateTypes)
#define NormDefShift (NSyntaxBits+NResourceBits+NInputBits+NMultPostBits+NCondStateBits)
#define NormDef(id) (((id)&NormDefMask)>>NormDefShift)
#define SetNormDef(id,val) ((id)=(((id)&~NormDefMask)|((val<<NormDefShift)&NormDefMask)))

//
// The remaining bits ??-31 are reserved
//
////////////////////////////////////////////////////////////////

typedef enum
{
	RK_NONE=-1,
	RK_ELSE,
	RK_ELSEIF,
	RK_ENDIF,
	RK_IF,
	RK_SET,
	RK_OPTION,
	RK_COLORMAP,
	RK_INCLUDE,
	RK_INCLUD2
} reskeyidx;

extern void LoadPalette(char *wadname);
extern void LoadColorMaps(char *wadname);
extern void freeifstack();
extern void freesymbols();
extern void pushifstack(int newelt,int tag);
extern void popifstack(void);
extern void negifstack(void);
extern void chgifstack(int newelt);
extern int  ifcondition(void);
extern int  GetSymbolValue(char *symbol,int *value);
extern void SetSymbolValue(char *symbol,int value);
extern void FreeParser();

extern void ParseMake(char *line,int lineno,VTex *info);
extern void StripComment(char *line);
extern int  GetKeyword(char *line,char **keys,int nkeys,int *kwd);
extern void ParseIF(char *line);
extern void ParseELSEIF(char *line);
extern void ParseELSE(char *line);
extern void ParseENDIF(char *line);
extern void ParseSET(char *line);
extern void ParseOPTION(char *line);
extern void ParseCMAP(char *line);
extern void ExtractPath(char *line,char *path,int maxlen);
extern void ExtractSign(char *line,char *sign,int maxlen);
extern int  ExtractSection(char *line,char *secname);
extern void GetResInfo(char *line,VTex *info);

extern void ParseCp
(
	char *line,			// the line we are parsing
	int lineno,			// the line number we are parsing
	int *star, 			// returns if line begins with an asterisk
	char *res,			// returns name of the resource
	char *path,			// returns path to physical file, if any
	char *sign,			// returns sign info, if any
	int *nnums,		  // returns number of numbers in array
	char ***nums,		// returns array of numbers after resource name
	int *cond				// current IF/ELSE state
);
extern void CreateCpSignBitmap(int i,int ltwid,Bitmap *bm);
extern void ReadCpBMP(int i,Bitmap *bm,RGBQUAD *pal);
extern void WriteCpBMPs(void);
extern int ReadCPControl(char *ctlname);
