#include "patch.h"

// patcher command data

char *cmdnames[]=
{
	"NULL",
	"MAK",
	"BRK",
	"MER",
	"CMP",
	"DIF",
	"KIL",
	"DUP",
	"CVT",
	"HIST",
	"SETPAL",
	"NBLD",
	"DEMON",
	"CLED",
	"TXTUSE",
	"FLTUSE",
	"REPL",
	"STAT",
	"VIEW",
	"GAMMA",
	"MOVEIT",
	"RENLIN",
	"NOSECRET",
	"STUCK",
	"MUSPUT",
	"HELP",
	"EXIT",
	"QUIT",
	"PATCHER",
};

int (*cmd[])(int arc,char **arv) =
{
	NULL,				//	"NULL",
	makmain,	 		//	"MAK",
	brkmain,	 		//	"BRK",
	mermain,	 		//	"MER",
	cmpmain,	 		//	"CMP",
	difmain,	 		//	"DIF",
	kilmain,	 		//	"KIL",
	dupmain,	 		//	"DUP",
	cvtmain,	 		//	"CVT",
	histmain,	 		//	"HIST",
	spalmain,	 		//	"SETPAL",
	nbldmain,	 		//	"NBLD",
	demmain,	 		//	"DEMON",
	cledmain,	 		//	"CLED",
	txtumain,	 		//	"TXTUSE",
	fltumain,	 		//	"FLTUSE",
	replmain,	 		//	"REPL",
	statmain,	 		//	"STAT",
	viewmain,	 		//	"STAT",
	gammain,	 		//	"GAMMA",
	movemain,	 		//	"MOVEIT",
	renmain,	 		//	"RENLIN",
	nosmain,	 		//	"NOSECRET"
	stucmain,	 		//	"STUCK",
	muspmain,	 		//	"MUSPUT",
	NULL,		 		//	"HELP",
	NULL,		 		//	"EXIT",
	NULL,		 		//	"QUIT",
	NULL,		 		//	"PATCHER",
};
int ncmds = sizeof(cmd)/sizeof(cmd[0]);

int (*cmdhelp[])(int arc,char **arv) =
{
	NULL,
	MakHelp,
	BrkHelp,
	MerHelp,
	CmpHelp,
	DifHelp,
	KilHelp,
	DupHelp,
	CvtHelp,
	HistHelp,
	SetPalHelp,
	NbldHelp,
	DemonHelp,
	CledHelp,
	TxtUseHelp,
	FltUseHelp,
	ReplHelp,
	StatHelp,
	ViewHelp,
	GammaHelp,
	MoveitHelp,
	RenlinHelp,
	NosecretHelp,
	StuckHelp,
	MusputHelp,
	NULL,
	NULL,
	NULL,
	NULL,
};
int ncmdhelps = sizeof(cmd)/sizeof(cmd[0]);

//============================================================================
// This is a shell for Patcher to reduce the size of the executable necessary,
// and to simplify global options. The options PATCHER is run with become the
// default options for all command lines entered in that shell invocation. 
//============================================================================

void FreePatcherStructures()
{
	FreeEditLines(&plines,&nplines);
	FreeEditLines(&elines,&nelines);
	FreeEditLines(&vlines,&nvlines);
	FreeRenLines();
	FreeHist();
	FreeFlatHist();
}

void InitPatcherStructures()
{
	InitEditLines(&plines,&nplines);
	InitEditLines(&elines,&nelines);
	InitEditLines(&vlines,&nvlines);
	InitRenLines();
	InitHist();
	InitFlatHist();
}

int IsPatcherCmd(char *str)
{
	int i;

	for (i=C_MAK;i<=C_PATCHER;i++)
		if (stricmp(str,cmdnames[i])==0)
			return i;
	return 0;
}

int DoPatcherSummary()
{
	dprintf(logfile,"\nPATCHER.EXE %s by Jim Flynn, Copyright 1996-7  -- Freeware --\n",PATCHER_VERSION);
	dprintf(logfile,"\nStart the PATCHER shell, set global defaults\n");
	dprintf(logfile,"\nUsage: PATCHER [options]\n");
	dprintf(logfile,"\n Global options:\n");
	dprintf(logfile,"\n /1 Convert FF_END to F_END, etc.\n");
	dprintf(logfile," /2 Convert F_END to FF_END, etc.\n");
	dprintf(logfile," /C Signs follow case\n");
	dprintf(logfile," /I Insert mode on at start of each line\n");
	dprintf(logfile,"\n Game definition options, use SET PATCHER=options to set defaults:\n");
	dprintf(logfile,"\n /DOOM2 /DOOM /HERETIC /HEXEN specify game\n");
	dprintf(logfile," /D:name[.DEF]  Specify resource definitions\n");
	return(1);
}

int DoHelpHelp()
{
	dprintf(logfile,"\nPatcher %s Help System\n",PATCHER_VERSION);
	dprintf(logfile,"\nType PATCHER and Enter to start the Patcher shell\n");
	dprintf(logfile,"Press F1 in the shell for command line editing help\n");
	dprintf(logfile,"Type QUIT or EXIT to leave the Patcher shell\n");
	dprintf(logfile,"\nType PATCHER in the shell, for Patcher invocation options\n");
	dprintf(logfile,"Type HELP in the shell, for a list of Patcher commands\n");
	dprintf(logfile,"Type any Patcher command name in the shell for syntax help\n");
	dprintf(logfile,"Type HELP <full patcher command> for detailed help on a command\n");
	dprintf(logfile,"Type HELP HELP for this brief description of online help\n");
	dprintf(logfile,"\nRefer to the text file by the same name as each command for details\n");
	dprintf(logfile,"\n");

	return 1;
}

int DoPatcherHelp(int arc,char **arv)
{
	int c,e=0,ky=0;

	if (arc==0)
	{
		dprintf(logfile,"\nName     Options               Parameters\n");
		dprintf(logfile,"----------------------------------------\n");
		dprintf(logfile,"PATCHER  [/12CI]               [single command]\n");
		dprintf(logfile,"MAK      [/I12RCOXKLESWMPTGFU] resfile[.SGN]\n");
		dprintf(logfile,"BRK      [/OJAIDLESWMPTGFU]    wadfile[.WAD]\n");
		dprintf(logfile,"MER      [/N12IDLESWMPTGFU]    base[.WAD|.SGN] patch[.WAD|.SGN] out[.WAD]\n");
		dprintf(logfile,"CMP      [/IDLESWMPTGFU]       base[.WAD|.SGN] patch[.WAD|.SGN]\n");
		dprintf(logfile,"DIF      [/I12DLESWMPTGFU]     revised[.WAD|.SGN] orig[.WAD|.SGN] diff[.WAD]\n");
		dprintf(logfile,"KIL      [/D12]                input[.WAD] kilspec[.SGN|.WAD] output[.WAD]\n");
		dprintf(logfile,"DUP      [/D12]                input[.WAD]\n");
		dprintf(logfile,"CVT      [/IDRGB]              name[.BMP]...\n");
		dprintf(logfile,"HIST     []                    wadfile[.WAD]      .RES may be used for .SGN \n");
		dprintf(logfile,"SETPAL   [/IDLRGB]             resdefn[.DEF]\n");
		dprintf(logfile,"\nEnter HELP CMD [options] [parameters]... for detailed CMD help\n");
		fprintf(stderr,"\nPress Enter for list of DOOM II only utilities, ESC to exit\n");
		ky=WaitKey();
		if (ky==ESC) return (1);
		dprintf(logfile,"\n Support for DOOM II/DOOM only:\n");
		dprintf(logfile,"\nNBLD     [/VOTBSE]             wadname[.wad] [mapname]\n");
		dprintf(logfile,"DEMON    [/HMVWLT]             file{.WAD|.LMP} [DEMO#]\n");
    dprintf(logfile,"CLED     []                    wadfile[.WAD] LEVOBJ[(m[,n])][.FIELD[=value]]\n");
		dprintf(logfile,"TXTUSE   [/RUICDST]            levels[.WAD] [MAPnn[-mm]]... [TEXTURE]...\n");
		dprintf(logfile,"FLTUSE   [/RUDST]              levels[.WAD] [MAPnn[-mm]]... [FLAT]...\n");
		dprintf(logfile,"REPL     []                    wadfile[.WAD] [replfile[.DAT]] [MAPnn]...\n");
		dprintf(logfile,"STAT     [/MPFSHC]             wadfile[.WAD] [MAPnn]\n");
		dprintf(logfile,"GAMMA    [/LUnnn]              wadfile[.WAD] [MAPnn]\n");
		dprintf(logfile,"MOVEIT   [/LG]                 wadfile[.WAD] [MAPnn]\n");
		dprintf(logfile,"RENLIN   [/P]                  wadfile[.WAD] [MAPnn] [nnn]...\n");
		dprintf(logfile,"NOSECRET []                    wadfile[.WAD] [MAPnn[-mm]]... \n");
		dprintf(logfile,"STUCK    [/WTECMA]             wadfile[.WAD] [MAPnn[-mm]]...\n");
		dprintf(logfile,"MUSPUT   []                    musfile{.MID|.MUS|.WAD@n} wadfile[.WAD]@m\n");
		dprintf(logfile,"\nEnter HELP CMD [options] [parameters]... for detailed CMD help\n");
		dprintf(logfile,"\n");
		return(1);
	}

	c = IsPatcherCmd(arv[0]);
	if (0<c && c<=C_MUSPUT)
		e=cmdhelp[c](arc,arv);
	else
		switch(c)
		{
			case C_PATCHER:
				e=DoPatcherSummary();
				break;
			case C_HELP:
				e=DoHelpHelp();
				break;
			case C_QUIT: case C_EXIT:
				printf("\nType QUIT or EXIT to exit the Patcher shell\n");
				e=-1;
				break;
			default:
				e=0;
				break;
		}
	return e;
}

int DoPatcherCmd(int cmdname,int arc,char **arv)
{
	int e,i,c;
	char *p=NULL;

	errorlev=e=0;
	if (arc>0)
	{
		if (logfile) fclose(logfile);
		logfile = NULL;

		for (i=arc-1;i>0;i--)
		if ((p=strrchr(arv[i],'>')))
		{
			Trim(p+1);
			if (p[1]!='\0')
				logfile = fopen(p+1,(p>arv[i] && p[-1]=='>')? "a" : "w");
			else if (i<arc-1)
				logfile = fopen(arv[i+1],(p>arv[i] && p[-1]=='>')? "a" : "w");
			arc = i;
			*p = '\0';
		}
	}

	c = IsPatcherCmd(arv[0]);
	if (0<c && c<=C_MUSPUT)
	{
		e=cmd[c](arc,arv);
	}
	else
		switch(c)
		{
			case C_PATCHER:
				e=DoPatcherSummary();
				break;
			case C_HELP:
				e=DoPatcherHelp(arc-1,arv+1);
				break;
			case C_QUIT: case C_EXIT:
				e=-1;
				break;
			default:
				e=4;
				break;
		}

	if (logfile) fclose(logfile);
	logfile = NULL;

	return e;
}

int PatcherPrompt()
{
	char *p,buffer[512],gopts[16];
	int glen;

	glen=0;
	if (InsOption)
		gopts[glen++] = 'I';
	if (SingleMarker || DoubleMarker)
		gopts[glen++] = SingleMarker? '1' : '2';
	if (FollowCase)
		gopts[glen++] = 'C';
	if (stricmp(defname,"HEXEN.DEF")==0)
	{
		gopts[glen++] = 'H';
		gopts[glen++] = 'X';
	}
	else if (stricmp(defname,"HERETIC.DEF")==0)
	{
		gopts[glen++] = 'H';
		gopts[glen++] = 'R';
	}
	else if (stricmp(defname,"DOOM.DEF")==0)
	{
		gopts[glen++] = 'D';
		gopts[glen++] = '1';
	}
	else if (stricmp(defname,"DOOM2.DEF")==0)
	{
		gopts[glen++] = 'D';
		gopts[glen++] = '2';
	}
	else if (stricmp(defname,"RESOURCE.DEF")==0)
	{
		gopts[glen++] = '-';
		gopts[glen++] = '-';
	}
	gopts[glen] = '\0';

	p = getcwd(buffer,512);
	if (p)
	{
		sprintf(promptstr,"[%s(%s) %s - %s] ",PATCHER_VERSION,gopts,p,GetDateTime());
		dprintf(logfile,"%s\n",promptstr);
		return strlen(promptstr);
	}
	else
	{
		sprintf(promptstr,"[%s(%s) %s] ",PATCHER_VERSION,gopts,GetDateTime());
		dprintf(logfile,"%s\n",promptstr);
		return strlen(promptstr);
	}
}
int main(int argc, char **argv)
{
	int i,e=0,c;
	char buffer[512];
	char **arv = NULL;
	int arc = 0;
	int oldSingle,oldDouble,oldFollow;
	char olddefname[256];
	char *p=NULL,*q=NULL;

	InitCommonStructures();
	InitPatcherStructures();

	if (getenv("NOLEAK")==NULL)					// disable mem leak detect
		atexit(xatexit);

	__system_flags |= __system_call_cmdproc; 	// use shell for system

	strcpy(defname,"RESOURCE");

	if (argc>0)													// UTIL renamed to command
	{
		p = strrchr(argv[0],'/');
		if (p) p++;	else p=argv[0];
		q = strrchr(p,'.');
		if (q) *q='\0';
		c = IsPatcherCmd(p);
		if (q) *q='.';
		if (c && c!=C_PATCHER)
		{
			e = DoPatcherCmd(c,argc,argv);
			FreeCommonStructures();
			FreePatcherStructures();
			exit(e);
		}
	}

	if (argc>1 && (c=IsPatcherCmd(argv[1])))	// single-shot execution
	{
		e = DoPatcherCmd(c,argc-1,argv+1);
		FreeCommonStructures();
		FreePatcherStructures();
		exit(e);
	}

	DoHelpHelp();

	CopyEnvironmentCLArgs("PATCHER",argc,argv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;
	}

	for (i=0;i<nclopts;i++)
	{
		if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
			strcpy(defname,cloptions[i]);
		else if (tolower(cloptions[i][0])=='i')
		{
			InsOption=cloptsense[i];
		}
 		else if (cloptions[i][0]=='1')
		{
			SingleMarker = cloptsense[i];
			DoubleMarker = !cloptsense[i];
		}
 		else if (cloptions[i][0]=='2')
		{
			DoubleMarker = cloptsense[i];
			SingleMarker = !cloptsense[i];
		}
		else if (tolower(cloptions[i][0])=='c')
			FollowCase = cloptsense[i];
		else if (tolower(cloptions[i][0])=='d' && cloptions[i][1]==':')
			strcpy(defname,cloptions[i]+2);
		else dprintf(logfile,"Warn: command line option: %s not understood\n",cloptions[i]);
	}

	FreeCLStrs();

	oldSingle = SingleMarker;
	oldDouble = DoubleMarker;
	oldFollow = FollowCase;
	strcpy(olddefname,defname);
	while (1)
	{
		SingleMarker = oldSingle;
		DoubleMarker = oldDouble;
		FollowCase = oldFollow;
		strcpy(defname,olddefname);

		PatcherPrompt();

   	InitTextData();
		SetEditLines(&plines,&nplines);
		strcpy(buffer,"");
		p = EditCommand(buffer);
		if (p)
		{
			Tokenize(buffer," \t",&arv,&arc);
			strcpy(cmdline,"");
			if (arc>0)
			{
				c = IsPatcherCmd(arv[0]);
				if (c)
					e=DoPatcherCmd(c,arc,arv);
				else
					system(buffer);
			}
			FreeDynStr(&arv,&arc);
		}
		else dprintf(logfile,"\n");

		if (e<0 || e>3) break;
		if (errorlev>3) break;
		FreeCommonStructures();
	}
	if (logfile) fclose(logfile);

	FreeCommonStructures();
	FreePatcherStructures();
	exit(errorlev);
}
