#include "patch.h"

int *lineno=NULL;
int nnos=0;

void FreeRenlinStructures()
{
	FreeLevelData();
	FreeDir(&pdir);
	FreeRenLines();
	FreeResourceDefinition(&ResDef);
	CloseStreams();
}

void FreeRenLines()
{
	FreeDynInt(&lineno,&nnos);
}

void InitRenLines()
{
	lineno=NULL;
	nnos = 0;
}

// Open the control files, make the output wad

int renmain(int argc,char **argv)
{
	int i,c,first=-1;
	int cleared=0;
	char mapsel[256],*p,buffer[256];

	dprintf(logfile,"\n");

	CopyEnvironmentCLArgs("RENLIN",argc,argv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;
	}

	if (!nclfiles)
	{
		dprintf(logfile,"\nRENLIN %s by Jim Flynn, Copyright 1996-7  -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"\nRenumber specified linedefs as lowest in level specified\n");
		dprintf(logfile,"\nUsage: RENLIN [options] wadfile[.wad] [MapNN] [nnn]...\n");
		dprintf(logfile,"\n General options, use SET RENLIN=options to set defaults:\n");
		dprintf(logfile,"\n/P - Prompt for line numbers\n");
		dprintf(logfile,"\n See RENLIN.TXT for details.\n");
						 
		FreeCLStrs();
		return(1);
	}

	cleared=0;
	strcpy(outname,"");
	strcpy(mapsel,"");
	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='p')
		    PromptForEm=cloptsense[i];
		else dprintf(logfile,"Warn: command line option: %s not understood\n",cloptions[i]);
	}
	strcpy(pwadname,clfiles[0]);

	AddExt(pwadname,".WAD");
	AddExt(defname,".DEF");

	InitCategories(defname,&ResDef);

	for (i=1;i<nclfiles;i++)
		if (IsNumeric(clfiles[i]))
			AddDynInt(&lineno,&nnos,atoi(clfiles[i]));
		else if (strlen(mapsel)==0 && IsDMap(clfiles[i]))
			strcpy(mapsel,clfiles[i]);
		else if (IsDMap(clfiles[i]))
			dprintf(logfile,"Warn: Only one map can be specified, all but first ignored\n");
		else
			dprintf(logfile,"Warn: Parameter %s not understood\n",clfiles[i]);
	FreeCLStrs();


	pst = fopen(pwadname,"r+b");
	if (pst!=NULL)
	{
		fread(&phdr,sizeof(WadHeader),1,pst);
		ReadDir(&phdr,&pdir,pst);

		first = -1;
		if (!*mapsel)
		{
			c = CategoryOf("LEVELS",&ResDef);
			if (c>=0)
				for (i=0;i<ResDef.cat[c].nnames;i++)
					if (IsIn(ResDef.cat[c].Names[i].str,&pdir)>=0)
						if (first<0)
							first=i;
						else
						{
							dprintf(logfile,"Warn: %s contains more than one level, only first will be renumbered\n",pwadname);
							break;
						}


			if (first>=0 && first<ResDef.cat[c].nnames)
				strcpy(mapsel,ResDef.cat[c].Names[first].str);
			else dprintf(logfile,"ERROR: No levels found in %s\n",pwadname);
		}

		if (*mapsel)
		{
			ReadLevelData(mapsel,&pdir,pst,NLevObj());

			if (PromptForEm)
			{
				while (1)
				{
					dprintf(logfile,"Enter a line number (blank line terminates input):");
					p = gets(buffer);
					if (!p) break;
					Trim(buffer);
					if (*buffer)
						AddDynInt(&lineno,&nnos,atoi(buffer));
					else break;
				}
			}

			dprintf(logfile,"Doing lines: ");
			for (i=0;i<nnos;i++)	// note order important to line[i] stability
			{
				dprintf(logfile,"%d",lineno[i]);
				if (i<nnos-1) printf(",");
			}
			dprintf(logfile,"\n");


			for (i=0;i<nnos;i++)	// note order important to line[i] stability
				RenumberLine(lineno[i]);

			i = IsIn(mapsel,&pdir);
			if (i>=0)
			{
				i = IsAfter(i,"LINEDEFS",&pdir);
				if (i>=0)
				{
					fseek(pst,pdir.dirp[i].resptr,SEEK_SET);
					SafeWrite(Linedefs,sizeof(Linedef),NLinedefs,pst);
				}
			}

			FreeLevelData();
        	dprintf(logfile,"%s has had the lines specified in %s renumbered lowest in wad\n",pwadname,mapsel);
		}
	}

	//FreeRenlinStructures();
	return errorlev;
}

