#include "patch.h"

//
// REPL.EXE
//
//

typedef enum
{
	SB_NONE,
	SB_WALLS,
	SB_FLOORS,
	SB_THINGS,
	SB_NAMES
} SubKinds;

char *subkinds[]=
{
	"None",
	"Walls",
	"Floors",
	"Things",
	"Names",
};

int ReadReplaceFile(char *subsname)
{
	int c=-1;
	FILE *sst;
	char buffer[256],*p,*q;
	int insec=0,cnt=0;
	int n1,n2;

	sst = fopen(subsname,"r");
	if (sst)
	{
		while(fgets(buffer,256,sst))
		{
			Trim(buffer);
			StripComment(buffer);
			if (*buffer=='[')
			{
				if (cnt>0)
					dprintf(logfile,"Added %d substitutions to %s section\n",cnt,subkinds[c]);
				cnt = 0;
				p = strchr(buffer,']');
				if (p) *p='\0';
				Trim(buffer+1);

				c = -1;
				insec=1;
				if (stricmp(buffer+1,"WALLS")==0)
					c = SB_WALLS;
				else if (stricmp(buffer+1,"FLOORS")==0)
					c = SB_FLOORS;
				else if (stricmp(buffer+1,"THINGS")==0)
					c = SB_THINGS;
				else if (stricmp(buffer+1,"NAMES")==0)
					c = SB_NAMES;
				else insec=0;
			}
			else if (insec && *buffer!=';')
			{
				p = strtok(buffer," \t=\n");
				if (p)
				{
					q = strtok(NULL," \t=\n");
					if (q)
					{
						n1 = (c==SB_THINGS)? ThingNumber(p) : 0;
						n2 = (c==SB_THINGS)? ThingNumber(q) : 0;
						AddVTex2Num(&mtxres[c],&mtxcnt[c],c,p,q,NULL,n1,n2);
						cnt++;
					}
				}
			}
		}
		if (cnt>0 && insec)
			dprintf(logfile,"Added %d substitutions to %s section\n",cnt,subkinds[c]);
		fclose(sst);
	}
	else
	{
		dprintf(logfile,"Can't open replacement file: %s\n",subsname);
		return 1;
	}
	return 0;
}

void FreeSubsStructures()
{
	FreeTextures(&pwt);
	FreeVTex();
	FreeDir(&pdir);
	FreeResourceDefinition(&pResDef);
	FreeResourceDefinition(&ResDef);
	CloseStreams();
}

// Open the control files, make the output wad

int replmain(int argc,char **argv)
{
	int i,j,m,n,k,o,chg=0,err=0;
	int cleared=0,levdir=-1,chgdir=0;
	Category *ctgy,*lcat;
	int subsidx,DoMap[100],lidx;
	char subsname[256];

	dprintf(logfile,"\n");

	CopyEnvironmentCLArgs("REPL",argc,argv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;	
	}

	if (nclfiles<1)
	{
		dprintf(logfile,"\nREPL.EXE %s by Jim Flynn, Copyright 1996-7  -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"Replace textures, flats, things, and level names. \n");
		dprintf(logfile,"\nUsage: REPL [options] wadfile[.WAD] [subsfile[.DAT]] [MAPnn]...\n");
		dprintf(logfile,"\nIn subsfile use sections [NAMES],[FLOORS],[WALLS],[THINGS]\n");
		dprintf(logfile,"with lines like MAP03=MAP01, SHAWN3=ROCK2 in the appropriate sections.\n");
		dprintf(logfile,"If subsfile is missing, REPLACE.DAT is used by default\n");
        dprintf(logfile,"\nSee REPL.TXT for details\n");

		FreeCLStrs();
		return(1);
	}

	strcpy(outname,"");
	cleared=0;
	for (i=0;i<nclopts;i++)
	{
		n = strspn(cloptions[i],"lwtemgspfuLWTEMGSPFU"); // its a sel option
		if (n==strlen(cloptions[i]))	// if all are res selection chars
		{
			if (!cleared)	// if first selection option
			{
				if (cloptsense[i])	// clear all if it adds types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=0;
					DoGraphics=DoSprites=DoPatches=DoFlats=DoNonStandard=0;
				}
				else if (!cloptsense[i]) // set all if it subtracts types
				{
					DoLumps=DoWads=DoTextures=DoEffects=DoMusics=1;
					DoGraphics=DoSprites=DoPatches=DoFlats=DoNonStandard=1;
				}
				cleared=1;	// but don't do it for following selection opts
			}
			for (j=0;j<strlen(cloptions[i]);j++)	// enable or disable
			{										// each resource for
				switch(tolower(cloptions[i][j]))	// which a letter occurs
				{
					case 'l':
						DoLumps = cloptsense[i];
						break;
					case 'w':
						DoWads = cloptsense[i];
						break;
					case 't':
						DoTextures = cloptsense[i];
						break;
					case 'e':
						DoEffects = cloptsense[i];
						break;
					case 'm':
						DoMusics = cloptsense[i];
						break;
					case 'g':
						DoGraphics = cloptsense[i];
						break;
					case 's':
						DoSprites = cloptsense[i];
						break;
					case 'p':
						DoPatches = cloptsense[i];
						break;
					case 'f':
						DoFlats = cloptsense[i];
						break;
					case 'u':
						DoNonStandard = cloptsense[i];
						break;
				}
			}
		}
		else if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
			strcpy(defname,cloptions[i]);
		else if (tolower(cloptions[i][0])=='d' && cloptions[i][1]==':')
			strcpy(defname,cloptions[i]+2);
		else dprintf(logfile,"Warn: command line option: %s not understood\n",cloptions[i]);
	}

	AddExt(defname,".DEF");
	InitCategories(defname,&ResDef);

	strcpy(pwadname,clfiles[0]);
	AddExt(pwadname,".WAD");


	lidx = CategoryOf("LEVELS",&ResDef);		 
	ctgy = lidx>=0? ResDef.cat+lidx : NULL;

	subsidx=-1;
	if (ctgy)
	{
		for (i=0;i<ctgy->nnames;i++)
			DoMap[i] = 1;
		cleared=0;
		for (i=1;i<nclfiles;i++)
		{
			k = StrIdxFind(ctgy->Names,ctgy->nnames,clfiles[i]);
			if (k>=0)
			{
				if (!cleared)
					for (j=0;j<ctgy->nnames;j++)
						DoMap[j] = 0;
				cleared=1;
				DoMap[k] = 1;
			}
			else if (subsidx<0)
				subsidx = i;
			else
			{
				dprintf(logfile,"More than one substitution data file offered.\n");
				FreeCLStrs();
				return 3;
			}
		}
	}
	else
	{
		dprintf(logfile,"No levels in defined game type, can't substitute\n");
		FreeCLStrs();
		return 3;
	}

	if (subsidx>=0)
		strcpy(subsname,clfiles[subsidx]);
	else
		strcpy(subsname,"REPLACE");
	AddExt(subsname,".DAT");

	FreeCLStrs();

    if ((err=ReadReplaceFile(subsname)))
		return err;

	pst = fopen(pwadname,"r+b");
	if (pst)
	{
		fread(&phdr,sizeof(WadHeader),1,pst);
		ReadDir(&phdr,&pdir,pst);
		LocateCategories(&pdir,&ResDef,&pResDef);

		lcat = pResDef.cat+lidx;

		chgdir=0;
		for (j=0;j<lcat->nnames;j++)
		{
			if (DoMap[j])
			{
				levdir = lcat->Names[j].didx;
				if (levdir>=0)
				{
					dprintf(logfile,"Substituting objects in level %s\n",lcat->Names[j].str);
					for (m=0;m<mtxcnt[SB_NAMES];m++)
						if (stricmp(lcat->Names[j].str,mtxres[SB_NAMES][m].name)==0)
						{
							memset(pdir.dirp[levdir].resname,'\0',8);
							strncpy(pdir.dirp[levdir].resname,mtxres[SB_NAMES][m].path,8);
							dprintf(logfile,"%s: Replacing %s with %s\n",lcat->Names[j].str,mtxres[SB_NAMES][m].name,mtxres[SB_NAMES][m].path);
							chgdir=1;
						}
						
					for (i=1;i<11;i++)
						if (strnicmp(pdir.dirp[levdir+i].resname,"THINGS",6)==0)
							break;
					if (i<11)
					{
						o = pdir.dirp[levdir+i].resptr;
					    n = pdir.dirp[levdir+i].reslen;
						Things = (Thing *)malloc(n);

						fseek(pst,o,SEEK_SET);
						NThings=n/sizeof(Thing);
						fread(Things,sizeof(Thing),NThings,pst);
						chg = 0;
						for (k=0;k<NThings;k++)
							for (m=0;m<mtxcnt[SB_THINGS];m++)
								if (Things[k].typ == atoi(mtxres[SB_THINGS][m].args[0]))
								{
									Things[k].typ = atoi(mtxres[SB_THINGS][m].args[1]);
									dprintf(logfile,"%s: Replacing thing #%d, %s with %s\n",lcat->Names[j].str,k,ThingName(atoi(mtxres[SB_THINGS][m].args[0])),ThingName(atoi(mtxres[SB_THINGS][m].args[1])));
									chg = 1;
								}
						if (chg)
						{
							fseek(pst,o,SEEK_SET);
							SafeWrite(Things,sizeof(Thing),NThings,pst);
						}
						free(Things);
						NThings=0;
						Things=NULL;
					}

					for (i=1;i<11;i++)
						if (strnicmp(pdir.dirp[levdir+i].resname,"SIDEDEFS",8)==0)
							break;
					if (i<11)
					{
						o = pdir.dirp[levdir+i].resptr;
					    n = pdir.dirp[levdir+i].reslen;
						Sidedefs = (Sidedef *)malloc(n);

						fseek(pst,o,SEEK_SET);
						NSidedefs=n/sizeof(Sidedef);
						fread(Sidedefs,sizeof(Sidedef),NSidedefs,pst);
						chg = 0;
						for (k=0;k<NSidedefs;k++)
							for (m=0;m<mtxcnt[SB_WALLS];m++)
							{
								if (strnicmp(Sidedefs[k].utxt,mtxres[SB_WALLS][m].name,8)==0)
								{
									memset(Sidedefs[k].utxt,'\0',8);
									strncpy(Sidedefs[k].utxt,mtxres[SB_WALLS][m].path,strlen(mtxres[SB_WALLS][m].path));
									dprintf(logfile,"%s: Replacing upper texture %s on Sidedef #%d with %s\n",lcat->Names[j].str,mtxres[SB_WALLS][m].name,k,mtxres[SB_WALLS][m].path);
									chg = 1;
								}
								if (strnicmp(Sidedefs[k].ntxt,mtxres[SB_WALLS][m].name,8)==0)
								{
									memset(Sidedefs[k].ntxt,'\0',8);
									strncpy(Sidedefs[k].ntxt,mtxres[SB_WALLS][m].path,strlen(mtxres[SB_WALLS][m].path));
									dprintf(logfile,"%s: Replacing normal texture %s on Sidedef #%d with %s\n",lcat->Names[j].str,mtxres[SB_WALLS][m].name,k,mtxres[SB_WALLS][m].path);
									chg = 1;
								}
								if (strnicmp(Sidedefs[k].ltxt,mtxres[SB_WALLS][m].name,8)==0)
								{
									memset(Sidedefs[k].ltxt,'\0',8);
									strncpy(Sidedefs[k].ltxt,mtxres[SB_WALLS][m].path,strlen(mtxres[SB_WALLS][m].path));
									dprintf(logfile,"%s: Replacing lower texture %s on Sidedef #%d with %s\n",lcat->Names[j].str,mtxres[SB_WALLS][m].name,k,mtxres[SB_WALLS][m].path);
									chg = 1;
								}
							}
						if (chg)
						{
							fseek(pst,o,SEEK_SET);
							SafeWrite(Sidedefs,sizeof(Sidedef),NSidedefs,pst);
						}
						free(Sidedefs);
						Sidedefs=NULL;
						NSidedefs=0;
					}

					for (i=1;i<11;i++)
						if (strnicmp(pdir.dirp[levdir+i].resname,"SECTORS",7)==0)
							break;
					if (i<11)
					{
						o = pdir.dirp[levdir+i].resptr;
					    n = pdir.dirp[levdir+i].reslen;
						Sectors = (Sector *)malloc(n);

						fseek(pst,o,SEEK_SET);
						NSectors=n/sizeof(Sector);
						fread(Sectors,sizeof(Sector),NSectors,pst);
						chg = 0;
						for (k=0;k<NSectors;k++)
							for (m=0;m<mtxcnt[SB_FLOORS];m++)
							{
								if (strnicmp(Sectors[k].ctxt,mtxres[SB_FLOORS][m].name,8)==0)
								{
									memset(Sectors[k].ctxt,'\0',8);
									strncpy(Sectors[k].ctxt,mtxres[SB_FLOORS][m].path,strlen(mtxres[SB_FLOORS][m].path));
									dprintf(logfile,"%s: Replacing ceiling %s in Sector #%d with %s\n",lcat->Names[j].str,mtxres[SB_FLOORS][m].name,k,mtxres[SB_FLOORS][m].path);
									chg = 1;
								}
								if (strnicmp(Sectors[k].ftxt,mtxres[SB_FLOORS][m].name,8)==0)
								{
									memset(Sectors[k].ftxt,'\0',8);
									strncpy(Sectors[k].ftxt,mtxres[SB_FLOORS][m].path,strlen(mtxres[SB_FLOORS][m].path));
									dprintf(logfile,"%s: Replacing floor %s in Sector #%d with %s\n",lcat->Names[j].str,mtxres[SB_FLOORS][m].name,k,mtxres[SB_FLOORS][m].path);
									chg = 1;
								}
							}
						if (chg)
						{
							fseek(pst,o,SEEK_SET);
							SafeWrite(Sectors,sizeof(Sector),NSectors,pst);
						}
						free(Sectors);
						NSectors=0;
						Sectors=NULL;
					}
				}
			}
		}

		if (chgdir)
		{
			fseek(pst,phdr.waddiraddr,SEEK_SET);
			SafeWrite(pdir.dirp,sizeof(WadDirEntry),pdir.nentries,pst);
		}
	}

	dprintf(logfile,"\nSubstitutions from %s carried out in %s\n",subsname,pwadname);
	//FreeSubsStructures();
	return errorlev;
}

