#include "patch.h"

/************************************************************
 WSUBS.C
			 /-------------------------\
			||   Wad Subroutnes        ||
			 \-------------------------/
									
************************************************************/

void FreeCommonStructures()
{
	FreeCLStrs();
	FreeVTex();
	FreeLevelData();
	FreePalette();
	FreeDir(&bdir);
	FreeDir(&pdir);
	FreeDir(&odir);
	FreeTextures(&bwt);
	FreeTextures(&pwt);
	FreeTextures(&owt);
	FreeResourceDefinition(&ResDef);
	FreeResourceDefinition(&bResDef);
	FreeResourceDefinition(&pResDef);
	CloseStreams();
}

void InitCommonStructures()
{
	InitCLStrs();
	InitVTex();
	InitLevelData();
	InitPalette();
	InitDir(&bdir);
	InitDir(&pdir);
	InitDir(&odir);
	InitTextures(&bwt);
	InitTextures(&pwt);
	InitTextures(&owt);
	InitResourceDefinition(&ResDef);
	InitResourceDefinition(&bResDef);
	InitResourceDefinition(&pResDef);
	InitStreams();
}


// If SingleMarker or DoubleMarker convert X_END and XX_END tags appropriately


int GetDEMO1Version(char *bwadname)
{
	char *p;
	UBYTE demonum = 0;
	int m;
	FILE *bst;
	WadHeader bhdr;
	WadDir bdir;

	p = strrchr(bwadname,'.');
	if (p) *p='\0';
	strcat(bwadname,".WAD");

	bst = fopen(bwadname,"rb");					// open wad
	if (bst)
	{
		InitDir(&bdir);
		fread(&bhdr,sizeof(WadHeader),1,bst);	// read header
		ReadDir(&bhdr,&bdir,bst);				// read dir
		m = IsIn("DEMO1",&bdir);				// search dir for DEMO1
		if (m>=0)								// if found
		{
			fseek(bst,bdir.dirp[m].resptr,SEEK_SET);	// file pos to DEMO1
			fread(&demonum,1,sizeof(UBYTE),bst);		// read demo ver byte
			demonum -= 100;								// get sig digit
			if (demonum<6 && demonum >9) demonum=0;		// check range

			fclose(bst);
		}
		else dprintf(logfile,"Version identifier wad: %s does not contain a DEMO1\n",bwadname);
		FreeDir(&bdir);
	}
	else dprintf(logfile,"Version identifier wad: %s not found\n",bwadname);

	return(demonum);		// return sig digit of DEMO1 ver in bwadname
}

int NLevObj()
{
	int c;

	if (ResDef.cat==NULL || ResDef.ncats==0)
		return 10;
	c = CategoryOf("LEVOBJ",&ResDef);
	return c<0? 10 : ResDef.cat[c].nnames;
}

/* return true if wde points to a DOOM 1 or DOOM 2 map tag */

int IsDMap(char *mname)
{
	int c;

	c = CategoryOf("LEVELS",&ResDef);
	if (c<0) return 0;
    return StrIdxFind(ResDef.cat[c].Names,ResDef.cat[c].nnames,Str8(mname))<0? 0 : 1;
}


/* return true if wde points level name tag */

int IsMapName(WadDirEntry *wde)
{
	int c;

	c = CategoryOf("LEVELS",&ResDef);
	if (c<0) return 0;
    return StrIdxFind(ResDef.cat[c].Names,ResDef.cat[c].nnames,Str8(wde->resname))<0? 0 : 1;
}

/* return true if wde points level resource */

int IsMapResource(WadDirEntry *wde)
{
	int c;

	c = CategoryOf("LEVOBJ",&ResDef);
	if (c<0) return 0;
    return StrIdxFind(ResDef.cat[c].Names,ResDef.cat[c].nnames,Str8(wde->resname))<0? 0 : 1;
}

int IsMarker(WadDirEntry *wde)
{
	int c;

	for (c=0;c<ResDef.ncats;c++)
	{
		if (ResDef.cat[c].idm==ID_MARKED)
		{
			if (IsCategoryMarker(c,Str8(wde->resname),&ResDef))
				return 1;
		}
	}
	return 0;
}

int IsColTranMap(char *name,WadDir *wdir,ResourceDefinition *rd)
{
	int s,c0,c1;

	if (stricmp(name,"COLORMAP")==0)
		return 1;
	if (stricmp(name,"WATERMAP")==0)
		return 1;

	s = CategoryOf("COLMAPS",rd);
	if (s>=0)
	{
		c0 = IsIn("C_START",wdir);
		c1 = IsAfter(c0,"C_END",wdir);
		if (IsBetween(c0,c1,name,wdir)>=0)
			return 1;
	}

	s = CategoryOf("TRANMAPS",rd);
	if (s>=0)
	{
		c0 = IsIn("T_START",wdir);
		c1 = IsAfter(c0,"T_END",wdir);
		if (IsBetween(c0,c1,name,wdir)>=0)
			return 1;
	}

	return 0;
}


void AddColorTranMaps(char *wadname,WadDir *wdir,FILE *st)
{
	int openedfile=0;
	int readdir=0;
	int oldncmaps=ncmaps;
	int oldntmaps=ntmaps;

	if (!ncmaps)
	{
		cmaps = malloc(2*sizeof(cmaps[0].cmap));
		memmove(cmaps[0].cmap[0],ColorMap[0],34*256);
		memmove(cmaps[1].cmap[0],WaterMap[0],34*256);
		ncmaps = 2;
		strcpy(cmaps[0].name,"COLORMAP");
		strcpy(cmaps[1].name,"WATERMAP");
	}

	if (!st)
	{
		if (!wadname || !*wadname)
			return;
		AddExt(wadname,".WAD");
		st = fopen(wadname,"rb");
		openedfile=1;
	}
	if (st)
	{
		WadDir wd;

		int s,e;

		if (!wdir || !wdir->dirp)
		{
			WadHeader whdr;

			InitDir(&wd);
			fread(&whdr,sizeof(WadHeader),1,st);
			ReadDir(&whdr,&wd,st);
			readdir = 1;
			wdir = &wd;
		}

		s = IsIn("C_START",wdir);
		e = IsIn("C_END",wdir);
		if (s>=0 && e>=0 && e>s+1)
		{
			int i;

			cmaps = realloc(cmaps,(ncmaps+e-s-1)*sizeof(cmaps[0].cmap));
			for (i=0;i<e-s-1;i++)
			{
				strcpy(cmaps[ncmaps+i].name,wdir->dirp[s+1+i].resname);
				fseek(st,wdir->dirp[s+1+i].resptr,SEEK_SET);
				fread(cmaps[ncmaps+i].cmap[0],1,sizeof(cmaps[0].cmap),st);
			}
			ncmaps += e-s-1;
		}

		s = IsIn("T_START",wdir);
		e = IsIn("T_END",wdir);
		if (s>=0 && e>=0 && e>s+1)
		{
			int i;

			tmaps = realloc(tmaps,(ntmaps+e-s-1)*sizeof(tmaps[0].tmap));
			for (i=0;i<e-s-1;i++)
			{
				strcpy(tmaps[ntmaps+i].name,wdir->dirp[s+1+i].resname);
				fseek(st,wdir->dirp[s+1+i].resptr,SEEK_SET);
				fread(tmaps[ntmaps+i].tmap[0],1,sizeof(tmaps[0].tmap),st);
			}
			ntmaps += e-s-1;
		}
		if (openedfile) fclose(st);
		if (readdir) FreeDir(wdir);
	}
	if (Verbose)
		dprintf(logfile,"Read %d colormaps and %d tranmaps from %s\n",
			       ncmaps-oldncmaps,ntmaps-oldntmaps,wadname? wadname : "a wad");
}

void FreeColorTranMaps()
{
	free(cmaps);
	cmaps=NULL;
	free(tmaps);
	tmaps=NULL;
	ncmaps=ntmaps=0;
}


// Find all the maps in a wad whose directory is passed

int ListResMaps(char ***maplist,int *nmaps,WadDir *dir,ResourceDefinition *rd)
{
	*nmaps = 0;
	*maplist = NULL;

	if (rd)
	{
		int n;
		int	c = CategoryOf("LEVELS",rd);

		for (n=0;n<rd->cat[c].nnames;n++)
			if (IsIn(rd->cat[c].Names[n].str,dir)>=0)
				AddDynStr(maplist,nmaps,rd->cat[c].Names[n].str);
	}

	return *nmaps;
}
