#include "patch.h"

static int RC=1,GC=1,BC=1;
static int lookup[256];
static UBYTE out[256];
static char resdefname[256];
static char lumpname[256];
static UBYTE rawpal[768];

char *trailers[]=
{
	"  	; red",
	"  	; green",
	"  	; blue",
	"  	; white",
	"  	; gray",
	"  	; yellow",
};

void FreeSpalResources()
{
	FreePalette();
	FreeResourceDefinition(&ResDef);
}

int spalmain(int argc,char **argv)
{
	int i,n,j;
	int sludone,paldone,palgot;
	int *idp[6];
	char *p,buffer[256];
	FILE *rst;

	dprintf(logfile,"\n");

	strcpy(resdefname,"");
	strcpy(lumpname,"");

	CopyEnvironmentCLArgs("CVT",argc,argv);
	if (ParseCommandLine(-1,2))
	{
		FreeCLStrs();
		return 3;
	}

	if (nclfiles!=1)
	{
		dprintf(logfile,"\nSETPAL.EXE %s by Jim Flynn, Copyright 1997 -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"\nSet PALETTE and SIGNLOOK sections for the specified palette\n");
		dprintf(logfile,"in the resource definition file specified\n");
		dprintf(logfile,"\nUsage: SETPAL [options] /pal-opt resdefn[.DEF]\n");
		dprintf(logfile,"\npal-opt:\n");
		dprintf(logfile," /DOOM2 /DOOM /HERETIC /HEXEN specify game palette\n");
		dprintf(logfile," /I:name specify WAD containing a PLAYPAL for palette\n");
		dprintf(logfile," /D:name[.DEF] specify resource definition palette\n");
		dprintf(logfile," /L[:name[.LMP]] specify binary palette, PLAYPAL.LMP by default\n");
		dprintf(logfile,"\noptions:\n");
		dprintf(logfile," /R:weight integer weight for red in palette match\n");
		dprintf(logfile," /G:weight integer weight for green in palette match\n");
		dprintf(logfile," /B:weight integer weight for blue in palette match\n");
        dprintf(logfile,"See SETPAL.TXT for details\n");

		FreeCLStrs();
		return(1);
	}

	for (i=0;i<nclopts;i++)
	{
		if (stricmp(cloptions[i],"DOOM")==0 || stricmp(cloptions[i],"DOOM2")==0 ||
			     stricmp(cloptions[i],"HERETIC")==0 || stricmp(cloptions[i],"HEXEN")==0)
			strcpy(defname,cloptions[i]);
		else if (tolower(cloptions[i][0])=='d' && cloptions[i][1]==':')
			strcpy(defname,cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='i' && cloptions[i][1]==':')
			strcpy(iwadname,cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='l')
		{
			strcpy(lumpname,"PLAYPAL");
			if (cloptions[i][1]==':')
				strcpy(lumpname,cloptions[i]+2);
		}
		else if (tolower(cloptions[i][0])=='r' && cloptions[i][1]==':')
			RC = atoi(cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='g' && cloptions[i][1]==':')
			GC = atoi(cloptions[i]+2);
		else if (tolower(cloptions[i][0])=='b' && cloptions[i][1]==':')
			BC = atoi(cloptions[i]+2);
		else dprintf(logfile,"Warn: Command line option %s not understood\n",cloptions[i]);

	}

	idp[0] = redidx;
	idp[1] = greenidx;
	idp[2] = blueidx;
	idp[3] = whiteidx;
	idp[4] = grayidx;
	idp[5] = yellowidx;
	for (i=0;i<6;i++)
		for (j=0;j<16;j++)
			out[16*i+j] = idp[i][j];

	strcpy(resdefname,clfiles[0]);
	FreeCLStrs();

	AddExt(resdefname,".DEF");
	AddExt(iwadname,".WAD");
	AddExt(defname,".DEF");
	SupplyExt(lumpname,".LMP");
	InitCategories(defname,&ResDef);

	palgot=0;
	if (*lumpname)								// first try binary lump 
	{
		FILE *ist;
		ist = fopen(lumpname,"rb");
		if (ist)
		{
			dprintf(logfile,"Using binary palette %s\n",lumpname);
			fread(rawpal,768,1,ist);
			fclose(ist);
			palette = malloc(256*sizeof(RGBQUAD));
			for (i=0;i<256;i++)
			{
				palette[i].rgbRed = rawpal[3*i+0];
				palette[i].rgbGreen = rawpal[3*i+1];
				palette[i].rgbBlue = rawpal[3*i+2];
				palette[i].rgbReserved = 0;
			}
			palgot=1;
		}
	}

	if (!palgot && *iwadname)					// then the IWAD, if spec'd
	{
		FILE *ist;
		WadHeader ihdr;
		WadDir idir;

		InitDir(&idir);
		ist = fopen(iwadname,"rb");
		if (ist!=NULL)
		{
			dprintf(logfile,"Using PLAYPAL from IWAD %s for palette\n",iwadname);
			fread(&ihdr,sizeof(WadHeader),1,ist);
			ReadDir(&ihdr,&idir,ist);
			n = IsIn("PLAYPAL",&idir);
			if (n>=0)
				palette = GetPalette(&idir,ist,n);
			fclose(ist);
			free(idir.dirp);
		}
	}
	else if (!palgot)
		palette = GetPalette(NULL,NULL,-1);		// then resdef's or DoomPalette

	sludone=0;
	paldone=0;
	rst = fopen(resdefname,"r+");
	if (rst)
	{
		while ((p=fgets(buffer,256,rst)))
		{
			if (strstr(buffer,"[PALETTE]"))
			{
				fseek(rst,0,SEEK_CUR);
				for (i=0;i<64;i++)
				{
					fprintf(rst,"  ");
					for (j=0;j<4;j++)
					{
						fprintf(rst,"%3d ",palette[4*i+j].rgbRed);
						fprintf(rst,"%3d ",palette[4*i+j].rgbGreen);
						fprintf(rst,"%3d",palette[4*i+j].rgbBlue);
						if (j<3) fprintf(rst," "); else fprintf(rst,"\n");
					}
				}
				fseek(rst,0,SEEK_CUR);
				paldone=1;
			}

			if (p && strstr(buffer,"[SIGNLOOK]"))
			{
				MatchPalettes(DoomPalette,palette,lookup);
				fseek(rst,0,SEEK_CUR);
				for (i=0;i<6;i++)
					for (j=0;j<16;j++)
					{
						fprintf(rst,"%3d",lookup[out[16*i+j]]);
						if (j<16-1) fprintf(rst," "); else fprintf(rst,"%s\n",trailers[i]);
					}
				sludone=1;
				fseek(rst,0,SEEK_CUR);
			}
		}

		if (!paldone)
		{
			fseek(rst,0,SEEK_CUR);
			fprintf(rst,"[PALETTE] !InFo!\n");
			for (i=0;i<64;i++)
			{
				fprintf(rst,"  ");
				for (j=0;j<4;j++)
				{
					fprintf(rst,"%3d ",palette[4*i+j].rgbRed);
					fprintf(rst,"%3d ",palette[4*i+j].rgbGreen);
					fprintf(rst,"%3d",palette[4*i+j].rgbBlue);
					if (j<3) fprintf(rst," "); else fprintf(rst,"\n");
				}
			}
		}

		if (!sludone)
		{
			fseek(rst,0,SEEK_CUR);
			MatchPalettes(DoomPalette,palette,lookup);
			fprintf(rst,"[SIGNLOOK] !InFo!\n");
			for (i=0;i<6;i++)
				for (j=0;j<16;j++)
				{
					fprintf(rst,"%3d",lookup[out[16*i+j]]);
					if (j<16-1) fprintf(rst," "); else fprintf(rst,"%s\n",trailers[i]);
				}
		}
		fclose(rst);
	}
	else dprintf(logfile,"Resource file to write %s, not found\n",resdefname);

	//FreeSpalResources();
	return 0;
}


