#include "patch.h"

/************************************************************
 SUBS.C
			 /-------------------------\
			||   General Subroutnes    ||
			 \-------------------------/
									
************************************************************/

/* return the gcd of two ints */

int agcd(int a,int b)
{
	a = abs(a);
	b = abs(b);
	if (!a) return b;
	if (!b) return a;
	if (a==b) return a;

	if (a>b)
		return agcd(a%b,b);
	else
		return agcd(b%a,a);
}


/* Return a string equal to an 8 char buffer in content */

char *Str8(char *buff)
{
	static char bf[9];

	strncpy(bf,buff,8);
	bf[8]='\0';
	return bf;
}


// Return the level number corresponding to level name
// Note for DOOM 1 the value of digits xy is returned for ExMy

int LevelNumber(char *levtag)
{
	char nums[3];

	if (IsD1Map(levtag))
	{
		nums[0]=levtag[1];
		nums[1]=levtag[3];
		nums[2]='\0';
	}
	else if (IsD2Map(levtag))
	{
		nums[0]=levtag[3];
		nums[1]=levtag[4];
		nums[2]='\0';
	}
	else return 0;

	return atoi(nums);
}

/* return true if mname is a DOOM 1 map tag ExMy */

int IsD1Map(char *mname)
{
	return (mname[0]=='E' || mname[0]=='e')
		&& (mname[2]=='M' || mname[2]=='m')
		&& isdigit(mname[1])
		&& isdigit(mname[3])
		&& mname[4]=='\0';
}

// Return true if mname is a DOOM 2 map tag Mapnn */

int IsD2Map(char *mname)
{
	return !strnicmp("MAP",mname,3)
		&& isdigit(mname[3])
		&& isdigit(mname[4])
		&& mname[5]=='\0';
}
					
void PrintVTex(FILE *st,VTex *mt,int i)
{
	int j;

	if (!mt || i<0) return;

	fprintf(st,"ResName: %9s Id: %2d",mt[i].name,mt[i].id);
	for (j=0;j<mt[i].nargs;j++)
		fprintf(st," %4s",mt[i].args[j]);
	fprintf(st,"\n");

	if (mt[i].path[0])
		fprintf(st," Specified path: %s\n",mt[i].path);
	if (mt[i].sign[0])
		fprintf(st," Sign description: %s\n",mt[i].sign);
}

// Print an VTex

void PrintVTexList(FILE *st,VTex* mt,int mtcnt,char *secname)
{
	int i;

	if (!mt || mtcnt<=0)
		fprintf(st,"%s is empty\n",secname);
	else
	{
		fprintf(st,"%s\n",secname);
		for (i=0;i<mtcnt;i++)
			PrintVTex(st,mt,i);
	}
}

// Add a resource to a VTex list

void AddVTex(VTex **dt,int *n,int id,char *nam,char *pth,char *sgn,int nargs,char **args)
{
	int i;

	*dt = (VTex *)realloc(*dt,((*n)+1)*sizeof(VTex));

	(*dt)[*n].id = id;

	(*dt)[*n].name = malloc(strlen(nam)+1);
	strcpy((*dt)[*n].name,nam);

	if (pth && *pth)
	{
		(*dt)[*n].path = malloc(strlen(pth)+1);
		strcpy((*dt)[*n].path,pth);
	}
	else (*dt)[*n].path=NULL;

	if (sgn && *sgn)
	{
		(*dt)[*n].sign = malloc(strlen(sgn)+1);
		strcpy((*dt)[*n].sign,sgn);
	}
	else (*dt)[*n].sign=NULL;

	(*dt)[*n].nargs = nargs;
	(*dt)[*n].args = malloc(nargs*sizeof(char *));
	
	for (i=0;i<nargs;i++)
	{
		(*dt)[*n].args[i] = malloc(1+strlen(args[i]));
	  strcpy((*dt)[*n].args[i],args[i]); 
	}

	(*n)++;
}

void AddVT(VTex **dt,int *n,VTex *data)
{
	AddVTex(dt,n,data->id,data->name,data->path,data->sign,data->nargs,data->args);
}

// Add the typical resource with two numeric args to a VTex list

void AddVTex2Num(VTex **dt,int *n,int id,char *nam,char *pth,char *sgn,int num1,int num2)
{
	char **nums=NULL,buf[34];
	int nnums=0;

	AddDynStr(&nums,&nnums,itoa(num1,buf,10));
	AddDynStr(&nums,&nnums,itoa(num2,buf,10));
	AddVTex(dt,n,id,nam,pth,sgn,nnums,nums);
	FreeDynStr(&nums,&nnums);
}

// Find a VTex with a certain name as initial substring

int IsInVTexN(char *nam,VTex *list,int len,int n)
{
	int i;

	for (i=0;i<len;i++)
		if (strnicmp(nam,list[i].name,n)==0)
			return i;
	return -1;
}

// Find a VTex with a certain name in a list

int IsInVTex(char *nam,VTex *list,int len)
{
	return IsInVTexN(nam,list,len,8);
}

// Append as VTex to *dt if different, else set VTex named nam to these vals

void PutVTex(VTex **dt,int *n,int id,char *nam,char *pth,char *sgn,int nargs,char **args)
{
	int i,j;

	i = IsInVTex(nam,*dt,*n);
	if (i<0)
		AddVTex(dt,n,id,nam,pth,sgn,nargs,args);
	else
	{
		VTex *v = (*dt)+i;

		v->id = id;

		v->name = realloc(v->name,strlen(nam)+1);
		if (nam)
			strcpy(v->name,nam);
		else
			v->name = NULL;

		if (pth && *pth)
		{
			v->path = realloc(v->path,1+strlen(pth));
			strcpy(v->path,pth);
		}
		else
			v->path=NULL;

		if (sgn && *sgn)
		{
			v->sign = realloc(v->sign,1+strlen(sgn));
			strcpy(v->sign,sgn);
		}
		else
			v->sign=NULL;

		if (nargs<v->nargs)
			for (j=nargs;j<v->nargs;j++)
			{
				free(v->args[j]);
				v->args[j]=NULL;
			}
		v->args=realloc(v->args,nargs*sizeof(char *));
		if (nargs>v->nargs)
			for (j=v->nargs;j<nargs;j++)
				v->args[j]=NULL;

		v->nargs = nargs;
		for (j=0;j<nargs;j++)
			v->args[j] = realloc(v->args[j],1+strlen(args[j]));
		  strcpy(v->args[j],args[j]);
	}
}

// Append as VTex to *dt if different, else do nothing

int AddNewVTex(VTex **dt,int *n,int id,char *nam,char *pth,char *sgn,int nargs,char **args)
{
	int i;

	i = IsInVTex(nam,*dt,*n);
	if (i<0)
		AddVTex(dt,n,id,nam,pth,sgn,nargs,args);
	return i==-1;
}

int AddNewVT(VTex **dt,int *n,VTex *data)
{
	return AddNewVTex(dt,n,data->id,data->name,data->path,data->sign,data->nargs,data->args);
}

int AddNewVTex2Num(VTex **dt,int *n,int id,char *nam,char *pth,char *sgn,int num1,int num2)
{
	int i;

	i = IsInVTex(nam,*dt,*n);
	if (i<0)
		AddVTex2Num(dt,n,id,nam,pth,sgn,num1,num2);

	return i==-1;
}

// Free all global VTex arrays

void FreeVTex()
{
	int i,j,k;

	for (j=0;j<MAXCAT;j++)
		if (mtxcnt[j])
		{
			for (i=0;i<mtxcnt[j];i++)
			{
				free(mtxres[j][i].name);
				free(mtxres[j][i].path);
				free(mtxres[j][i].sign);
				for (k=0;k<mtxres[j][i].nargs;k++)
					free(mtxres[j][i].args[k]);
			  mtxres[j][i].args=NULL;
				mtxres[j][i].nargs=0;
			}
			free(mtxres[j]);
			mtxres[j]=NULL;
			mtxcnt[j]=0;
		}
}

// Initialize global VTex pointers

void InitVTex()
{
	int i;

	for (i=0;i<MAXCAT;i++)
	{
		mtxres[i]=NULL;
		mtxcnt[i]=0;
	}
}


// Add a string at the end of a dynamic array of dynamic strings

void AddDynStr(char ***dynstrp,int *ndynstr,char *str)
{
	// add one more dynamic string pointer to array

	*dynstrp = (char **) realloc(*dynstrp, (*ndynstr+1)*sizeof(char *));

	// allocate the bytes for new string on new pointer

	(*dynstrp)[*ndynstr] = (char *) malloc(1+strlen(str));
	strcpy((*dynstrp)[*ndynstr],str);	// copy string bytes and terminator

	(*ndynstr)++;						// note one more dynamic string
}

// deallocate a dynamic array of dynamic strings

void FreeDynStr(char ***dynstrp,int *ndynstr)
{
	int i;

	for (i=0;i<*ndynstr;i++)			// free the strings
		free((*dynstrp)[i]);
	free(*dynstrp);						// free array of string pointers
	*ndynstr = 0;
	*dynstrp = NULL;					// ready for next use
}

// Add an integer to a dynamic array of them

void AddDynInt(int **dynintp,int *n,int val)
{
	// add one more integer

	*dynintp = (int *) realloc(*dynintp, ((*n)+1)*sizeof(int));
	(*dynintp)[*n] = val;

	(*n)++;
}

// Add a short to a dynamic array of them

void AddDynShort(short **dynintp,int *n,short val)
{
	// add one more short

	*dynintp = (short *) realloc(*dynintp, ((*n)+1)*sizeof(short));
	(*dynintp)[*n] = val;

	(*n)++;
}

int AddNewDynShort(short **dynintp,int *n,short val)
{
	int i;

	for (i=0;i<*n;i++)
		if ((*dynintp)[i]==val)
			return 0;

	// add one more short

	*dynintp = (short *) realloc(*dynintp, ((*n)+1)*sizeof(short));
	(*dynintp)[*n] = val;

	(*n)++;
	return 1;
}

// Add an integer to a dynamic array of them

int AddNewDynInt(int **dynintp,int *n,int val)
{
	int i;

	for (i=0;i<*n;i++)
		if ((*dynintp)[i]==val)
			return 0;

	// add one more integer

	*dynintp = (int *) realloc(*dynintp, ((*n)+1)*sizeof(int));
	(*dynintp)[*n] = val;

	(*n)++;
	return 1;
}

void DelDynInt(int **dynintp,int *n,int idx)
{
	// delete the idx-th integer

	if (*dynintp==NULL || n==0 || idx<0)
		return;

	if (idx<*n-1)
		memmove((*dynintp)+idx,(*dynintp)+idx+1,*n-1-idx);
	(*n)--;
	if (*n>0)
	{
		*dynintp = (int *) realloc(*dynintp, (*n)*sizeof(int));
	}
	else FreeDynInt(dynintp,n);
}

// deallocate a dynamic array of integers

void FreeDynInt(int **dynintp,int *n)
{
	free(*dynintp);
	*n = 0;
	*dynintp = NULL;
}

// deallocate a dynamic array of shorts

void FreeDynShort(short **dynintp,int *n)
{
	free(*dynintp);
	*n = 0;
	*dynintp = NULL;
}

// Read the specified environment variable and use it as default command
// line if present. Add the command line arguments at the end.

void CopyEnvironmentCLArgs(char *envar,int arc,char **arv)
{
	char *p,*q;
	int i;

	if ((p = getenv(envar))!=NULL)
	{
		strcpy(cmdline,p);
		q=strdup(p);
		Tokenize(q," \t",&clstrs,&nclstrs);
		xlibfree(q);
	}
	for (i=1;i<arc;i++)
	{
		AddDynStr(&clstrs,&nclstrs,arv[i]);
		strcat(cmdline," ");
		strcat(cmdline,arv[i]);
	}
}

// Read the command line arguments and separate options and parameters

int ParseCommandLine(int maxopts,int maxfiles)
{
	int i,n;
	
	nclfiles=nclopts=0;
	for (i=0;i<nclstrs;i++)
	{
		if (clstrs[i][0]=='-' || clstrs[i][0]=='/' || clstrs[i][0]=='+')
		{
			if (maxopts>=0 && nclopts==maxopts)
			{
				dprintf(logfile,"Only %d options allowed on command line\n",maxopts);
				return(1);
			}
			n = nclopts;
			AddDynStr(&cloptions,&n,clstrs[i]+1);
			AddDynInt(&cloptsense,&nclopts,clstrs[i][0]=='-'? 0 : 1);
		}
		else
		{
			if (maxfiles>=0 && nclfiles==maxfiles)
			{
				dprintf(logfile,"Only %d parameters allowed on command line\n",maxfiles);
				return(1);
			}
			AddDynStr(&clfiles,&nclfiles,clstrs[i]);
		}
	}

	return 0;
}

int ParseSlashOptionLine(int maxopts,int maxfiles)
{
	int i,j,k,m,n;
	char temp[128];
	
	nclfiles=nclopts=0;
	for (i=0;i<nclstrs;i++)
	{
		if (clstrs[i][0]=='/')
		{
			if (maxopts>=0 && nclopts==maxopts)
			{
				dprintf(logfile,"Only %d options allowed on command line\n",maxopts);
				return(1);
			}
			k = n = nclopts;
			strcpy(temp,clstrs[i]+1);
			Tokenize(temp,"/",&cloptions,&n);
			for (j=0,m=0;j<n-k;j++)
			{
				AddDynInt(&cloptsense,&nclopts,clstrs[i][m]=='-'? 0 : 1);
				m += 1+strlen(cloptions[k+j]);
			}					  
		}
		else
		{
			if (maxfiles>=0 && nclfiles==maxfiles)
			{
				dprintf(logfile,"Only %d parameters allowed on command line\n",maxfiles);
				return(1);
			}
			AddDynStr(&clfiles,&nclfiles,clstrs[i]);
		}
	}

	return 0;
}

// Return the next larger power of two

int NextPowerTwo(int n)
{
	int i;

	for (i=0;i<32;i++)
		if ((1<<i)>=n) break;

	return 1<<i;
}

// Add the tokens in str delimited by chars in dlm to the dynamic
// array of dynamic strings *dynstrp of length *ndynstr

int Tokenize (char *str,char *dlm,char ***dynstrp,int *ndynstr)
{
	char *t,*s;

	s = strdup(str);
	t = strtok(s,dlm);
	while (t!=NULL)
	{
		AddDynStr(dynstrp,ndynstr,t);
		t = strtok(NULL,dlm);
	}
	xlibfree(s);
	return *ndynstr;
}

// Add the next n-1 tokens in str delimited by chars in dlm to the dynamic
// array of dynamic strings *dynstrp of length *ndynstr. Then if str not
// exhausted add the remainder of str (minus initial delimiters) as
// one more string.

int TokenizeN (char *str,char *dlm,char ***dynstrp,int *ndynstr, int n)
{
	char *t,*s;
	int cnt=0,p;

	s = strdup(str);
	t = strtok(s,dlm);			 				// 0-th string
	while (t!=NULL && ++cnt<n)
	{
		AddDynStr(dynstrp,ndynstr,t); 			// add cnt-1 th string
		t = strtok(NULL,cnt==n-1? "" : dlm);	// on last call get rest
	}
	p = strspn(t,dlm);			 				// skip delims before last
	AddDynStr(dynstrp,ndynstr,t+p);				// add rest as n-1th string

	xlibfree(s);
	return *ndynstr;
}



// get a normal or extended key press

int GetKey(void)
{
	int ch;

	if (!kbhit()) return 0;
	ch = getch();
	if (!ch) return getch()+EXTEND_KEY;
	return ch;
}

int WaitKey(void)
{
	int ch;

	while (!kbhit())
		;
	ch = getch();
	if (!ch) return getch()+EXTEND_KEY;
	return ch;
}

void GetString(char *buffer)
{
	char *p;

	fgets(buffer,256,stdin);
	p = strrchr(buffer,'\n');
	if (p && p[1]=='\0') *p='\0';
}


char *stringf(char *fmt,...)
{
    va_list arg_ptr;
    int n;
	static char buffer[256];

    /* Use vsprintf to expand the format string and arguments */

    va_start(arg_ptr,fmt);
    n = vsprintf(buffer,fmt,arg_ptr);
    va_end(arg_ptr);
    
	return buffer;
}

char *GetStringElt(StringArray *sa,int n)
{
	static char *theElt=NULL;

	theElt = (char *)realloc(theElt,sa->len+1);
	strcpy(theElt,"");

	if (n>=0 && n<sa->num)
	{
		strncpy(theElt,sa->strbuf+sa->len*n,sa->len);
		theElt[sa->len]='\0';
	}
	return theElt;
}

int BStringEltFind(StringArray *sa,int strt,int end,char *theString)
{
	int mid;

	if (end-strt<2)
	{
		if (strnicmp(theString,sa->strbuf+sa->len*strt,sa->len)<=0)
			return strt;
		if (strnicmp(theString,sa->strbuf+sa->len*end,sa->len)<=0)
			return end;
		return end+1;
	}
	mid = (strt + end)/2;
	if (strnicmp(theString,sa->strbuf+sa->len*mid,sa->len)==0)
		return mid;
	if (strnicmp(theString,sa->strbuf+sa->len*mid,sa->len)<0)
		return BStringEltFind(sa,strt,mid,theString);
	else
		return BStringEltFind(sa,mid,end,theString);
	return 0;
}

int BStringFind(StringArray *sa,char *theString)
{
	return BStringEltFind(sa,0,sa->num-1,theString);
}

int StringFind(StringArray *sa,char *theString)
{
	int i;

	for (i=0;i<sa->num;i++)
		if (strnicmp(GetStringElt(sa,i),theString,sa->len)==0)
			return i;
	return -1;
}

int ExistStringElt(StringArray *sa,char *theString,int *n)
{
	*n = BStringFind(sa,theString);
	if (*n<0 || *n>sa->num-1) return 0;
	return (strnicmp(GetStringElt(sa,*n),theString,sa->len)==0)? 1 : 0;
}

int IsStringEltEqual(StringArray *sa,int n,char *theString)
{
	return strnicmp(GetStringElt(sa,n),theString,sa->len)==0? 1 : 0;
}

void PutStringElt(StringArray *sa,int n,char *theString)
{
	strncpy(sa->strbuf+sa->len*n,theString,sa->len);
}

void AddStringSorted(StringArray *sa,char *theString)
{
	int i;

	i = BStringFind(sa,theString);
	if (strnicmp(theString,GetStringElt(sa,i),sa->len)!=0)
	{
		for (i=0;i<8 && theString[i];i++)
			theString[i]=toupper(theString[i]);

		sa->strbuf = (char *)realloc(sa->strbuf,sa->len*sa->num);
		memmove(sa->strbuf+sa->len*i+sa->len,sa->strbuf+sa->len*i,(sa->num-i)*sa->len);
		strncpy(sa->strbuf+sa->len*i,theString,sa->len);
		sa->num++;
	}
}

void AppendStringElt(StringArray *sa,char *theString)
{
	int i;

	if (StringFind(sa,theString)<0)
	{
		for (i=0;i<sa->len && theString[i];i++)
			theString[i]=toupper(theString[i]);

		sa->strbuf = realloc(sa->strbuf,sa->len*(sa->num+1));
		strncpy(sa->strbuf+sa->len*sa->num,theString,sa->len);
		sa->num++;
	}
}

void FreeStringArray(StringArray *sa)
{
	free(sa->strbuf);
	sa->strbuf=NULL;
	sa->num=0;
}

void InitStringArray(StringArray *sa,int len)
{
	sa->strbuf=NULL;
	sa->len=len;
	sa->num=0;
}

// Scan a BMP palette relative to the DOOM palette, and return a 256 byte
// lookup that translates each BMP palette index to the closest DOOM palette
// index. To color convert the image, replace the palette with the DOOM
// palette, and substitute the pixel value i, with lookup[i] for each
// image byte.

int MatchColor(RGBQUAD *doompal,RGBQUAD *col)
{
	RGBQUAD *q;
	int j,k=0,m,t;

	m = 0x7fffffff;
	for (j=0,q=doompal;j<256;j++,q++)
	{
		t=colordist(col,q);
		if (t<m)
		{
			m = t;
		  k = j;
		}
	}
	return k;
}

void MatchColors(RGBQUAD *bmppal,RGBQUAD *doompal,UBYTE *lookup)
{
	int i;
	RGBQUAD *p;

	for (i=0,p=bmppal;i<256;i++,p++)
		lookup[i] = MatchColor(doompal,p);
}

static int RC=10,GC=10,BC=10;

int colordist(RGBQUAD *col1,RGBQUAD *col2)
{
	int r1 = col1->rgbRed;
	int g1 = col1->rgbGreen;
	int b1 = col1->rgbBlue;
	r1 -= col2->rgbRed;
	g1 -= col2->rgbGreen;
	b1 -= col2->rgbBlue;

	return RC*r1*r1 + GC*g1*g1 + BC*b1*b1;
}

void setweights(int r,int g,int b)
{
	RC=r; GC=g; BC=b;
}

static char printfbuffer[512];

int dprintf(FILE *logfile,char *fmt,...)
{
  va_list arg_ptr;
  int n;
    
  /* Use vsprintf to expand the format string and arguments */

  va_start(arg_ptr,fmt);
  n = vsprintf(printfbuffer,fmt,arg_ptr);
  va_end(arg_ptr);
    
  /* print the formatted string to console and output file as optioned */
	if (!QuietMode) printf("%s",printfbuffer);
	if (logfile) fprintf(logfile,"%s",printfbuffer);
    return(n);
}

int aprintf(char *fmt,...)
{
  va_list arg_ptr;
  int n;
    
  /* Use vsprintf to expand the format string and arguments */

  va_start(arg_ptr,fmt);
  n = vsprintf(printfbuffer,fmt,arg_ptr);
  va_end(arg_ptr);
    
  /* print the formatted string to stderr if stdout redirected */
	n=printf("%s",printfbuffer);
	if (!isatty(1))
		fprintf(stderr,"%s",printfbuffer);

  return(n);
}

void InitCLStrs()
{
	clstrs=NULL;
	nclstrs=0;
	nclopts=nclfiles=0;
	cloptions=clfiles=NULL;
	cloptsense=NULL;
}

void FreeCLStrs()
{
	int zero=nclopts;

	FreeDynStr(&clstrs,&nclstrs);
	FreeDynStr(&clfiles,&nclfiles);
	FreeDynStr(&cloptions,&nclopts);
	FreeDynInt(&cloptsense,&zero);
}

void FreeSegs(Segment **segp,int *nsegp)
{
	free(*segp);
	*segp=NULL;
	*nsegp = 0;
}

void InitSegs(Segment **segp,int *nsegp)
{
	*segp=NULL;
	*nsegp = 0;
}

// close all streams

void InitStreams(void)
{
	bst = NULL;
	pst = NULL;
	ost = NULL;
}

// close all streams

int CloseStreams(void)
{
  if (bst) fclose(bst);
	bst = NULL;
  if (pst) fclose(pst);
	pst = NULL;
  if (ost) fclose(ost);
	ost = NULL;
	return 0;
}

int NumsOnly(char *str)
{
	for (;isdigit(*str);str++)
		;
	return (*str? 0 : 1);
}

int IsNumeric(char *str)
{
	if (str==NULL) return 0;							// NULL not numeric
	if (*str=='\0') return 0;							// empty string not
	if (strlen(str)==1) return isdigit(str[0])? 1 : 0;	// single digit is
	if
	(
		strspn(str,"+-0123456789")==strlen(str)			// all chars in +-0-9
		&&
		strspn(str+1,"0123456789")==strlen(str+1)		// all but first 0-9
	)
		return 1;
	return 0;
}


static int starttic;
static int curtic;
static int timertic[16];
static int timerval[16];
static int timerrun[16];
static int timermod[16];

int dostics()
{
	struct dostime_t d_time;

	_dos_gettime(&d_time);
	curtic = d_time.hour*360000L+d_time.minute*6000L+d_time.second*100L+d_time.hsecond;
	return curtic - starttic;
}

void InitTics()
{
	int i;

	starttic = 0;
	starttic = dostics();
	for (i=0;i<16;i++)
	{
		timertic[i] = 0;
		timerval[i] = 0;
		timerrun[i] = 0;
		timermod[i] = TMR_ONESHOT;
	}
}

void StartTicTimer(int tid,int val,int mod)
{
	timertic[tid] = dostics();
	timerval[tid] = timertic[tid]+val;
	timerrun[tid] = 1;
}

int CheckTicTimer(int tid)
{
	if (!timerrun[tid]) return 0;
	if (dostics()>timerval[tid])
	{
		timerrun[tid]=0;
		timerval[tid]=0;
		return 1;
	}
	return 0;
}

int DeltaTicTimer(int tid)
{
	if (!timerrun[tid]) return 0;
	return dostics()-timertic[tid];
}

void StopTicTimer(int tid)
{
	timerrun[tid]=0;
}

int IsInBox(int x,int y,int x1,int y1,int x2,int y2)
{
    return((x1<=x && x<=x2 && y1<=y && y<=y2)? 1 : 0);
}

char *stristr(char *str1,char *str2)
{
	char *p1=strdup(str1);
	char *p2=strdup(str2);
	char *q;
	
	strupr(p1); strupr(p2);
	q = strstr(p1,p2);

	if (q) q = str1+(q-p1);

	xlibfree(p1);
	xlibfree(p2);
	return q;
}

void FreePalette()
{
	free(palette);
	palette=NULL;
}

void InitPalette()
{
	palette=NULL;
}

void ConvertMarker(char *resn)
{
	if
	(
		SingleMarker
		&&
		resn[2]=='_'
		&&
		resn[0]==resn[1]
		&&
		(
			resn[0]=='P'
			||
			resn[0]=='F'
			||
			resn[0]=='S'
		)
		&&
		(
			strcmp(resn+3,"END")==0
			||
			strcmp(resn+3,"START")==0
		)
	)
		memmove(resn,resn+1,strlen(resn));	
	else if
	(
		DoubleMarker
		&&
		resn[1]=='_'
		&&
		resn[0]!='D'		// speeds up check
		&&
		resn[0]!='M'		// by rejecting musics and menus
		&&
		(
			strcmp(resn,"S_START")==0
			||
			strcmp(resn,"F_START")==0
			||
			strcmp(resn,"P_START")==0
			||
			strcmp(resn,"S_END")==0
			||
			strcmp(resn,"F_END")==0
			||
			strcmp(resn,"P_END")==0
		)
	)
		memmove(resn+1,resn,1+strlen(resn));
}

/* test two boxes for having an intersection */

int Intersect(short *xy,short *zw)
{
    if (xy[2]<zw[0] || xy[3]<zw[1]) return 0;
    if (zw[2]<xy[0] || zw[3]<xy[1]) return 0;
    return 1;
}
    

