#include "patch.h"

/************************************************************
 TEXTURE.C
			 /-------------------------\
			||  Textures and PNames    ||
			 \-------------------------/
									
************************************************************/

void InitTextures(WadTextures *wt)
{
	wt->pnames.npnames = 0;
	wt->pnames.patchname = NULL;

	wt->t1.N = 0;
	wt->t1.offs = NULL;
	wt->t1.textures = NULL;

	wt->t2.N = 0;
	wt->t2.offs = NULL;
	wt->t2.textures = NULL;

	wt->t3.N = 0;
	wt->t3.offs = NULL;
	wt->t3.textures = NULL;
}

void FreeTextures(WadTextures *wt)
{
	int i;

	if (wt->pnames.patchname)
		free(wt->pnames.patchname[0]);
	free(wt->pnames.patchname);

	for (i=0;i<wt->t1.N;i++)
		free(wt->t1.textures[i].patches);
	for (i=0;i<wt->t2.N;i++)
		free(wt->t2.textures[i].patches);
	for (i=0;i<wt->t3.N;i++)
		free(wt->t3.textures[i].patches);

	free(wt->t1.textures);
	free(wt->t2.textures);
	free(wt->t3.textures);
	free(wt->t1.offs);
	free(wt->t2.offs);
	free(wt->t3.offs);

	InitTextures(wt);
}

/* Return a string that represents a patch name, the p-th in PNames */

char *GetPName(PNamesLump *pl,int p)
{
	static char pnbuff[9];

	strcpy(pnbuff,"");
	if (pl->npnames>p)
		strncpy(pnbuff,pl->patchname[p],8);
	pnbuff[8]='\0';

	return (pnbuff);
}

/* Return the patch index in PNames of a patch name */

int GetPIdx(PNamesLump *pl,char *pnamb)
{
	int i;
	char pnam[9];

	strncpy(pnam,pnamb,8);
	pnam[8]='\0';

	for (i=0;i<pl->npnames;i++)
		if (stricmp(pnam,GetPName(pl,i))==0)
			return i;
	return -1;
}


/* Print a patchname */

void PrintPName(WadTextures *wt,int p)
{
	if (wt->pnames.npnames>p)
		dprintf(logfile,"%-8s\n",Str8(wt->pnames.patchname[p]));
}

/* Print a list of PNames */

void PrintPNames(WadTextures *wt)
{
	int i;

	for (i=0;i<wt->pnames.npnames;i++)
	    PrintPName(wt,i);
	dprintf(logfile,"\n");
}

/* Print a texture description */

void PrintTexture(WadTextures *wt,TextureDesc *td)
{
	int i;

	if (td->npatches>0)
	{
		dprintf(logfile,"%-8s %d %d\n",Str8(td->textname),td->twid,td->thgt);
		for (i=0;i<td->npatches;i++)
			dprintf(logfile,"*    %-8s %d %d\n",GetPName(&wt->pnames,td->patches[i].pidx),td->patches[i].xoff,td->patches[i].yoff);
	}
}

/* Print a list of Textures and Patches they use ala DeuTex */

void PrintTextureLump(WadTextures *wt, TextureLump *t)
{
	int i;

	for (i=0;i<t->N;i++)
		PrintTexture(wt,t->textures+i);
	
}

/* Print all three existing texture lumps */

void PrintTextures(WadTextures *wt)
{
	if (wt->pnames.npnames>0)
	{
		dprintf(logfile,"PNames:\n");
		PrintPNames(wt);
	}
	if (wt->t1.N>0)
	{
		dprintf(logfile,"Texture Lump TEXTURE1:\n");
		PrintTextureLump(wt,&wt->t1);
	}
	if (wt->t2.N>0)
	{
		dprintf(logfile,"Texture Lump TEXTURE2:\n");
		PrintTextureLump(wt,&wt->t2);
	}
	if (wt->t3.N>0)
	{
		dprintf(logfile,"Texture Lump TEXTURE3:\n");
		PrintTextureLump(wt,&wt->t3);
	}
}

// Add the patch name pnam to the PNames lump pl

void AddPName(PNamesLump *pl,char *pnam)
{
	int i,n;

	n = pl->npnames;
	for (i=0;i<n;i++)
		if (strnicmp(pl->patchname[i],pnam,8)==0) return;

	pl->patchname = (char **)realloc(pl->patchname,(n+1)*sizeof(char *));
	if (n==0) pl->patchname[0]=NULL;

	pl->patchname[0] = (char *)realloc(pl->patchname[0],(n+1)*8);
	for (i=1;i<=n;i++)
		pl->patchname[i] = pl->patchname[i-1]+8;

	strncpy(pl->patchname[n],pnam,8);
	pl->npnames++;
}

// Add the texture *td to the texture lump *ol, translating indices if !isbase

void AddTexture(TextureLump *ol,TextureDesc *td, int isbase)
{
	int j;

	// reallocate for one new offset
	ol->offs = (long *)realloc(ol->offs,(1+ol->N) * sizeof(long));

	// all offsets increase by the size of the new offset
	for (j=0;j<ol->N;j++)
		ol->offs[j] += sizeof(long);

	// set new offset += TextureDesc - Patch pointer + patches
	if (ol->N)
		ol->offs[ol->N] = ol->offs[ol->N-1] + sizeof(TextureDesc) - sizeof(Patch *)
			+ ol->textures[ol->N-1].npatches * sizeof(Patch);
	else
		ol->offs[ol->N] = sizeof(long)+4;

	// reallocate one more texture description
	ol->textures = (TextureDesc *)realloc(ol->textures,(1+ol->N) * sizeof(TextureDesc));

	// copy the new description
	ol->textures[ol->N] = *td;

	// allocate the patch array
	ol->textures[ol->N].patches = (Patch *)malloc(td->npatches * sizeof(Patch));

	// copy the patch array, translate the patchname indices to output indices
	for (j=0;j<td->npatches;j++)
	{
		ol->textures[ol->N].patches[j] = td->patches[j];
		if (!isbase)
			ol->textures[ol->N].patches[j].pidx = XIdx[td->patches[j].pidx];
	}

	// note one more texture in the lump
	ol->N++;
}

int cmptxt(const void *tp1,const void *tp2)
{
    TextureDesc *td1,*td2;
    td1 = (TextureDesc *)tp1;
    td2 = (TextureDesc *)tp2;
    return strnicmp(td1->textname,td2->textname,8);
}

void SortTextures(WadDir *dir,FILE *wst, WadTextures *wt)
{
	int i;

    if (wt->t1.N > 1)
        qsort(wt->t1.textures+1,wt->t1.N-1,sizeof(TextureDesc),cmptxt);
    if (wt->t2.N > 1)
        qsort(wt->t2.textures,wt->t2.N,sizeof(TextureDesc),cmptxt);
    if (wt->t3.N > 1)
        qsort(wt->t3.textures,wt->t3.N,sizeof(TextureDesc),cmptxt);

	for (i=0;i<wt->t1.N;i++)
		if (i)
			wt->t1.offs[i] = wt->t1.offs[i-1]+sizeof(TextureHeader)+wt->t1.textures[i-1].npatches*sizeof(Patch);
		else
			wt->t1.offs[i]= sizeof(long) + (wt->t1.N)*sizeof(long);
	for (i=0;i<wt->t2.N;i++)
		if (i)
			wt->t2.offs[i] = wt->t2.offs[i-1]+sizeof(TextureHeader)+wt->t2.textures[i-1].npatches*sizeof(Patch);
		else
			wt->t2.offs[i]= sizeof(long) + (wt->t2.N)*sizeof(long);
	for (i=0;i<wt->t3.N;i++)
		if (i)
			wt->t3.offs[i] = wt->t3.offs[i-1]+sizeof(TextureHeader)+wt->t3.textures[i-1].npatches*sizeof(Patch);
		else
			wt->t3.offs[i]= sizeof(long) + (wt->t3.N)*sizeof(long);
}

// Read the PNAMES from a wad file

void ReadPNames(WadDir *dir,FILE *wst,WadTextures *wt)
{
	int o,n,i,m,p;

	if ((p=IsIn("PNAMES",dir))>=0)
	{
		o = dir->dirp[p].resptr;
		n = dir->dirp[p].reslen;
		fseek(wst,o,SEEK_SET);
		fread(&m,sizeof(long),1,wst);
		wt->pnames.npnames = m;
		wt->pnames.patchname = (char **)malloc(m*sizeof(char *));
		wt->pnames.patchname[0] = (char *)malloc(8*m);
		for (i=1;i<m;i++)
			wt->pnames.patchname[i] = wt->pnames.patchname[i-1]+8;
		fread(wt->pnames.patchname[0],8*m,1,wst);
	}
	else
	{
		wt->pnames.npnames = 0;
		wt->pnames.patchname = NULL;
	}
}

// Read the TEXTUREn from a wad file

void ReadTextures(WadDir *dir,FILE *wst,WadTextures *wt)
{
	int o,n,i,p;
	TextureHeader th;

	if ((p=IsIn("TEXTURE1",dir))!=-1)
	{
		o = dir->dirp[p].resptr;
		n = dir->dirp[p].reslen;
		fseek(wst,o,SEEK_SET);

		fread(&(wt->t1.N),sizeof(long),1,wst);
		if (wt->t1.N>0)
		{
			wt->t1.offs = (long *)malloc(wt->t1.N * sizeof(long));
			fread(wt->t1.offs,sizeof(long),wt->t1.N,wst);
			wt->t1.textures = (TextureDesc *)malloc(wt->t1.N * sizeof(TextureDesc));
			for (i=0;i<wt->t1.N;i++)
			{
				fseek(wst,o+wt->t1.offs[i],SEEK_SET);
				fread(&th,sizeof(TextureHeader),1,wst);
				strncpy(wt->t1.textures[i].textname,th.textname,8);
				wt->t1.textures[i].twid = th.twid;
				wt->t1.textures[i].thgt = th.thgt;
				wt->t1.textures[i].npatches = th.npatches;
				wt->t1.textures[i].res1 = wt->t1.textures[i].res2=0;
				wt->t1.textures[i].res3 = wt->t1.textures[i].res4=0;
				if (wt->t1.textures[i].npatches)
				{
					wt->t1.textures[i].patches =
						(Patch *)malloc(wt->t1.textures[i].npatches*sizeof(Patch));
					fread(wt->t1.textures[i].patches,sizeof(Patch),wt->t1.textures[i].npatches,wst);
				}
				else
				{
					dprintf(logfile,"WARNING: texture %s has 0 patches\n",Str8(wt->t1.textures[i].textname));
				}
			}
		}
		else
		{
			wt->t1.textures=NULL;
		}
	}
	else
	{
		wt->t1.N = 0;
		wt->t1.textures=NULL;
	}

	if ((p=IsIn("TEXTURE2",dir))!=-1)
	{
		o = dir->dirp[p].resptr;
		n = dir->dirp[p].reslen;
		fseek(wst,o,SEEK_SET);

		fread(&(wt->t2.N),sizeof(long),1,wst);
		if (wt->t2.N>0)
		{
			wt->t2.offs = (long *)malloc(wt->t2.N * sizeof(long));
			fread(wt->t2.offs,sizeof(long),wt->t2.N,wst);
			wt->t2.textures = (TextureDesc *)malloc(wt->t2.N * sizeof(TextureDesc));
			for (i=0;i<wt->t2.N;i++)
			{
				fseek(wst,o+wt->t2.offs[i],SEEK_SET);
				fread(&th,sizeof(TextureHeader),1,wst);
				strncpy(wt->t2.textures[i].textname,th.textname,8);
				wt->t2.textures[i].twid = th.twid;
				wt->t2.textures[i].thgt = th.thgt;
				wt->t2.textures[i].npatches = th.npatches;
				wt->t2.textures[i].res1 = wt->t2.textures[i].res2=0;
				wt->t2.textures[i].res3 = wt->t2.textures[i].res4=0;
				if (wt->t2.textures[i].npatches)
				{
					wt->t2.textures[i].patches = (Patch *)malloc(wt->t2.textures[i].npatches*sizeof(Patch));
					fread(wt->t2.textures[i].patches,sizeof(Patch),wt->t2.textures[i].npatches,wst);
				}
				else
				{
					dprintf(logfile,"WARNING: texture %s has 0 patches\n",Str8(wt->t2.textures[i].textname));
				}
			}
		}
		else
		{
			wt->t2.textures=NULL;
		}
	}
	else
	{
		wt->t2.N = 0;
		wt->t2.textures=NULL;
	}

	if ((p=IsIn("TEXTURE3",dir))!=-1)
	{
		o = dir->dirp[p].resptr;
		n = dir->dirp[p].reslen;
		fseek(wst,o,SEEK_SET);

		fread(&(wt->t3.N),sizeof(long),1,wst);
		if (wt->t3.N>0)
		{
			wt->t3.offs = (long *)malloc(wt->t3.N * sizeof(long));
			fread(wt->t3.offs,sizeof(long),wt->t3.N,wst);
			wt->t3.textures = (TextureDesc *)malloc(wt->t3.N * sizeof(TextureDesc));
			for (i=0;i<wt->t3.N;i++)
			{
				fseek(wst,o+wt->t3.offs[i],SEEK_SET);
				fread(&th,sizeof(TextureHeader),1,wst);
				strncpy(wt->t3.textures[i].textname,th.textname,8);
				wt->t3.textures[i].twid = th.twid;
				wt->t3.textures[i].thgt = th.thgt;
				wt->t3.textures[i].npatches = th.npatches;
				wt->t3.textures[i].res1 = wt->t3.textures[i].res2=0;
				wt->t3.textures[i].res3 = wt->t3.textures[i].res4=0;
				if (wt->t3.textures[i].npatches)
				{
					wt->t3.textures[i].patches = (Patch *)malloc(wt->t3.textures[i].npatches*sizeof(Patch));
					fread(wt->t3.textures[i].patches,sizeof(Patch),wt->t3.textures[i].npatches,wst);
				}
				else
				{
					dprintf(logfile,"WARNING: texture %s has 0 patches\n",Str8(wt->t3.textures[i].textname));
				}
			}
		}
		else
		{
			wt->t3.textures=NULL;
		}
	}
	else
	{
		wt->t3.N = 0;
		wt->t3.textures=NULL;
	}
}

void WriteTextures(FILE *wadf, WadTextures *wt,unsigned long *offs)
{
	int j,npats,l;
	unsigned long n;
	TextureLump *tl[3];

	tl[0] = &wt->t1;
	tl[1] = &wt->t2;
	tl[2] = &wt->t3;

	///////////////////////////////////////////////////////////////////
	// TEXTUREn
	// if textures exist in the output WadTextures, write the TEXTUREn resource
	for (l=0;l<3;l++)
		if (tl[l]->N)
		{
			fseek(wadf,*offs,SEEK_SET);
			SafeWrite(&tl[l]->N,sizeof(int),1,wadf);
			SafeWrite(tl[l]->offs,sizeof(int),tl[l]->N,wadf);
			for (j=0,npats=0;j<tl[l]->N;j++)
			{
				SafeWrite(tl[l]->textures+j,sizeof(TextureDesc)-sizeof(Patch *),1,wadf);
				SafeWrite(tl[l]->textures[j].patches,sizeof(Patch),tl[l]->textures[j].npatches,wadf);
				npats += tl[l]->textures[j].npatches;
			}
			n = sizeof(int) + sizeof(int) * tl[l]->N
				+ (sizeof(TextureDesc)-sizeof(Patch *)) * tl[l]->N
				+ npats * sizeof(Patch);
			AddDirEntry(&odir,*offs,n,"TEXTURE1");
			*offs += n;
		}
}

void ReplaceTextureLumps(FILE *wadf,WadDir *dir,WadTextures *wt)
{
	int j,b;
	TextureLump *tl;

	tl = &wt->t1;
	b = IsIn("TEXTURE1",dir);
	if (b>=0)
	{
		fseek(wadf,dir->dirp[b].resptr,SEEK_SET);
		SafeWrite(&tl->N,sizeof(int),1,wadf);
		if (tl->N)
		{
			SafeWrite(tl->offs,sizeof(int),tl->N,wadf);
			for (j=0;j<tl->N;j++)
			{
				SafeWrite(tl->textures+j,sizeof(TextureDesc)-sizeof(Patch *),1,wadf);
				SafeWrite(tl->textures[j].patches,sizeof(Patch),tl->textures[j].npatches,wadf);
			}
		}
	}
	tl = &wt->t2;
	b = IsIn("TEXTURE2",dir);
	if (b>=0)
	{
		fseek(wadf,dir->dirp[b].resptr,SEEK_SET);
		SafeWrite(&tl->N,sizeof(int),1,wadf);
		if (tl->N)
		{
			SafeWrite(tl->offs,sizeof(int),tl->N,wadf);
			for (j=0;j<tl->N;j++)
			{
				SafeWrite(tl->textures+j,sizeof(TextureDesc)-sizeof(Patch *),1,wadf);
				SafeWrite(tl->textures[j].patches,sizeof(Patch),tl->textures[j].npatches,wadf);
			}
		}
	}
	tl = &wt->t3;
	b = IsIn("TEXTURE3",dir);
	if (b>=0)
	{
		fseek(wadf,dir->dirp[b].resptr,SEEK_SET);
		SafeWrite(&tl->N,sizeof(int),1,wadf);
		if (tl->N)
		{
			SafeWrite(tl->offs,sizeof(int),tl->N,wadf);
			for (j=0;j<tl->N;j++)
			{
				SafeWrite(tl->textures+j,sizeof(TextureDesc)-sizeof(Patch *),1,wadf);
				SafeWrite(tl->textures[j].patches,sizeof(Patch),tl->textures[j].npatches,wadf);
			}
		}
	}
}

void WritePNames(FILE *wadf, WadTextures *wt,unsigned long *offs)
{
	unsigned long n;

	///////////////////////////////////////////////////////////////////
	// PNAMES
	// if there are pnames in the output WadTextures, write the PNAMES resource
	if (wt->pnames.npnames)
	{
		fseek(wadf,*offs,SEEK_SET);
		SafeWrite(&wt->pnames.npnames,sizeof(int),1,wadf);
		SafeWrite(wt->pnames.patchname[0],8,wt->pnames.npnames,wadf);
		n = sizeof(int) + 8 * wt->pnames.npnames;
		AddDirEntry(&odir,*offs,n,"PNAMES");
		*offs += n;
	}
}

void ReplacePNamesLump(FILE *wadf,WadDir *dir,WadTextures *wt)
{
	int n;

	///////////////////////////////////////////////////////////////////
	// PNAMES
	// if there are pnames in the output WadTextures, write the PNAMES resource
	if (wt->pnames.npnames)
	{
		n = IsIn("PNAMES",dir);
		if (n>=0)
		{
			fseek(wadf,dir->dirp[n].resptr,SEEK_SET);
			SafeWrite(&wt->pnames.npnames,sizeof(int),1,wadf);
			SafeWrite(wt->pnames.patchname[0],8,wt->pnames.npnames,wadf);
		}
	}
}


// Merging textures starts by merging the pname resources. We then build new
// TEXTURE1, TEXTURE2, and TEXTURE3 resources in memory, converting each patch
// index in the original descriptions to the new merged pnames indices, 
// The new TEXTURE1 contains all textures named in the base TEXTURE1,
// replaced as needed by any textures in the patch wad's TEXTURE1, TEXTURE2,
// or TEXTURE3 resources that have the same name as a base wad TEXTURE1. It
// also contains, at the end, all textures in the patch wad's TEXTURE1 that
// do not occur in any base wad TEXTUREn section.

void MergePNames(WadTextures *bwt,WadTextures *pwt,WadTextures *owt)
{
	int i,m,n;

	// Merge the pnames
	// 1) start the merged list with the base wad's PNAMES
	// 2) for every patchname in patch wad but not base wad add to end

	m = bwt->pnames.npnames+pwt->pnames.npnames;  // output len overestimate
	if (m==0) return;

	owt->pnames.patchname = (char **)malloc(m*sizeof(char *)); // overalloc
	owt->pnames.patchname[0] = (char *)malloc(m*8);
	for (i=1;i<m;i++)							// singly allocated array so
		owt->pnames.patchname[i] = owt->pnames.patchname[i-1]+8; // set ptrs
	for (i=0;i<bwt->pnames.npnames;i++)			// copy base's pnames
		strncpy(owt->pnames.patchname[i],bwt->pnames.patchname[i],8);
	owt->pnames.npnames = bwt->pnames.npnames;	// set length so far
  
	n=0;
	while (DoTextures && n<pwt->pnames.npnames)	// while not at end of list
	{
		while						// search for entry NOT in base wad
		(
			n<pwt->pnames.npnames
			&&
			GetPIdx(&bwt->pnames,Str8(pwt->pnames.patchname[n]))>=0
		)
			n++;

		// then add it to end of out wad's pnames
		if (n<pwt->pnames.npnames)
			strncpy(owt->pnames.patchname[owt->pnames.npnames++],
				    pwt->pnames.patchname[n++], 8);
	}

	// shrink to required allocation
	if (owt->pnames.npnames)
	{
		owt->pnames.patchname = (char **)realloc(owt->pnames.patchname,sizeof(char *)*owt->pnames.npnames);
		owt->pnames.patchname[0] = (char *)realloc(owt->pnames.patchname[0],8*owt->pnames.npnames);
		for (i=1;i<owt->pnames.npnames;i++)			// singly allocated array so
			owt->pnames.patchname[i] = owt->pnames.patchname[i-1]+8; // set ptrs
	}
	else
	{
		free(owt->pnames.patchname[0]);
		free(owt->pnames.patchname);
		owt->pnames.patchname=NULL;
	}

	// create index translation table from patch pnames to output pnames
	XIdx = (int *)malloc(owt->pnames.npnames*sizeof(int));
	for (i=0;i<pwt->pnames.npnames;i++)
	{
		XIdx[i] = GetPIdx(&owt->pnames,pwt->pnames.patchname[i]);
		if (!IsIn(Str8(pwt->pnames.patchname[i]),&pdir))
			XIdx[i] = GetPIdx(&bwt->pnames,pwt->pnames.patchname[i]);
		if (DoTextures && XIdx[i]<0)
		{
			dprintf(logfile,"Fatal - %s not defined in base or patch wad\n",Str8(pwt->pnames.patchname[i]));
			exit(1);
		}
	}
}

void FreeXIdx()
{
	free(XIdx);
	XIdx=NULL;
}

void AddFirst()
{
	TextureDesc td;

	// AADUMMY1 always first texture output in lump n
	if (DoTextures)
	{
		strncpy(td.textname,"AADUMMY1",8);
		td.res1 = td.res2 = td.res3 = td.res4 = 0;
		td.twid = td.thgt = 64;
		td.npatches = 1;
		td.patches = malloc(sizeof(Patch));
		
		
		td.patches[0].xoff = 0;
		td.patches[0].yoff = 0;
		td.patches[0].pidx = 0;
		td.patches[0].cmap = 0;
		td.patches[0].stepdir=1;

		AddTexture(&owt.t1,&td,1);
		free(td.patches);
	}
}

void MergeTextures(WadTextures *bt,WadTextures *pt,WadTextures *ot)
{
	int i,j,b,p,o;
	int first;
	char textnam[9];
	TextureLump *tlb[3];
	TextureLump *tlp[3];
	TextureLump *tlo[3];

	tlb[0] = &bt->t1;  tlp[0] = &pt->t1;  tlo[0] = &ot->t1;
	tlb[1] = &bt->t2;  tlp[1] = &pt->t2;  tlo[1] = &ot->t2;
	tlb[2] = &bt->t3;  tlp[2] = &pt->t3;  tlo[2] = &ot->t3;

	// First we create the output wads TEXTURE1 resource
	// This contains every texture named in the base wad, but uses
	// the definitions in the patch wad if they exist instead. It
	// also contains all textures in the patch, but not the base,
	// that are in the TEXTURE1,2 or 3 section of the patch.

	// add to output wad all texture definitions superseded by the
	// patch wad, and all definitions from the base wad that are not.
	// TEXTURE1 section required, TEXTURE2, TEXTURE3 optional


	first=1;
	for (b=0;b<3;b++)	// for each base texture
	{
		o = b;
		if (!ResourceExists("TEXTURES",stringf("TEXTURE%d",b+1),&ResDef))
			o = 0;
		for (i=0;i<tlb[b]->N;i++)
		{
			strcpy(textnam,Str8(tlb[b]->textures[i].textname));
			if (stricmp(textnam,"AADUMMY1")==0) continue; // already done

			if (DoTextures && !NewOnly)
			{
				for (p=0;p<3;p++)
				{
					for (j=0;j<tlp[p]->N;j++)
					{
						if (stricmp(textnam,Str8(tlp[p]->textures[j].textname))==0)
						{
							// use patch wads TEXTURE1 definition
							if (first)
							{
								AddFirst();
								first = 0;
							}
							AddTexture(tlo[o],tlp[p]->textures+j,0);
							goto nextone;
						}
					}
				}
			}

			// use base wad's TEXTURE1 definition
			if (first)
			{
				AddFirst();
				first = 0;
			}
			AddTexture(tlo[o],tlb[b]->textures+i,1);
nextone:;
		}
	}

	// Add to output wad all textures from the patches TEXTUREb that are
	// not in any base wad texture section.

	if (DoTextures)
	{
		for (b=0;b<3;b++)
		{
			o = b;
			if (!ResourceExists("TEXTURES",stringf("TEXTURE%d",b+1),&ResDef))
				o = 0;
			for (i=0;i<tlp[b]->N;i++)
			{
				strcpy(textnam,Str8(tlp[b]->textures[i].textname));
				if (stricmp(textnam,"AADUMMY1")==0) continue; // already done
				for (p=0;p<3;p++)
				{
					for (j=0;j<tlb[p]->N;j++)
					{
						if (stricmp(textnam,Str8(tlb[p]->textures[j].textname))==0)
							goto nxtone;
					}
				}
				if (first)
				{
					AddFirst();
					first = 0;
				}
				AddTexture(tlo[o],tlp[b]->textures+i,0);
nxtone:;
			}
		}
	}
	FreeXIdx();
}

int FindTextureN(WadTextures *wt,char *txtname,int *lumpnum)
{
	int i,j;
	TextureLump *tl[3];

	tl[0] = &wt->t1;
	tl[1] = &wt->t2;
	tl[2] = &wt->t3;

	for (j=0;j<3;j++)
	{
		*lumpnum=j;
		for (i=0;i<tl[j]->N;i++)
			if (strnicmp(txtname,tl[j]->textures[i].textname,8)==0)
				break;
		if (i<tl[j]->N)
			return i;
	}
	return -1;
}

int FindTexture(TextureLump *tl,char *txtname)
{
	int i;

	for (i=0;i<tl->N;i++)
		if (strnicmp(txtname,tl->textures[i].textname,8)==0)
			break;
	if (i<tl->N)
		return i;
	return -1;
}
