#include "patch.h"

////// Thing Identification and Property Retrieval ////////////////////////////
//
//	Some routines to test for kinds of Things and properties they may possess
//
//  Data about things is stored in three arrays: thingdata1, 2, and 3.
//  These represent the three discontiguous ranges in Thing type in DOOM.
//

int ThingProps(unsigned typ)
{
	if (typ<90)
		return thingdata1[typ].props;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].props;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].props;
	else if (typ>=4000 && typ<4005)
		return thingdata4[typ-4001].props;
	else if (typ>=5000 && typ<5003)
		return thingdata4[typ-5001].props;
	else
		return -1;
}

int Radius(Thing t)
{
	switch (t.typ)
	{
		case PLAYER1:
		case PLAYER2:
		case PLAYER3:
		case PLAYER4:		return rad_dat[0];
		case TROOPER:		return rad_dat[1];
		case WOLFSOLDIER:	return rad_dat[2];
		case SARGEANT:		return rad_dat[3];
		case COMMANDO:		return rad_dat[4];
		case IMPS:			return rad_dat[5];
		case DEMON:			return rad_dat[6];
		case SPECTRE:		return rad_dat[7];
		case LOSTSOUL:		return rad_dat[8];
		case CACODEMON:		return rad_dat[9];
		case HELLKNIGHT:	return rad_dat[10];
		case BARON:			return rad_dat[11];
		case ARACHNOTRON:	return rad_dat[12];
		case PAINELEMENTAL:	return rad_dat[13];
		case REVENANT:		return rad_dat[14];
		case MANCUBUS:		return rad_dat[15];
		case ARCHVILE:		return rad_dat[16];
		case SPIDERDEMON:	return rad_dat[17];
		case CYBERDEMON:	return rad_dat[18];
		case BOSSBRAIN:		return rad_dat[19];
		case HANGINGKEEN:	return rad_dat[20];
		case BARREL:		return rad_dat[21];
		case LARGEBRNTREE:	return rad_dat[22];
		default:
			if (ThingProps(t.typ)&GET)
				return rad_dat[23];
			return rad_dat[24];
	}
}

// Return the height of a thing type
	
int Height(Thing t)					  
{
	switch (t.typ)
	{								
		case PLAYER1:    
		case PLAYER2:    
		case PLAYER3:    
		case PLAYER4:		return hgt_dat[0];
		case TROOPER:		return hgt_dat[1];
		case WOLFSOLDIER:	return hgt_dat[2];
		case SARGEANT:		return hgt_dat[3];
		case COMMANDO:		return hgt_dat[4];
		case IMPS:			return hgt_dat[5];
		case DEMON:			return hgt_dat[6];
		case SPECTRE:		return hgt_dat[7];
		case LOSTSOUL:		return hgt_dat[8];
		case CACODEMON:		return hgt_dat[9];
		case HELLKNIGHT:	return hgt_dat[10];
		case BARON:			return hgt_dat[11];
		case ARACHNOTRON:	return hgt_dat[12];
		case PAINELEMENTAL:	return hgt_dat[13];
		case REVENANT:		return hgt_dat[14];
		case MANCUBUS:		return hgt_dat[15];
		case ARCHVILE:		return hgt_dat[16];
		case SPIDERDEMON:	return hgt_dat[17];
		case CYBERDEMON:	return hgt_dat[18];
		case BOSSBRAIN:		return hgt_dat[19];
		case HANGINGKEEN:	return hgt_dat[20];
		case BARREL:		return hgt_dat[21];
		case LARGEBRNTREE:	return hgt_dat[22];
		default:   			return hgt_dat[24];
	}
}

int ReadExternalThingData(char *fn)
{
	FILE *st;
	int radius_mode,i,j;
	char *p,buffer[256],filepath[256];

	rad_dat[0]	 = 16;
	rad_dat[1]	 = 20;  
	rad_dat[2]	 = 20;
	rad_dat[3]	 = 20;
	rad_dat[4]	 = 20;
	rad_dat[5]	 = 20;
	rad_dat[6]	 = 30;
	rad_dat[7]	 = 30;
	rad_dat[8]	 = 16;
	rad_dat[9]	 = 31;
	rad_dat[10]  = 24;
	rad_dat[11]  = 24;
	rad_dat[12]  = 64;
	rad_dat[13]  = 31;
	rad_dat[14]  = 20;
	rad_dat[15]  = 48;
	rad_dat[16]  = 20;
	rad_dat[17]  = 128;
	rad_dat[18]  = 40;
	rad_dat[19]  = 16;
	rad_dat[20]  = 16;
	rad_dat[21]  = 10;
	rad_dat[22]  = 32;
	rad_dat[23]  = 16;
	rad_dat[24]  = 20;

	hgt_dat[0]	 = 56;
	hgt_dat[1]	 = 56; 
	hgt_dat[2]	 = 56;
	hgt_dat[3]	 = 56;
	hgt_dat[4]	 = 56;
	hgt_dat[5]	 = 56;
	hgt_dat[6]	 = 56;
	hgt_dat[7]	 = 56;
	hgt_dat[8]	 = 56;
	hgt_dat[9]	 = 56;
	hgt_dat[10]	 = 64;
	hgt_dat[11]	 = 64;
	hgt_dat[12]	 = 64;
	hgt_dat[13]	 = 56;
	hgt_dat[14]	 = 56;
	hgt_dat[15]	 = 64;
	hgt_dat[16]	 = 56;
	hgt_dat[17]	 = 100;
	hgt_dat[18]	 = 110;
	hgt_dat[19]  = 56;
	hgt_dat[20]	 = 72;
	hgt_dat[21]	 = 42;
	hgt_dat[22]	 = 16;
	hgt_dat[23]	 = 16;
	hgt_dat[24]	 = 16;

	if (!strlen(fn))
	{
		if ((p=getenv("STKDATA")))
			strcpy(buffer,p);
		else
			strcpy(buffer,"RADIUS.TXT");
	}
	else strcpy(buffer,fn);

	if (strchr(buffer,':')==NULL && strchr(buffer,'\\')==NULL)
	{
		if ((p=searchpath(buffer)))
			strcpy(filepath,p);
		else
			return 1;
	}
	else
		strcpy(filepath,buffer);

	st = fopen(filepath,"r");
	if (!st) return 1;

	radius_mode = -1;
	i=j=0;
	while (fgets(buffer,256,st))
	{
		Trim(buffer);
		if (!strnicmp(buffer,"[radius]",8))
		{
			radius_mode=1;
			continue;
		}
		else if (!strnicmp(buffer,"[height]",8))
		{
			radius_mode=0;
			continue;
		}
		if (*buffer==';' || !(p=strchr(buffer,'='))) continue;

		if (radius_mode==1)
		{
			p++;
			Trim(p);
			rad_dat[i++]=atoi(p);
		}
		else if (radius_mode==0)
		{
			p++;
			Trim(p);
			hgt_dat[j++]=atoi(p);
		}
	}
	fclose(st);
	return 0;
}

/* Find and report the thing a point is in */

int ThingOf(int x, int y)
{
	int i,r,cx,cy;

	for (i=0;i<NThings;i++)
	{
		r = Radius(Things[i]);
		cx = Things[i].x;
		cy = Things[i].y;
		if (cx-r<=x && x<=cx+r && cy-r<=y && y<=cy+r)
			return i;
	}
	return -1;
}

int IsLight(int typ)			// Type is a LIGHT - source of light sprite
{
	if (typ<90)
		return thingdata1[typ].props&LIGHT;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].props&LIGHT;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].props&LIGHT;
	else if (typ>=4000 && typ<4005)
		return thingdata3[typ-4001].props&LIGHT;
	else if (typ>=5000 && typ<5003)
		return thingdata3[typ-5001].props&LIGHT;
	else
		return 0;
}

int IsItem(int typ)			// Type is an ITEM - scores at end of level
{
	if (typ<90)
		return thingdata1[typ].props&ITEM;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].props&ITEM;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].props&ITEM;
	else if (typ>=4000 && typ<4005)
		return thingdata3[typ-4001].props&ITEM;
	else if (typ>=5000 && typ<5003)
		return thingdata3[typ-5001].props&ITEM;
	else
		return 0;
}

int IsAmmo(int typ)			// Type is AMMO for a weapon
{
	if (typ<90)
		return thingdata1[typ].props&AMMO;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].props&AMMO;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].props&AMMO;
	else if (typ>=4000 && typ<4005)
		return thingdata3[typ-4001].props&AMMO;
	else if (typ>=5000 && typ<5003)
		return thingdata3[typ-5001].props&AMMO;
	else
		return 0;
}							

int IsSmallAmmo(int typ) 	// Type is AMMO for a weapon
{
	switch(typ)
	{
	    case 2007:
		case 2008:
		case 2010:
		case 2047:
			return 1;
		default:
			return 0;
	}
}

int IsHealth(int typ)		// Type is one that adds health to player
{
	if (typ<90)
		return thingdata1[typ].props&HEALTH;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].props&HEALTH;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].props&HEALTH;
	else if (typ>=4000 && typ<4005)
		return thingdata4[typ-4001].props&HEALTH;
	else if (typ>=5000 && typ<5003)
		return thingdata4[typ-5001].props&HEALTH;
	else
		return 0;
}

int IsGettable(int typ)		// Type is one that player can pick up
{
	if (typ<90)
		return thingdata1[typ].props&GET;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].props&GET;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].props&GET;
	else if (typ>=4000 && typ<4005)
		return thingdata4[typ-4001].props&GET;
	else if (typ>=5000 && typ<5003)
		return thingdata4[typ-5001].props&GET;
	else
		return 0;
}

int IsArmor(int typ)		// Type is one that adds armor to player
{
	if (typ<90)
		return thingdata1[typ].props&ARMOR;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].props&ARMOR;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].props&ARMOR;
	else if (typ>=4000 && typ<4005)
		return thingdata4[typ-4001].props&ARMOR;
	else if (typ>=5000 && typ<5003)
		return thingdata4[typ-5001].props&ARMOR;
	else
		return 0;
}

int IsWeapon(int typ)		// Type is a weapon
{
	if (typ<90)
		return thingdata1[typ].props&WEAP;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].props&WEAP;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].props&WEAP;
	else if (typ>=4000 && typ<4005)
		return thingdata4[typ-4001].props&WEAP;
	else if (typ>=5000 && typ<5003)
		return thingdata4[typ-5001].props&WEAP;
	else
		return 0;
}

int IsMonster(int typ)		// Type is a monster
{
	if (typ<90)
		return thingdata1[typ].props&MONST;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].props&MONST;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].props&MONST;
	else if (typ>=4000 && typ<4005)
		return thingdata4[typ-4001].props&MONST;
	else if (typ>=5000 && typ<5003)
		return thingdata4[typ-5001].props&MONST;
	else
		return 0;
}

int IsBoss(int typ)		// Type is a monster
{
	switch(typ)
	{
		case SPIDERDEMON:
		case CYBERDEMON:
		case BOSSBRAIN:
		case ARCHVILE:
			return 1;
		default:
			break;
	}
	return 0;
}

int IsKey(int typ)			// Type is a key (return which)
{
	switch (typ)
	{
		case REDCARD:		
			return 4;
		case REDSKULL:
			return 1;
		case YELLOWCARD:
			return 5;
		case YELLOWSKULL:
			return 2;
		case BLUECARD:
			return 6;
		case BLUESKULL:
			return 3;
		default:
			return 0;
	}
}

int IsStart(int typ)		// Type is a start position, DM or co-op
{
	if
	(
		(PLAYER1<=typ && typ<=PLAYER4) ||
		(PLAYER5<=typ && typ<=PLAYER8) ||
		typ==DMSTART
	)
		return 1;
	return 0;
}

int CountsForSave(int typ)
{
	if (typ==1) return 1;
	if (typ==23) return 0;
	return !IsStart(typ);
}

//
// Convert name of a thing to its type number in DOOM
//
// This is used in SUBST to allow users to specify things by name
//

int Name2Type(char *tname)
{
    int i,res;
    
    if (isdigit(tname[0]))	// if a number is specified, assume it is type
        return atoi(tname);

    i=0;
    for (i=0;i<ntassoc;i++)	// for each association of a type and a name
    {
        res = stricmp(tname,thingassoc[i].name);
        if (res<0) return -1;						// none equal
        if (res==0) return thingassoc[i].typ;		// found match
    }
    return -1; // no match found
}

// Return the text string name of a thing type 

char *Name(unsigned typ)
{
	if (typ<90)
		return thingdata1[typ].name;
	else if (typ>=2000 && typ<2050)
		return thingdata2[typ-2001].name;
	else if (typ>=3000 && typ<3007)
		return thingdata3[typ-3001].name;
	else if (typ>=4000 && typ<4005)
		return thingdata4[typ-4001].name;
	else if (typ>=5000 && typ<5003)
		return thingdata4[typ-5001].name;
	else
		return "Whazzat?";
}

// Return the bit-encoded thing properties of a thing type 

// Return the strength of a monster by its thing type

int Toughness(Thing t)
{
	switch (t.typ)
	{
		case SPIDERDEMON:	return 3000;
		case SARGEANT:		return 30;
		case CYBERDEMON:	return 4000;
		case SPECTRE:		return 150;
		case ARCHVILE:		return 700;
		case COMMANDO:		return 70;
		case REVENANT:		return 300;
		case MANCUBUS:		return 600;
		case ARACHNOTRON:	return 500;
		case HELLKNIGHT:	return 500;
		case PAINELEMENTAL:	return 400;
		case HANGINGKEEN:	return 100;
		case WOLFSOLDIER:	return 50;
		case BOSSBRAIN:		return 250;
		case IMPS:			return 60;
		case DEMON:			return 150;
		case BARON:			return 1000;
		case TROOPER:		return 20;
		case CACODEMON:		return 400;
		case LOSTSOUL:		return 100;
		default: 			return 0;
	}
}

// Return the ammo contained in a thing by its type

int AmmoContent(Thing t)
{
	switch(t.typ)
	{
		case BACKPACK:			return 98;
		case SARGEANT:			return 28;
		case CELLBATTERY:		return 200;
		case COMMANDO:			return 10;
		case SUPERSHOTGUN:		return 56;
		case SHOTGUN:			return 56;
		case CHAINGUN:			return 20;
		case ROCKETLAUNCHER:	return 40;
		case PLASMARIFLE:		return 80;
		case CHAINSAW:			return 0;
		case BFG9000:			return 80;
		case AMMOCLIP:			return 10;
		case SHELLS:			return 28;
		case ROCKET:			return 20;
		case BOXROCKETS:		return 100;
		case CELLPACK:			return 40;
		case BOXAMMO:			return 50;
		case BOXSHELLS:			return 140;
		case TROOPER:			return 5;
		case WOLFSOLDIER:		return 5;
		default:				return 0;
	}
}

// Return the health contained in a thing type

int HealthCont(int t)
{
	switch(t)
	{
		case MEGASPHERE:		return 200;
		case STIMPACK:			return 10;
		case MEDIKIT:			return 25;
		case BLUESPHERE:		return 100;
		case BLUEBOTTLE:		return 1;
		case BERSERKPACK:		return 100;
		default:				return 0;
	}
}

// Return the health contained in a thing

int HealthContent(Thing t)
{
	return HealthCont(t.typ);
}

// Return the health contained in a thing type

int ArmourCont(int t)
{
	switch(t)
	{
		case MEGASPHERE:		return 200;
		case ARMORHELMET:		return 1;
		case GREENARMOR:		return 100;
		case BLUEARMOR:			return 200;
		default:				return 0;
	}
}


// Returns a string detailing a set of thing attribute bits

char *AttrStr(unsigned attr)
{
	static char attrstr[32];

	strcpy(attrstr,presence_strings[attr&7]);
	if ((attr&deafmask) || (attr&mplyrmask))
	{
		if (attr&deafmask)
			strcat(attrstr,"+Df");
		else
			strcat(attrstr,"   ");

		if (attr&mplyrmask)
			strcat(attrstr,"+Mp");
		else
			strcat(attrstr,"   ");
	}
	return attrstr;
}



