// ------------------------------
// Type and structure definitions
// ------------------------------

typedef unsigned short UINT;
typedef unsigned short WORD;
typedef unsigned short UWORD;
typedef unsigned short SWORD;
typedef unsigned long DWORD;
typedef long LONG;
typedef unsigned long ULONG;
typedef long SLONG;
typedef unsigned char BYTE;
typedef signed char SBYTE;
typedef unsigned char UBYTE;

typedef unsigned short unsh;

typedef struct numlist_t		// abstract linked list of numbers
{
	int num;
	struct numlist_t *next;
} numlist_t;

typedef struct sthalfplane
{
	int a,b,c;
	int side;
	struct sthalfplane *next;
} HalfPlane;


typedef struct stseglist
{
	unsh segnum;
	struct stseglist *next;
} SegList;

typedef struct stWadFileHeader
{
	char wadtag[4];					/* wad identification tag */
	unsigned long waddirlen;		/* number of entries in directory*/
	unsigned long waddiraddr;		/* byte file offset of directory*/
} WadHeader;

/* wad directory entry structure*/

typedef struct stWadDirEntry
{
	unsigned long resptr;			/* byte file offset of resource*/
	unsigned long reslen;			/* byte length of resource*/
	char resname[8];				/* name of resource nul termed if < 8 char*/
} WadDirEntry;

/* Directory locations for start/end tags */

/* wad directory structure*/

typedef struct stWadDir
{
	WadDirEntry *dirp;				/* dynamic array of directory entries*/
	int nentries;					/* number of entries used */
	int allentries;					/* number of entries allocated */
} WadDir;

/* wad data structures */

#define CYANCOLOR 0xf7

typedef struct stPicData
{
	short pwid;
	short phgt;
	short pxoff;
	short pyoff;
	
	long *coloffs;
	unsigned char *posts;
	long postlen;
} __attribute__ ((packed)) PicData;

typedef struct st_bitmap
{
	UBYTE *bmp;
	int logw;
	int realw;
	int logh;
} Bitmap;

typedef struct stPatch
{
	short xoff;
	short yoff;
	short pidx;
	short stepdir;  //1
	short cmap;		//0

} __attribute__ ((packed)) Patch;

typedef struct stTextureDesc
{
	char textname[8];
	short res1;
	short res2;
	short twid;
	short thgt;
	short res3;
	short res4;
	short npatches;
	Patch *patches;
} __attribute__ ((packed)) TextureDesc;

typedef struct stTextureHeader
{
	char textname[8];
	short res1;
	short res2;
	short twid;
	short thgt;
	short res3;
	short res4;
	short npatches;
} __attribute__ ((packed)) TextureHeader;

typedef struct stTextureLump
{
	long N;
	long *offs;
	TextureDesc *textures;
} TextureLump;

typedef struct stPNamesLump
{
	long npnames;
	char **patchname;
} PNamesLump;

typedef struct stWadTextures
{
	TextureLump t1;
	TextureLump t2;
	TextureLump t3;
	PNamesLump pnames;
} WadTextures;

typedef enum
{
	BTN_LEFT,
	BTN_RIGHT,
	BTN_CENTER
} ButtonAlignTypes;

typedef struct buttonstyle
{
	int len;				// length of caption string displayed
	int aln;				// padding/truncation method
	int thk;				// thickness in each state
	GrFBoxColors *frm;		// frame colors in each state
	GrTextOption *top;		// button text drawing style
} ButtonStyle;


typedef struct button
{
	char cap[256];			// string for caption text
	int stt;				// current button state
	ButtonStyle *sty[2];	// button drawing style in each state
} Button;

typedef struct buttonarray
{
	Button *btn;
	int nbtns;
	int nrows;
	int ncols;
	int xinc;
	int yinc;
	int colord;
} ButtonArray;

typedef struct stVTex
{
	int id;
	char *name;
	char *path;
	char *sign;
	int nargs;
	char **args;
} VTex;

typedef struct stVTexList
{
	int nvtx;
	VTex *vt;
} VTexList;

typedef struct stMTex
{
	int isp;
	char *name;
	char *path;
	char *sign;
	int num1;
	int num2;
} MTex;

typedef struct stWaveHdr
{
	char riff[4];
	long flen;
	char wave[4];
} WaveHdr;

typedef struct stFmtData
{
	short a1;		// format type, only 0001 supported here
	short a2;		// number of channels
	long s1;		// samples/sec
	long s2;		// average bytes/sec
	short a3;		// block align
} FmtData;

typedef struct stCkHdr
{
	char ckid[4];
	long cklen;
} CkHdr;

typedef struct stWaveHeader
{
	char riff[4];
	long flen;
	char wave[8];
	long a0;
	short a1;
	short a2;
	long s1;
	long s2;
	short a3;
	short a4;
	char data[4];
	long nsamps;
} __attribute__ ((packed)) WaveHeader;

typedef struct stRawHeader
{
	short three;
	short srate;
	short nsamps;
	short zero;
} __attribute__ ((packed)) RawHeader;

typedef struct stSegment
{
	int color;
	int bright;
	int fsel;
	int haln;
	int valn;
	int row;
	int col;
	int len;
	int ncols;
	int align;
	char str[128];
} __attribute__ ((packed)) Segment;

/* thing structure*/

typedef struct stThing
{
	short x,y;
	short ang;
	unsigned short typ;
	unsigned short attr;
} __attribute__ ((packed)) Thing;

/* linedef structure*/

typedef struct stLinedef
{
	unsigned short from;
	unsigned short to;
	unsigned short attr;
	unsigned short typ;
	unsigned short tag;
	short sides[2];
} __attribute__ ((packed)) Linedef;

/* sidedef structure*/

typedef struct stSidedef
{
	short xoff;
	short yoff;
	char utxt[8];
	char ltxt[8];
	char ntxt[8];
	unsigned short sect;
} __attribute__ ((packed)) Sidedef;

/* vertex structure*/

typedef struct stVertex
{
	short x;
	short y;
} __attribute__ ((packed)) Vertex;

typedef struct stSeg
{
	unsigned short from;
	unsigned short to;
	unsigned short ang;
	unsigned short ldef;
	unsigned short dir;
	unsigned short offs;
} __attribute__ ((packed)) Seg;

typedef struct stSsector
{
	unsigned short nsegs;
	unsigned short seg0;
} __attribute__ ((packed)) Ssector;

typedef struct stNode
{
	short x;
	short y;
	short dx;
	short dy;
	short rmxy;
	short rmny;
	short rmnx;
	short rmxx;
	short lmxy;
	short lmny;
	short lmnx;
	short lmxx;
	unsigned short children[2];
} __attribute__ ((packed)) Node;

/* sector structure*/

typedef struct stSector
{
	short fhgt;
	short chgt;
	char ftxt[8];
	char ctxt[8];
	unsigned short lite;
	unsigned short typ;
	unsigned short tag;
} __attribute__ ((packed)) Sector;

struct hplane;
typedef struct hplane
{
	int A,B,C;
	struct hplane *parent;
} HPlane;

typedef struct stfileinf
{
	char drv[4];
	char dir[64];
	char nam[12];
	char ext[4];
	struct stat stt;
} __attribute__ ((packed)) FileData;

typedef struct structthingpair
{
	char name[32];
	int typ;
} __attribute__ ((packed)) THINGPAIR;

typedef struct structthingdata
{
	char name[32];
	unsigned props;
	char sprite[16];
	unsigned hits;
} __attribute__ ((packed)) THINGDATA;

typedef struct stStringArray
{
	char *strbuf;
	int len;
	int num;
} __attribute__ ((packed)) StringArray;

typedef struct timer
{
	unsigned time0;
	unsigned period;
	int expf;
} __attribute__ ((packed)) Timer;

typedef struct st_cmap
{
	char name[16];
	char cmap[34][256];
} CMap;

typedef struct st_tmap
{
	char name[16];
	char tmap[256][256];
} TMap;

typedef struct st_sectordata
{
	int secnum;
	char *fill;
	Bitmap fillbmp;
	int cmap;
	int light;
	int xoff;
	int yoff;
} SectorData;

typedef struct st_blockmapdesc
{
	short xorg;
	short yorg;
	unsigned short ncols;
	unsigned short nrows;
	unsigned short **blocklists;
} BlockMapDesc;


typedef enum
{
	ID_NONE,
	ID_LISTED,
	ID_MARKED,
	ID_INFO
} IdMethods;

typedef struct stStrIdxAry
{
	int lidx;
	int didx;
	char *str;
} __attribute__ ((packed)) StrIdxAry;

typedef struct stCategory
{
	char secname[16];						// section in .DEF or .SGN file
	char defaultdir[256];					// default resource location
	IdMethods idm;							// method of identifying resource
	int mark;
	int nextns;
	StrIdxAry *Extns;						// extensions on resource files
	int nnames;
	StrIdxAry *Names;						// names for category
} __attribute__ ((packed)) Category;

typedef struct stResDef
{
	int ncats;
	Category *cat;
} __attribute__ ((packed)) ResourceDefinition;

typedef struct value
{
	int n;
	char str[256];
} __attribute__ ((packed)) Value;

typedef struct linedata
{
	int typ;
	char desc[20];
	int props;
} LineData;

typedef struct symbol_st
{
	char *sym;
	int val;
} Symbol;

typedef struct if_st
{
	int path;
	int id;
} IfElt;

typedef struct st_switchdata
{
  char name1[9];
  char name2[9];
  short episode;
} __attribute__ ((packed)) SwitchData; //jff 3/23/98 pack to read from memory

typedef struct st_animdata
{
    char        istexture;            //jff 3/23/98 make char for comparison
    char        endname[9];           //  if false, it is a flat
    char        startname[9];
    int         speed;
} __attribute__ ((packed)) AnimData; //jff 3/23/98 pack to read from memory

typedef enum
{
	SD_RIGHT=-1,
	SD_ON=0,
	SD_LEFT=1,
} SideTypes;

typedef enum
{
	SEG_RIGHT=-2,
	SEG_RCOINC=-1,
	SEG_SPLIT=0,
	SEG_LCOINC=1,
	SEG_LEFT=2
} SegCmpTypes;

typedef enum
{					    			// Trigger function
	TRT_NONE=1,				// a tag is not meaningful for this trigger
	TRT_SWITCH=2,			// trigger is switch, not walkover
	TRT_GUN=4,				// trigger is gun, ignore TRT_SWITCH
	TRT_2NDSIDE=8, 		// no tag is required, second sidedef sector affected
	TRT_RETRIG=16,		// trigger can be reused if set
	TRT_MONST=32,			// trigger can be used by monsters if set
	TRT_LOCK=64,			// trigger may lock out further actions if set
	TRT_DOORNOW=128,	// door trigger that can be reused without waiting
} TriggerTypes;

typedef enum				// Speed of motion
{
	TRS_NONE=0,
	TRS_SLOW,
	TRS_MED,
	TRS_FAST,
	TRS_TURBO,
} TriggerSpeedTypes;

typedef enum				
{
	TRX_NONE=0,	 			// Change tx type 
	TRX_ZERO,				
	TRX_SECDMG,				
	TRX_NUMERIC=4,
} TriggerChgTypes;

typedef enum
{
	TRK_NONE=0,				// Keyed trigger color 1=red 2=blue 3=yellow
	TRK_RED,
	TRK_BLUE,
	TRK_YELLOW,
} TriggerKeyTypes;

typedef enum
{
	TRC_NONE=0,				// Crusher types 1 = fast 2 = slow
	TRC_FAST,
	TRC_SLOW,
} TriggerCrushTypes;

typedef enum
{
	TRE_NONE=0,				// Sound types
	TRE_DOOR,				// 1 = Door
	TRE_BLAZE,				// 2 = Fast Door
	TRE_MOVER,				// 3 = Low Rumble
	TRE_CRUSH,				// 4 = Crusher sounds
	TRE_LIFT,				// 5 = Lift sounds
	TRE_CLUNK,				// 6 = Switch pressed
	TRE_TPORT,				// 7 = Teleport swoosh
	TRE_QUIET,				// 8 = Quiet crusher
} TriggerSounds;

// TRIGGER CLASS
typedef enum
{
	TRF_NONE=0,			    // Action classes
	TRF_FLOOR,				// 1  = Floor motion
	TRF_LIFT,				// 2  = Lift action
	TRF_STAIRS,				// 3  = Rising stairs
	TRF_DOOR,				// 4  = slow, fast, keyed doors
	TRF_CEILING,			// 5  = Ceiling motion
	TRF_CRUSHER,			// 6  = Crusher action
	TRF_TELE,				// 7  = Teleporters
	TRF_LIGHTS,				// 8  = Lighting changes
	TRF_EXIT,				// 9  = Exit specials
	TRF_SPECIAL				// 10 = Others
} TriggerClasses;

typedef enum
{
	OP_LTE,
	OP_GTE,
	OP_EQL,
	OP_NEQ,
	OP_AAS,
	OP_SAS,
	OP_ASS,
	OP_LT,
	OP_GT,
} OpTypes;


typedef enum {O_THING=0,O_VERTEX,O_LINEDEF,O_SIDEDEF,O_SECTOR} ObjTypes;
typedef enum {TH_X=0,TH_Y,TH_TYPE,TH_ANGLE,TH_FLAGS} ThgFields;
typedef enum {VE_X=0,VE_Y} VrtFields;
typedef enum {LI_FROM=0,LI_TO,LI_FLAGS,LI_TYPE,LI_TAG,LI_RSID,LI_LSID} LdfFields;
typedef enum {SI_UPPER=0,SI_NORMAL,SI_LOWER,SI_XOFF,SI_YOFF,SI_SECT} SdfFields;
//			  SI_UPPER1,SI_NORMAL1,SI_LOWER1,SI_XOFF1,SI_YOFF1,SI_SECT1,
//			  SI_UPPER2,SI_NORMAL2,SI_LOWER2,SI_XOFF2,SI_YOFF2,SI_SECT2} SdfFields;
typedef enum {SE_FHGT=0,SE_CHGT,SE_FLOOR,SE_CEILING,SE_TAG,SE_LIGHT,SE_TYPE} SecFields;

typedef enum 
{
	STX_ALLOALLP, // OBJ 			  all objects, all properties
	STX_ALLOONEP, // OBJ.FLD       	  all objects, one property
	STX_LEVOALLP, // OBJ(m)			  all objects in one level, all properties
	STX_LEVOONEP, // OBJ(m).FLD		  all objects in one level, one property
	STX_ONEOALLP, // OBJ(m,n)		  one object, all properties
	STX_ONEOONEP, // OBJ(m,n).FLD	  one object, one property
	STX_ONECHANGE // OBJ(m,n).FLD=VAL one object, one property, one value
} CLEDSyntaxes;

typedef enum
{
	TMR_NULL,
	TMR_ONESHOT,
	TMR_RETRIGGER,
} TimerModes;

typedef enum
{
	LfIm=1,LfMo=2,Lf2S=4,LfUp=8,LfLw=16,LfSe=32,LfSo=64,LfIn=128,LfMa=256,LfPt=512,
} LineFlags;

typedef enum
{
	TfEz=1,TfMd=2,TfHd=4,TfDf=8,TfMp=16,TfNd=32,TfNc=64,
} ThingFlags;

typedef enum
{
	C_NULL,
	C_MAK,
	C_BRK,
	C_MER,
	C_CMP,
	C_DIF,
	C_KIL,
	C_DUP,
	C_CVT,
	C_HIST,
	C_SETPAL,
	C_NBLD,
	C_DEMON,
	C_CLED,
	C_TXTUSE,
	C_FLTUSE,
	C_REPL,
	C_STAT,
	C_VIEW,
	C_GAMMA,
	C_MOVEIT,
	C_RENLIN,
	C_NOSECRET,
	C_STUCK,
	C_MUSPUT,
	C_HELP,
	C_EXIT,
	C_QUIT,
	C_PATCHER,
} CmdTypes;

typedef enum
{
	GET=1,ITEM=2,ALIVE=4,OBST=8,HANG=16,KEY=32,MONST=64,WEAP=128,
	ARMOR=256,HEALTH=512,AMMO=1024,FLY=2048,LIGHT=4096
} ThingProperties;

typedef enum
{
	SRCISDST=-4,SRCISDIR,DSTNFND,SRCNFND,MOVOK
} MovErrors;

typedef enum {I_NONE,I_DOOM2,I_TNT} IWADTypes;

typedef enum
{
	V_TOP,
	V_MIDDLE,
	V_BOTTOM
} VAlign;

typedef enum
{
	H_LEFT,
	H_CENTER,
	H_RIGHT
} HAlign;

typedef enum
{
	DT_GENERIC,					// data requires no translation
	DT_LEVELS,					// data is level image in a wad
	DT_TEXTURES,				// data is textures and pnames
	DT_SOUNDS,					// data is wav's or txt for sound card or spkr
	DT_PICS,					  // data is BMP for doom picture format
	DT_FLATS					  // data is BMP for doom flat format
} ResourceDataTypes;

typedef enum
{
	RES_UNKNOWN=0,
	RES_LUMPS,
	RES_LEVELS,
	RES_TEXTURES,
	RES_PNAMES,
	RES_SOUNDS,
	RES_MUSICS,
	RES_GRAPHICS,
	RES_PICTURES,
	RES_SPRITES,
	RES_PATCHES,
	RES_FLATS
} ResType;

typedef enum
{
	I_PLAYPAL=1,
	I_COLORMAP,
	I_ENDOOM,
	I_DEMO1,
	I_DEMO2,
	I_DEMO3,
	I_DEMO4,
	I_GENMIDI,
	I_DMXGUSC,
	I_DPPISTOL,
	I_DSPISTOL,
	I_DPSHOTGN,
	I_DSSHOTGN,
	I_DPSGCOCK,
	I_DSSGCOCK,
	I_DPSAWUP,
	I_DSSAWUP,
	I_DPSAWIDL,
	I_DSSAWIDL,
	I_DPSAWFUL,
	I_DSSAWFUL,
	I_DPSAWHIT,
	I_DSSAWHIT,
	I_DPRLAUNC,
	I_DSRLAUNC,
	I_DPRXPLOD,
	I_DSRXPLOD,
	I_DPFIRSHT,
	I_DSFIRSHT,
	I_DPFIRXPL,
	I_DSFIRXPL,
	I_DPPSTART,
	I_DSPSTART,
	I_DPPSTOP,
	I_DSPSTOP,
	I_DPDOROPN,
	I_DSDOROPN,
	I_DPDORCLS,
	I_DSDORCLS,
	I_DPSTNMOV,
	I_DSSTNMOV,
	I_DPSWTCHN,
	I_DSSWTCHN,
	I_DPSWTCHX,
	I_DSSWTCHX,
	I_DPPLPAIN,
	I_DSPLPAIN,
	I_DPDMPAIN,
	I_DSDMPAIN,
	I_DPPOPAIN,
	I_DSPOPAIN,
	I_DPSLOP,
	I_DSSLOP,
	I_DPITEMUP,
	I_DSITEMUP,
	I_DPWPNUP,
	I_DSWPNUP,
	I_DPOOF,
	I_DSOOF,
	I_DPTELEPT,
	I_DSTELEPT,
	I_DPPOSIT1,
	I_DSPOSIT1,
	I_DPPOSIT2,
	I_DSPOSIT2,
	I_DPPOSIT3,
	I_DSPOSIT3,
	I_DPBGSIT1,
	I_DSBGSIT1,
	I_DPBGSIT2,
	I_DSBGSIT2,
	I_DPSGTSIT,
	I_DSSGTSIT,
	I_DPBRSSIT,
	I_DSBRSSIT,
	I_DPSGTATK,
	I_DSSGTATK,
	I_DPCLAW,
	I_DSCLAW,
	I_DPPLDETH,
	I_DSPLDETH,
	I_DPPDIEHI,
	I_DSPDIEHI,
	I_DPPODTH1,
	I_DSPODTH1,
	I_DPPODTH2,
	I_DSPODTH2,
	I_DPPODTH3,
	I_DSPODTH3,
	I_DPBGDTH1,
	I_DSBGDTH1,
	I_DPBGDTH2,
	I_DSBGDTH2,
	I_DPSGTDTH,
	I_DSSGTDTH,
	I_DPBRSDTH,
	I_DSBRSDTH,
	I_DPPOSACT,
	I_DSPOSACT,
	I_DPBGACT,
	I_DSBGACT,
	I_DPDMACT,
	I_DSDMACT,
	I_DPNOWAY,
	I_DSNOWAY,
	I_DPBAREXP,
	I_DSBAREXP,
	I_DPPUNCH,
	I_DSPUNCH,
	I_DPTINK,
	I_DSTINK,
	I_DPBDOPN,
	I_DSBDOPN,
	I_DPBDCLS,
	I_DSBDCLS,
	I_DPITMBK,
	I_DSITMBK,
	I_DPGETPOW,
	I_DSGETPOW,
	I_DPPLASMA,
	I_DSPLASMA,
	I_DPBFG,
	I_DSBFG,
	I_DPCACSIT,
	I_DSCACSIT,
	I_DPCYBSIT,
	I_DSCYBSIT,
	I_DPSPISIT,
	I_DSSPISIT,
	I_DPSKLATK,
	I_DSSKLATK,
	I_DPCACDTH,
	I_DSCACDTH,
	I_DPSKLDTH,
	I_DSSKLDTH,
	I_DPCYBDTH,
	I_DSCYBDTH,
	I_DPSPIDTH,
	I_DSSPIDTH,
	I_DPHOOF,
	I_DSHOOF,
	I_DPMETAL,
	I_DSMETAL,
	I_DPDSHTGN,
	I_DSDSHTGN,
	I_DPDBOPN,
	I_DSDBOPN,
	I_DPDBCLS,
	I_DSDBCLS,
	I_DPDBLOAD,
	I_DSDBLOAD,
	I_DPVIPAIN,
	I_DSVIPAIN,
	I_DPMNPAIN,
	I_DSMNPAIN,
	I_DPPEPAIN,
	I_DSPEPAIN,
	I_DPBSPSIT,
	I_DSBSPSIT,
	I_DPKNTSIT,
	I_DSKNTSIT,
	I_DPVILSIT,
	I_DSVILSIT,
	I_DPMANSIT,
	I_DSMANSIT,
	I_DPPESIT,
	I_DSPESIT,
	I_DPSKEPCH,
	I_DSSKEPCH,
	I_DPVILATK,
	I_DSVILATK,
	I_DPSKESWG,
	I_DSSKESWG,
	I_DPBSPDTH,
	I_DSBSPDTH,
	I_DPVILDTH,
	I_DSVILDTH,
	I_DPKNTDTH,
	I_DSKNTDTH,
	I_DPPEDTH,
	I_DSPEDTH,
	I_DPSKEDTH,
	I_DSSKEDTH,
	I_DPBSPACT,
	I_DSBSPACT,
	I_DPBSPWLK,
	I_DSBSPWLK,
	I_DPVILACT,
	I_DSVILACT,
	I_DPFLAME,
	I_DSFLAME,
	I_DPFLAMST,
	I_DSFLAMST,
	I_DPBOSPIT,
	I_DSBOSPIT,
	I_DPBOSCUB,
	I_DSBOSCUB,
	I_DPBOSSIT,
	I_DSBOSSIT,
	I_DPBOSPN,
	I_DSBOSPN,
	I_DPBOSDTH,
	I_DSBOSDTH,
	I_DPMANATK,
	I_DSMANATK,
	I_DPMANDTH,
	I_DSMANDTH,
	I_DPSSSIT,
	I_DSSSSIT,
	I_DPSSDTH,
	I_DSSSDTH,
	I_DPKEENPN,
	I_DSKEENPN,
	I_DPKEENDT,
	I_DSKEENDT,
	I_DPSKEACT,
	I_DSSKEACT,
	I_DPSKESIT,
	I_DSSKESIT,
	I_DPSKEATK,
	I_DSSKEATK,
	I_DPRADIO,
	I_DSRADIO,
	I_D_RUNNIN,
	I_D_STALKS,
	I_D_COUNTD,
	I_D_BETWEE,
	I_D_DOOM,
	I_D_THE_DA,
	I_D_SHAWN,
	I_D_DDTBLU,
	I_D_IN_CIT,
	I_D_DEAD,
	I_D_STLKS2,
	I_D_THEDA2,
	I_D_DOOM2,
	I_D_DDTBL2,
	I_D_RUNNI2,
	I_D_DEAD2,
	I_D_STLKS3,
	I_D_ROMERO,
	I_D_SHAWN2,
	I_D_MESSAG,
	I_D_COUNT2,
	I_D_DDTBL3,
	I_D_AMPIE,
	I_D_THEDA3,
	I_D_ADRIAN,
	I_D_MESSG2,
	I_D_ROMER2,
	I_D_TENSE,
	I_D_SHAWN3,
	I_D_OPENIN,
	I_D_EVIL,
	I_D_ULTIMA,
	I_D_READ_M,
	I_D_DM2TTL,
	I_D_DM2INT,
	I_HELP,
	I_VICTORY2,
	I_PFUB1,
	I_PFUB2,
	I_END0,
	I_END1,
	I_END2,
	I_END3,
	I_END4,
	I_END5,
	I_END6,
	I_TITLEPIC,
	I_CREDIT,
	I_BOSSBACK,
	I_AMMNUM0,
	I_AMMNUM1,
	I_AMMNUM2,
	I_AMMNUM3,
	I_AMMNUM4,
	I_AMMNUM5,
	I_AMMNUM6,
	I_AMMNUM7,
	I_AMMNUM8,
	I_AMMNUM9,
	I_STBAR,
	I_STGNUM0,
	I_STGNUM1,
	I_STGNUM2,
	I_STGNUM3,
	I_STGNUM4,
	I_STGNUM5,
	I_STGNUM6,
	I_STGNUM7,
	I_STGNUM8,
	I_STGNUM9,
	I_STTMINUS,
	I_STTNUM0,
	I_STTNUM1,
	I_STTNUM2,
	I_STTNUM3,
	I_STTNUM4,
	I_STTNUM5,
	I_STTNUM6,
	I_STTNUM7,
	I_STTNUM8,
	I_STTNUM9,
	I_STTPRCNT,
	I_STYSNUM0,
	I_STYSNUM1,
	I_STYSNUM2,
	I_STYSNUM3,
	I_STYSNUM4,
	I_STYSNUM5,
	I_STYSNUM6,
	I_STYSNUM7,
	I_STYSNUM8,
	I_STYSNUM9,
	I_STKEYS0,
	I_STKEYS1,
	I_STKEYS2,
	I_STKEYS3,
	I_STKEYS4,
	I_STKEYS5,
	I_STDISK,
	I_STCDROM,
	I_STARMS,
	I_STCFN033,
	I_STCFN034,
	I_STCFN035,
	I_STCFN036,
	I_STCFN037,
	I_STCFN038,
	I_STCFN039,
	I_STCFN040,
	I_STCFN041,
	I_STCFN042,
	I_STCFN043,
	I_STCFN044,
	I_STCFN045,
	I_STCFN046,
	I_STCFN047,
	I_STCFN048,
	I_STCFN049,
	I_STCFN050,
	I_STCFN051,
	I_STCFN052,
	I_STCFN053,
	I_STCFN054,
	I_STCFN055,
	I_STCFN056,
	I_STCFN057,
	I_STCFN058,
	I_STCFN059,
	I_STCFN060,
	I_STCFN061,
	I_STCFN062,
	I_STCFN063,
	I_STCFN064,
	I_STCFN065,
	I_STCFN066,
	I_STCFN067,
	I_STCFN068,
	I_STCFN069,
	I_STCFN070,
	I_STCFN071,
	I_STCFN072,
	I_STCFN073,
	I_STCFN074,
	I_STCFN075,
	I_STCFN076,
	I_STCFN077,
	I_STCFN078,
	I_STCFN079,
	I_STCFN080,
	I_STCFN081,
	I_STCFN082,
	I_STCFN083,
	I_STCFN084,
	I_STCFN085,
	I_STCFN086,
	I_STCFN087,
	I_STCFN088,
	I_STCFN089,
	I_STCFN090,
	I_STCFN091,
	I_STCFN092,
	I_STCFN093,
	I_STCFN094,
	I_STCFN095,
	I_STCFN121,
	I_STFB1,
	I_STFB0,
	I_STFB2,
	I_STFB3,
	I_STPB1,
	I_STPB0,
	I_STPB2,
	I_STPB3,
	I_STFST01,
	I_STFST00,
	I_STFST02,
	I_STFTL00,
	I_STFTR00,
	I_STFOUCH0,
	I_STFEVL0,
	I_STFKILL0,
	I_STFST11,
	I_STFST10,
	I_STFST12,
	I_STFTL10,
	I_STFTR10,
	I_STFOUCH1,
	I_STFEVL1,
	I_STFKILL1,
	I_STFST21,
	I_STFST20,
	I_STFST22,
	I_STFTL20,
	I_STFTR20,
	I_STFOUCH2,
	I_STFEVL2,
	I_STFKILL2,
	I_STFST31,
	I_STFST30,
	I_STFST32,
	I_STFTL30,
	I_STFTR30,
	I_STFOUCH3,
	I_STFEVL3,
	I_STFKILL3,
	I_STFST41,
	I_STFST40,
	I_STFST42,
	I_STFTL40,
	I_STFTR40,
	I_STFOUCH4,
	I_STFEVL4,
	I_STFKILL4,
	I_STFGOD0,
	I_STFDEAD0,
	I_M_DOOM,
	I_M_RDTHIS,
	I_M_OPTION,
	I_M_QUITG,
	I_M_NGAME,
	I_M_SKULL1,
	I_M_SKULL2,
	I_M_THERMO,
	I_M_THERMR,
	I_M_THERMM,
	I_M_THERML,
	I_M_ENDGAM,
	I_M_PAUSE,
	I_M_MESSG,
	I_M_MSGON,
	I_M_MSGOFF,
	I_M_EPISOD,
	I_M_EPI1,
	I_M_EPI2,
	I_M_EPI3,
	I_M_HURT,
	I_M_JKILL,
	I_M_ROUGH,
	I_M_SKILL,
	I_M_NEWG,
	I_M_ULTRA,
	I_M_NMARE,
	I_M_SVOL,
	I_M_OPTTTL,
	I_M_SAVEG,
	I_M_LOADG,
	I_M_DISP,
	I_M_MSENS,
	I_M_GDHIGH,
	I_M_GDLOW,
	I_M_DETAIL,
	I_M_DISOPT,
	I_M_SCRNSZ,
	I_M_SGTTL,
	I_M_LGTTL,
	I_M_SFXVOL,
	I_M_MUSVOL,
	I_M_LSLEFT,
	I_M_LSCNTR,
	I_M_LSRGHT,
	I_BRDR_TL,
	I_BRDR_T,
	I_BRDR_TR,
	I_BRDR_L,
	I_BRDR_R,
	I_BRDR_BL,
	I_BRDR_B,
	I_BRDR_BR,
	I_WIOSTK,
	I_WIOSTI,
	I_WIF,
	I_WIMSTT,
	I_WIOSTS,
	I_WIOSTF,
	I_WITIME,
	I_WIPAR,
	I_WIMSTAR,
	I_WIMINUS,
	I_WIPCNT,
	I_WINUM0,
	I_WINUM1,
	I_WINUM2,
	I_WINUM3,
	I_WINUM4,
	I_WINUM5,
	I_WINUM6,
	I_WINUM7,
	I_WINUM8,
	I_WINUM9,
	I_WICOLON,
	I_WISUCKS,
	I_WIFRGS,
	I_WILV00,
	I_WILV01,
	I_WILV02,
	I_WILV03,
	I_WILV04,
	I_WILV05,
	I_WILV06,
	I_WILV07,
	I_WILV08,
	I_WILV11,
	I_WILV12,
	I_WILV14,
	I_WILV15,
	I_WILV16,
	I_WILV17,
	I_WILV18,
	I_WILV20,
	I_WILV21,
	I_WILV22,
	I_WILV23,
	I_WILV24,
	I_WILV25,
	I_WILV26,
	I_WILV27,
	I_WILV28,
	I_WILV13,
	I_WILV10,
	I_WIP1,
	I_WIP2,
	I_WIP3,
	I_WIP4,
	I_WIBP1,
	I_WIBP2,
	I_WIBP3,
	I_WIBP4,
	I_WIKILRS,
	I_WIVCTMS,
	I_WISCRT2,
	I_WIENTER,
	I_CWILV00,
	I_CWILV01,
	I_CWILV02,
	I_CWILV03,
	I_CWILV04,
	I_CWILV05,
	I_CWILV06,
	I_CWILV07,
	I_CWILV08,
	I_CWILV09,
	I_CWILV10,
	I_CWILV11,
	I_CWILV12,
	I_CWILV13,
	I_CWILV14,
	I_CWILV15,
	I_CWILV16,
	I_CWILV17,
	I_CWILV18,
	I_CWILV19,
	I_CWILV20,
	I_CWILV21,
	I_CWILV22,
	I_CWILV23,
	I_CWILV24,
	I_CWILV25,
	I_CWILV26,
	I_CWILV27,
	I_CWILV28,
	I_CWILV29,
	I_CWILV30,
	I_CWILV31,
	I_INTERPIC
} ResourceNames;


#define ERR_OPENWAD 1
#define ERR_NOTAWAD 2
#define ERR_READDIR 3

#define RES_MAPNAME  1
#define RES_LINEDEFS 2
#define RES_SIDEDEFS 3
#define RES_VERTICES 4
#define RES_SECTORS	 5
#define RES_THINGS	 6
#define RES_SEGS	 7
#define RES_SSECTORS 8
#define RES_NODES	 9
#define RES_REJECT	 10
#define RES_BLOCKMAP 11
#define RES_TEXTSTART 12
#define RES_FLATSTART 13
#define RES_PTCHSTART 14
#define RES_SPRTSTART 15
#define RES_TEXTEND 16
#define RES_FLATEND 17
#define RES_PTCHEND 18
#define RES_SPRTEND 19
#define RES_TEXTURE 20
#define RES_FLAT 21
#define RES_PATCH 22
#define RES_SPRITE 23
#define RES_SOUND 24
#define RES_PCSOUND 25
#define RES_MUSIC 26
#define RES_GRAPHIC 27
#define RES_STRING 28
#define RES_COLORMAP 29
#define RES_PLAYPAL 30
#define RES_MISC 31

#define NU 0
#define SW 1
#define WA 2
#define GU 3
#define MA 4
#define XX 5


