#include <grx20.h>
#include <mousex.h>
#include "patch.h"

FILE *sslog=NULL;
#define sgn(x) (((x)<0)? (-1) : (((x)>0)? (1) : (0)))

#define LINE_TOLERANCE 12

#define SCREEN_LEFT (0)
#define SCREEN_RIGHT (HRES)
#define SCREEN_TOP (0)
#define SCREEN_BOTTOM (VRES)

#define WINDOW_LEFT (HRES-VRES+16)
#define WINDOW_RIGHT (HRES-16)
#define WINDOW_TOP (36)
#define WINDOW_BOTTOM (VRES-36)

int modeBx = 0;
int modeBy = 0;
int fthgBx = 0;
int fthgBy = 0;
int fdifBx = 0;
int fdifBy = 0;
int flinBx = 0;
int flinBy = 0;
int fsecBx = 0;
int fsecBy = 0;
int fvrtBx = 0;
int fvrtBy = 0;
int fprvBx = 0;
int fprvBy = 0;
int fldBx = 0;
int fldBy = 0;

int	thgDx =	 0;
int	thgDy =	 0;
int	linDx =	 0;
int	linDy =	 0;
int	secDx =	 0;
int	secDy =	 0;
int	sscDx =	 0;
int	sscDy =	 0;
int	vrtDx =	 0;
int	vrtDy =	 0;


typedef enum
{
	M_THING,
	M_LINE,
	M_SECTOR,
	M_VERTEX,
	M_DOOM,
	M_NODE,
} ModeLabels;

int Mode=M_THING;

//
int texturesread=0;
int CommandMode=0;
char buffer[256];
int FullScreen=0;
int clrco=1;
int firstmark=-1;

// screen globals
int HRES=640;	 	// screen resolution
int VRES=480;

// colors
long gray[16];
long stdc[16];

// edit marked list

int *marked=NULL;
int nmarked=0;
int *secsmarked=NULL;
int nsecsmarked=0;
int *thgsmarked=NULL;
int nthgsmarked=0;
int *vrtsmarked=NULL;
int nvrtsmarked=0;

int *vxaligned=NULL;
int nxaligned=0;

unsigned char stdp[16][3]=
{
	{  0,  0,  0},		//DK_BLACK
	{  0,  0,128},		//DK_BLUE
	{  0,128,  0},		//DK_GREEN
	{ 42, 42, 42},		//DK_CYAN			(DK_GRAY)
	{128,  0,  0},		//DK_RED
	{ 85, 85, 85},		//DK_MAGENTA		(MID_GRAY)
	{255,144,  0},		//DK_YELLOW         (ORANGE)
	{128,128,128},		//DK_WHITE
	{170,170,170},		//LT_BLACK			(LT_GRAY)
	{0  ,  0,255},		//LT_BLUE
	{0  ,255,  0},		//LT_GREEN
	{0  ,255,255},		//LT_CYAN
	{255,  0,  0},		//LT_RED
	{255,  0,255},		//LT_MAGENTA
	{255,255,  0},		//LT_YELLOW
	{255,255,255},		//LT_WHITE
};

GrTextOption topt,eopt;
GrTextOption cotop= {&GrDefaultFont,{ 14 },{ 0 },0,0,0,0};

// grid globals		
int grid=0;			// current gridsize
int gridinc=0;		// direction of last grid change

// Display state globals

int *verts1=NULL,*verts2=NULL;
int nverts1=0, nverts2=0;
int oldobj1=-1, oldobj2=-1;
int colfudge=0;

int thingfilter = -1;

int mouse_enabled = 0;
int mouse_blocked = 0;
int last_mx=0;
int last_my=0;
int anchor_x=0;
int anchor_y=0;
int anchor_t=0;
long acctime=0;
int rubber=0;

// Box frame colors

GrFBoxColors outfrm={7,8,5,3,5};
GrFBoxColors infrm= {0,3,5,8,5};
GrFBoxColors midfrm={7,3,5,8,5};

char basename[256];
char mapsel[256];

// define a rectangle with scroll info representing a view into a world

typedef struct stView
{
	int minx;
	int miny;
	int maxx;
	int maxy;
	int scrollx;
	int scrolly;
	int zoomf;
} View;

// some global views, the scroll/zoom view, the screen, and the map extent

View view,scrn,map,mapw,dataw,edw,btnw,cow,mnw;

extern void W2S(int wx,int wy,int *sx,int *sy,View *v);
extern void S2W(int sx,int sy,int *wx,int *wy,View *v);
extern void SetScrollIncs(int mx,int my,View *v);

//
// Map viewer design
//
// We use a world space of -32768 to 32767 for both x and y. Coordinates
// are a short int in this space.
//
// The level map is placed in the world according to the short coordinates
// of its vertices and things. The extent of the level is defined by
// the globals map.minx, map.miny, map.maxx, and map.mapyy.
//
// Within this space is placed a view rectangle, This rectangle is mapped
// to the full screen linearly. The screen is zoomed and scrolled by redefining
// the location of this view rectangle. The view rectangle is defined by
// coordinates view.minx, view.miny, view.maxx, and view.maxy.
//
// The grid is always at 0 or a power of 2 value, and this represents the
// difference in coordinates between adjacent grid lines. 0 grid means no grid
// display, otherwise the grid lines are drawn thru all world coordinates
// that are 0 modulo the grid.
//
// The initial view is the smallest rectangle that is the same aspect
// as the screen and contains the entire level.
//

void InitView(View *v)
{
	int mdx,mdy,sdx,sdy;
	int delta=0;

	// initialize the map extent, level data must have been read

	map.minx=map_minx;
	map.miny=map_miny;
	map.maxx=map_maxx;
	map.maxy=map_maxy;

	if (map.minx==map.maxx) map.maxx++;		// prevent divide/0
	if (map.miny==map.maxy) map.maxy++;		// otherwise dx actual width
	mdx = map.maxx - map.minx;
	mdy = map.maxy - map.miny;

	// initialize the screen extent

	scrn.minx = SCREEN_LEFT;							// set screen extent
	scrn.maxx = SCREEN_RIGHT-1;
	scrn.miny = SCREEN_TOP;
	scrn.maxy = SCREEN_BOTTOM-1;
	if (scrn.minx==scrn.maxx) scrn.maxx++;		// prevent divide/0
	if (scrn.miny==scrn.maxy) scrn.maxy++;		// otherwise dx actual width

	if (FullScreen)
	{
		mapw.minx = SCREEN_LEFT;
		mapw.maxx = SCREEN_RIGHT;
		mapw.miny = SCREEN_TOP-1;
		mapw.maxy = SCREEN_BOTTOM-1;
	}
	else
	{
		mapw.minx = WINDOW_LEFT;
		mapw.maxx = WINDOW_RIGHT;
		mapw.miny = WINDOW_TOP;
		mapw.maxy = WINDOW_BOTTOM;
	}
	sdx = mapw.maxx - mapw.minx;				// compute widths for later
	sdy = mapw.maxy - mapw.miny;

	// initialize the view into the world with roughly correct aspect ratio

	if (mdy/mdx < sdy/sdx)					// if map more wide than tall
	{
		delta = mdy - (sdy*mdx)/sdx;		// leftover pixels, short direction
		v->minx = map.minx;
		v->maxx = map.maxx;
		v->miny = map.miny + delta/2;	    // center view around map
		v->maxy = map.maxy - delta/2;
	}
	else
	{
		delta = mdx - (sdx*mdy)/sdy;		// leftover pixels, short direction
		v->miny = map.miny;
		v->maxy = map.maxy;
		v->minx = map.minx + delta/2;		// center view around map
		v->maxx = map.maxx - delta/2;
	}

	edw.minx = WINDOW_LEFT-2;		 		// edit window
	edw.miny = WINDOW_BOTTOM+8;
	edw.maxx = WINDOW_RIGHT+2;
	edw.maxy = SCREEN_BOTTOM-9;

	btnw.minx = WINDOW_LEFT-2;				// mode button window
	btnw.miny = SCREEN_TOP+8;
	btnw.maxx = WINDOW_RIGHT+2;
	btnw.maxy = WINDOW_TOP-9;

	cow.minx = SCREEN_LEFT+16;				// coordinate window
	cow.miny = SCREEN_TOP+12;
	cow.maxx = WINDOW_LEFT-16;
	cow.maxy = SCREEN_TOP+28;

	mnw.minx = WINDOW_RIGHT-96;				// map number window
	mnw.miny = SCREEN_TOP+11;
	mnw.maxx = WINDOW_RIGHT-56;
	mnw.maxy = WINDOW_TOP-12;

	modeBx = btnw.minx+4;
	modeBy = btnw.miny+3;
	fthgBx = 24;
	fthgBy = 42;
	fdifBx = 16;
	fdifBy = 308;
	flinBx = 24;
	flinBy = 42;
	fsecBx = 24;
	fsecBy = 42;
	fvrtBx = 24;
	fvrtBy = 42;
	fprvBx = WINDOW_RIGHT-44;
	fprvBy = btnw.miny+3;
	fldBx = edw.minx + 4;
	fldBy = edw.miny + 3;

	thgDx = 12;
	thgDy =	352;
	linDx =	12;
	linDy =	200;
	secDx =	12;
	secDy =	352;
	sscDx =	12;
	sscDy =	352;
	vrtDx =	12;
	vrtDy =	352;

	SetScrollIncs(16,16,v);
	v->zoomf = 0;
}

void InitColors()
{
	long i;

	GrResetColors();
	for(i=0;i<16;i++)
		GrAllocCell();

	for (i=0;i<16;i++)
		GrSetColor(i,stdp[i][0],stdp[i][1],stdp[i][2]);
}

// Transform one view to another

void V2V(int ix,int iy,int *ox,int *oy,View *in,View *out)
{
	// place the screen x coordinate proportionally to x position within view
	*ox = out->minx + ((out->maxx-out->minx)*(ix-in->minx))/(in->maxx-in->minx);

	// place the screen y coordinate proportionally to y position within view
	*oy = out->maxy - ((out->maxy-out->miny)*(iy-in->miny))/(in->maxy-in->miny);
}

// World to screen coordinate transform

void W2S(int wx,int wy,int *sx,int *sy,View *v)
{
	V2V(wx,wy,sx,sy,v,&mapw);
}

// Screen to world coordinate transform

void S2W(int sx,int sy,int *wx,int *wy,View *v)
{
	V2V(sx,sy,wx,wy,&mapw,v);
}

char *EditBox(int x1,int y1,int x2,int y2,char *prompt,int len,int px,int py,int wx,int wy,char *buffer)
{
	GrFramedBox(x1,y1,x2,y2,4,&outfrm);
	GrDrawString(prompt,len,px,py,&topt);
	GrFramedBox(wx,wy,x2-8,wy+GrCharHeight('Q',&eopt),4,&infrm);
	InitGraphicData(wx,wy,x2-20,wy+GrCharHeight('Q',&topt),&eopt);
	return EditCommand(buffer);
}


void ZoomOut(View *v,int x,int y)
{
	int wx,wy;

	if (!FullScreen && !IsInBox(x,y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
	{
		x = (mapw.minx+mapw.maxx)/2;
		y = (mapw.miny+mapw.maxy)/2;
	}

	S2W(x,y,&wx,&wy,v);

	if (v->zoomf<8)
	{
		v->minx = wx - 2*(wx-v->minx);
		v->maxx = wx + 2*(v->maxx-wx);

    	v->miny = wy - 2*(wy-v->miny);
		v->maxy = wy + 2*(v->maxy-wy);
		v->zoomf++;
	}
}

void ZoomIn(View *v,int x,int y)
{
	int wx,wy;

	if (!FullScreen && !IsInBox(x,y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
	{
		x = (mapw.minx+mapw.maxx)/2;
		y = (mapw.miny+mapw.maxy)/2;
	}

	S2W(x,y,&wx,&wy,v);

	if (v->zoomf>1 || ((v->scrollx >> (1-v->zoomf)) && (v->scrolly >> (1-v->zoomf))))
	{
		v->minx = wx - (wx-v->minx)/2;
		v->maxx = wx + (v->maxx-wx)/2;

    	v->miny = wy - (wy-v->miny)/2;
		v->maxy = wy + (v->maxy-wy)/2;
		v->zoomf--;
	}
}

// The current view center is (view.minx+view.maxx)/2, (view.miny+view.maxy)/2
// Change it to x,y, then zoom to zoom factor zf

void CenterView(int x,int y,int zf,View *v)
{
	int curx,cury;

	curx = (v->minx+v->maxx)/2;
	cury = (v->miny+v->maxy)/2;

	// to translate curx,cury -> x,y we add x-curx,y-cury to all view coords
	v->minx += x - curx;
	v->miny += y - cury;
	v->maxx += x - curx;
	v->maxy += y - cury;

	// now zoom
	while (v->zoomf>zf)
		ZoomIn(&view,x,y);
	while (v->zoomf<zf)
		ZoomOut(&view,x,y);
}

void SetScrollIncs(int mx,int my,View *v)
{
	int wx,wy,wx1,wy1;

	S2W(mapw.minx,mapw.miny,&wx,&wy,v);
	S2W(mapw.minx+mx,mapw.miny+my,&wx1,&wy1,v);
	v->scrollx = wx1 - wx;
	v->scrolly = wy1 - wy;
}

void ScrollLeft(View *v)
{
	int inc;

	inc = v->zoomf>0 ? v->scrollx << v->zoomf : v->scrollx >> -v->zoomf;

	v->minx -= inc;
	v->maxx -= inc;
}

void ScrollRight(View *v)
{
	int inc;

	inc = v->zoomf>0 ? v->scrollx << v->zoomf : v->scrollx >> -v->zoomf;

	v->minx += inc;
	v->maxx += inc;
}

void ScrollUp(View *v)
{
	int inc;
	inc = v->zoomf>0 ? v->scrolly << v->zoomf : v->scrolly >> -v->zoomf;

	v->miny += inc;
	v->maxy += inc;
}

void ScrollDown(View *v)
{
	int inc;
	inc = v->zoomf>0 ? v->scrolly << v->zoomf : v->scrolly >> -v->zoomf;

	v->miny -= inc;
	v->maxy -= inc;
}

void WorldLine(int wx1,int wy1,int wx2,int wy2,int c,View *v)
{
	int sx1,sx2,sy1,sy2;

	W2S(wx1,wy1,&sx1,&sy1,v);
	W2S(wx2,wy2,&sx2,&sy2,v);
	GrLine(sx1,sy1,sx2,sy2,c);
}

void DrawThickScreenLine(int Xstart, int Ystart, int Xend, int Yend, int c)
{
	GrLineOption go;

	go.lno_color = c;
	go.lno_width = 3;
	go.lno_pattlen=0;
	go.lno_dashpat=NULL;

	GrCustomLine(Xstart,Ystart,Xend,Yend,&go);
	return;
}

void ThickWorldLine(int wx1,int wy1,int wx2,int wy2,int c,View *v)
{
	int sx1,sx2,sy1,sy2;

	W2S(wx1,wy1,&sx1,&sy1,v);
	W2S(wx2,wy2,&sx2,&sy2,v);
	DrawThickScreenLine(sx1,sy1,sx2,sy2,c);
}

void DrawGrid(View *v)
{
	int x1=0,y1=0,x2=0,y2=0;
	int g0=0,g1=0;

	// adjust grid size so the lines do not fill the screen
	// adjustment is in direction of last grid change

	while (x2-x1<2 || y1-y2<2)
	{
		W2S((g0=(v->minx/grid)*grid),(g1=(v->miny/grid)*grid),&x1,&y1,v);
		W2S(g0+grid,g1+grid,&x2,&y2,v);
		if (x2-x1<2 || y1-y2<2)
		{
			if (gridinc)
				grid = grid*2;
			else
			{
				grid = 0;
				break;
			}
		}
	}

	// draw the grid

	if (grid)
	{
		for (;g1<=v->maxy;g1+=grid)		// horizontal grid lines
		{
			W2S(0,g1,&x1,&y1,v);
			GrHLine(0,scrn.maxx,y1,1);
		}

		for (;g0<=v->maxx;g0+=grid)		// vertical grid lines
		{
			W2S(g0,0,&x1,&y1,v);
			GrVLine(x1,0,scrn.maxy,1);
		}
	}
}

void DrawMark(int sx,int sy,int r,int c)
{
	GrFilledBox(sx-r,sy-r,sx+r,sy+r,c);
}

void DrawXtraPlayer1(int tno,View *v)
{
	int x,y;
	int sx,sx1,sy,sy1;

	x = Things[tno].x;
	y = Things[tno].y;
	W2S(x,y,&sx,&sy,v);
	W2S(x+24,y,&sx1,&sy1,v);
	DrawMark(sx,sy,sx1-sx,12);
}

void DrawActor(int x,int y,int r,int t,int f,int ai)
{
	int si,co;
	int s0,c0,s1,c1,s2,c2,s3,c3;
	int pgn[3][2];
	int c;

	c = 12;
	if (IsStart(t)) c = 10;
	if (t==DMSTART)	c = 11;
	if (IsMonster(t)) c = (f&TfDf)?  6 : 12;

	if (t!=TPDEST) GrBox(x-r,y-r,x+r,y+r,c);
	ai = (ai+22)/45;
	while (ai<0) ai+=8;

	si = -(int)(r*sin(ai*pi/4)+.5);
	co = (int)(r*cos(ai*pi/4)+.5);

	s1 = y+si;	// the point r from center in direction faced
	c1 = x+co;

	s3 = y-si; // the point -r from center away from direction (nadir)
	c3 = x-co;

	s0 = y-co; 	// the points one r in normal direction from nadir
	c0 = x+si;

	s2 = y+co;
	c2 = x-si;

	pgn[0][0]=c1;
	pgn[0][1]=s1;
	pgn[1][0]=c0;
	pgn[1][1]=s0;
	pgn[2][0]=c2;
	pgn[2][1]=s2;

	GrFilledPolygon(3,pgn,14);
	GrLine(c1,s1,c3,s3,14);
	if (t==TPDEST)
	{
		GrCircle(x,y,r,11);
		GrCircle(x,y,(r*3)/4,14);
		GrCircle(x,y,r/2,10);
		GrCircle(x,y,r/4,15);
	}
}

void DrawArmor(int x,int y,int r,int t)
{
	int pgn[6][2];
	int c=7;

	pgn[0][0]=x-r;
	pgn[0][1]=y-r;
	pgn[1][0]=x;
	pgn[1][1]=y-r+r/3;
	pgn[2][0]=x+r;
	pgn[2][1]=y-r;
	pgn[3][0]=x+r;
	pgn[3][1]=y+r-r/3;
	pgn[4][0]=x;
	pgn[4][1]=y+r;
	pgn[5][0]=x-r;
	pgn[5][1]=y+r-r/3;

	switch(t)
	{
		case MEGASPHERE:
			c = 11;
			break;
		case BLUEARMOR:
			c = 9;
			break;
		case GREENARMOR:
			c = 10;
			break;
		case ARMORHELMET:
			c = 6;
			break;
	}
	GrFilledPolygon(6,pgn,c);
}

void DrawWeapon(int x,int y,int r,int t)
{
	int pgn[8][2];
	int c=7;

	switch(t)
	{
		case SHOTGUN:
			c = 10;
			break;
		case SUPERSHOTGUN:
			c = 9;
			break;
		case ROCKETLAUNCHER:
			c = 12;
			break;
		case CHAINGUN:
			c = 14;
			break;
		case PLASMARIFLE:
			c = 11;
			break;
		case BFG9000:
			c = 13;
			break;
		case CHAINSAW:
			c = 15;
			break;
	}

	pgn[0][0]=x-r;							// barrel of rifle
	pgn[0][1]=y-max(1,r/8);			
	pgn[1][0]=x-r;					
	pgn[1][1]=y+max(1,r/8);			
	pgn[2][0]=x+r;					
	pgn[2][1]=y+max(1,r/8);			
	pgn[3][0]=x+r;					
	pgn[3][1]=y-max(1,r/8);
	GrFilledConvexPolygon(4,pgn,c);

									
	pgn[0][0]=x-r+max(1,r/8);				// grip of rifle
	pgn[0][1]=y+max(1,r/8);
	pgn[1][0]=x-r;
	pgn[1][1]=y+r;
	pgn[2][0]=x-r+max(1,r/4);
	pgn[2][1]=y+r;
	pgn[3][0]=x-r+max(1,(3*r)/8);
	pgn[3][1]=y+max(1,r/8);
	GrFilledConvexPolygon(4,pgn,c);

}

void DrawLight(int x,int y,int r,int t)
{
	GrFilledCircle(x,y,r,15);
	GrFilledCircle(x,y,r/2,3);
}

void DrawAmmo(int x,int y,int r,int t)
{
	int pgn[3][2];
	int c=7;

	switch (t)
	{
		case AMMOCLIP:
		case BOXAMMO:
			c = 2;
			break;
		case BOXSHELLS:
		case SHELLS:
			c = 12;
			break;
		case BOXROCKETS:
		case ROCKET:
			c = 6;
			break;
		case CELLBATTERY:
		case CELLPACK:
			c = 11;
		case BACKPACK:
			c = 15;
			break;
	}
	pgn[0][0]=x;
	pgn[0][1]=IsSmallAmmo(t)? y : y-r;
	pgn[1][0]=x-r;
	pgn[1][1]=y+r;
	pgn[2][0]=x+r;
	pgn[2][1]=y+r;

	GrFilledPolygon(3,pgn,c);
	if (t==BACKPACK) GrBox(x-r,y-r,x+r,y+r,14);
}

void DrawKey(int x,int y,int r, int t)
{
	int c=7;

	switch(IsKey(t))
	{
		case 1: case 4:
			c = 12;
			break;
		case 2: case 5:
			c = 14;
			break;
		case 3: case 6:
			c = 9;
			break;
		default:
			break;
	}
	GrCircle(x,y,r,11);
	GrLine(x-2*r,y,x+2*r,y,c);
	GrLine(x,y-2*r,x,y+2*r,c);
	GrLine(x-r,y+r,x+r,y-r,c);
	GrLine(x-r,y-r,x+r,y+r,c);
}

void DrawHealth(int x,int y,int r,int t)
{
	int c=7;

	switch(t)
	{
		case MEGASPHERE:
			c = 15;
			break;
		case BLUESPHERE:
			c = 9;
			break;
		case BERSERKPACK:
			c = 12;
			break;
		case MEDIKIT:
			c = 10;
			break;
		case STIMPACK:
			c = 14;
			break;
		case BLUEBOTTLE:
			c = 1;
			break;
	}
	GrFilledBox(x-r/3,y-r,x+r/3,y+r,c);
	GrFilledBox(x-r,y-r/3,x+r,y+r/3,c);
}

int IsMarked(int ln)
{
	int i;

	for (i=0;i<nmarked;i++)
		if (marked[i]==ln)
			break;
	return i<nmarked;
}

int SectorIsMarked(int sn)
{
	int i;

	for (i=0;i<nsecsmarked;i++)
		if (secsmarked[i]==sn)
			break;
	return i<nsecsmarked;
}

int ThingIsMarked(int tn)
{
	int i;

	for (i=0;i<nthgsmarked;i++)
		if (thgsmarked[i]==tn)
			break;
	return i<nthgsmarked;
}

void DrawThing(int tno,View *v)
{
	long c=0;
	int t,sx,sy,sx1,sy1;
	int r,done=0,f;
	int ThMonsterMode=fthgButtons->btn[1].stt;
	int ThPlayerMode=fthgButtons->btn[2].stt;
	int ThArmorMode=fthgButtons->btn[3].stt; 
	int ThLightsMode=fthgButtons->btn[4].stt;
	int ThAmmoMode=fthgButtons->btn[5].stt;
	int ThAmmoBoxMode=fthgButtons->btn[6].stt;
	int ThKeysMode=fthgButtons->btn[7].stt;
	int ThWeaponMode=fthgButtons->btn[8].stt;
	int ThHealthMode=fthgButtons->btn[9].stt;
	int ThItemMode=fthgButtons->btn[10].stt;
	int ThOtherMode=fthgButtons->btn[11].stt;
	int ThEzMode=fdifButtons->btn[0].stt;
	int ThMdMode=fdifButtons->btn[1].stt;
	int ThHdMode=fdifButtons->btn[2].stt;
	int ThDfMode=fdifButtons->btn[4].stt;
	int ThMpMode=fdifButtons->btn[5].stt;

	t = Things[tno].typ;
	f = Things[tno].attr;

	if (!ThMpMode)
		if (f&TfMp) return;
	if (!ThDfMode)
		if (f&TfDf) return;
	if (ThEzMode)
		if (!(f&TfEz)) return;
	if (ThMdMode)
		if (!(f&TfMd)) return;
	if (ThHdMode)
		if (!(f&TfHd)) return;

	r =  0;
	r |= ThMonsterMode && IsMonster(t);
	r |= ThPlayerMode && (IsStart(t) || t == TPDEST);
	r |= ThArmorMode && IsArmor(t);
	r |= ThLightsMode && IsLight(t);
	r |= ThAmmoMode && IsSmallAmmo(t);
	r |= ThAmmoBoxMode && IsAmmo(t) && !IsSmallAmmo(t);
	r |= ThKeysMode && IsKey(t);
	r |= ThWeaponMode && IsWeapon(t);
	r |= ThHealthMode && IsHealth(t);
	r |= ThItemMode && IsItem(t);
	r |= ThOtherMode && !IsMonster(t) && !IsStart(t) && t!=TPDEST &&
			!IsArmor(t) && !IsLight(t) && !IsAmmo(t) &&
			!IsKey(t) && !IsWeapon(t) && !IsHealth(t) && !IsItem(t);

	if (!r) return;

	c = 7;
	if (IsItem(t)) c=11;

	W2S(Things[tno].x,Things[tno].y,&sx,&sy,v);
	W2S(Things[tno].x+Radius(Things[tno]),Things[tno].y,&sx1,&sy1,v);
	r = sx1-sx;
	if (IsMonster(t) || IsStart(t) || t == TPDEST)
	{
		if (t==PLAYER1 && tno>=firstmark)
			DrawXtraPlayer1(tno,v);
		else
			DrawActor(sx,sy,r,t,Things[tno].attr&TfDf,Things[tno].ang);
		done = 1;
	}
	if (IsArmor(t))
	{
		DrawArmor(sx,sy,r,t);
		done = 1;
	}
	if (IsLight(t))
	{
		DrawLight(sx,sy,r,t);
		done=1;
	}
	if (IsAmmo(t))
	{
		DrawAmmo(sx,sy,r,t);
		done = 1;
	}
	if (IsKey(t))
	{
		DrawKey(sx,sy,r,t);
		done=1;
	}
	if (IsWeapon(t))
	{
		DrawWeapon(sx,sy,r,t);
		done=1;
	}
	if (IsHealth(t))
	{
		DrawHealth(sx,sy,r,t);
		done=1;
	}
	if (!done)
		GrFilledCircle(sx,sy,r,c);
	if (ThingIsMarked(tno))
		GrFilledBox(sx-r,sy-r,sx+r,sy+r,8|GrXOR);
}

void DrawVertexMark(int sx,int sy,int r,int c)
{
	GrLine(sx-r,sy-r,sx+r,sy+r,c);
	GrLine(sx+r,sy-r,sx-r,sy+r,c);
}

void DrawVertexSpot(int sx,int sy,int r,int c)
{
	GrHLine(sx-r,sx+r,sy,c);
	GrVLine(sx,sy-r,sy+r,c);
}

void DrawVertex(int vn,int r,int c,int isspot,View *v)
{
	int x,y,sx,sy;

	x = Vertexes[vn].x;
	y = Vertexes[vn].y;

	W2S(x,y,&sx,&sy,v);
	if (isspot) DrawVertexSpot(sx,sy,r,c);
	else DrawVertexMark(sx,sy,r,c);
}

void ShowVertLegend()
{
	int r,w,h,x0,y0;
	int x=fvrtBx,y=fvrtBy;

	if (FullScreen || Mode!=M_VERTEX) return;

	r = 6;
	x0=70; y0=6;
	w=10; h=20;
	DrawButtonArray(x,y,fvrtButtons);

	y0+=h;

	DrawVertexSpot(x+x0+0*w,y+y0+0*h,1,14);
	DrawVertexSpot(x+x0+0*w,y+y0+1*h,1,10);
	if (!ViewOnly)
	{
		DrawVertexMark(x+x0+0*w,y+y0+2*h,3,12);
		DrawVertexMark(x+x0+0*w,y+y0+3*h,3,9);
	}
}

void ShowThingLegend()
{
	int r,w,h,x0,y0;
	int x=fthgBx,y=fthgBy;

	if (FullScreen || Mode!=M_THING)
		return;

	r = 4;
	x0=70; y0=6;
	w=10; h=20;
	DrawButtonArray(x,y,fthgButtons);
	DrawButtonArray(fdifBx,fdifBy,fdifButtons);

	y0+=h;
	DrawActor(x+x0+0*w,y+y0,r,DEMON,0,90);

	y0+=h;
	DrawActor(x+x0+0*w,y+y0,r,PLAYER1,0,90);
	DrawActor(x+x0+1*w,y+y0,r,DMSTART,0,90);
	DrawMark(x+x0+2*w,y+y0,r,12);
	DrawActor(x+x0+3*w,y+y0,r,TPDEST,0,90);

	y0+=h;
	DrawArmor(x+x0+0*w,y+y0,r,MEGASPHERE);
	DrawHealth(x+x0+0*w,y+y0,r,MEGASPHERE);
	DrawArmor(x+x0+1*w,y+y0,r,BLUEARMOR);
	DrawArmor(x+x0+2*w,y+y0,r,GREENARMOR);
	DrawArmor(x+x0+3*w,y+y0,r,ARMORHELMET);

	y0+=h;
	DrawLight(x+x0+0*w,y+y0,r,FLOORLAMP);

	y0+=h;
	DrawAmmo(x+x0+0*w,y+y0,r,CELLBATTERY);
	DrawAmmo(x+x0+1*w,y+y0,r,ROCKET);
	DrawAmmo(x+x0+2*w,y+y0,r,SHELLS);
	DrawAmmo(x+x0+3*w,y+y0,r,AMMOCLIP);

	y0+=h;
	DrawAmmo(x+x0+0*w,y+y0,r,CELLPACK);
	DrawAmmo(x+x0+1*w,y+y0,r,BOXROCKETS);
	DrawAmmo(x+x0+2*w,y+y0,r,BOXSHELLS);
	DrawAmmo(x+x0+3*w,y+y0,r,BOXAMMO);

	y0+=h;
	DrawKey(x+x0+0*w,y+y0,r,REDCARD);
	DrawKey(x+x0+1*w,y+y0,r,YELLOWCARD);
	DrawKey(x+x0+2*w,y+y0,r,BLUECARD);

	y0+=h;
	DrawWeapon(x+x0+0*w,y+y0,r,BFG9000);
	DrawWeapon(x+x0+1*w,y+y0,r,PLASMARIFLE);
	DrawWeapon(x+x0+2*w,y+y0,r,ROCKETLAUNCHER);
	DrawWeapon(x+x0+3*w,y+y0,r,CHAINGUN);
	DrawWeapon(x+x0+4*w,y+y0,r,SUPERSHOTGUN);
	DrawWeapon(x+x0+5*w,y+y0,r,SHOTGUN);

	y0+=h;
	DrawArmor(x+x0+0*w,y+y0,r,MEGASPHERE);
	DrawHealth(x+x0+0*w,y+y0,r,MEGASPHERE);
	DrawHealth(x+x0+1*w,y+y0,r,BLUESPHERE);
	DrawHealth(x+x0+2*w,y+y0,r,BERSERKPACK);
	DrawHealth(x+x0+3*w,y+y0,r,MEDIKIT);
	DrawHealth(x+x0+4*w,y+y0,r,STIMPACK);
	DrawHealth(x+x0+5*w,y+y0,r,BLUEBOTTLE);

	y0+=h;
	GrFilledCircle(x+x0+0*w,y+y0,r,11);

	y0+=h;
	GrFilledCircle(x+x0+0*w,y+y0,r,8);
}

void ShowSectorLegend()
{
	int r,w,h,x0,y0;
	int x=fsecBx,y=fsecBy;

	if (FullScreen || Mode!=M_SECTOR) return;

	r = 6;
	x0=70; y0=6;
	w=10; h=20;
	DrawButtonArray(x,y,fsecButtons);

	y0+=h;
	GrHLine(x+x0+0*w,x+x0+4*w,y+y0+0*h,14);
	GrHLine(x+x0+0*w,x+x0+4*w,y+y0+1*h,10);
	GrHLine(x+x0+0*w,x+x0+4*w,y+y0+2*h,12);
	GrHLine(x+x0+0*w,x+x0+4*w,y+y0+3*h,6);
	GrHLine(x+x0+0*w,x+x0+4*w,y+y0+4*h,11);
}

void ShowLineLegend()
{
	int r,w,h,x0,y0,x1;
	int x=flinBx,y=flinBy;

	if (FullScreen || Mode!=M_LINE) return;

	r = 6;
	w=16; h=20;
	DrawButtonArray(x,y,flinButtons);

	y0=6;
	y0+=h;
	x0=24;
	x1=88;
	GrHLine(x+x0+0*w,x+x0+1*w,y+y0+0*h,7);
	GrHLine(x+x1+0*w,x+x1+1*w,y+y0+0*h,1);
	GrHLine(x+x0+0*w,x+x0+1*w,y+y0+1*h,4);
	GrHLine(x+x1+0*w,x+x1+1*w,y+y0+1*h,11);
	GrHLine(x+x0+0*w,x+x0+1*w,y+y0+2*h,2);
	GrHLine(x+x1+0*w,x+x1+1*w,y+y0+2*h,13);
	GrHLine(x+x0+0*w,x+x0+1*w,y+y0+3*h,9);
	GrHLine(x+x1+0*w,x+x1+1*w,y+y0+3*h,15);
	GrHLine(x+x0+0*w,x+x0+1*w,y+y0+4*h,6);
	GrHLine(x+x1+0*w,x+x1+1*w,y+y0+4*h-2,10);
	GrHLine(x+x1+0*w,x+x1+1*w,y+y0+4*h,14);
	GrHLine(x+x1+0*w,x+x1+1*w,y+y0+4*h+2,12);
}

void ShowThingData(int x,int y,int mx,int my)
{
	int tn,wx,wy;
	char buffer[256];
	int h=13;
	static int oldtn=-1;

	if (Mode!=M_THING) return;
	if (!IsInBox(mx,my,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy)) return;

	if (!FullScreen)
	{
		S2W(mx,my,&wx,&wy,&view);
		tn =ThingOf(wx,wy);
		if (oldtn!=tn && oldtn>=0) GrFilledBox(4,352,HRES-VRES+10,VRES-5,7);
		if (tn!=oldtn && tn>=0)
		{
			sprintf(buffer,"TYP:%-16s",Name(Things[tn].typ));
			buffer[20]='\0';
			topt.txo_fgcolor.v=0;
			topt.txo_bgcolor.v=7;
	    	GrDrawString(buffer,strlen(buffer),x,y+0*h,&topt);
			sprintf(buffer,"NUM:%-4d",tn);
	    	GrDrawString(buffer,strlen(buffer),x,y+1*h,&topt);
			sprintf(buffer,"FLG:%-16s",AttrStr(Things[tn].attr));
			GrDrawString(buffer,strlen(buffer),x,y+2*h,&topt);
			sprintf(buffer,"ANG:%-3d           ",Things[tn].ang);
			GrDrawString(buffer,strlen(buffer),x,y+3*h,&topt);
			sprintf(buffer,"X  :%-6d       ",Things[tn].x);
			GrDrawString(buffer,strlen(buffer),x,y+4*h,&topt);
			sprintf(buffer,"Y  :%-6d       ",Things[tn].y);
			GrDrawString(buffer,strlen(buffer),x,y+5*h,&topt);
		}
		oldtn = tn;
	}
}

void DrawLineList(int *lins,int nlins,int col,View *v)
{
	int i,x1,y1,x2,y2,dx,dy,cx,cy;

	GrSetClipBox(mapw.minx,mapw.miny,mapw.maxx,mapw.maxy);
	for (i=0;i<nlins;i++)
	{
		x1 = Vertexes[Linedefs[lins[i]].from].x;
		y1 = Vertexes[Linedefs[lins[i]].from].y;
		x2 = Vertexes[Linedefs[lins[i]].to].x;
		y2 = Vertexes[Linedefs[lins[i]].to].y;
		dx = x2-x1;
		dy = y2-y1;
		cx = (x2+x1)/2;
		cy = (y2+y1)/2;
		ThickWorldLine(x1,y1,x2,y2,col|GrXOR,v);
		if (Mode==M_LINE)
		{
			WorldLine(x1+.85*dx-.15*dy,y1+.85*dy+.15*dx,x2,y2,col|GrXOR,v);
			WorldLine(x1+.85*dx+.15*dy,y1+.85*dy-.15*dx,x2,y2,col|GrXOR,v);
			WorldLine(cx,cy,cx+.15*dy,cy-.15*dx,col|GrXOR,v);
		}
	}
	GrResetClipBox();
}

void DrawVertexList(int *verts,int nverts,int col,View *v)
{
	int i,x1,y1,x2,y2,dx,dy,cx,cy;

	GrSetClipBox(mapw.minx,mapw.miny,mapw.maxx,mapw.maxy);
	for (i=0;i<nverts;i+=2)
	{
		x1 = Vertexes[verts[i]].x;
		y1 = Vertexes[verts[i]].y;
		x2 = Vertexes[verts[i+1]].x;
		y2 = Vertexes[verts[i+1]].y;
		dx = x2-x1;
		dy = y2-y1;
		cx = (x2+x1)/2;
		cy = (y2+y1)/2;
		ThickWorldLine(x1,y1,x2,y2,col|GrXOR,v);
		if (Mode==M_LINE || Mode==M_NODE)
		{
			WorldLine(x1+.85*dx-.15*dy,y1+.85*dy+.15*dx,x2,y2,col|GrXOR,v);
			WorldLine(x1+.85*dx+.15*dy,y1+.85*dy-.15*dx,x2,y2,col|GrXOR,v);
			WorldLine(cx,cy,cx+.15*dy,cy-.15*dx,col|GrXOR,v);
		}
	}
	GrResetClipBox();
}

void ClearLines()
{
	DrawVertexList(verts1,nverts1,14,&view);
	FreeDynInt(&verts1,&nverts1);	
	DrawVertexList(verts2,nverts2,12,&view);
	FreeDynInt(&verts2,&nverts2);
	oldobj1=oldobj2=-1;
}

void ClearVertexes()
{
	DrawVertexList(verts1,nverts1,14,&view);
	FreeDynInt(&verts1,&nverts1);	
	DrawVertexList(verts2,nverts2,12,&view);
	FreeDynInt(&verts2,&nverts2);
	oldobj1=oldobj2=-1;
}

void ClearMarks()
{
	FreeDynInt(&marked,&nmarked);
	FreeDynInt(&secsmarked,&nsecsmarked);
	FreeDynInt(&thgsmarked,&nthgsmarked);
}

void ShowSectorData(int x,int y,int mx,int my)
{
	int sn=-1,wx,wy;
	char buffer[256];
	int h = 13;
	static int oldsn=-1;


	if (Mode!=M_SECTOR) return;
	if (!IsInBox(mx,my,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy)) return;

	S2W(mx,my,&wx,&wy,&view);
	sn = SectorOf(wx,wy);
	if (!FullScreen)
	{
		if (oldsn!=sn && oldsn>=0) GrFilledBox(4,352,HRES-VRES+10,VRES-5,7);
		if (sn!=oldsn && sn>=0)
		{
			sprintf(buffer,"SECTOR:%4d         ",sn);
			topt.txo_fgcolor.v=0;
			topt.txo_bgcolor.v=7;
    	GrDrawString(buffer,strlen(buffer),x,y+0*h,&topt);
			sprintf(buffer,"TYPE: %-14s",sectortypes[Sectors[sn].typ]);
			GrDrawString(buffer,strlen(buffer),x,y+1*h,&topt);
			sprintf(buffer,"LITE: %-3d",Sectors[sn].lite);
			GrDrawString(buffer,strlen(buffer),x,y+2*h,&topt);
			sprintf(buffer,"TAG : %-3d",Sectors[sn].tag);
			GrDrawString(buffer,strlen(buffer),x,y+3*h,&topt);
			sprintf(buffer,"FHGT: %-6d",Sectors[sn].fhgt);
			GrDrawString(buffer,strlen(buffer),x,y+4*h,&topt);
			sprintf(buffer,"CHGT: %-6d",Sectors[sn].chgt);
			GrDrawString(buffer,strlen(buffer),x,y+5*h,&topt);
			sprintf(buffer,"FTXT: %-8s",Str8(Sectors[sn].ftxt));
			GrDrawString(buffer,strlen(buffer),x,y+6*h,&topt);
			sprintf(buffer,"CTXT: %-8s",Str8(Sectors[sn].ctxt));
			GrDrawString(buffer,strlen(buffer),x,y+7*h,&topt);
		}
		oldsn = sn;
	}

	if (sn!=oldobj1)
	{
		if (oldobj1>=0)
			ClearLines();

		if (sn>=0)
		{
			GetVertexesInSector(sn,&verts1,&nverts1);
			DrawVertexList(verts1,nverts1,14,&view);

			GetVertexesWithTag(Sectors[sn].tag,&verts2,&nverts2);
			DrawVertexList(verts2,nverts2,12,&view);
		}
		oldobj1 = sn;
	}
}

void ShowLineData(int x, int y, int mx, int my)
{
	int ln,wx,wy;
	char buffer[256];
	int h = 13;
	int s1,s2;
	char txt1[16],txt2[16];
	static int oldln=-1;

	if (Mode!=M_LINE) return;
	if (!IsInBox(mx,my,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy)) return;

	S2W(mx,my,&wx,&wy,&view);
	ln=LineOf(wx,wy,LINE_TOLERANCE);
	if (!FullScreen)
	{
		if (ln!=oldln && oldln>=0) GrFilledBox(4,200,HRES-VRES+10,VRES-5,7);
		if (ln!=oldln && ln>=0)
		{
			s1 = Linedefs[ln].sides[0];
			if (s1<0) return;
			s2 = Linedefs[ln].sides[1];
			sprintf(txt1,"%d",s1);
			strcpy(txt2,"");
			if (s2>=0) sprintf(txt2,"|S2: %d",s2);
			sprintf(buffer,"S1:%-7s%-10s",txt1,txt2);
			topt.txo_fgcolor.v=0;
			topt.txo_bgcolor.v=7;
	    	GrDrawString(buffer,strlen(buffer),x,y+0*h,&topt);
			strcpy(txt1,Str8(Sidedefs[s1].utxt));
			if (s2>=0) strcpy(txt2,"| "); else strcpy(txt2,"");
			strcat(txt2,s2>=0? Str8(Sidedefs[s2].utxt) : "");
			sprintf(buffer,"U:%-8s%-10s",txt1,txt2);
			GrDrawString(buffer,strlen(buffer),x,y+1*h,&topt);
			strcpy(txt1,Str8(Sidedefs[s1].ntxt));
			if (s2>=0) strcpy(txt2,"| "); else strcpy(txt2,"");
			strcat(txt2,s2>=0? Str8(Sidedefs[s2].ntxt) : "");
			sprintf(buffer,"N:%-8s%-10s",txt1,txt2);
			GrDrawString(buffer,strlen(buffer),x,y+2*h,&topt);
			strcpy(txt1,Str8(Sidedefs[s1].ltxt));
			if (s2>=0) strcpy(txt2,"| "); else strcpy(txt2,"");
			strcat(txt2,s2>=0? Str8(Sidedefs[s2].ltxt) : "");
			sprintf(buffer,"L:%-8s%-10s",txt1,txt2);
			GrDrawString(buffer,strlen(buffer),x,y+3*h,&topt);
			sprintf(txt1,"%d",Sidedefs[s1].xoff);
			strcpy(txt2,"");
			if (s2>=0) sprintf(txt2,"| %d",Sidedefs[s2].xoff);
			sprintf(buffer,"X:%-8s%-10s",txt1,txt2);
			GrDrawString(buffer,strlen(buffer),x,y+4*h,&topt);
			sprintf(txt1,"%d",Sidedefs[s1].yoff);
			strcpy(txt2,"");
			if (s2>=0) sprintf(txt2,"| %d",Sidedefs[s2].yoff);
			sprintf(buffer,"Y:%-8s%-10s",txt1,txt2);
			GrDrawString(buffer,strlen(buffer),x,y+5*h,&topt);
			sprintf(txt1,"%d",Sidedefs[s1].sect);
			strcpy(txt2,"");
			if (s2>=0) sprintf(txt2,"| %d",Sidedefs[s2].sect);
			sprintf(buffer,"S:%-8s%-10s",txt1,txt2);
			GrDrawString(buffer,strlen(buffer),x,y+6*h,&topt);

			sprintf(buffer,"NUM:%4d",ln);
			topt.txo_fgcolor.v=0;
			topt.txo_bgcolor.v=7;
	    	GrDrawString(buffer,strlen(buffer),x,y+8*h,&topt);
			sprintf(buffer,"TYP:%-3d",Linedefs[ln].typ);
			GrDrawString(buffer,strlen(buffer),x,y+9*h,&topt);
			sprintf(buffer,"FLG:%-16s",ShortLineFlagStr(Linedefs[ln].attr));
			GrDrawString(buffer,strlen(buffer),x,y+10*h,&topt);
			sprintf(buffer,"TAG:%-3d       ",Linedefs[ln].tag);
			GrDrawString(buffer,strlen(buffer),x,y+11*h,&topt);
			sprintf(buffer,"FRM:%-6d       ",Linedefs[ln].from);
			GrDrawString(buffer,strlen(buffer),x,y+12*h,&topt);
			sprintf(buffer,"TO :%-6d       ",Linedefs[ln].to);
			GrDrawString(buffer,strlen(buffer),x,y+13*h,&topt);
			sprintf(buffer,"SD1:%-5d        ",Linedefs[ln].sides[0]);
			GrDrawString(buffer,strlen(buffer),x,y+14*h,&topt);
			sprintf(buffer,"SD2:%-5d        ",Linedefs[ln].sides[1]);
			GrDrawString(buffer,strlen(buffer),x,y+15*h,&topt);
			sprintf(buffer,"%-19s",linetypdata[Linedefs[ln].typ].desc);
			GrDrawString(buffer,strlen(buffer),x,y+16*h,&topt);
		}
		oldln = ln;
	}

	if (ln!=oldobj2)
	{
		if (oldobj2>=0)
			ClearLines();

		if (ln>=0)
		{
			AddDynInt(&verts1,&nverts1,Linedefs[ln].from);
			AddDynInt(&verts1,&nverts1,Linedefs[ln].to);
			DrawVertexList(verts1,nverts1,14,&view);

			switch(Linedefs[ln].typ)
			{
				case 1: case 26: case 27: case 28: // manual doors
				case 117: case 118:
				case 31: case 32: case 33: case 34:
					if (Linedefs[ln].sides[1]>=0)
						GetVertexesInSector(Sidedefs[Linedefs[ln].sides[1]].sect,&verts2,&nverts2);
					break;
				default:
					GetVertexesInTaggedSectors(Linedefs[ln].tag,&verts2,&nverts2);
					break;
			}
			DrawVertexList(verts2,nverts2,12,&view);
		}
		oldobj2 = ln;
	}
}

void EraseStatusPanel()
{
	if (FullScreen)	return;
	GrFilledBox(4,4,HRES-VRES+11,VRES-5,7);
}

static int fillcol=12;

void FreeSegList(SegList **sgl)
{

	if (!*sgl) return;
	else
	{
		SegList *nxt,*s;

		for (s=*sgl;s;s=nxt)
		{
			nxt = s->next;
			free(s);
		}
	}
	*sgl = NULL;
}


void DrawHalfPlanes(HalfPlane *hp)
{
	int (*pgn)[2];
	int (*spgn)[2];
	int npts,i;
	int coltbl[8]={1,2,3,14,8,9,10,11};

	pgn = malloc(NVertexes*sizeof(int)*2);
	spgn = malloc(NVertexes*sizeof(int)*2);
  npts = 0;

	HPListToPolygon(hp,pgn,&npts);

	for (i=0;i<npts;i++)
		W2S(pgn[i][0],pgn[i][1],&spgn[i][0],&spgn[i][1],&view);

	if (npts>2)
		GrFilledConvexPolygon(npts,spgn,coltbl[fillcol++%8]);

	free(pgn);
	free(spgn);
}

// This subroutine requires a route array and a level within the array,
// equal to its length that specifies the branches taken from the root
// of the BSP tree to this point, be it a node or subsector. This 
// information is used to non-recursively return a list of halfplanes
// formed by the nodes above this point, the convex hull of the node
// lines above.

HalfPlane *HullAboveNode(int nodenum,int level,int *route)
{
	HalfPlane *hphead=NULL,*hp,*last=NULL;
	int i;
	unsh n=NNodes-1;  // root of BSP tree

	// add the nodes along the route to the halfplane list for
	// this node - its ancestors' convex hull, which it splits

	for (i=0;i<level && !(n & NF_SUBSECTOR);i++)
	{
		short x1 = Nodes[n].x;
		short y1 = Nodes[n].y;
		short x2 = Nodes[n].dx+x1;
		short y2 = Nodes[n].dy+y1;

		hp = malloc(sizeof(HalfPlane));
		if (last)
			last->next = hp;
		else
			hphead = hp;

		hp->a = -(y2-y1);
		hp->b = x2-x1;
		hp->c = x1*y2-x2*y1;
		hp->side = route[i]? SD_LEFT : SD_RIGHT;
		hp->next = NULL;

		last = hp;
		n = Nodes[n].children[route[i]];
	}
	return hphead;
}

void R_SegsBelowNode(int nodenum,int level,int *route,SegList **list,SegList **last)
{
	if (nodenum & NF_SUBSECTOR)
	{
		unsh subsec = (nodenum & ~NF_SUBSECTOR);
		unsh s0 = Ssectors[subsec].seg0;
		unsh ns = Ssectors[subsec].nsegs;
		int i;

		for (i=s0;i<s0+ns;i++)
		{
			SegList *sg = malloc(sizeof(SegList));

			if (*last)
				(*last)->next = sg;
			else
				*list = sg;

			sg->segnum = i;
			sg->next = NULL;

			(*last) = sg;
		}
		return;
	}

	route[level]=0;
	R_SegsBelowNode(Nodes[nodenum].children[0],level+1,route,list,last);

	route[level]=1;
	R_SegsBelowNode(Nodes[nodenum].children[1],level+1,route,list,last);
}

int NodeOnScreen(Node *n,int side)
{
	short box1[4],box2[4]={0,0,HRES-1,VRES-1};
	int box[4],i;
	int x1,y1,x2,y2;

	if (side)
	{
		x1=n->lmnx;
		y1=n->lmxy;
		x2=n->lmxx;
		y2=n->lmny;
	}
	else
	{
		x1=n->rmnx;
		y1=n->rmxy;
		x2=n->rmxx;
		y2=n->rmny;
	}
	W2S(x1,y1,box,box+1,&view);
	W2S(x2,y2,box+2,box+3,&view);
	for (i=0;i<4;i++)
		box1[i]=box[i];
	return Intersect(box1,box2);
}

int LineOnScreen(int x1,int y1,int x2,int y2)
{
	short box1[4],box2[4]={0,0,HRES-1,VRES-1};
	int box[4],i;

	W2S(min(x1,x2),max(y1,y2),box,box+1,&view);
	W2S(max(x1,x2),min(y1,y2),box+2,box+3,&view);
	for (i=0;i<4;i++)
		box1[i]=box[i];
	return Intersect(box1,box2);
}

static void R_DrawSubSectors(unsh nodenum, int level, int *route)
{
	int i;

	// recursively traverse the BSP tree

	if (nodenum & NF_SUBSECTOR)			// if its a leaf - a subsector
	{
		unsh subsec = nodenum & ~NF_SUBSECTOR;
		HalfPlane *hphead=NULL,*hp,*last=NULL;
		unsh ns=Ssectors[subsec].nsegs;
		unsh s0=Ssectors[subsec].seg0;
		
		// add the nodes along the route to the halfplane list for
		// this subsector

		{
			unsh n=NNodes-1;

			for (i=0;!(n & NF_SUBSECTOR) && i<level;i++)
			{
				short x1 = Nodes[n].x;
				short y1 = Nodes[n].y;
				short x2 = Nodes[n].dx+x1;
				short y2 = Nodes[n].dy+y1;


				hp = malloc(sizeof(HalfPlane));
				if (last)
					last->next = hp;
				else
					hphead = hp;


				hp->a = -(y2-y1);
				hp->b = x2-x1;
				hp->c = x1*y2-x2*y1;
				hp->side = route[i]? SD_LEFT : SD_RIGHT;
				hp->next = NULL;

				last = hp;
				n = Nodes[n].children[route[i]];
			}
		}

		// add the subsector's lines to the halfplane list
		// subsector is on the right side of all of these lines

		for (i=s0;i<s0+ns;i++)
		{
			short x1 = Vertexes[Segs[i].from].x;
			short y1 = Vertexes[Segs[i].from].y;
			short x2 = Vertexes[Segs[i].to].x;
			short y2 = Vertexes[Segs[i].to].y;

			hp = malloc(sizeof(HalfPlane));
			if (last)
				last->next = hp;
			else
				hphead = hp;

//		-(y2-y1)x + (x2-x1)y + x1y2 - y1x2

			hp->a = -(y2-y1);
			hp->b = x2-x1;
			hp->c = x1*y2-x2*y1;
			hp->side = SD_RIGHT;
			hp->next = NULL;

			last = hp;
		}

		DrawHalfPlanes(hphead);
		FreeHalfPlane(&hphead);
		return;
	}

	if (NodeOnScreen(Nodes+nodenum,0))
	{
		route[level] = 0;
		R_DrawSubSectors(Nodes[nodenum].children[0],level+1,route);
	}

	if (NodeOnScreen(Nodes+nodenum,1))
	{
		route[level] = 1;
		R_DrawSubSectors(Nodes[nodenum].children[1],level+1,route);
	}
}		

// This routine clips a node line to a convex hull specified
// by a list of HalfPlanes

void ClipNodeFromAbove(int nodenum,HalfPlane *hp,double *minp,double *maxp)
{
	int	x1 = Nodes[nodenum].x;
	int y1 = Nodes[nodenum].y;
	int x2 = Nodes[nodenum].dx+x1;
	int y2 = Nodes[nodenum].dy+y1;
	HalfPlane *h,n = {-(y2-y1),x2-x1,x1*y2-x2*y1,0,NULL};

	for (h=hp;h;h=h->next)
	{
		double x,y,t=0;

		if (IntersectHalfPlanes(h,&n,&x,&y))
		{
			HalfPlane honly = *h;
			honly.next = NULL;

			// get the parametric position along the node line
			// where it intersects the halfplane edge

			if (x2-x1 && abs(x2-x1)>=abs(y2-y1))
				t = (x-x1)/(x2-x1);
			else if (abs(y2-y1)>abs(x2-x1))
				t = (y-y1)/(y2-y1);

			// this limits the extent of the node line, which end
			// depending on the whether increasing t moves inside or outside
			// the halfplane.

			if (InHalfPlanes(x1+(t+10)*(x2-x1),y1+(t+10)*(y2-y1),&honly))
			{
				if (t>*minp)		// minp is the largest t from any intersection
					*minp = t;		// where increasing t moves inside the halfplane
			}
			else
			{
				if (t<*maxp)		// maxp is the smallest t from any intersection
					*maxp = t;		// where increasing t moves outside the halfplane
			}
		}

	}
}

void ClipNodeFromBelow(int nodenum,SegList *sgl,double *minp,double *maxp)
{
	int nx1 = Nodes[nodenum].x;
	int ny1 = Nodes[nodenum].y;
	int nx2 = Nodes[nodenum].dx+nx1;
	int ny2 = Nodes[nodenum].dy+ny1;
	int dnx = nx2-nx1;
	int dny = ny2-ny1;
	HalfPlane nhp = {-dny,dnx,nx1*ny2-nx2*ny1,0,NULL};
	SegList *sg;

	for (sg=sgl;sg;sg=sg->next)
	{
		int sx1 = Vertexes[Segs[sg->segnum].from].x;
		int sy1 = Vertexes[Segs[sg->segnum].from].y;
		int sx2 = Vertexes[Segs[sg->segnum].to].x;
		int sy2 = Vertexes[Segs[sg->segnum].to].y;
		int dsx = sx2-sx1;
		int dsy = sy2-sy1;
		HalfPlane shp = {-dsy,dsx,sx1*sy2-sx2*sy1,0,NULL};
		double x,y,s=-1,t=0;

		if (IntersectHalfPlanes(&nhp,&shp,&x,&y))
		{
			// (x,y) = = (sx1,sy1) + s*(dsx,dsy)

			if (dsx && abs(dsx)>=abs(dsy))
				s = (x-sx1)/dsx;
			else if (abs(dsy)>abs(dsx))
				s = (y-sy1)/dsy;

			// (x,y) = = (nx1,ny1) + t*(dnx,dny)

			if (0<=s && s<=1)
			{
				if (dnx && abs(dnx)>=abs(dny))
					t = (x-nx1)/dnx;
				else if (abs(dny)>abs(dnx))
					t = (y-ny1)/dny;

				if (t<*minp)
					*minp = t;
				if (t>*maxp)
					*maxp = t;
			}
		}
		else
		{
			if
			(
				(nhp.a && abs(nhp.a)>=abs(nhp.b) && nhp.c/nhp.a == shp.c/shp.a)
				||
				(abs(nhp.a)<abs(nhp.b) && nhp.c/nhp.b == shp.c/shp.b)
			)
			{
				if (dnx && abs(dnx)>=abs(dny))
					t = (sx1-nx1)/(double)dnx;
				else if (abs(dny)>abs(dnx))
					t = (sy1-ny1)/(double)dny;

				if (t<*minp)
					*minp = t;
				if (t>*maxp)
					*maxp = t;

				if (dnx && abs(dnx)>=abs(dny))
					t = (sx2-nx1)/(double)dnx;
				else if (abs(dny)>abs(dnx))
					t = (sy2-ny1)/(double)dny;

				if (t<*minp)
					*minp = t;
				if (t>*maxp)
					*maxp = t;
			}
		}
	}
}

void ClipNodeToMap(int nodenum,double *minp,double *maxp)
{
	int x1 = Nodes[nodenum].x;
	int y1 = Nodes[nodenum].y;
	int dx = Nodes[nodenum].dx;
	int dy = Nodes[nodenum].dy;
	int x2 = dx+x1, y2 = dy+y1;
	HalfPlane nhp = {-dy,dx,x1*y2-x2*y1,0,NULL};
	double x,y,s=0,t=0;
	int i;

	for (i=0;i<NLinedefs;i++)
	{
		int lx1 = Vertexes[Linedefs[i].from].x;
		int ly1 = Vertexes[Linedefs[i].from].y;
		int lx2 = Vertexes[Linedefs[i].to].x;
		int ly2 = Vertexes[Linedefs[i].to].y;
		int dlx = lx2-lx1;
		int dly = ly2-ly1;
		HalfPlane lhp = {-dly,dlx,lx1*ly2-lx2*ly1,0,NULL};

		if (IntersectHalfPlanes(&nhp,&lhp,&x,&y))
		{
				if (dlx && abs(dlx)>=abs(dly))
					s = (x-lx1)/dlx;
				else if (abs(dly)>abs(dlx))
					s = (y-ly1)/dly;

				if (0<=s && s<=1)
				{
					if (dx && abs(dx)>=abs(dy))
						t = (x-x1)/dx;
					else if (abs(dy)>abs(dx))
						t = (y-y1)/dy;

					if (t<*minp)
						*minp = t;
					if (t>*maxp)
						*maxp = t;
				}
		}
	}
}

// Return a list of segs below a given node or in a given subsector
// Requires level and route information from a tree traverse

SegList *SegsBelowNode(int nodenum,int level,int *route)
{
	SegList *list=NULL;
	SegList *last=NULL;

	R_SegsBelowNode(nodenum,level,route,&list,&last);

	return list;
}

void R_DrawNodeLines(int nodenum,int level,int *route)
{
	int x1,y1,x2,y2;
	int sx1,sy1,sx2,sy2;
	double mint=-1.1e17;
	double maxt=1.1e17;
	double minr=1.1e17;
	double maxr=-1.1e17;

	if (nodenum & NF_SUBSECTOR)
		return;
	else
	{
		HalfPlane *hp=HullAboveNode(nodenum,level,route);

		// Clip this node line to the ancestor node lines convex hull

		ClipNodeFromAbove(nodenum,hp,&mint,&maxt);

		// Find the extreme intersections of this node with linedefs
		// in the map for all unbounded nodes after clipping from above

		if (mint<-1e17 || maxt>1e17)
		{
			ClipNodeToMap(nodenum,&minr,&maxr);

			if (mint<-1e17)
				if (minr>mint)
					mint = minr;
			if (maxt>1e17)
				if (maxr<maxt)
					maxt = maxr;
		}
		if (mint<=maxt)
		{
			x1 = Nodes[nodenum].x;
			y1 = Nodes[nodenum].y;
			x2 = Nodes[nodenum].dx+x1;
			y2 = Nodes[nodenum].dy+y1;

			W2S((int)floor(x1+mint*(x2-x1)+.5),(int)floor(y1+mint*(y2-y1)+.5),&sx1,&sy1,&view);
			W2S((int)floor(x1+maxt*(x2-x1)+.5),(int)floor(y1+maxt*(y2-y1)+.5),&sx2,&sy2,&view);
			GrLine(sx1,sy1,sx2,sy2,12);
		}

		FreeHalfPlane(&hp);

		if (NodeOnScreen(Nodes+nodenum,0))
		{
			route[level] = 0;
			R_DrawNodeLines(Nodes[nodenum].children[0],level+1,route);
		}

		if (NodeOnScreen(Nodes+nodenum,1))
		{
			route[level] = 1;
			R_DrawNodeLines(Nodes[nodenum].children[1],level+1,route);
		}
	}
}

void DrawSubSectors()
{
	int *route = malloc(NNodes*sizeof(int));

	fillcol=0;
	R_DrawSubSectors(NNodes-1,0,route);
	free(route);
	route=NULL;
}

void DrawNodeLines()
{
	int *route = malloc(NNodes*sizeof(int));

	fillcol=0;
	R_DrawNodeLines(NNodes-1,0,route);
	free(route);
	route=NULL;
}

int R_RouteToSubSector(int nodenum,int *level,int *route,int subsec)
{
	if (nodenum & ~NF_SUBSECTOR)
	{
		int ssec = nodenum&~NF_SUBSECTOR;
		return ssec==subsec;
	}

	route[(*level)++]=0;
	if (R_RouteToSubSector(Nodes[nodenum].children[0],level,route,subsec))
		return 1;
	else
		(*level)--;

	route[(*level)++]=1;
	if (R_RouteToSubSector(Nodes[nodenum].children[1],level,route,subsec))
		return 1;
	else
		(*level)--;

	return 0;
}

void RouteToSubSector(int subsec,int *level,int *route)
{
	R_RouteToSubSector(NNodes-1,level,route,subsec);
}

void GetSubSectorNodeVertexes(int subsec,int **objs,int *nobjs)
{
	int *route=malloc(NNodes*sizeof(int));
	int level;
	double mint;
	double maxt;
	double minr;
	double maxr;
	int x1,y1,x2,y2,nodenum;
	HalfPlane *hp,*h;

	RouteToSubSector(subsec,&level,route);

	// Draw the node lines above the subsector wide

	// Clip the node to the convex hull above

	nodenum = NNodes-1;
	hp = HullAboveNode(subsec|NF_SUBSECTOR,level,route);
	for (h=hp;h;h=h->next)
	{
		ClipNodeFromAbove(nodenum,hp,&mint,&maxt);

		// Find the extreme intersections of this node with linedefs
		// in the map for all unbounded nodes after clipping from above

		if (mint<-1e17 || maxt>1e17)
		{
			ClipNodeToMap(nodenum,&minr,&maxr);

			if (mint<-1e17)
				if (minr>mint)
					mint = minr;
			if (maxt>1e17)
				if (maxr<maxt)
					maxt = maxr;
		}
		if (mint<=maxt)
		{
			int i=0,j=0;

			x1 = Nodes[nodenum].x;
			y1 = Nodes[nodenum].y;
			while (i<NVertexes && (Vertexes[i].x!=x1 || Vertexes[i].y!=y1))
				i++;
			if (i<NVertexes)
			{
				x2 = Nodes[nodenum].dx+x1;
				y2 = Nodes[nodenum].dy+y1;
				while (j<NVertexes && (Vertexes[j].x!=x2 || Vertexes[j].y!=y2))
					j++;
				if (j<NVertexes)
				{
					AddDynInt(objs,nobjs,i);
					AddDynInt(objs,nobjs,j);
				}
			}
		}

		nodenum = Nodes[nodenum].children[route[level++]];
	}
}

void ShowSubSectorData(int x,int y,int mx,int my)
{
	int sn=-1,wx,wy;
	char buffer[256];
	int h = 13;
	static int oldsn=-1;


	if (Mode!=M_NODE) return;
	if (!IsInBox(mx,my,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy)) return;

	S2W(mx,my,&wx,&wy,&view);
	sn = SubSectorOf(wx,wy);
	if (!FullScreen)
	{
		if (oldsn!=sn && oldsn>=0) GrFilledBox(4,352,HRES-VRES+10,VRES-5,7);
		if (sn!=oldsn && sn>=0)
		{
			sprintf(buffer,"SUBSECTOR: %5d",sn);
			topt.txo_fgcolor.v=0;
			topt.txo_bgcolor.v=7;
    	GrDrawString(buffer,strlen(buffer),x,y+0*h,&topt);
			sprintf(buffer,"NSEGS: %-3d",Ssectors[sn].nsegs);
			GrDrawString(buffer,strlen(buffer),x,y+1*h,&topt);
			sprintf(buffer,"SEG0:  %-3d",Ssectors[sn].seg0);
		}
		oldsn = sn;
	}

	if (sn!=oldobj1)
	{
		if (oldobj1>=0)
			ClearVertexes();

		if (sn>=0)
		{
			GetVertexesInSubSector(sn,&verts1,&nverts1);
			DrawVertexList(verts1,nverts1,14,&view);
//			GetSubSectorNodeVertexes(NNodes-1,&verts1,&nverts1);
		}
		oldobj1 = sn;
	}
}

int TextWid(WadTextures *wt,char *textname)
{
	int i,n;

	i = FindTextureN(wt,textname,&n);
	if (i>=0)
	{
		switch(n)
		{
			case 0:
				return wt->t1.textures[i].twid;
			case 1:
				return wt->t2.textures[i].twid;
			case 2:
				return wt->t3.textures[i].twid;
			default:
			   	break;
		}
	}
	return 1;
}

int Equiv(int a,int b,int modulus)
{
	return (a-b)%modulus == 0;
}

int SidedefsXAligned(WadTextures *wt,int lsn,int rsn,int llen)
{
	int ul,nl,ll,ur,nr,lr;
	char lutx[16],lntx[16],lltx[16];
	char rutx[16],rntx[16],rltx[16];

	if (lsn<0 || rsn<0) return 1;

	ul = strcmp(Sidedefs[lsn].utxt,"-");
	nl = strcmp(Sidedefs[lsn].ntxt,"-");
	ll = strcmp(Sidedefs[lsn].ltxt,"-");
	ur = strcmp(Sidedefs[rsn].utxt,"-");
	nr = strcmp(Sidedefs[rsn].ntxt,"-");
	lr = strcmp(Sidedefs[rsn].ltxt,"-");

	strcpy(lutx,Str8(Sidedefs[lsn].utxt));
	strcpy(lntx,Str8(Sidedefs[lsn].ntxt));
	strcpy(lltx,Str8(Sidedefs[lsn].ltxt));
	strcpy(rutx,Str8(Sidedefs[rsn].utxt));
	strcpy(rntx,Str8(Sidedefs[rsn].ntxt));
	strcpy(rltx,Str8(Sidedefs[rsn].ltxt));

	if (ul && ur && !strcmp(lutx,rutx))
		if (!Equiv(Sidedefs[rsn].xoff,Sidedefs[lsn].xoff+llen,TextWid(wt,lutx)))
			return 0;
	
	if (ul && nr && !strcmp(lutx,rntx))
		if (!Equiv(Sidedefs[rsn].xoff,Sidedefs[lsn].xoff+llen,TextWid(wt,lutx)))
			return 0;
	
	if (ul && lr && !strcmp(lutx,rltx))
		if (!Equiv(Sidedefs[rsn].xoff,Sidedefs[lsn].xoff+llen,TextWid(wt,lutx)))
			return 0;
	
	if (nl && ur && !strcmp(lntx,rutx))
		if (!Equiv(Sidedefs[rsn].xoff,Sidedefs[lsn].xoff+llen,TextWid(wt,lntx)))
			return 0;
	
	if (nl && nr && !strcmp(lntx,rntx))
		if (!Equiv(Sidedefs[rsn].xoff,Sidedefs[lsn].xoff+llen,TextWid(wt,lntx)))
			return 0;
	
	if (nl && lr && !strcmp(lntx,rltx))
		if (!Equiv(Sidedefs[rsn].xoff,Sidedefs[lsn].xoff+llen,TextWid(wt,lntx)))
			return 0;
	
	if (ll && ur && !strcmp(lltx,rutx))
		if (!Equiv(Sidedefs[rsn].xoff,Sidedefs[lsn].xoff+llen,TextWid(wt,lltx)))
			return 0;
	
	if (ll && nr && !strcmp(lltx,rntx))
		if (!Equiv(Sidedefs[rsn].xoff,Sidedefs[lsn].xoff+llen,TextWid(wt,lltx)))
			return 0;
	
	if (ll && lr && !strcmp(lltx,rltx))
		if (!Equiv(Sidedefs[rsn].xoff,Sidedefs[lsn].xoff+llen,TextWid(wt,lltx)))
			return 0;

	return 1;
}


int LinedefsXAligned(WadTextures *wt,int ln1,int ln2,int vn)
{
	int len1,len2;

	len1 = LineLen(Linedefs+ln1);
	len2 = LineLen(Linedefs+ln2);

	if (!len1 || !len2) return 1;

	if (Linedefs[ln1].to==vn)
	{
		if (Linedefs[ln2].to==vn)
		{
			if 
			(
				SidedefsXAligned(wt,Linedefs[ln1].sides[0],Linedefs[ln2].sides[1],len1) &&
				SidedefsXAligned(wt,Linedefs[ln2].sides[0],Linedefs[ln1].sides[1],len2)
			)
				return 1;
		}
		else
		{
			if 
			(
				SidedefsXAligned(wt,Linedefs[ln1].sides[0],Linedefs[ln2].sides[0],len1) &&
				SidedefsXAligned(wt,Linedefs[ln2].sides[1],Linedefs[ln1].sides[1],len2)
			)
				return 1;
		}
	}
	else
	{
		if (Linedefs[ln2].to==vn)
		{
			if 
			(
				SidedefsXAligned(wt,Linedefs[ln1].sides[1],Linedefs[ln2].sides[1],len1) &&
				SidedefsXAligned(wt,Linedefs[ln2].sides[0],Linedefs[ln1].sides[0],len2)
			)
				return 1;
		}
		else
		{
			if 
			(
				SidedefsXAligned(wt,Linedefs[ln1].sides[1],Linedefs[ln2].sides[0],len1) &&
				SidedefsXAligned(wt,Linedefs[ln2].sides[1],Linedefs[ln1].sides[0],len2)
			)
				return 1;
		}
	}
	return 0;
}

int VertexXAligned(WadTextures *wt,int vn)
{
	int i,j,aligned=1;
	int *lins=NULL,nlins=0;

	for (i=0;i<NLinedefs;i++)
		if (Linedefs[i].from==vn || Linedefs[i].to==vn)
			AddDynInt(&lins,&nlins,i);

	for (i=0;i<nlins;i++)
		for (j=i+1;j<nlins;j++)
			if (!LinedefsXAligned(wt,lins[i],lins[j],vn))
			{
				aligned=0;
				break;
			}

	FreeDynInt(&lins,&nlins);
	return aligned;
}

void DrawThings(View *v)
{
	int i;

	for (i=0;i<NThings;i++)
		DrawThing(i,v);
}

void DrawMap(View *v)
{
	int c,a,s,i,x1=0,y1=0,x2=0,y2=0;
	int t,sx,sy,r=0,d=0,sx1,sy1,sx2,sy2;
	int cx,cy,dx,dy;
	int mrk=0;
	int Li2SMode = flinButtons->btn[2].stt;
	int LiMaMode = flinButtons->btn[3].stt;
	int LiSoMode = flinButtons->btn[4].stt;
	int LiLwMode = flinButtons->btn[5].stt;
	int LiUpMode = flinButtons->btn[6].stt;
	int LiSeMode = flinButtons->btn[7].stt;
	int LiMoMode = flinButtons->btn[8].stt;
	int LiImMode = flinButtons->btn[9].stt;
	int LiInMode = flinButtons->btn[10].stt;
	int LiTrMode = flinButtons->btn[11].stt;
	int SeLightsMode = fsecButtons->btn[1].stt;
	int SeDamageMode = fsecButtons->btn[2].stt;
	int SeSecretMode = fsecButtons->btn[3].stt;
	int SeOpenMode	 = fsecButtons->btn[4].stt;
	int SeTaggedMode = fsecButtons->btn[5].stt;
	int VeNormalMode = fvrtButtons->btn[1].stt;
	int VeUnusedMode = fvrtButtons->btn[2].stt;
	int VeXMalignMode = fvrtButtons->btn[3].stt;
	int VeYMalignMode = fvrtButtons->btn[4].stt;
	int *route = malloc(NNodes*sizeof(int));

	ClearLines();
	GrSetClipBox(mapw.minx,mapw.miny,mapw.maxx,mapw.maxy);

	if (Mode==M_NODE)
		DrawSubSectors(NNodes-1,0,route);

	if (grid) DrawGrid(v);

	for (i=0;i<(Mode==M_NODE? NSegs : NLinedefs);i++)
	{
		Seg *sg = i<NSegs? Segs+i : NULL;
		Linedef *ld = Mode==M_NODE? Linedefs+Segs[i].ldef : Linedefs+i;
		Vertex *v1 = Vertexes + (Mode==M_NODE? sg->from : ld->from);
		Vertex *v2 = Vertexes + (Mode==M_NODE? sg->to : ld->to);
		x1 = v1->x;
		y1 = v1->y;
		x2 = v2->x;
		y2 = v2->y;
		if (!LineOnScreen(x1,y1,x2,y2))
			continue;

		a = ld->attr;
		c = (ld->attr&Lf2S)? 7 : 15;
		if (Mode==M_DOOM)
		{
			if (ld->attr&LfIn) c=0;
			else if (ld->sides[1]==-1 || ld->attr&LfSe) c=12;
			else
			{
				Sector *s1 = Sectors+Sidedefs[ld->sides[0]].sect;
				Sector *s2 = Sectors+Sidedefs[ld->sides[1]].sect;
				if (s1->fhgt==s2->fhgt && s1->chgt==s2->chgt)
					c = 0;
				else if (s1->chgt!=s2->chgt)
					c = 14;
				else if (s1->fhgt!=s2->fhgt)
					c = 7;
			}
		}
		else if (Mode==M_LINE || Mode==M_NODE)
		{
			mrk=0;
			if (IsMarked(i))
			{
				mrk=1;
				c = 6;
			}
			else
			{
				if (Li2SMode)
					if (a&Lf2S) c = 7;
				if (LiMaMode)
					if (a&LfMa) c = 1;
				if (LiSoMode)
					if (a&LfSo) c = 4;
				if (LiLwMode)
					if (a&LfLw) c = 11;
				if (LiUpMode)
					if (a&LfUp) c = 2;
				if (LiSeMode)
					if (a&LfSe) c = 13;
				if (LiMoMode)
					if (a&LfMo) c = 9;
				if (LiImMode)
					if (a&LfIm) c = 15;
				if (LiInMode)
					if (a&LfIn) c = 6;
				if (LiTrMode && (ld->tag>0 || ld->typ>0))
				{
					if (ld->tag>0 && ld->typ>0)
					{
						switch(ld->typ)
						{
							case 11: case 52: case 197:
							case 51: case 124: case 198:
							case 48:
								c = 14;
								break;
							case 1: case 26: case 27: case 28:
							case 117: case 118:
							case 31: case 32: case 33: case 34:
								c = (ld->sides[1]==-1)? 12 : 14;
								break;
							default:
								c = 10;
								break;
						}
					}
					else if (ld->tag>0)
					{
						c = 14;
					}
					else if (ld->typ>0)
					{
						switch (ld->typ)
						{
							case 11: case 52:
							case 51: case 124:
							case 48:
								c = 10;
								break;
							case 1: case 26: case 27: case 28:
							case 117: case 118:
							case 31: case 32: case 33: case 34:
								c = (ld->sides[1]==-1)? 12 : 10;
								break;
							default:
								c = 12;
								break;
						}
					}
					if (Mode!=M_NODE)
					{
						d=13;
						switch(ld->typ)
						{
							case 11: case 52: case 197:
								d=12;
							case 51: case 124: case 198:
								W2S(x1,y1,&sx1,&sy1,v);
								W2S(x1+64,y1,&sx,&sy,v);
								W2S(x2,y2,&sx2,&sy2,v);
								r = sx-sx1;
								GrCircle((sx1+sx2)/2,(sy1+sy2)/2,r,d);
								GrCircle((sx1+sx2)/2,(sy1+sy2)/2,r-1,d);
								GrCircle((sx1+sx2)/2,(sy1+sy2)/2,r-2,d);
								GrCircle((sx1+sx2)/2,(sy1+sy2)/2,r-3,d);
								break;
							default:
								break;
						}
					}
				}
			}
		}
		else if (Mode==M_SECTOR)
		{
			int s1 = ld->sides[0];
			int s2 = ld->sides[1];
			Sidedef *sd1 = Sidedefs+s1;
			Sidedef *sd2 = Sidedefs+s1;
			Sector *sc1 = Sectors+sd1->sect;
			Sector *sc2 = Sectors+sd2->sect;

			mrk=0;
			if ((s1>=0 && SectorIsMarked(sd1->sect)) ||
				(s2>=0 && SectorIsMarked(sd2->sect)))
			{
				mrk=1;
				c=6;
			}
			else
			{
				s = sc1->typ;
				t = sc1->tag;
				if (!s && s2>=0)
				{
					s = sc2->typ;
					t = sc1->tag;
				}
				switch(s)
				{
					case 1: case 2: case 3:
					case 8: case 12: case 13: case 17:
						if (SeLightsMode)
							c = 14;
						break;
					case 5: case 7: case 4: case 11: case 16:
						if (SeDamageMode)
							c = 10;
						break;
					case 9:
						if (SeSecretMode)
							c = 12;
						break;
					case 10: case 14:
						if (SeOpenMode)
							c = 6;
						break;
					default:
						break;
				}
				if ((c==7 || c==15) && t)
					if (SeTaggedMode)
						c = 11;
			}
		}
		if (c)
		{
			if (mrk)
			{
				ThickWorldLine(x1,y1,x2,y2,c,v);
				if (Mode==M_LINE)
				{
					dx = x2-x1;
					dy = y2-y1;
					cx = (x2+x1)/2;
					cy = (y2+y1)/2;
					WorldLine(x1+.85*dx-.15*dy,y1+.85*dy+.15*dx,x2,y2,c,v);
					WorldLine(x1+.85*dx+.15*dy,y1+.85*dy-.15*dx,x2,y2,c,v);
					WorldLine(cx,cy,cx+.15*dy,cy-.15*dx,c,v);
				}
			}
			else
			{
				WorldLine(x1,y1,x2,y2,c,v);
				if ((Mode==M_VERTEX || Mode==M_NODE) && VeNormalMode)
				{
					DrawVertex(v1-Vertexes,1,14,1,v);
					DrawVertex(v2-Vertexes,1,14,1,v);
				}
			}
		}
	}
	if (Mode==M_NODE)
		DrawNodeLines(NNodes-1,0,route);

	if (Mode==M_VERTEX || Mode==M_NODE)
		if (VeUnusedMode || Mode==M_NODE)
			for (i=0;i<NVertexes;i++)
				if (!VertUsed(i))
					DrawVertex(i,1,10,1,v);

	if (Mode==M_VERTEX)
	{
		if (VeXMalignMode)
		{
			if (!vxaligned)
			{
				vxaligned = (int *)realloc(vxaligned,NVertexes);
				for (i=0;i<NVertexes;i++)
					vxaligned[i] = VertexXAligned(&pwt,i);
			}
			for (i=0;i<NVertexes;i++)
				if (!vxaligned[i])
					DrawVertex(i,3,12,0,v);
		}
		else if (VeYMalignMode)
		{
			;
		}
	}

	if (Mode==M_THING)
		DrawThings(&view);

	GrResetClipBox();
}


void DrawModeButtons()
{
	DrawButtonArray(modeBx,modeBy,modeButtons);
	DrawButtonArray(fprvBx,fprvBy,fprvButtons);
	GrFramedBox(mnw.minx,mnw.miny,mnw.maxx,mnw.maxy,2,&infrm);
	GrDrawString(mapsel,5,mnw.minx+1,mnw.miny,&eopt);
}

int IsInModeButtons(int mx,int my)
{
	return IsInButtonArray(mx,my,modeBx,modeBy,modeButtons);
}

int IsInNxtPrvButtons(int mx,int my)
{
	return IsInButtonArray(mx,my,fprvBx,fprvBy,fprvButtons);
}

void DrawFieldButtons()
{
	int x,y;

	if (ViewOnly) return;
	x = fldBx;
	y = fldBy;
	GrFramedBox(edw.minx,edw.miny,edw.maxx,edw.maxy,2,&midfrm);
	switch(Mode)
	{
		case M_VERTEX:
			DrawButtonArray(x,y,vertButtons);
			break;
		case M_THING:
			DrawButtonArray(x,y,thngButtons);
			break;
		case M_LINE:
			DrawButtonArray(x,y,lineButtons);
			break;
		case M_SECTOR:
			DrawButtonArray(x,y,sectButtons);
			break;
	}
}

void DrawScreenBackground()
{
	GrFramedBox(scrn.minx+4,scrn.miny+4,scrn.maxx-4,scrn.maxy-4,4,&outfrm);
	GrFramedBox(mapw.minx,mapw.miny,mapw.maxx,mapw.maxy,4,&infrm);
	if (!ViewOnly) GrFramedBox(edw.minx,edw.miny,edw.maxx,edw.maxy,2,&midfrm);
	GrFramedBox(btnw.minx,btnw.miny,btnw.maxx,btnw.maxy,2,&midfrm);
	DrawModeButtons();
	if (!ViewOnly) DrawFieldButtons();
}

void SaveScreen()
{
	Bitmap out;
	RGBQUAD *pal;
	int i,j;
	char bmpname[256];

	out.realw = mapw.maxx-mapw.minx+1;
	out.logh = mapw.maxy-mapw.miny+1;
	out.logw = nextmultfour(out.realw);
	out.bmp = malloc(out.logh*out.logw);
	pal = malloc(sizeof(RGBQUAD)*256);


	for (i=mapw.miny;i<=mapw.maxy;i++)
		for (j=mapw.minx;j<=mapw.maxx;j++)
			out.bmp[out.logw*(i-mapw.miny)+j-mapw.minx] = GrPixel(j,i);

	for (i=0;i<256;i++)
	{
		pal[i].rgbRed = 0;
		pal[i].rgbGreen = 0;
		pal[i].rgbBlue = 0;
		pal[i].rgbReserved = 0;
	}

	for (i=0;i<16;i++)
	{
		pal[i].rgbRed = stdp[i][0];
		pal[i].rgbGreen = stdp[i][1];
		pal[i].rgbBlue = stdp[i][2];
	}

	strcpy(bmpname,NextFileName(pwadname,".BMP"));
	if (*bmpname)
		WriteBMP(&out,bmpname,pal);
	free(out.bmp);
	free(pal);
}

char *GetFirstLevel(WadDir *dir)
{
	static char maptag[16];
	int i,j;

	for (i=1;i<=32;i++)
	{
		sprintf(maptag,"MAP%02d",i);
		if (IsIn(maptag,dir)>=0)
			return(maptag);
	}

	for (i=1;i<=4;i++)
		for (j=1;j<=9;j++)
		{
			sprintf(maptag,"E%0dM%0d",i,j);
			if (IsIn(maptag,dir)>=0)
				return(maptag);
		}
	return "";
}

char *GetNextLevel(char *curname,WadDir *dir,int prev)
{
	int i,j;

	for (i=0;i<ND2Names;i++)
		if (stricmp(curname,D2Names[i])==0)
			break;

	if (i<ND2Names)
	{
		for (j=1;j<ND2Names;j++)
			if (prev)
			{
				if (IsIn(D2Names[(i+ND2Names-j)%ND2Names],dir)>=0)
					return D2Names[(i+ND2Names-j)%ND2Names];
			}
			else
			{
				if (IsIn(D2Names[(j+i)%ND2Names],dir)>=0)
					return D2Names[(j+i)%ND2Names];
			}
	}
	else
	{
		for (i=0;i<ND1Names;i++)
			if (stricmp(curname,D1Names[i])==0)
				break;

		if (i<ND1Names)
		{
			for (j=1;j<ND1Names;j++)
				if (prev)
				{
					if (IsIn(D1Names[(i+ND1Names-j)%ND1Names],dir)>=0)
						return D1Names[(i+ND1Names-j)%ND1Names];
				}
				else
				{
					if (IsIn(D1Names[(j+i)%ND1Names],dir)>=0)
						return D1Names[(j+i)%ND1Names];
				}
		}
	}
	return "";
}

void FullScreenView()
{
	int tmp;

	tmp = view.minx - ((view.maxx-view.minx)*(WINDOW_LEFT-SCREEN_LEFT))/(WINDOW_RIGHT-WINDOW_LEFT);
	view.maxx = view.maxx + ((view.maxx-view.minx)*(SCREEN_RIGHT-WINDOW_RIGHT))/(WINDOW_RIGHT-WINDOW_LEFT);
	view.minx = tmp;
	tmp = view.miny - ((view.maxy-view.miny)*(SCREEN_BOTTOM-WINDOW_BOTTOM))/(WINDOW_BOTTOM-WINDOW_TOP);
	view.maxy = view.maxy + ((view.maxy-view.miny)*(WINDOW_TOP-SCREEN_TOP))/(WINDOW_BOTTOM-WINDOW_TOP);
	view.miny = tmp;
	mapw.minx = SCREEN_LEFT;
	mapw.miny = SCREEN_TOP;
	mapw.maxx =	SCREEN_RIGHT-1;
	mapw.maxy = SCREEN_BOTTOM-1;
}

void WindowedView()
{
	int tmp;

	tmp = view.minx + ((view.maxx-view.minx)*(WINDOW_LEFT-SCREEN_LEFT))/(SCREEN_RIGHT-SCREEN_LEFT);
	view.maxx = view.maxx - ((view.maxx-view.minx)*(SCREEN_RIGHT-WINDOW_RIGHT))/(SCREEN_RIGHT-SCREEN_LEFT);
	view.minx = tmp;
	tmp = view.miny + ((view.maxy-view.miny)*(SCREEN_BOTTOM-WINDOW_BOTTOM))/(SCREEN_BOTTOM-SCREEN_TOP);
	view.maxy = view.maxy - ((view.maxy-view.miny)*(WINDOW_TOP-SCREEN_TOP))/(SCREEN_BOTTOM-SCREEN_TOP);
	view.miny = tmp;
	mapw.minx = WINDOW_LEFT;
	mapw.miny = WINDOW_TOP;
	mapw.maxx = WINDOW_RIGHT;
	mapw.maxy = WINDOW_BOTTOM;
}

void DrawStatus(int mx,int my,int updco,int updsts,int upddat,int updbtn,int updmod)
{
	int x,y;
	char costr[256],fillstr[32],*p;
	int i;

	if (!updmod && !updco && !updsts && !upddat && !updbtn) return;
	if (updmod)
	{
		EraseStatusPanel();
		updco=updsts=upddat=updbtn=1;
	}

	if (updco && !FullScreen)
	{
		if (IsInBox(mx,my,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
		{
			S2W(mx,my,&x,&y,&view);
			GrFramedBox(cow.minx,cow.miny,cow.maxx,cow.maxy,2,&infrm);
			strcpy(fillstr,"               ");
			i = strlen(p=stringf("(%d,%d)",max(-32768,min(32767,x)),max(-32768,min(32767,y))));
			i = max(0,((cow.maxx-cow.minx)/9 - i)/2);
			if (i<strlen(fillstr)) fillstr[i]='\0';
			sprintf(costr,"%s%s",fillstr,p);
			topt.txo_bgcolor.v=7;
			topt.txo_fgcolor.v=0;
			GrDrawString(costr,strlen(costr),cow.minx+4,cow.miny+2,&cotop);
		}
		else GrFramedBox(cow.minx,cow.miny,cow.maxx,cow.maxy,2,&outfrm);
	}

	switch(Mode)
	{
		case M_THING:
			if ((updsts || updbtn) && !FullScreen)
				ShowThingLegend();
			if (upddat) ShowThingData(thgDx,thgDy,mx,my);
			break;
		case M_SECTOR:
			if ((updsts || updbtn) && !FullScreen)
				ShowSectorLegend();
			if (upddat) ShowSectorData(secDx,secDy,mx,my);
			break;
		case M_LINE:
			if ((updsts || updbtn) && !FullScreen)
				ShowLineLegend();
			if (upddat) ShowLineData(linDx,linDy,mx,my);
			break;
		case M_VERTEX:
			if ((updsts || updbtn) && !FullScreen)
				ShowVertLegend();
			//if (upddat) ShowVertData(vrtDx,vrtDy,mx,my);
			break;
		case M_DOOM:
			break;
		case M_NODE:
			if (upddat) ShowSubSectorData(sscDx,sscDy,mx,my);
			break;
	}
	if (!FullScreen)
	{
		topt.txo_bgcolor.v=7;
		topt.txo_fgcolor.v=9;
		GrDrawString("        ",8,84,39,&topt);
		GrDrawString(basename,strlen(basename),84,39,&topt);
	}
}


int VertexInBox(int vn,int x1,int y1,int x2,int y2)
{
	int lx1,ly1;
	int sx1,sy1;

	lx1 = Vertexes[vn].x;
	ly1 = Vertexes[vn].y;
	W2S(lx1,ly1,&sx1,&sy1,&view);

	return IsInBox(sx1,sy1,x1,y1,x2,y2);
}

int ThingInBox(int tn,int x1,int y1,int x2,int y2)
{
	int lx1,ly1;
	int sx1,sy1;

	lx1 = Things[tn].x;
	ly1 = Things[tn].y;
	W2S(lx1,ly1,&sx1,&sy1,&view);

	return IsInBox(sx1,sy1,x1,y1,x2,y2);
}

int LinedefInBox(int ln,int x1,int y1,int x2,int y2)
{
	int lx1,ly1,lx2,ly2;
	int sx1,sy1,sx2,sy2;

	lx1 = Vertexes[Linedefs[ln].from].x;
	ly1 = Vertexes[Linedefs[ln].from].y;
	lx2 = Vertexes[Linedefs[ln].to].x;
	ly2 = Vertexes[Linedefs[ln].to].y;

	W2S(lx1,ly1,&sx1,&sy1,&view);
	W2S(lx2,ly2,&sx2,&sy2,&view);

	return (IsInBox(sx1,sy1,x1,y1,x2,y2) && IsInBox(sx2,sy2,x1,y1,x2,y2));
}

int SectorInBox(int sn,int x1,int y1,int x2,int y2)
{
	int lx1,ly1,lx2,ly2;
	int sx1,sy1,sx2,sy2;
	int i,s1,s2;

	for (i=0;i<NLinedefs;i++)
	{
		s1 = Linedefs[i].sides[0];
		s2 = Linedefs[i].sides[1];
		if ((s1>=0 && Sidedefs[s1].sect==sn) || (s2>=0 && Sidedefs[s2].sect==sn))
		{
			lx1 = Vertexes[Linedefs[i].from].x;
			ly1 = Vertexes[Linedefs[i].from].y;
			lx2 = Vertexes[Linedefs[i].to].x;
			ly2 = Vertexes[Linedefs[i].to].y;

			W2S(lx1,ly1,&sx1,&sy1,&view);
			W2S(lx2,ly2,&sx2,&sy2,&view);

			if (!IsInBox(sx1,sy1,x1,y1,x2,y2) || !IsInBox(sx2,sy2,x1,y1,x2,y2))
				return 0;
		}
	}
	return 1;
}

void GetTextureWad()
{
	char buffer[256];

	if
	(
		EditBox
		(
			mapw.minx+16,
			mapw.miny+16,
			WINDOW_RIGHT-16,
			mapw.miny+80,
			"Enter path to textures:",
			strlen("Enter path to textures:"),
			mapw.minx+32,
			mapw.miny+24,
			mapw.minx+32,
			mapw.miny+52,
			buffer
		)
	)
	{
		AddExt(buffer,".WAD");

		bst = fopen(buffer,"rb");
		if (bst)
		{
			fread(&bhdr,sizeof(WadHeader),1,bst);
			ReadDir(&bhdr,&bdir,bst);
			if (IsIn("PNAMES",&bdir)>=0 && IsIn("TEXTURE1",&bdir)>=0)
			{
				ReadPNames(&bdir,bst,&pwt);
				ReadTextures(&bdir,bst,&pwt);
				texturesread=1;
			}
			FreeDir(&bdir);
			fclose(bst);
		}
	}
}

int ChgLevel(int dir)
{
	char name[16];
	int i=NThings-1;

	strcpy(name,GetNextLevel(mapsel,&pdir,dir));
	if (*name && IsIn(name,&pdir)>=0)
	{
		strcpy(mapsel,name);
		free(vxaligned);
		vxaligned=NULL;
		nxaligned=0;
		FreeLevelData();
		ReadLevelData(mapsel,&pdir,pst,11);
		for (i=NThings-1;i>=0;i--)
			if (Things[i].typ!=PLAYER1) break;
		InitView(&view);
	}
	return i+1;
}

void JumpToObject()
{
	char prompt[128]="";
	char buffer[256];
	int zf = -4,n,i,s1,s2,first;
	int x,y,x1,y1,dx,dy;

	switch(Mode)
	{
		case M_VERTEX:
			strcpy(prompt,"Enter vertex number:");
			break;
		case M_THING:
			strcpy(prompt,"Enter thing number:");
			break;
		case M_SECTOR:
			strcpy(prompt,"Enter sector number:");
			break;
		case M_LINE:
			strcpy(prompt,"Enter line number:");
			break;
		default:
			break;
	}
	SetEditLines(&elines,&nelines);
	EditBox
	(
		mapw.minx+16,
		mapw.miny+16,
		WINDOW_RIGHT-16,
		mapw.miny+80,
		prompt,
		strlen(prompt),
		mapw.minx+32,
		mapw.miny+24,
		mapw.minx+32,
		mapw.miny+52,
		buffer
	);

	Trim(buffer);
	if (IsNumeric(buffer))
	{
		n = atoi(buffer);
		switch(Mode)
		{
			case M_VERTEX:
				if (0<=n && n<=NVertexes)
				{
					x = Vertexes[n].x;
					y = Vertexes[n].y;
					CenterView(x,y,zf,&view);// redraw map centered on thing at zoom
					MouseWarp((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2);
					DrawStatus((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2,0,0,1,0,0);
				}
				break;
			case M_THING:
				if (0<=n && n<=NThings)
				{
					x = Things[n].x;
					y = Things[n].y;
					CenterView(x,y,zf,&view);// redraw map centered on thing at zoom
					MouseWarp((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2);
					DrawStatus((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2,0,0,1,0,0);
				}
				break;
			case M_NODE:
				DrawStatus((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2,0,0,1,0,0);
				break;
			case M_SECTOR:
				if (0<=n && n<=NSectors)
				{
					for (i=0;i<NLinedefs;i++)
					{
						s1 = Linedefs[i].sides[0];
						s2 = Linedefs[i].sides[1];
						first = 1;
						if (s1>=0 && Sidedefs[s1].sect==n)
							break;
						first = 0;
						if (s2>=0 && Sidedefs[s2].sect==n)
							break;
					}
					if (i<NLinedefs)
					{
						x = Vertexes[Linedefs[i].from].x;
						y = Vertexes[Linedefs[i].from].y;
						x1 = (x+Vertexes[Linedefs[i].to].x)/2;
						y1 = (y+Vertexes[Linedefs[i].to].y)/2;
					    // (y1-y),-(x1-x)
						// if a first sidedef we want a small distance
						// on the right of from -> to motion
						if (first)
						{
							dx = (y1-y);
							dy = -(x1-x);
							if (dx || dy)
							{
								while ((!dy || abs(dy)>2) && (!dx || abs(dx)>2))
								{
									dy >>= 1;
									dx >>= 1;
								}
							}
						}
						else
						{
							dx = -(y1-y);
							dy = (x1-x);
							if (dx || dy)
							{
								while ((!dy || abs(dy)>2) && (!dx || abs(dx)>2))
								{
									dy >>= 1;
									dx >>= 1;
								}
							}
						}
						CenterView(x1+dx,y1+dy,zf,&view);// redraw map centered on thing at zoom
						MouseWarp((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2);
						DrawStatus((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2,0,0,1,0,0);
					}
				}
				break;
			case M_LINE:
				if (0<=n && n<=NLinedefs)
				{
					x = Vertexes[Linedefs[n].from].x;
					y = Vertexes[Linedefs[n].from].y;
					x1 = Vertexes[Linedefs[n].to].x;
					y1 = Vertexes[Linedefs[n].to].y;
					CenterView((x+x1)/2,(y+y1)/2,zf,&view);// redraw map centered on thing at zoom
					MouseWarp((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2);
					DrawStatus((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2,0,0,1,0,0);
				}
				break;
			default:
				break;
		}
	}
}

void TogglefvrtButton(int bn)
{
	int j;

	if (!ViewOnly)
	{
		if (bn==3 || bn==4 || bn==fvrtButtons->nbtns-1)
		{
			if (!texturesread)
				GetTextureWad();
			if (texturesread)
				if (bn!=fvrtButtons->nbtns-1)
					fvrtButtons->btn[bn].stt = 1 - fvrtButtons->btn[bn].stt;
				else
					for (j=1;j<fvrtButtons->nbtns-1;j++)
						fvrtButtons->btn[j].stt = 1;
		}
		else if (bn==0)
			for (j=1;j<fvrtButtons->nbtns-1;j++)
				fvrtButtons->btn[j].stt = 0;
		else
			fvrtButtons->btn[bn].stt = 1 - fvrtButtons->btn[bn].stt;
	}
	else
	{
		if (bn==fvrtButtons->nbtns-1)
			for (j=1;j<fvrtButtons->nbtns-1;j++)
				fvrtButtons->btn[j].stt = 1;
		else if (bn==0)
			for (j=1;j<fvrtButtons->nbtns-1;j++)
				fvrtButtons->btn[j].stt = 0;
		else
			fvrtButtons->btn[bn].stt = 1 - fvrtButtons->btn[bn].stt;
	}
}

void FreeButtons()
{
	FreeButtonArray(&modeButtons);
	FreeButtonArray(&fthgButtons);
	FreeButtonArray(&flinButtons);
	FreeButtonArray(&fvrtButtons);
	FreeButtonArray(&fsecButtons);
	FreeButtonArray(&fdifButtons);
	FreeButtonArray(&fprvButtons);
	FreeButtonArray(&thngButtons);
	FreeButtonArray(&lineButtons);
	FreeButtonArray(&sectButtons);
	FreeButtonArray(&sideButtons);
	FreeButtonArray(&vertButtons);
}

void DrawTwoPartString(char *str,int ch,int x,int y,int col1,int col2,GrTextOption *to)
{
	char *p;
	int n=0;
	int oc;
	
	if (!str || !*str) return;

	oc = to->txo_fgcolor.v;
	p = strchr(str,ch);
	if (p)
	{
		n = p-str;
		*p = '\0';
		to->txo_fgcolor.v=col1;
		GrDrawString(str,strlen(str),x,y,to);
		*p = ch;
	}
	else p = str;
	
	to->txo_fgcolor.v=col2;
	GrDrawString(p,strlen(p),x+8*n,y,to);
	to->txo_fgcolor.v = oc;
}

void ViewerHelp()
{
	char buffer[256];
	int x,y;

	GrFilledBox(mapw.minx,mapw.miny,mapw.maxx,mapw.maxy,0);

	topt.txo_bgcolor.v=0;
	topt.txo_fgcolor.v=11;
	x = mapw.minx; y = mapw.miny;

	sprintf(buffer," Patcher Map Viewer %s       KEYS",PATCHER_VERSION);
	GrDrawString(buffer,strlen(buffer),x,y,&topt);

	y = mapw.miny+30;
	x = mapw.minx+0;
	topt.txo_fgcolor.v=14;
	sprintf(buffer,"  Scroll/Zoom");
	GrDrawString(buffer,strlen(buffer),x,y+0,&topt);
	topt.txo_fgcolor.v=10;
	sprintf(buffer," PGUP or +..Zoom In");
	DrawTwoPartString(buffer,'.',x,y+20,15,10,&topt);
	sprintf(buffer," PGDN or -..Zoom Out");
	DrawTwoPartString(buffer,'.',x,y+40,15,10,&topt);
	sprintf(buffer," LEFT.......Scroll Right");
	DrawTwoPartString(buffer,'.',x,y+60,15,10,&topt);
	sprintf(buffer," RIGHT......Scroll Left");
	DrawTwoPartString(buffer,'.',x,y+80,15,10,&topt);
	sprintf(buffer," UP.........Scroll Down");
	DrawTwoPartString(buffer,'.',x,y+100,15,10,&topt);
	sprintf(buffer," DOWN.......Scroll Up");
	DrawTwoPartString(buffer,'.',x,y+120,15,10,&topt);

	y = mapw.miny+30;
	x = mapw.minx+232;
	topt.txo_fgcolor.v=14;
	sprintf(buffer,"  Modes");
	GrDrawString(buffer,strlen(buffer),x,y+0,&topt);
	topt.txo_fgcolor.v=10;
	sprintf(buffer," T..........Thing mode");
	DrawTwoPartString(buffer,'.',x,y+20,15,10,&topt);
	sprintf(buffer," L..........Line mode");
	DrawTwoPartString(buffer,'.',x,y+40,15,10,&topt);
	sprintf(buffer," S..........Sector mode");
	DrawTwoPartString(buffer,'.',x,y+60,15,10,&topt);
	sprintf(buffer," V..........Vertex mode");
	DrawTwoPartString(buffer,'.',x,y+80,15,10,&topt);
	sprintf(buffer," D..........DOOM automap");
	DrawTwoPartString(buffer,'.',x,y+100,15,10,&topt);
	sprintf(buffer,"                          ");
	DrawTwoPartString(buffer,'.',x,y+120,15,10,&topt);

	y = mapw.miny+180;
	x = mapw.minx+0;
	topt.txo_fgcolor.v=14;
	sprintf(buffer,"  Commands");
	GrDrawString(buffer,strlen(buffer),x,y+0,&topt);
	topt.txo_fgcolor.v=10;
	sprintf(buffer," J..........Jump to object");
	DrawTwoPartString(buffer,'.',x,y+20,15,10,&topt);
	sprintf(buffer," C..........Clear marks");
	DrawTwoPartString(buffer,'.',x,y+40,15,10,&topt);
	sprintf(buffer," G..........Reduce grid");
	DrawTwoPartString(buffer,'.',x,y+60,15,10,&topt);
	sprintf(buffer," g..........Inc grid");
	DrawTwoPartString(buffer,'.',x,y+80,15,10,&topt);
	sprintf(buffer," K..........Kill grid");
	DrawTwoPartString(buffer,'.',x,y+100,15,10,&topt);
	sprintf(buffer," ALT-S......Save to BMP");
	DrawTwoPartString(buffer,'.',x,y+120,15,10,&topt);

	y = mapw.miny+180;
	x = mapw.minx+232;
	topt.txo_fgcolor.v=10;
	sprintf(buffer," ALT-N......Next level");
	DrawTwoPartString(buffer,'.',x,y+20,15,10,&topt);
	sprintf(buffer," ALT-P......Prev level");
	DrawTwoPartString(buffer,'.',x,y+40,15,10,&topt);
	sprintf(buffer," CTRL-F.....Full screen");
	DrawTwoPartString(buffer,'.',x,y+60,15,10,&topt);
	sprintf(buffer," F10........Command entry");
	DrawTwoPartString(buffer,'.',x,y+80,15,10,&topt);
	sprintf(buffer," ESC........Quit");
	DrawTwoPartString(buffer,'.',x,y+100,15,10,&topt);
	sprintf(buffer,"                      ");
	DrawTwoPartString(buffer,'.',x,y+120,15,10,&topt);
}

int viewmain(int argc,char **argv)
{
	int i,j;
	int wx,wy;
	char *p;
	GrMouseEvent me;
	int updatethgs=1;
	int updateco=1,updatemap=1;
	int updatedata=1,updatests=1;
	int updatebtn=1,updatebg=1;
	int updatemode=1;

	dprintf(logfile,"\n");

	CopyEnvironmentCLArgs("VIEW",argc,argv);
	if (ParseCommandLine(-1,-1))
	{
		FreeCLStrs();
		return 3;
	}

	if (nclfiles<1)
	{
		dprintf(logfile,"\nVIEW.EXE v1.0 by Jim Flynn, Copyright 1996-7  -- Freeware --\n",PATCHER_VERSION);
		dprintf(logfile,"\nView a DOOM or DOOM II map\n");
		dprintf(logfile,"\nUsage: VIEW [options] wadfile[.wad] [MapNN]\n");
		dprintf(logfile,"\nPress Alt-S to save the screen as a .BMP file\n");
		dprintf(logfile,"\nT selects Thing mode, S selects Sector mode\n");
		dprintf(logfile,"L selects Line mode, X selects BSP22X vpmark mode\n");
		dprintf(logfile,"D selects DOOM mode, CTRL-F selects full screen mode\n");
		dprintf(logfile,"\n General options, use SET VIEW=options to set defaults:\n");
		dprintf(logfile,"/R:nnn - specify horizontal resolution 640, 800, 1024, or 1280\n");
		dprintf(logfile,"/F - map view is full screen initially\n");
		dprintf(logfile,"/V - simplified view-only functions\n");
		dprintf(logfile,"/T:texturewad[.WAD] - read textures from texturewad\n");
		FreeCLStrs();
		return(1);
	}

	ReadExternalThingData("RADIUS.TXT");

	InitTics();

	strcpy(outname,"");
	strcpy(iwadname,"");
	for (i=0;i<nclopts;i++)
	{
		if (tolower(cloptions[i][0])=='r' && cloptions[i][1]==':')
		{
			HRES = atoi(cloptions[i]+2);
			switch (HRES)
			{
				case 640:
					VRES = 480;
					break;
				case 800:
					VRES = 600;
					break;
				case 1024:
					VRES = 768;
					break;
				case 1280:
					VRES = 1024;
					break;
				case 1600:
					VRES = 1280;
					break;
				default:
					HRES = 640;
					VRES = 480;
					break;
			}
		}
		else if (tolower(cloptions[i][0])=='f')
			FullScreen = cloptsense[i];
		else if (tolower(cloptions[i][0])=='v')
			ViewOnly = cloptsense[i];
		else if (tolower(cloptions[i][0])=='t' && cloptions[i][1]==':')
			strcpy(iwadname,cloptions[1]+2);
		else dprintf(logfile,"Warn: command line option: %s not understood\n",cloptions[i]);
	}

	strcpy(pwadname,clfiles[0]);
	strcpy(mapsel,"");
	if (nclfiles>1)
		strcpy(mapsel,clfiles[1]);
	FreeCLStrs();

	AddExt(pwadname,".WAD");
	AddExt(iwadname,".WAD");

	pst = fopen(pwadname,"rb");
	if (pst)
	{
		strcpy(basename,BaseOf(pwadname));
		strupr(basename);
		fread(&phdr,sizeof(WadHeader),1,pst);
		ReadDir(&phdr,&pdir,pst);

		if (IsIn("PNAMES",&pdir)>=0 && IsIn("TEXTURE1",&pdir)>=0)
		{
			ReadPNames(&pdir,pst,&pwt);
			ReadTextures(&pdir,pst,&pwt);
			texturesread=1;
		}
		else if (*iwadname)
		{
			bst = fopen(iwadname,"rb");
			if (bst)
			{
				fread(&bhdr,sizeof(WadHeader),1,bst);
				ReadDir(&bhdr,&bdir,bst);

				if (IsIn("PNAMES",&bdir)>=0 && IsIn("TEXTURE1",&bdir)>=0)
				{
					ReadPNames(&bdir,bst,&pwt);
					ReadTextures(&bdir,bst,&pwt);
					texturesread=1;
				}
				fclose(bst);
			}
		}

		if (!*mapsel)
			strcpy(mapsel,GetFirstLevel(&pdir));

		if (*mapsel && IsIn(mapsel,&pdir)>=0)
		{
			ReadLevelData(mapsel,&pdir,pst,11);
			for (i=NThings-1;i>=0;i--)
				if (Things[i].typ!=PLAYER1) break;
			firstmark = i+1;

			InitView(&view);
			GrSetMode(GR_width_height_color_graphics,HRES,VRES,16);
			topt.txo_font = &GrDefaultFont;
			topt.txo_fgcolor.v = 0;
			topt.txo_bgcolor.v = 7;
			topt.txo_direct=topt.txo_xalign=topt.txo_yalign=topt.txo_chrtype=0;
			eopt.txo_font = &GrDefaultFont;
			eopt.txo_fgcolor.v = 10;
			eopt.txo_bgcolor.v = 0;
			eopt.txo_direct=eopt.txo_xalign=eopt.txo_yalign=eopt.txo_chrtype=0;

			InitColors();
			CreateButtons();

			if ((mouse_enabled=MouseDetect()))
			{
				MouseEventMode(1);
				MouseInit();
				MouseSetColors(15,0);
			}
			mouse_blocked=0;
			MouseWarp((mapw.minx+mapw.maxx)/2,(mapw.miny+mapw.maxy)/2);
top:
			while (1)
			{
				if (CommandMode && !ViewOnly)
				{
					strcpy(buffer,"");
					InitGraphicData(edw.minx,edw.miny,edw.maxx,edw.maxy,&eopt);
					GrFramedBox(edw.minx,edw.miny,edw.maxx,edw.maxy,2,&infrm);
					SetEditLines(&vlines,&nvlines);
					p = EditCommand(buffer);
					if (p)
					{
						char fldname[32];
						char valstr[256];
						int op;

						ParseAtomicPredicate(buffer,fldname,&op,valstr);
						if (op==OP_ASS)
						{
							switch(Mode)
							{
								case M_VERTEX:
									AssignField(O_VERTEX,fldname,valstr,vrtsmarked,nvrtsmarked);
									sprintf(buffer,"Vertex %s %d %s",fldname,op,valstr);
									DisplayLine(buffer);
									updatemap=updatethgs=1;
									break;
								case M_THING:
									AssignField(O_THING,fldname,valstr,thgsmarked,nthgsmarked);
									sprintf(buffer,"Thing %s %d %s",fldname,op,valstr);
									DisplayLine(buffer);
									updatemap=updatethgs=1;
									break;
								case M_LINE:
									AssignField(O_LINEDEF,fldname,valstr,marked,nmarked);
									sprintf(buffer,"Line %s %d %s",fldname,op,valstr);
									DisplayLine(buffer);
									updatemap=updatethgs=1;
									break;
								case M_SECTOR:
									AssignField(O_SECTOR,fldname,valstr,secsmarked,nsecsmarked);
									sprintf(buffer,"Sector %s %d %s",fldname,op,valstr);
									DisplayLine(buffer);
									updatemap=updatethgs=1;
									break;
							}
						}

					}
					CommandMode = 0;
					updatemode = 1;
				}
				if (!FullScreen)
				{
					if (updatebg)
						DrawScreenBackground();
					updatebg=0;
					if (updatemode)
					{
						for (j=0;j<modeButtons->nbtns;j++)
							modeButtons->btn[j].stt = (Mode==j)? 1 : 0; 
						GrFilledBox(4,200,HRES-VRES+10,VRES-5,7);
						DrawModeButtons();
						DrawFieldButtons();
					}
				}
				if (updatemap || updatethgs)
				{
					if (mouse_enabled)
						mouse_blocked = MouseBlock(NULL,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy);

					ClearLines();
					GrFilledBox(mapw.minx,mapw.miny,mapw.maxx,mapw.maxy,0);
					if (updatemap || updatethgs)
						DrawMap(&view);
					updatemap=0;
					updatethgs=0;

					if (mouse_blocked)
						MouseUnBlock(mouse_blocked);
					mouse_blocked=0;
				}

				if (updatests || updateco || updatedata || updatebtn || updatemode)
				{
					if (mouse_enabled)
				 		mouse_blocked = MouseBlock(NULL,0,0,HRES-VRES+16,VRES-1);

					DrawStatus(last_mx,last_my,updateco,updatests,updatedata,updatebtn,updatemode);
					updateco=0;
					updatests=0;
					updatedata=0;
					updatebtn=0;
					updatemode=0;

					if (mouse_blocked)
						MouseUnBlock(mouse_blocked);
					mouse_blocked=0;
				}

				MouseGetEvent(M_EVENT,&me);
				acctime += me.dtime;

				if (me.flags&M_KEYPRESS)
				{
					if (me.key>500) me.key -= 256;
					switch(me.key)
					{
						case K_F1:
							ViewerHelp();
							WaitKey();
							updatemap=1;
							break;
						case K_F10:
							if (ViewOnly) break;
							CommandMode=1;
							goto top;
						case ALT_N:
							firstmark = ChgLevel(0);
							updatests=1;
							updatemap=1;
							updatemode=1;
							break;
						case ALT_P:
							firstmark = ChgLevel(1);
							updatests=1;
							updatemap=1;
							updatemode=1;
							break;
						case ALT_C:
							if (Mode==M_NODE)
								colfudge=1-colfudge;
							updatemap=1;
							break;
						case CTL_F:
							FullScreen=1-FullScreen;
							if (!FullScreen)
								WindowedView();
							else
								FullScreenView();
							updateco=1;
							updatebg=1;
							updatedata=1;
							updatests=1;
							updatebtn=1;
							updatemap=1;
							updatemode=1;
							break;
						case K_UP:
							ScrollDown(&view);
							updateco=1;
							updatedata=1;
							updatemap=1;
							break;
						case K_DN:
							ScrollUp(&view);
							updateco=1;
							updatedata=1;
							updatemap=1;
							break;
						case K_LT:
							ScrollLeft(&view);
							updateco=1;
							updatedata=1;
							updatemap=1;
							break;
						case K_RT:
							ScrollRight(&view);
							updateco=1;
							updatedata=1;
							updatemap=1;
							break;
						case K_PGUP: case '=': case '+':
							ZoomIn(&view,me.x,me.y);
							updateco=1;
							updatedata=1;
							updatemap=1;
							break;
						case K_PGDN: case '-': case '_':
							ZoomOut(&view,me.x,me.y);
							updateco=1;
							updatedata=1;
							updatemap=1;
							break;
						case ALT_S:
							SaveScreen();
							break;
						case ESC:
							goto exitprog;
						case 'g':
							if (grid==0)
								grid = 1024;
							else if (grid>1)
								grid = grid>>1;
							else grid = 0;
							gridinc=0;
							updatemap=1;
							updatests=1;
							break;
						case 'G':
							if (grid==0)
								grid = 1;
							else
								grid = (grid<<1)&1023;
							gridinc=1;
							updatemap=1;
							updatests=1;
							break;
						case 'j': case 'J':
							JumpToObject();
							updatemap=1;
							break;
						case 'k': case 'K':
							grid = 0;
							updatemap=1;
							updatests=1;
							break;
						case 'c': case 'C':
							ClearMarks();
							updatemap=1;
							break;
						case 't': case 'T':
							Mode = M_THING;
							updatedata=1;
							updatebtn=1;
							updatemap=1;
							updatests=1;
							updatemode=1;
							ClearMarks();
							break;
						case 'l': case 'L':
							Mode = M_LINE;
							updatedata=1;
							updatebtn=1;
							updatemap=1;
							updatests=1;
							updatemode=1;
							ClearMarks();
							break;
						case 's': case 'S':
							Mode = M_SECTOR;
							updatebtn=1;
							updatedata=1;
							updatemap=1;
							updatemode=1;
							updatests=1;
							break;
						case 'v': case 'V':
							Mode = M_VERTEX;
							updatedata=1;
							updatemap=1;
							updatebtn=1;
							updatests=1;
							updatemode=1;
							ClearMarks();
							break;
						case 'n': case 'N':
							if (ViewOnly) break;
							Mode = M_NODE;
							updatedata=1;
							updatebtn=1;
							updatemap=1;
							updatests=1;
							updatemode=1;
							ClearMarks();
							break;
						case 'd': case 'D':
							Mode = M_DOOM;
							updatedata=1;
							updatebtn=1;
							updatemap=1;
							updatests=1;
							updatemode=1;
							ClearMarks();
							break;
						case 'e': case 'E':
							if (Mode==M_THING)
							{
								fdifButtons->btn[0].stt = 1 - fdifButtons->btn[0].stt;
								fdifButtons->btn[1].stt = fdifButtons->btn[2].stt = 0;
								updatests=1;
								updatebtn=1;
								updatethgs=1;
							}
							break;
						case 'm': case 'M':
							if (Mode==M_THING)
							{
								fdifButtons->btn[1].stt = 1 - fdifButtons->btn[1].stt;
								fdifButtons->btn[0].stt = fdifButtons->btn[2].stt = 0;
								updatests=1;
								updatebtn=1;
								updatethgs=1;
							}
							break;
						case 'h': case 'H':
							if (Mode==M_THING)
							{
								fdifButtons->btn[2].stt = 1 - fdifButtons->btn[2].stt;
								fdifButtons->btn[1].stt = fdifButtons->btn[0].stt = 0;
								updatests=1;
								updatebtn=1;
								updatethgs=1;
							}
							break;
						case 'p': case 'P':
							if (Mode==M_THING)
							{
								fdifButtons->btn[5].stt = 1 - fdifButtons->btn[5].stt;
								updatests=1;
								updatebtn=1;
								updatethgs=1;
							}
							break;
						default:
							break;
					}
				}
				if (me.flags&M_MOTION)
				{
					if (!rubber && anchor_t>0 &&
						IsInBox(me.x,me.y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy) &&
						acctime-anchor_t>100)
					{
						MouseSetCursorMode(M_CUR_RUBBER,anchor_x,anchor_y,8);
						rubber=1;
					}

					last_mx = me.x;
					last_my = me.y;

					if (IsInBox(me.x,me.y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
					{
						updateco=1;
						updatedata=1;
						clrco=1;
					}
					else if (clrco)
					{
						GrFramedBox(cow.minx,cow.miny,cow.maxx,cow.maxy,2,&outfrm);
						clrco=0;
					}
				}

				if (me.flags&M_LEFT_UP)
				{
					if (rubber)
					{
						if (Mode==M_LINE)
						{
							for (i=0;i<NLinedefs;i++)
							{
								if
								(
									LinedefInBox
									(
										i,
										min(anchor_x,me.x),
										min(anchor_y,me.y),
										max(anchor_x,me.x),
										max(anchor_y,me.y)
									)
								)
								{
									if (!IsMarked(i))
									{
										AddDynInt(&marked,&nmarked,i);
										updatemap=1;
									}
								}
							}
						}
						else if (Mode==M_SECTOR)
						{
							S2W(me.x,me.y,&wx,&wy,&view);
							for (i=0;i<NSectors;i++)
								if
								(
									SectorInBox
									(
										i,
										min(anchor_x,me.x),
										min(anchor_y,me.y),
										max(anchor_x,me.x),
										max(anchor_y,me.y)
									)
								)
								{
									if (!SectorIsMarked(i))
									{
										AddDynInt(&secsmarked,&nsecsmarked,i);
										updatemap=1;
									}
								}
						}
						else if (Mode==M_THING)
						{
							S2W(me.x,me.y,&wx,&wy,&view);
							for (i=0;i<NThings;i++)
								if
								(
									ThingInBox
									(
										i,
										min(anchor_x,me.x),
										min(anchor_y,me.y),
										max(anchor_x,me.x),
										max(anchor_y,me.y)
									)
								)
								{
									if (!ThingIsMarked(i))
									{
										AddDynInt(&thgsmarked,&nthgsmarked,i);
										updatemap=1;
									}
								}
						}
						MouseSetCursorMode(M_CUR_NORMAL);
						rubber = 0;
					}
					anchor_t = 0;
				}

				if (me.flags&M_LEFT_DOWN)
				{
					if (anchor_t==0 && IsInBox(me.x,me.y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
					{
						anchor_x = me.x;
						anchor_y = me.y;
						anchor_t = acctime;
					}
					if (!FullScreen && SCREEN_TOP+3<me.y && WINDOW_TOP-4>me.y)
					{
						i = IsInModeButtons(me.x,me.y)-1;
						if (0<=i && i<modeButtons->nbtns && Mode!=i)
						{
								Mode = i;
								updatemode=1;
								updatedata=1;
								updatebtn=1;
								updatests=1;
								updatemap=1;
								ClearMarks();
						}
						i = IsInNxtPrvButtons(me.x,me.y)-1;
						if (0<=i && i<fprvButtons->nbtns)
						{
							if (i)
								ChgLevel(0);
							else
								ChgLevel(1);
							updatests=1;
							updatemap=1;
							updatemode=1;
						}
					}
					else if (Mode==M_VERTEX)
					{
						if (IsInBox(me.x,me.y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
						{
							//S2W(me.x,me.y,&wx,&wy,&view);
							//i = VertexOf(wx,wy);
							//if (i>=0)
							//{
							//	for (j=0;j<nvrtsmarked;j++)
							//		if (vrtsmarked[j]==i)
							//			break;
							//	if (j==nvrtsmarked)
							//		AddDynInt(&vrtsmarked,&nvrtsmarked,i);
							//	else
							//		DelDynInt(&vrtsmarked,&nvrtsmarked,j);
							//	updatemap=1;
							//}
						}
						else if (!FullScreen)
						{
							if ((i=IsInButtonArray(me.x,me.y,8,42,fvrtButtons)-1)>=0)
							{
								TogglefvrtButton(i);
								updatests=1;
								updatethgs=1;
							}
						}
					}
					else if (Mode==M_THING)
					{
						if (IsInBox(me.x,me.y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
						{
							S2W(me.x,me.y,&wx,&wy,&view);
							i = ThingOf(wx,wy);
							if (i>=0)
							{
								for (j=0;j<nthgsmarked;j++)
									if (thgsmarked[j]==i)
										break;
								if (j==nthgsmarked)
									AddDynInt(&thgsmarked,&nthgsmarked,i);
								else
									DelDynInt(&thgsmarked,&nthgsmarked,j);
								updatemap=1;
								updatests=1;
							}
						}
 						else if (!FullScreen)
						{
							if ((i=IsInButtonArray(me.x,me.y,fdifBx,fdifBy,fdifButtons)-1)>=0)
							{
								fdifButtons->btn[i].stt = 1-fdifButtons->btn[i].stt;
								if (i<3)
									for (j=0;j<3;j++)
										if (i!=j) fdifButtons->btn[j].stt = 0;
								updatemap=1;
								updatests=1;
							}
							else if ((i=IsInButtonArray(me.x,me.y,fthgBx,fthgBy,fthgButtons)-1)>=0)
							{
								if (i==0)
									for (j=1;j<fthgButtons->nbtns-1;j++)
										fthgButtons->btn[j].stt = 0;
								else if (i<fthgButtons->nbtns-1)
								{
									fthgButtons->btn[i].stt = 1-fthgButtons->btn[i].stt;
								}
								else
									for (j=1;j<fthgButtons->nbtns-1;j++)
										fthgButtons->btn[j].stt = 1;
								updatests=1;
								updatemap=1;
							}
						}
					}
					else if (Mode==M_SECTOR)
					{
						if (IsInBox(me.x,me.y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
						{
							S2W(me.x,me.y,&wx,&wy,&view);
							i = SectorOf(wx,wy);
							if (i>=0)
							{
								for (j=0;j<nsecsmarked;j++)
									if (secsmarked[j]==i)
										break;
								if (j==nsecsmarked)
									AddDynInt(&secsmarked,&nsecsmarked,i);
								else
									DelDynInt(&secsmarked,&nsecsmarked,j);
								updatemap=1;
								updatests=1;
							}
						}
						else if (Mode==M_NODE)
						{
							if (IsInBox(me.x,me.y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
							{
								S2W(me.x,me.y,&wx,&wy,&view);
								i = SubSectorOf(wx,wy);
								if (i>=0)
								{
									updatemap=1;
									updatests=1;
								}
							}
						}
						else if (!FullScreen)
						{
							if ((i=IsInButtonArray(me.x,me.y,fsecBx,fsecBy,fsecButtons)-1)>=0)
							{
								if (i==0)
									for (j=1;j<fsecButtons->nbtns-1;j++)
										fsecButtons->btn[j].stt = 0;
								else if (i<fsecButtons->nbtns-1)
									fsecButtons->btn[i].stt = 1-fsecButtons->btn[i].stt;
								else
									for (j=1;j<fsecButtons->nbtns-1;j++)
										fsecButtons->btn[j].stt = 1;
								updatests=1;
								updatemap=1;
							}
						}
					}
					else if (Mode==M_LINE)
					{
						if (IsInBox(me.x,me.y,mapw.minx,mapw.miny,mapw.maxx,mapw.maxy))
						{
							S2W(me.x,me.y,&wx,&wy,&view);
							i = LineOf(wx,wy,LINE_TOLERANCE);
							if (i>=0)
							{
								for (j=0;j<nmarked;j++)
									if (marked[j]==i)
										break;
								if (j==nmarked)
									AddDynInt(&marked,&nmarked,i);
								else
									DelDynInt(&marked,&nmarked,j);
								updatemap=1;
								updatests=1;
							}
						}
						else if (!FullScreen)
						{
							if ((i=IsInButtonArray(me.x,me.y,flinBx,flinBy,flinButtons)-1)>=0)
							{
								if (i<2)
									for (j=2;j<flinButtons->nbtns-2;j++)
										flinButtons->btn[j].stt = 0;
								else if (i<flinButtons->nbtns-2)
									flinButtons->btn[i].stt = 1-flinButtons->btn[i].stt;
								else
									for (j=2;j<flinButtons->nbtns-2;j++)
										flinButtons->btn[j].stt = 1;
								updatests=1;
								updatemap=1;
							}
						}
					}
				}
			}
		}
		else dprintf(logfile,"Level not found\n");
	}
exitprog:
	FreeButtons();
	ClearMarks();
	if (mouse_enabled) MouseUnInit();
	GrSetMode(GR_default_text);
	fclose(pst);
	return 0;
}

