#include "patch.h"

/************************************************************
 WDIR.C
			 /-------------------------\
			|| Wad Directory Routines  ||
			 \-------------------------/
									
************************************************************/


/* Initialize a new directory */

void InitDir(WadDir *d)
{
	d->allentries=0;
	d->nentries=0;
	d->dirp = NULL;
}

/* Free a directory */

void FreeDir(WadDir *d)
{
	d->allentries=0;
	d->nentries=0;
	free(d->dirp);
	d->dirp = NULL;
}

/* set a wad directory entry to values passed in */

void SetDirEntry(WadDirEntry *wde,unsigned long iresptr, unsigned long ireslen, char *iresname)
{
	wde->resptr = iresptr;
	wde->reslen = ireslen;
	strncpy(wde->resname,iresname,8);
}

/* Wad directory routines*/

/* Add a directory entry to the wad, allocate more memory if necessary
   and initialize the new directory entry */

int AddDirEntry(WadDir *wdd,unsigned long iresptr, unsigned long ireslen, char *iresname)
{
	char resn[16];

	if (wdd->nentries>=wdd->allentries)
	{
		wdd->allentries += 256;
		wdd->dirp = realloc(wdd->dirp,wdd->allentries*sizeof(WadDirEntry));
	}

	strcpy(resn,Str8(iresname));
	strupr(resn);
	ConvertMarker(resn);	// translate X_END/START <-> XX_END/START
		
	SetDirEntry(&wdd->dirp[wdd->nentries++],iresptr,ireslen,resn);
	return 0;
}

/* Insert a directory entry to the wad, allocate more memory if necessary
   and initialize the new directory entry */

int InsDirEntry(WadDir *wdd,int inspt,unsigned long iresptr, unsigned long ireslen, char *iresname)
{
	int j;

	if (wdd->nentries>=wdd->allentries)
	{
		wdd->allentries += 256;
		wdd->dirp = realloc(wdd->dirp,wdd->allentries*sizeof(WadDirEntry));
	}
	if (inspt<wdd->nentries-1)
		for (j=wdd->nentries-1;j>=inspt;j--)
			wdd->dirp[j+1]=wdd->dirp[j];
	
	SetDirEntry(&wdd->dirp[inspt],iresptr,ireslen,iresname);
	return 0;
}

/* Read the directory from a wad file with header hdr to the specified
   dir structure */

int ReadDir(WadHeader *hdr,WadDir *dir,FILE *st)
{
	WadDirEntry wde;
	int entry;

	fseek(st,hdr->waddiraddr,SEEK_SET);
	for (entry=0;entry<hdr->waddirlen;entry++)
	{
		fread(&wde,sizeof(WadDirEntry),1,st);
		AddDirEntry(dir,wde.resptr,wde.reslen,Str8(wde.resname));
	}
	return 0;
}

/* Get location of resource in directory or return -1 */

int IsIn(char *res,WadDir *d)
{
	int i;

	for (i=0;i<d->nentries;i++)
		if (strnicmp(res,d->dirp[i].resname,8)==0) return i;

	return -1;
}

int IsAfter(int n,char *res,WadDir *d)
{
	int i;

	for (i=n;i<d->nentries;i++)
		if (strnicmp(res,d->dirp[i].resname,8)==0) return i;

	return -1;
}

int IsBetween(int n,int m,char *res,WadDir *d)
{
	int i;

	for (i=n;i<=m;i++)
		if (strnicmp(res,d->dirp[i].resname,8)==0) return i;

	return -1;
}

/* Dump a directory entry */

void PrintDirEntry(WadDirEntry *wde)
{
	dprintf(logfile,"%9s Off:%8x Len:%8x\n",Str8(wde->resname),(unsigned)wde->resptr,(unsigned)wde->reslen);
}

/* Dump a directory */

void PrintWadDir(WadDir *d)
{
	long i;
	
	for (i=0;i<d->nentries;i++) PrintDirEntry(&d->dirp[i]);
}

/* Dump directory and header */

void PrintDir(WadDir *wdir)
{
	dprintf(logfile,"Wad directory:\n");
	dprintf(logfile,"RESOURCE  OFFSET   LENGTH\n");
	dprintf(logfile,"-------------------------\n");
	PrintWadDir(wdir);
	dprintf(logfile,"\n\n");
}


